macro_rules! int_impl {
    ($SelfT:ty, $ActualT:ident, $UnsignedT:ty, $BITS:expr, $BITS_MINUS_ONE:expr, $Min:expr, $Max:expr,
     $rot:expr, $rot_op:expr, $rot_result:expr, $swap_op:expr, $swapped:expr,
     $reversed:expr, $le_bytes:expr, $be_bytes:expr,
     $to_xe_bytes_doc:expr, $from_xe_bytes_doc:expr) => {
        /// The smallest value that can be represented by this integer type, <br>此整数类型可以表示的最小值，<br>
        #[doc = concat!("&minus;2<sup>", $BITS_MINUS_ONE, "</sup>.")]
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN, ", stringify!($Min), ");")]
        /// ```
        #[stable(feature = "assoc_int_consts", since = "1.43.0")]
        pub const MIN: Self = !0 ^ ((!0 as $UnsignedT) >> 1) as Self;

        /// The largest value that can be represented by this integer type, <br>此整数类型可以表示的最大值，<br>
        #[doc = concat!("2<sup>", $BITS_MINUS_ONE, "</sup> &minus; 1.")]
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MAX, ", stringify!($Max), ");")]
        /// ```
        #[stable(feature = "assoc_int_consts", since = "1.43.0")]
        pub const MAX: Self = !Self::MIN;

        /// The size of this integer type in bits. <br>此整数类型的大小 (以位为单位)。<br>
        ///
        /// # Examples
        ///
        /// ```
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::BITS, ", stringify!($BITS), ");")]
        /// ```
        #[stable(feature = "int_bits_const", since = "1.53.0")]
        pub const BITS: u32 = $BITS;

        /// Converts a string slice in a given base to an integer. <br>将给定基数的字符串切片转换为整数。<br>
        ///
        /// The string is expected to be an optional `+` or `-` sign followed by digits. <br>该字符串应为可选的 `+` 或 `-` 符号，后跟数字。<br>
        /// Leading and trailing whitespace represent an error. <br>前导和尾随空格表示错误。<br>
        /// Digits are a subset of these characters, depending on `radix`: <br>根据 `radix`，数字是这些字符的子集：<br>
        ///
        ///  * `0-9`
        ///  * `a-z`
        ///  * `A-Z`
        ///
        /// # Panics
        ///
        /// This function panics if `radix` is not in the range from 2 to 36. <br>如果 `radix` 不在 2 到 36 之间，则此函数 panics。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::from_str_radix(\"A\", 16), Ok(10));")]
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        pub fn from_str_radix(src: &str, radix: u32) -> Result<Self, ParseIntError> {
            from_str_radix(src, radix)
        }

        /// Returns the number of ones in the binary representation of `self`. <br>返回 `self` 二进制表示形式中的位数。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let n = 0b100_0000", stringify!($SelfT), ";")]
        ///
        /// assert_eq!(n.count_ones(), 1);
        /// ```
        ///
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[doc(alias = "popcount")]
        #[doc(alias = "popcnt")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn count_ones(self) -> u32 { (self as $UnsignedT).count_ones() }

        /// Returns the number of zeros in the binary representation of `self`. <br>返回 `self` 二进制表示形式中的零数。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MAX.count_zeros(), 1);")]
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn count_zeros(self) -> u32 {
            (!self).count_ones()
        }

        /// Returns the number of leading zeros in the binary representation of `self`. <br>返回 `self` 二进制表示形式中前导零的数目。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let n = -1", stringify!($SelfT), ";")]
        ///
        /// assert_eq!(n.leading_zeros(), 0);
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn leading_zeros(self) -> u32 {
            (self as $UnsignedT).leading_zeros()
        }

        /// Returns the number of trailing zeros in the binary representation of `self`. <br>返回 `self` 二进制表示形式中的尾随零数。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let n = -4", stringify!($SelfT), ";")]
        ///
        /// assert_eq!(n.trailing_zeros(), 2);
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn trailing_zeros(self) -> u32 {
            (self as $UnsignedT).trailing_zeros()
        }

        /// Returns the number of leading ones in the binary representation of `self`. <br>返回 `self` 二进制表示形式中前导数字。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let n = -1", stringify!($SelfT), ";")]
        ///
        #[doc = concat!("assert_eq!(n.leading_ones(), ", stringify!($BITS), ");")]
        /// ```
        #[stable(feature = "leading_trailing_ones", since = "1.46.0")]
        #[rustc_const_stable(feature = "leading_trailing_ones", since = "1.46.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn leading_ones(self) -> u32 {
            (self as $UnsignedT).leading_ones()
        }

        /// Returns the number of trailing ones in the binary representation of `self`. <br>返回 `self` 二进制表示形式中的尾随数字。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let n = 3", stringify!($SelfT), ";")]
        ///
        /// assert_eq!(n.trailing_ones(), 2);
        /// ```
        #[stable(feature = "leading_trailing_ones", since = "1.46.0")]
        #[rustc_const_stable(feature = "leading_trailing_ones", since = "1.46.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn trailing_ones(self) -> u32 {
            (self as $UnsignedT).trailing_ones()
        }

        /// Shifts the bits to the left by a specified amount, `n`, wrapping the truncated bits to the end of the resulting integer. <br>将位左移指定的量 `n`，将截断的位包装到结果整数的末尾。<br>
        ///
        ///
        /// Please note this isn't the same operation as the `<<` shifting operator! <br>请注意，此操作与 `<<` 移位运算符不同！<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let n = ", $rot_op, stringify!($SelfT), ";")]
        #[doc = concat!("let m = ", $rot_result, ";")]
        ///
        #[doc = concat!("assert_eq!(n.rotate_left(", $rot, "), m);")]
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn rotate_left(self, n: u32) -> Self {
            (self as $UnsignedT).rotate_left(n) as Self
        }

        /// Shifts the bits to the right by a specified amount, `n`, wrapping the truncated bits to the beginning of the resulting integer. <br>将位右移指定的量 `n`，将截断的位包装到结果整数的开头。<br>
        ///
        ///
        /// Please note this isn't the same operation as the `>>` shifting operator! <br>请注意，此操作与 `>>` 移位运算符不同！<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let n = ", $rot_result, stringify!($SelfT), ";")]
        #[doc = concat!("let m = ", $rot_op, ";")]
        ///
        #[doc = concat!("assert_eq!(n.rotate_right(", $rot, "), m);")]
        /// ```
        ///
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn rotate_right(self, n: u32) -> Self {
            (self as $UnsignedT).rotate_right(n) as Self
        }

        /// Reverses the byte order of the integer. <br>反转整数的字节顺序。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let n = ", $swap_op, stringify!($SelfT), ";")]
        ///
        /// let m = n.swap_bytes();
        ///
        #[doc = concat!("assert_eq!(m, ", $swapped, ");")]
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn swap_bytes(self) -> Self {
            (self as $UnsignedT).swap_bytes() as Self
        }

        /// Reverses the order of bits in the integer. <br>反转整数中的位顺序。<br>
        /// The least significant bit becomes the most significant bit, second least-significant bit becomes second most-significant bit, etc. <br>最低有效位变为最高有效位，第二最低有效位变为第二最高有效位，依此类推。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let n = ", $swap_op, stringify!($SelfT), ";")]
        /// let m = n.reverse_bits();
        ///
        #[doc = concat!("assert_eq!(m, ", $reversed, ");")]
        #[doc = concat!("assert_eq!(0, 0", stringify!($SelfT), ".reverse_bits());")]
        /// ```
        #[stable(feature = "reverse_bits", since = "1.37.0")]
        #[rustc_const_stable(feature = "reverse_bits", since = "1.37.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn reverse_bits(self) -> Self {
            (self as $UnsignedT).reverse_bits() as Self
        }

        /// Converts an integer from big endian to the target's endianness. <br>将整数从大端字节序转换为目标的字节序。<br>
        ///
        /// On big endian this is a no-op. <br>在大端节序序上，这是个禁忌。<br> On little endian the bytes are swapped. <br>在小端字节序上，字节被交换。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let n = 0x1A", stringify!($SelfT), ";")]
        ///
        /// if cfg!(target_endian = "big") {
        #[doc = concat!("    assert_eq!(", stringify!($SelfT), "::from_be(n), n)")]
        /// } else {
        #[doc = concat!("    assert_eq!(", stringify!($SelfT), "::from_be(n), n.swap_bytes())")]
        /// }
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_conversions", since = "1.32.0")]
        #[must_use]
        #[inline]
        pub const fn from_be(x: Self) -> Self {
            #[cfg(target_endian = "big")]
            {
                x
            }
            #[cfg(not(target_endian = "big"))]
            {
                x.swap_bytes()
            }
        }

        /// Converts an integer from little endian to the target's endianness. <br>将整数从小端字节序转换为目标的字节序。<br>
        ///
        /// On little endian this is a no-op. <br>在小端字节序上，这是个禁忌。<br> On big endian the bytes are swapped. <br>在大字节序中，字节被交换。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let n = 0x1A", stringify!($SelfT), ";")]
        ///
        /// if cfg!(target_endian = "little") {
        #[doc = concat!("    assert_eq!(", stringify!($SelfT), "::from_le(n), n)")]
        /// } else {
        #[doc = concat!("    assert_eq!(", stringify!($SelfT), "::from_le(n), n.swap_bytes())")]
        /// }
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_conversions", since = "1.32.0")]
        #[must_use]
        #[inline]
        pub const fn from_le(x: Self) -> Self {
            #[cfg(target_endian = "little")]
            {
                x
            }
            #[cfg(not(target_endian = "little"))]
            {
                x.swap_bytes()
            }
        }

        /// Converts `self` to big endian from the target's endianness. <br>将 `self` 从目标的字节序转换为大字节序。<br>
        ///
        /// On big endian this is a no-op. <br>在大端节序序上，这是个禁忌。<br> On little endian the bytes are swapped. <br>在小端字节序上，字节被交换。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let n = 0x1A", stringify!($SelfT), ";")]
        ///
        /// if cfg!(target_endian = "big") {
        ///     assert_eq!(n.to_be(), n)
        /// } else {
        ///     assert_eq!(n.to_be(), n.swap_bytes())
        /// }
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_conversions", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn to_be(self) -> Self { // or not to be? <br>还是不是？<br>
            #[cfg(target_endian = "big")]
            {
                self
            }
            #[cfg(not(target_endian = "big"))]
            {
                self.swap_bytes()
            }
        }

        /// Converts `self` to little endian from the target's endianness. <br>将 `self` 从目标的字节序转换为 Little Endian。<br>
        ///
        /// On little endian this is a no-op. <br>在小端字节序上，这是个禁忌。<br> On big endian the bytes are swapped. <br>在大字节序中，字节被交换。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let n = 0x1A", stringify!($SelfT), ";")]
        ///
        /// if cfg!(target_endian = "little") {
        ///     assert_eq!(n.to_le(), n)
        /// } else {
        ///     assert_eq!(n.to_le(), n.swap_bytes())
        /// }
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_conversions", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn to_le(self) -> Self {
            #[cfg(target_endian = "little")]
            {
                self
            }
            #[cfg(not(target_endian = "little"))]
            {
                self.swap_bytes()
            }
        }

        /// Checked integer addition. <br>检查整数加法。<br>
        /// Computes `self + rhs`, returning `None` if overflow occurred. <br>计算 `self + rhs`，如果发生溢出则返回 `None`。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!((", stringify!($SelfT), "::MAX - 2).checked_add(1), Some(", stringify!($SelfT), "::MAX - 1));")]
        #[doc = concat!("assert_eq!((", stringify!($SelfT), "::MAX - 2).checked_add(3), None);")]
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_checked_int_methods", since = "1.47.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_add(self, rhs: Self) -> Option<Self> {
            let (a, b) = self.overflowing_add(rhs);
            if unlikely!(b) {None} else {Some(a)}
        }

        /// Unchecked integer addition. <br>未经检查的整数加法。<br>
        /// Computes `self + rhs`, assuming overflow cannot occur. <br>假设不会发生溢出，则计算 `self + rhs`。<br>
        ///
        /// # Safety
        ///
        /// This results in undefined behavior when <br>当以下情况时，这导致未定义的行为<br>
        #[doc = concat!("`self + rhs > ", stringify!($SelfT), "::MAX` or `self + rhs < ", stringify!($SelfT), "::MIN`,")]
        /// i.e. when [`checked_add`] would return `None`. <br>即当 [`checked_add`] 将返回 `None` 时。<br>
        ///
        #[doc = concat!("[`checked_add`]: ", stringify!($SelfT), "::checked_add")]
        #[unstable(
            feature = "unchecked_math",
            reason = "niche optimization path",
            issue = "85122",
        )]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[rustc_const_unstable(feature = "const_inherent_unchecked_arith", issue = "85122")]
        #[inline(always)]
        pub const unsafe fn unchecked_add(self, rhs: Self) -> Self {
            // SAFETY: the caller must uphold the safety contract for `unchecked_add`. <br>调用者必须遵守 `unchecked_add` 的安全保证。<br>
            //
            unsafe { intrinsics::unchecked_add(self, rhs) }
        }

        /// Checked addition with an unsigned integer. <br>用无符号整数检查加法。<br>
        /// Computes `self + rhs`, returning `None` if overflow occurred. <br>计算 `self + rhs`，如果发生溢出则返回 `None`。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        /// # #![feature(mixed_integer_ops)]
        #[doc = concat!("assert_eq!(1", stringify!($SelfT), ".checked_add_unsigned(2), Some(3));")]
        #[doc = concat!("assert_eq!((", stringify!($SelfT), "::MAX - 2).checked_add_unsigned(3), None);")]
        /// ```
        #[unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[rustc_const_unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_add_unsigned(self, rhs: $UnsignedT) -> Option<Self> {
            let (a, b) = self.overflowing_add_unsigned(rhs);
            if unlikely!(b) {None} else {Some(a)}
        }

        /// Checked integer subtraction. <br>检查整数减法。<br>
        /// Computes `self - rhs`, returning `None` if overflow occurred. <br>计算 `self - rhs`，如果发生溢出则返回 `None`。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!((", stringify!($SelfT), "::MIN + 2).checked_sub(1), Some(", stringify!($SelfT), "::MIN + 1));")]
        #[doc = concat!("assert_eq!((", stringify!($SelfT), "::MIN + 2).checked_sub(3), None);")]
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_checked_int_methods", since = "1.47.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_sub(self, rhs: Self) -> Option<Self> {
            let (a, b) = self.overflowing_sub(rhs);
            if unlikely!(b) {None} else {Some(a)}
        }

        /// Unchecked integer subtraction. <br>未经检查的整数减法。<br>
        /// Computes `self - rhs`, assuming overflow cannot occur. <br>假设不会发生溢出，则计算 `self - rhs`。<br>
        ///
        /// # Safety
        ///
        /// This results in undefined behavior when <br>当以下情况时，这导致未定义的行为<br>
        #[doc = concat!("`self - rhs > ", stringify!($SelfT), "::MAX` or `self - rhs < ", stringify!($SelfT), "::MIN`,")]
        /// i.e. when [`checked_sub`] would return `None`. <br>即当 [`checked_sub`] 将返回 `None` 时。<br>
        ///
        #[doc = concat!("[`checked_sub`]: ", stringify!($SelfT), "::checked_sub")]
        #[unstable(
            feature = "unchecked_math",
            reason = "niche optimization path",
            issue = "85122",
        )]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[rustc_const_unstable(feature = "const_inherent_unchecked_arith", issue = "85122")]
        #[inline(always)]
        pub const unsafe fn unchecked_sub(self, rhs: Self) -> Self {
            // SAFETY: the caller must uphold the safety contract for `unchecked_sub`. <br>调用者必须遵守 `unchecked_sub` 的安全保证。<br>
            //
            unsafe { intrinsics::unchecked_sub(self, rhs) }
        }

        /// Checked subtraction with an unsigned integer. <br>用无符号整数检查减法。<br>
        /// Computes `self - rhs`, returning `None` if overflow occurred. <br>计算 `self - rhs`，如果发生溢出则返回 `None`。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        /// # #![feature(mixed_integer_ops)]
        #[doc = concat!("assert_eq!(1", stringify!($SelfT), ".checked_sub_unsigned(2), Some(-1));")]
        #[doc = concat!("assert_eq!((", stringify!($SelfT), "::MIN + 2).checked_sub_unsigned(3), None);")]
        /// ```
        #[unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[rustc_const_unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_sub_unsigned(self, rhs: $UnsignedT) -> Option<Self> {
            let (a, b) = self.overflowing_sub_unsigned(rhs);
            if unlikely!(b) {None} else {Some(a)}
        }

        /// Checked integer multiplication. <br>检查整数乘法。<br>
        /// Computes `self * rhs`, returning `None` if overflow occurred. <br>计算 `self * rhs`，如果发生溢出则返回 `None`。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MAX.checked_mul(1), Some(", stringify!($SelfT), "::MAX));")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MAX.checked_mul(2), None);")]
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_checked_int_methods", since = "1.47.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_mul(self, rhs: Self) -> Option<Self> {
            let (a, b) = self.overflowing_mul(rhs);
            if unlikely!(b) {None} else {Some(a)}
        }

        /// Unchecked integer multiplication. <br>未经检查的整数乘法。<br>
        /// Computes `self * rhs`, assuming overflow cannot occur. <br>假设不会发生溢出，则计算 `self * rhs`。<br>
        ///
        /// # Safety
        ///
        /// This results in undefined behavior when <br>当以下情况时，这导致未定义的行为<br>
        #[doc = concat!("`self * rhs > ", stringify!($SelfT), "::MAX` or `self * rhs < ", stringify!($SelfT), "::MIN`,")]
        /// i.e. when [`checked_mul`] would return `None`. <br>即当 [`checked_mul`] 将返回 `None` 时。<br>
        ///
        #[doc = concat!("[`checked_mul`]: ", stringify!($SelfT), "::checked_mul")]
        #[unstable(
            feature = "unchecked_math",
            reason = "niche optimization path",
            issue = "85122",
        )]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[rustc_const_unstable(feature = "const_inherent_unchecked_arith", issue = "85122")]
        #[inline(always)]
        pub const unsafe fn unchecked_mul(self, rhs: Self) -> Self {
            // SAFETY: the caller must uphold the safety contract for `unchecked_mul`. <br>调用者必须遵守 `unchecked_mul` 的安全保证。<br>
            //
            unsafe { intrinsics::unchecked_mul(self, rhs) }
        }

        /// Checked integer division. <br>检查整数除法。<br>
        /// Computes `self / rhs`, returning `None` if `rhs == 0` or the division results in overflow. <br>计算 `self / rhs`，如果 `rhs == 0` 或除法导致溢出，则返回 `None`。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!((", stringify!($SelfT), "::MIN + 1).checked_div(-1), Some(", stringify!($Max), "));")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.checked_div(-1), None);")]
        #[doc = concat!("assert_eq!((1", stringify!($SelfT), ").checked_div(0), None);")]
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_checked_int_div", since = "1.52.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_div(self, rhs: Self) -> Option<Self> {
            if unlikely!(rhs == 0 || ((self == Self::MIN) && (rhs == -1))) {
                None
            } else {
                // SAFETY: div by zero and by INT_MIN have been checked above <br>上面已经检查了 div 的零和 INT_MIN 的值<br>
                Some(unsafe { intrinsics::unchecked_div(self, rhs) })
            }
        }

        /// Checked Euclidean division. <br>检查欧几里得除法。<br>
        /// Computes `self.div_euclid(rhs)`, returning `None` if `rhs == 0` or the division results in overflow. <br>计算 `self.div_euclid(rhs)`，如果 `rhs == 0` 或除法导致溢出，则返回 `None`。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!((", stringify!($SelfT), "::MIN + 1).checked_div_euclid(-1), Some(", stringify!($Max), "));")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.checked_div_euclid(-1), None);")]
        #[doc = concat!("assert_eq!((1", stringify!($SelfT), ").checked_div_euclid(0), None);")]
        /// ```
        #[stable(feature = "euclidean_division", since = "1.38.0")]
        #[rustc_const_stable(feature = "const_euclidean_int_methods", since = "1.52.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_div_euclid(self, rhs: Self) -> Option<Self> {
            // Using `&` helps LLVM see that it is the same check made in division. <br>使用 `&` 有助于 LLVM 看到它是在除法中进行的相同检查。<br>
            if unlikely!(rhs == 0 || ((self == Self::MIN) & (rhs == -1))) {
                None
            } else {
                Some(self.div_euclid(rhs))
            }
        }

        /// Checked integer remainder. <br>检查整数余数。<br>
        /// Computes `self % rhs`, returning `None` if `rhs == 0` or the division results in overflow. <br>计算 `self % rhs`，如果 `rhs == 0` 或除法导致溢出，则返回 `None`。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        ///
        #[doc = concat!("assert_eq!(5", stringify!($SelfT), ".checked_rem(2), Some(1));")]
        #[doc = concat!("assert_eq!(5", stringify!($SelfT), ".checked_rem(0), None);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.checked_rem(-1), None);")]
        /// ```
        #[stable(feature = "wrapping", since = "1.7.0")]
        #[rustc_const_stable(feature = "const_checked_int_div", since = "1.52.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_rem(self, rhs: Self) -> Option<Self> {
            if unlikely!(rhs == 0 || ((self == Self::MIN) && (rhs == -1))) {
                None
            } else {
                // SAFETY: div by zero and by INT_MIN have been checked above <br>上面已经检查了 div 的零和 INT_MIN 的值<br>
                Some(unsafe { intrinsics::unchecked_rem(self, rhs) })
            }
        }

        /// Checked Euclidean remainder. <br>检查欧几里得的余数。<br>
        /// Computes `self.rem_euclid(rhs)`, returning `None` if `rhs == 0` or the division results in overflow. <br>计算 `self.rem_euclid(rhs)`，如果 `rhs == 0` 或除法导致溢出，则返回 `None`。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(5", stringify!($SelfT), ".checked_rem_euclid(2), Some(1));")]
        #[doc = concat!("assert_eq!(5", stringify!($SelfT), ".checked_rem_euclid(0), None);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.checked_rem_euclid(-1), None);")]
        /// ```
        #[stable(feature = "euclidean_division", since = "1.38.0")]
        #[rustc_const_stable(feature = "const_euclidean_int_methods", since = "1.52.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_rem_euclid(self, rhs: Self) -> Option<Self> {
            // Using `&` helps LLVM see that it is the same check made in division. <br>使用 `&` 有助于 LLVM 看到它是在除法中进行的相同检查。<br>
            if unlikely!(rhs == 0 || ((self == Self::MIN) & (rhs == -1))) {
                None
            } else {
                Some(self.rem_euclid(rhs))
            }
        }

        /// Checked negation. <br>检查否定。<br> Computes `-self`, returning `None` if `self == MIN`. <br>计算 `-self`，如果为 `self == MIN`，则返回 `None`。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        ///
        #[doc = concat!("assert_eq!(5", stringify!($SelfT), ".checked_neg(), Some(-5));")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.checked_neg(), None);")]
        /// ```
        #[stable(feature = "wrapping", since = "1.7.0")]
        #[rustc_const_stable(feature = "const_checked_int_methods", since = "1.47.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_neg(self) -> Option<Self> {
            let (a, b) = self.overflowing_neg();
            if unlikely!(b) {None} else {Some(a)}
        }

        /// Checked shift left. <br>检查左移。<br>
        /// Computes `self << rhs`, returning `None` if `rhs` is larger than or equal to the number of bits in `self`. <br>计算 `self << rhs`，如果 `rhs` 大于或等于 `self` 中的位数，则返回 `None`。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(0x1", stringify!($SelfT), ".checked_shl(4), Some(0x10));")]
        #[doc = concat!("assert_eq!(0x1", stringify!($SelfT), ".checked_shl(129), None);")]
        /// ```
        #[stable(feature = "wrapping", since = "1.7.0")]
        #[rustc_const_stable(feature = "const_checked_int_methods", since = "1.47.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_shl(self, rhs: u32) -> Option<Self> {
            let (a, b) = self.overflowing_shl(rhs);
            if unlikely!(b) {None} else {Some(a)}
        }

        /// Unchecked shift left. <br>未检查的左移。<br>
        /// Computes `self << rhs`, assuming that `rhs` is less than the number of bits in `self`. <br>计算 `self << rhs`，假设 `rhs` 小于 `self` 中的位数。<br>
        ///
        /// # Safety
        ///
        /// This results in undefined behavior if `rhs` is larger than or equal to the number of bits in `self`, i.e. <br>如果 `rhs` 大于或等于 `self` 中的位数，则会导致未定义的行为，即<br>
        ///
        /// when [`checked_shl`] would return `None`. <br>当 [`checked_shl`] 返回 `None` 时。<br>
        ///
        #[doc = concat!("[`checked_shl`]: ", stringify!($SelfT), "::checked_shl")]
        #[unstable(
            feature = "unchecked_math",
            reason = "niche optimization path",
            issue = "85122",
        )]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[rustc_const_unstable(feature = "const_inherent_unchecked_arith", issue = "85122")]
        #[inline(always)]
        pub const unsafe fn unchecked_shl(self, rhs: Self) -> Self {
            // SAFETY: the caller must uphold the safety contract for `unchecked_shl`. <br>调用者必须遵守 `unchecked_shl` 的安全保证。<br>
            //
            unsafe { intrinsics::unchecked_shl(self, rhs) }
        }

        /// Checked shift right. <br>检查右移。<br>
        /// Computes `self >> rhs`, returning `None` if `rhs` is larger than or equal to the number of bits in `self`. <br>计算 `self >> rhs`，如果 `rhs` 大于或等于 `self` 中的位数，则返回 `None`。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(0x10", stringify!($SelfT), ".checked_shr(4), Some(0x1));")]
        #[doc = concat!("assert_eq!(0x10", stringify!($SelfT), ".checked_shr(128), None);")]
        /// ```
        #[stable(feature = "wrapping", since = "1.7.0")]
        #[rustc_const_stable(feature = "const_checked_int_methods", since = "1.47.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_shr(self, rhs: u32) -> Option<Self> {
            let (a, b) = self.overflowing_shr(rhs);
            if unlikely!(b) {None} else {Some(a)}
        }

        /// Unchecked shift right. <br>未检查右移。<br>
        /// Computes `self >> rhs`, assuming that `rhs` is less than the number of bits in `self`. <br>计算 `self >> rhs`，假设 `rhs` 小于 `self` 中的位数。<br>
        ///
        /// # Safety
        ///
        /// This results in undefined behavior if `rhs` is larger than or equal to the number of bits in `self`, i.e. <br>如果 `rhs` 大于或等于 `self` 中的位数，则会导致未定义的行为，即<br>
        ///
        /// when [`checked_shr`] would return `None`. <br>当 [`checked_shr`] 返回 `None` 时。<br>
        ///
        #[doc = concat!("[`checked_shr`]: ", stringify!($SelfT), "::checked_shr")]
        #[unstable(
            feature = "unchecked_math",
            reason = "niche optimization path",
            issue = "85122",
        )]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[rustc_const_unstable(feature = "const_inherent_unchecked_arith", issue = "85122")]
        #[inline(always)]
        pub const unsafe fn unchecked_shr(self, rhs: Self) -> Self {
            // SAFETY: the caller must uphold the safety contract for `unchecked_shr`. <br>调用者必须遵守 `unchecked_shr` 的安全保证。<br>
            //
            unsafe { intrinsics::unchecked_shr(self, rhs) }
        }

        /// Checked absolute value. <br>检查的绝对值。<br>
        /// Computes `self.abs()`, returning `None` if `self == MIN`. <br>计算 `self.abs()`，如果为 `self == MIN`，则返回 `None`。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        ///
        #[doc = concat!("assert_eq!((-5", stringify!($SelfT), ").checked_abs(), Some(5));")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.checked_abs(), None);")]
        /// ```
        #[stable(feature = "no_panic_abs", since = "1.13.0")]
        #[rustc_const_stable(feature = "const_checked_int_methods", since = "1.47.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_abs(self) -> Option<Self> {
            if self.is_negative() {
                self.checked_neg()
            } else {
                Some(self)
            }
        }

        /// Checked exponentiation. <br>检查取幂。<br>
        /// Computes `self.pow(exp)`, returning `None` if overflow occurred. <br>计算 `self.pow(exp)`，如果发生溢出则返回 `None`。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(8", stringify!($SelfT), ".checked_pow(2), Some(64));")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MAX.checked_pow(2), None);")]
        /// ```

        #[stable(feature = "no_panic_pow", since = "1.34.0")]
        #[rustc_const_stable(feature = "const_int_pow", since = "1.50.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_pow(self, mut exp: u32) -> Option<Self> {
            if exp == 0 {
                return Some(1);
            }
            let mut base = self;
            let mut acc: Self = 1;

            while exp > 1 {
                if (exp & 1) == 1 {
                    acc = try_opt!(acc.checked_mul(base));
                }
                exp /= 2;
                base = try_opt!(base.checked_mul(base));
            }
            // since exp!=0, finally the exp must be 1. <br>由于 `exp != 0`，最终 exp 必须为 1。<br>
            // Deal with the final bit of the exponent separately, since squaring the base afterwards is not necessary and may cause a needless overflow. <br>分开处理指数的最后一位，因为在此之后对底数进行平方是不必要的，并且可能导致不必要的溢出。<br>
            //
            //
            Some(try_opt!(acc.checked_mul(base)))
        }

        /// Saturating integer addition. <br>饱和整数加法。<br>
        /// Computes `self + rhs`, saturating at the numeric bounds instead of overflowing. <br>计算 `self + rhs`，在数字范围内饱和，而不是溢出。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".saturating_add(1), 101);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MAX.saturating_add(100), ", stringify!($SelfT), "::MAX);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.saturating_add(-1), ", stringify!($SelfT), "::MIN);")]
        /// ```

        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_saturating_int_methods", since = "1.47.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn saturating_add(self, rhs: Self) -> Self {
            intrinsics::saturating_add(self, rhs)
        }

        /// Saturating addition with an unsigned integer. <br>使用无符号整数进行饱和加法。<br>
        /// Computes `self + rhs`, saturating at the numeric bounds instead of overflowing. <br>计算 `self + rhs`，在数字范围内饱和，而不是溢出。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        /// # #![feature(mixed_integer_ops)]
        #[doc = concat!("assert_eq!(1", stringify!($SelfT), ".saturating_add_unsigned(2), 3);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MAX.saturating_add_unsigned(100), ", stringify!($SelfT), "::MAX);")]
        /// ```
        #[unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[rustc_const_unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn saturating_add_unsigned(self, rhs: $UnsignedT) -> Self {
            // Overflow can only happen at the upper bound We cannot use `unwrap_or` here because it is not `const` <br>溢出只能发生在上限，我们不能在这里使用 `unwrap_or`，因为它不是 `const`<br>
            //
            match self.checked_add_unsigned(rhs) {
                Some(x) => x,
                None => Self::MAX,
            }
        }

        /// Saturating integer subtraction. <br>饱和整数减法。<br>
        /// Computes `self - rhs`, saturating at the numeric bounds instead of overflowing. <br>计算 `self - rhs`，在数字范围内饱和，而不是溢出。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".saturating_sub(127), -27);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.saturating_sub(100), ", stringify!($SelfT), "::MIN);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MAX.saturating_sub(-1), ", stringify!($SelfT), "::MAX);")]
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_saturating_int_methods", since = "1.47.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn saturating_sub(self, rhs: Self) -> Self {
            intrinsics::saturating_sub(self, rhs)
        }

        /// Saturating subtraction with an unsigned integer. <br>使用无符号整数进行饱和减法。<br>
        /// Computes `self - rhs`, saturating at the numeric bounds instead of overflowing. <br>计算 `self - rhs`，在数字范围内饱和，而不是溢出。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        /// # #![feature(mixed_integer_ops)]
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".saturating_sub_unsigned(127), -27);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.saturating_sub_unsigned(100), ", stringify!($SelfT), "::MIN);")]
        /// ```
        #[unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[rustc_const_unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn saturating_sub_unsigned(self, rhs: $UnsignedT) -> Self {
            // Overflow can only happen at the lower bound We cannot use `unwrap_or` here because it is not `const` <br>溢出只能发生在下限，我们不能在这里使用 `unwrap_or`，因为它不是 `const`<br>
            //
            match self.checked_sub_unsigned(rhs) {
                Some(x) => x,
                None => Self::MIN,
            }
        }

        /// Saturating integer negation. <br>饱和整数求反。<br>
        /// Computes `-self`, returning `MAX` if `self == MIN` instead of overflowing. <br>计算 `-self`，如果 `self == MIN` 则返回 `MAX` 而不是溢出。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".saturating_neg(), -100);")]
        #[doc = concat!("assert_eq!((-100", stringify!($SelfT), ").saturating_neg(), 100);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.saturating_neg(), ", stringify!($SelfT), "::MAX);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MAX.saturating_neg(), ", stringify!($SelfT), "::MIN + 1);")]
        /// ```

        #[stable(feature = "saturating_neg", since = "1.45.0")]
        #[rustc_const_stable(feature = "const_saturating_int_methods", since = "1.47.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn saturating_neg(self) -> Self {
            intrinsics::saturating_sub(0, self)
        }

        /// Saturating absolute value. <br>饱和绝对值。<br>
        /// Computes `self.abs()`, returning `MAX` if `self == MIN` instead of overflowing. <br>计算 `self.abs()`，如果 `self == MIN` 则返回 `MAX` 而不是溢出。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".saturating_abs(), 100);")]
        #[doc = concat!("assert_eq!((-100", stringify!($SelfT), ").saturating_abs(), 100);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.saturating_abs(), ", stringify!($SelfT), "::MAX);")]
        #[doc = concat!("assert_eq!((", stringify!($SelfT), "::MIN + 1).saturating_abs(), ", stringify!($SelfT), "::MAX);")]
        /// ```

        #[stable(feature = "saturating_neg", since = "1.45.0")]
        #[rustc_const_stable(feature = "const_saturating_int_methods", since = "1.47.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn saturating_abs(self) -> Self {
            if self.is_negative() {
                self.saturating_neg()
            } else {
                self
            }
        }

        /// Saturating integer multiplication. <br>饱和整数乘法。<br>
        /// Computes `self * rhs`, saturating at the numeric bounds instead of overflowing. <br>计算 `self * rhs`，在数字范围内饱和，而不是溢出。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        ///
        #[doc = concat!("assert_eq!(10", stringify!($SelfT), ".saturating_mul(12), 120);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MAX.saturating_mul(10), ", stringify!($SelfT), "::MAX);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.saturating_mul(10), ", stringify!($SelfT), "::MIN);")]
        /// ```
        #[stable(feature = "wrapping", since = "1.7.0")]
        #[rustc_const_stable(feature = "const_saturating_int_methods", since = "1.47.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn saturating_mul(self, rhs: Self) -> Self {
            match self.checked_mul(rhs) {
                Some(x) => x,
                None => if (self < 0) == (rhs < 0) {
                    Self::MAX
                } else {
                    Self::MIN
                }
            }
        }

        /// Saturating integer division. <br>饱和整数除法。<br>
        /// Computes `self / rhs`, saturating at the numeric bounds instead of overflowing. <br>计算 `self / rhs`，在数值边界处饱和而不是溢出。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(5", stringify!($SelfT), ".saturating_div(2), 2);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MAX.saturating_div(-1), ", stringify!($SelfT), "::MIN + 1);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.saturating_div(-1), ", stringify!($SelfT), "::MAX);")]
        ///
        /// ```
        ///
        /// ```should_panic
        #[doc = concat!("let _ = 1", stringify!($SelfT), ".saturating_div(0);")]
        ///
        /// ```
        #[stable(feature = "saturating_div", since = "1.58.0")]
        #[rustc_const_stable(feature = "saturating_div", since = "1.58.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn saturating_div(self, rhs: Self) -> Self {
            match self.overflowing_div(rhs) {
                (result, false) => result,
                (_result, true) => Self::MAX, // MIN / -1 is the only possible saturating overflow <br>MIN/-1 是唯一可能的饱和溢出<br>
            }
        }

        /// Saturating integer exponentiation. <br>饱和整数幂。<br>
        /// Computes `self.pow(exp)`, saturating at the numeric bounds instead of overflowing. <br>计算 `self.pow(exp)`，在数字范围内饱和，而不是溢出。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        ///
        #[doc = concat!("assert_eq!((-4", stringify!($SelfT), ").saturating_pow(3), -64);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.saturating_pow(2), ", stringify!($SelfT), "::MAX);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.saturating_pow(3), ", stringify!($SelfT), "::MIN);")]
        /// ```
        #[stable(feature = "no_panic_pow", since = "1.34.0")]
        #[rustc_const_stable(feature = "const_int_pow", since = "1.50.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn saturating_pow(self, exp: u32) -> Self {
            match self.checked_pow(exp) {
                Some(x) => x,
                None if self < 0 && exp % 2 == 1 => Self::MIN,
                None => Self::MAX,
            }
        }

        /// Wrapping (modular) addition. <br>包装 (modular) 添加。<br>
        /// Computes `self + rhs`, wrapping around at the boundary of the type. <br>计算 `self + rhs`，在类型的边界处回绕。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".wrapping_add(27), 127);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MAX.wrapping_add(2), ", stringify!($SelfT), "::MIN + 1);")]
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn wrapping_add(self, rhs: Self) -> Self {
            intrinsics::wrapping_add(self, rhs)
        }

        /// Wrapping (modular) addition with an unsigned integer. <br>用无符号整数包装 (modular) 加法。<br>
        /// Computes `self + rhs`, wrapping around at the boundary of the type. <br>计算 `self + rhs`，在类型的边界处回绕。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        /// # #![feature(mixed_integer_ops)]
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".wrapping_add_unsigned(27), 127);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MAX.wrapping_add_unsigned(2), ", stringify!($SelfT), "::MIN + 1);")]
        /// ```
        #[unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[rustc_const_unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn wrapping_add_unsigned(self, rhs: $UnsignedT) -> Self {
            self.wrapping_add(rhs as Self)
        }

        /// Wrapping (modular) subtraction. <br>包装 (modular) 减法。<br>
        /// Computes `self - rhs`, wrapping around at the boundary of the type. <br>计算 `self - rhs`，在类型的边界处回绕。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(0", stringify!($SelfT), ".wrapping_sub(127), -127);")]
        #[doc = concat!("assert_eq!((-2", stringify!($SelfT), ").wrapping_sub(", stringify!($SelfT), "::MAX), ", stringify!($SelfT), "::MAX);")]
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn wrapping_sub(self, rhs: Self) -> Self {
            intrinsics::wrapping_sub(self, rhs)
        }

        /// Wrapping (modular) subtraction with an unsigned integer. <br>用无符号整数包装 (modular) 减法。<br>
        /// Computes `self - rhs`, wrapping around at the boundary of the type. <br>计算 `self - rhs`，在类型的边界处回绕。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        /// # #![feature(mixed_integer_ops)]
        #[doc = concat!("assert_eq!(0", stringify!($SelfT), ".wrapping_sub_unsigned(127), -127);")]
        #[doc = concat!("assert_eq!((-2", stringify!($SelfT), ").wrapping_sub_unsigned(", stringify!($UnsignedT), "::MAX), -1);")]
        /// ```
        #[unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[rustc_const_unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn wrapping_sub_unsigned(self, rhs: $UnsignedT) -> Self {
            self.wrapping_sub(rhs as Self)
        }

        /// Wrapping (modular) multiplication. <br>包装 (modular) 乘法。<br>
        /// Computes `self * rhs`, wrapping around at the boundary of the type. <br>计算 `self * rhs`，在类型的边界处回绕。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(10", stringify!($SelfT), ".wrapping_mul(12), 120);")]
        /// assert_eq!(11i8.wrapping_mul(12), -124);
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn wrapping_mul(self, rhs: Self) -> Self {
            intrinsics::wrapping_mul(self, rhs)
        }

        /// Wrapping (modular) division. <br>包装 (modular) 分区。<br> Computes `self / rhs`, wrapping around at the boundary of the type. <br>计算 `self / rhs`，在类型的边界处回绕。<br>
        ///
        /// The only case where such wrapping can occur is when one divides `MIN / -1` on a signed type (where `MIN` is the negative minimal value for the type); <br>可能发生这种换行的唯一情况是将 `MIN / -1` 除以有符号类型 (其中 `MIN` 是该类型的负最小值)。<br> this is equivalent to `-MIN`, a positive value that is too large to represent in the type. <br>这等效于 `-MIN`，它是一个太大的正值，无法在类型中表示。<br>
        /// In such a case, this function returns `MIN` itself. <br>在这种情况下，此函数将返回 `MIN` 本身。<br>
        ///
        /// # Panics
        ///
        /// This function will panic if `rhs` is 0. <br>如果 `rhs` 为 0，则此函数将为 panic。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".wrapping_div(10), 10);")]
        /// assert_eq!((-128i8).wrapping_div(-1), -128);
        /// ```
        ///
        ///
        #[stable(feature = "num_wrapping", since = "1.2.0")]
        #[rustc_const_stable(feature = "const_wrapping_int_methods", since = "1.52.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn wrapping_div(self, rhs: Self) -> Self {
            self.overflowing_div(rhs).0
        }

        /// Wrapping Euclidean division. <br>包装欧几里得除法。<br>
        /// Computes `self.div_euclid(rhs)`, wrapping around at the boundary of the type. <br>计算 `self.div_euclid(rhs)`，在类型的边界处回绕。<br>
        ///
        /// Wrapping will only occur in `MIN / -1` on a signed type (where `MIN` is the negative minimal value for the type). <br>包装只会在 `MIN / -1` 上的带符号类型 (其中 `MIN` 是该类型的负最小值) 上发生。<br>
        /// This is equivalent to `-MIN`, a positive value that is too large to represent in the type. <br>这等效于 `-MIN`，它是一个太大的正值，无法在类型中表示。<br>
        /// In this case, this method returns `MIN` itself. <br>在这种情况下，此方法将返回 `MIN` 本身。<br>
        ///
        /// # Panics
        ///
        /// This function will panic if `rhs` is 0. <br>如果 `rhs` 为 0，则此函数将为 panic。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".wrapping_div_euclid(10), 10);")]
        /// assert_eq!((-128i8).wrapping_div_euclid(-1), -128);
        /// ```
        #[stable(feature = "euclidean_division", since = "1.38.0")]
        #[rustc_const_stable(feature = "const_euclidean_int_methods", since = "1.52.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn wrapping_div_euclid(self, rhs: Self) -> Self {
            self.overflowing_div_euclid(rhs).0
        }

        /// Wrapping (modular) remainder. <br>包装 (modular) 余数。<br> Computes `self % rhs`, wrapping around at the boundary of the type. <br>计算 `self % rhs`，在类型的边界处回绕。<br>
        ///
        /// Such wrap-around never actually occurs mathematically; <br>这种折回实际上在数学上从来没有发生过。<br> implementation artifacts make `x % y` invalid for `MIN / -1` on a signed type (where `MIN` is the negative minimal value). <br>实现工件会导致 `x % y` 对于带符号类型的 `MIN / -1` 无效 (其中 `MIN` 为负最小值)。<br>
        ///
        /// In such a case, this function returns `0`. <br>在这种情况下，此函数返回 `0`。<br>
        ///
        /// # Panics
        ///
        /// This function will panic if `rhs` is 0. <br>如果 `rhs` 为 0，则此函数将为 panic。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".wrapping_rem(10), 0);")]
        /// assert_eq!((-128i8).wrapping_rem(-1), 0);
        /// ```
        ///
        #[stable(feature = "num_wrapping", since = "1.2.0")]
        #[rustc_const_stable(feature = "const_wrapping_int_methods", since = "1.52.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn wrapping_rem(self, rhs: Self) -> Self {
            self.overflowing_rem(rhs).0
        }

        /// Wrapping Euclidean remainder. <br>包装欧几里得的余数。<br> Computes `self.rem_euclid(rhs)`, wrapping around at the boundary of the type. <br>计算 `self.rem_euclid(rhs)`，在类型的边界处回绕。<br>
        ///
        /// Wrapping will only occur in `MIN % -1` on a signed type (where `MIN` is the negative minimal value for the type). <br>包装只会在 `MIN % -1` 上的带符号类型 (其中 `MIN` 是该类型的负最小值) 上发生。<br>
        /// In this case, this method returns 0. <br>在这种情况下，此方法返回 0。<br>
        ///
        /// # Panics
        ///
        /// This function will panic if `rhs` is 0. <br>如果 `rhs` 为 0，则此函数将为 panic。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".wrapping_rem_euclid(10), 0);")]
        /// assert_eq!((-128i8).wrapping_rem_euclid(-1), 0);
        /// ```
        ///
        #[stable(feature = "euclidean_division", since = "1.38.0")]
        #[rustc_const_stable(feature = "const_euclidean_int_methods", since = "1.52.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn wrapping_rem_euclid(self, rhs: Self) -> Self {
            self.overflowing_rem_euclid(rhs).0
        }

        /// Wrapping (modular) negation. <br>包装 (modular) 取反。<br> Computes `-self`, wrapping around at the boundary of the type. <br>计算 `-self`，在类型的边界处回绕。<br>
        ///
        /// The only case where such wrapping can occur is when one negates `MIN` on a signed type (where `MIN` is the negative minimal value for the type); <br>可能发生这种换行的唯一情况是对有符号类型的 `MIN` 求反 (其中 `MIN` 是该类型的负最小值) ; 否则，可能会发生这种换行。<br> this is a positive value that is too large to represent in the type. <br>这是一个太大的正值，无法在类型中表示。<br>
        /// In such a case, this function returns `MIN` itself. <br>在这种情况下，此函数将返回 `MIN` 本身。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".wrapping_neg(), -100);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.wrapping_neg(), ", stringify!($SelfT), "::MIN);")]
        /// ```
        ///
        ///
        #[stable(feature = "num_wrapping", since = "1.2.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn wrapping_neg(self) -> Self {
            (0 as $SelfT).wrapping_sub(self)
        }

        /// Panic-free bitwise shift-left; <br>无 Panic - 按位左移；<br> yields `self << mask(rhs)`, where `mask` removes any high-order bits of `rhs` that would cause the shift to exceed the bitwidth of the type. <br>产生 `self << mask(rhs)`，其中 `mask` 删除 `rhs` 的所有高位，这些高位将导致移位超过该类型的位宽。<br>
        ///
        /// Note that this is *not* the same as a rotate-left; <br>注意，这与左旋不同；<br> the RHS of a wrapping shift-left is restricted to the range of the type, rather than the bits shifted out of the LHS being returned to the other end. <br>环绕左移的 RHS 限于该类型的范围，而不是从 LHS 移出的位返回到另一端。<br>
        ///
        /// The primitive integer types all implement a [`rotate_left`](Self::rotate_left) function, which may be what you want instead. <br>所有原始整数类型都实现了 [`rotate_left`](Self::rotate_left) 函数，而您可能想要的是 [`rotate_left`](Self::rotate_left) 函数。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!((-1", stringify!($SelfT), ").wrapping_shl(7), -128);")]
        #[doc = concat!("assert_eq!((-1", stringify!($SelfT), ").wrapping_shl(128), -1);")]
        /// ```
        ///
        ///
        #[stable(feature = "num_wrapping", since = "1.2.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn wrapping_shl(self, rhs: u32) -> Self {
            // SAFETY: the masking by the bitsize of the type ensures that we do not shift out of bounds <br>通过类型的位大小进行掩蔽可确保我们不会越界<br>
            //
            unsafe {
                intrinsics::unchecked_shl(self, (rhs & ($BITS - 1)) as $SelfT)
            }
        }

        /// Panic-free bitwise shift-right; <br>无 Panic - 按位右移；<br> yields `self >> mask(rhs)`, where `mask` removes any high-order bits of `rhs` that would cause the shift to exceed the bitwidth of the type. <br>产生 `self >> mask(rhs)`，其中 `mask` 删除 `rhs` 的所有高位，这些高位将导致移位超过该类型的位宽。<br>
        ///
        /// Note that this is *not* the same as a rotate-right; <br>注意，这与右旋转不同。<br> the RHS of a wrapping shift-right is restricted to the range of the type, rather than the bits shifted out of the LHS being returned to the other end. <br>换行右移的 RHS 限于类型的范围，而不是从 LHS 移出的位返回到另一端。<br>
        ///
        /// The primitive integer types all implement a [`rotate_right`](Self::rotate_right) function, which may be what you want instead. <br>所有原始整数类型都实现了 [`rotate_right`](Self::rotate_right) 函数，而您可能想要的是 [`rotate_right`](Self::rotate_right) 函数。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!((-128", stringify!($SelfT), ").wrapping_shr(7), -1);")]
        /// assert_eq!((-128i16).wrapping_shr(64), -128);
        /// ```
        ///
        ///
        #[stable(feature = "num_wrapping", since = "1.2.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn wrapping_shr(self, rhs: u32) -> Self {
            // SAFETY: the masking by the bitsize of the type ensures that we do not shift out of bounds <br>通过类型的位大小进行掩蔽可确保我们不会越界<br>
            //
            unsafe {
                intrinsics::unchecked_shr(self, (rhs & ($BITS - 1)) as $SelfT)
            }
        }

        /// Wrapping (modular) absolute value. <br>包装 (modular) 绝对值。<br> Computes `self.abs()`, wrapping around at the boundary of the type. <br>计算 `self.abs()`，在类型的边界处回绕。<br>
        ///
        /// The only case where such wrapping can occur is when one takes the absolute value of the negative minimal value for the type; <br>可能发生这种换行的唯一情况是，当类型取负的最小值的绝对值时；<br> this is a positive value that is too large to represent in the type. <br>这是一个太大的正值，无法在类型中表示。<br>
        /// In such a case, this function returns `MIN` itself. <br>在这种情况下，此函数将返回 `MIN` 本身。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".wrapping_abs(), 100);")]
        #[doc = concat!("assert_eq!((-100", stringify!($SelfT), ").wrapping_abs(), 100);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.wrapping_abs(), ", stringify!($SelfT), "::MIN);")]
        /// assert_eq!((-128i8).wrapping_abs() as u8, 128);
        /// ```
        ///
        ///
        #[stable(feature = "no_panic_abs", since = "1.13.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[allow(unused_attributes)]
        #[inline]
        pub const fn wrapping_abs(self) -> Self {
             if self.is_negative() {
                 self.wrapping_neg()
             } else {
                 self
             }
        }

        /// Computes the absolute value of `self` without any wrapping or panicking. <br>计算 `self` 的绝对值，而不会引起任何包装或 panic。<br>
        ///
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".unsigned_abs(), 100", stringify!($UnsignedT), ");")]
        #[doc = concat!("assert_eq!((-100", stringify!($SelfT), ").unsigned_abs(), 100", stringify!($UnsignedT), ");")]
        /// assert_eq!((-128i8).unsigned_abs(), 128u8);
        /// ```
        ///
        #[stable(feature = "unsigned_abs", since = "1.51.0")]
        #[rustc_const_stable(feature = "unsigned_abs", since = "1.51.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn unsigned_abs(self) -> $UnsignedT {
             self.wrapping_abs() as $UnsignedT
        }

        /// Wrapping (modular) exponentiation. <br>包装 (modular) 指数。<br>
        /// Computes `self.pow(exp)`, wrapping around at the boundary of the type. <br>计算 `self.pow(exp)`，在类型的边界处回绕。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(3", stringify!($SelfT), ".wrapping_pow(4), 81);")]
        /// assert_eq!(3i8.wrapping_pow(5), -13);
        /// assert_eq!(3i8.wrapping_pow(6), -39);
        /// ```
        #[stable(feature = "no_panic_pow", since = "1.34.0")]
        #[rustc_const_stable(feature = "const_int_pow", since = "1.50.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn wrapping_pow(self, mut exp: u32) -> Self {
            if exp == 0 {
                return 1;
            }
            let mut base = self;
            let mut acc: Self = 1;

            while exp > 1 {
                if (exp & 1) == 1 {
                    acc = acc.wrapping_mul(base);
                }
                exp /= 2;
                base = base.wrapping_mul(base);
            }

            // since exp!=0, finally the exp must be 1. <br>由于 `exp != 0`，最终 exp 必须为 1。<br>
            // Deal with the final bit of the exponent separately, since squaring the base afterwards is not necessary and may cause a needless overflow. <br>分开处理指数的最后一位，因为在此之后对底数进行平方是不必要的，并且可能导致不必要的溢出。<br>
            //
            //
            acc.wrapping_mul(base)
        }

        /// Calculates `self` + `rhs` <br>计算 `self` + `rhs`<br>
        ///
        /// Returns a tuple of the addition along with a boolean indicating whether an arithmetic overflow would occur. <br>返回一个加法元组以及一个布尔值，该布尔值指示是否会发生算术溢出。<br>
        /// If an overflow would have occurred then the wrapped value is returned. <br>如果将发生溢出，则返回包装的值。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        ///
        #[doc = concat!("assert_eq!(5", stringify!($SelfT), ".overflowing_add(2), (7, false));")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MAX.overflowing_add(1), (", stringify!($SelfT), "::MIN, true));")]
        /// ```
        #[stable(feature = "wrapping", since = "1.7.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn overflowing_add(self, rhs: Self) -> (Self, bool) {
            let (a, b) = intrinsics::add_with_overflow(self as $ActualT, rhs as $ActualT);
            (a as Self, b)
        }

        /// Calculates `self` + `rhs` with an unsigned `rhs` <br>使用无符号 `rhs` 计算 `self` + `rhs`<br>
        ///
        /// Returns a tuple of the addition along with a boolean indicating whether an arithmetic overflow would occur. <br>返回一个加法元组以及一个布尔值，该布尔值指示是否会发生算术溢出。<br>
        /// If an overflow would have occurred then the wrapped value is returned. <br>如果将发生溢出，则返回包装的值。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        /// # #![feature(mixed_integer_ops)]
        #[doc = concat!("assert_eq!(1", stringify!($SelfT), ".overflowing_add_unsigned(2), (3, false));")]
        #[doc = concat!("assert_eq!((", stringify!($SelfT), "::MIN).overflowing_add_unsigned(", stringify!($UnsignedT), "::MAX), (", stringify!($SelfT), "::MAX, false));")]
        #[doc = concat!("assert_eq!((", stringify!($SelfT), "::MAX - 2).overflowing_add_unsigned(3), (", stringify!($SelfT), "::MIN, true));")]
        /// ```
        ///
        #[unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[rustc_const_unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn overflowing_add_unsigned(self, rhs: $UnsignedT) -> (Self, bool) {
            let rhs = rhs as Self;
            let (res, overflowed) = self.overflowing_add(rhs);
            (res, overflowed ^ (rhs < 0))
        }

        /// Calculates `self` - `rhs` <br>计算 `self`-`rhs`<br>
        ///
        /// Returns a tuple of the subtraction along with a boolean indicating whether an arithmetic overflow would occur. <br>返回一个减法的元组以及一个布尔值，该布尔值指示是否会发生算术溢出。<br>
        /// If an overflow would have occurred then the wrapped value is returned. <br>如果将发生溢出，则返回包装的值。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        ///
        #[doc = concat!("assert_eq!(5", stringify!($SelfT), ".overflowing_sub(2), (3, false));")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.overflowing_sub(1), (", stringify!($SelfT), "::MAX, true));")]
        /// ```
        #[stable(feature = "wrapping", since = "1.7.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn overflowing_sub(self, rhs: Self) -> (Self, bool) {
            let (a, b) = intrinsics::sub_with_overflow(self as $ActualT, rhs as $ActualT);
            (a as Self, b)
        }

        /// Calculates `self` - `rhs` with an unsigned `rhs` <br>使用无符号 `rhs` 计算 `self`-`rhs`<br>
        ///
        /// Returns a tuple of the subtraction along with a boolean indicating whether an arithmetic overflow would occur. <br>返回一个减法的元组以及一个布尔值，该布尔值指示是否会发生算术溢出。<br>
        /// If an overflow would have occurred then the wrapped value is returned. <br>如果将发生溢出，则返回包装的值。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        /// # #![feature(mixed_integer_ops)]
        #[doc = concat!("assert_eq!(1", stringify!($SelfT), ".overflowing_sub_unsigned(2), (-1, false));")]
        #[doc = concat!("assert_eq!((", stringify!($SelfT), "::MAX).overflowing_sub_unsigned(", stringify!($UnsignedT), "::MAX), (", stringify!($SelfT), "::MIN, false));")]
        #[doc = concat!("assert_eq!((", stringify!($SelfT), "::MIN + 2).overflowing_sub_unsigned(3), (", stringify!($SelfT), "::MAX, true));")]
        /// ```
        ///
        #[unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[rustc_const_unstable(feature = "mixed_integer_ops", issue = "87840")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn overflowing_sub_unsigned(self, rhs: $UnsignedT) -> (Self, bool) {
            let rhs = rhs as Self;
            let (res, overflowed) = self.overflowing_sub(rhs);
            (res, overflowed ^ (rhs < 0))
        }

        /// Calculates the multiplication of `self` and `rhs`. <br>计算 `self` 和 `rhs` 的乘法。<br>
        ///
        /// Returns a tuple of the multiplication along with a boolean indicating whether an arithmetic overflow would occur. <br>返回乘法的元组以及一个布尔值，该布尔值指示是否会发生算术溢出。<br>
        /// If an overflow would have occurred then the wrapped value is returned. <br>如果将发生溢出，则返回包装的值。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(5", stringify!($SelfT), ".overflowing_mul(2), (10, false));")]
        /// assert_eq!(1_000_000_000i32.overflowing_mul(10), (1410065408, true));
        /// ```
        #[stable(feature = "wrapping", since = "1.7.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn overflowing_mul(self, rhs: Self) -> (Self, bool) {
            let (a, b) = intrinsics::mul_with_overflow(self as $ActualT, rhs as $ActualT);
            (a as Self, b)
        }

        /// Calculates the divisor when `self` is divided by `rhs`. <br>`self` 除以 `rhs` 时计算除数。<br>
        ///
        /// Returns a tuple of the divisor along with a boolean indicating whether an arithmetic overflow would occur. <br>返回除数的元组以及指示是否将发生算术溢出的布尔值。<br>
        /// If an overflow would occur then self is returned. <br>如果将发生溢出，则返回 self。<br>
        ///
        /// # Panics
        ///
        /// This function will panic if `rhs` is 0. <br>如果 `rhs` 为 0，则此函数将为 panic。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        ///
        #[doc = concat!("assert_eq!(5", stringify!($SelfT), ".overflowing_div(2), (2, false));")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.overflowing_div(-1), (", stringify!($SelfT), "::MIN, true));")]
        /// ```
        #[inline]
        #[stable(feature = "wrapping", since = "1.7.0")]
        #[rustc_const_stable(feature = "const_overflowing_int_methods", since = "1.52.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        pub const fn overflowing_div(self, rhs: Self) -> (Self, bool) {
            // Using `&` helps LLVM see that it is the same check made in division. <br>使用 `&` 有助于 LLVM 看到它是在除法中进行的相同检查。<br>
            if unlikely!((self == Self::MIN) & (rhs == -1)) {
                (self, true)
            } else {
                (self / rhs, false)
            }
        }

        /// Calculates the quotient of Euclidean division `self.div_euclid(rhs)`. <br>计算欧几里得除法 `self.div_euclid(rhs)` 的商。<br>
        ///
        /// Returns a tuple of the divisor along with a boolean indicating whether an arithmetic overflow would occur. <br>返回除数的元组以及指示是否将发生算术溢出的布尔值。<br>
        /// If an overflow would occur then `self` is returned. <br>如果将发生溢出，则返回 `self`。<br>
        ///
        /// # Panics
        ///
        /// This function will panic if `rhs` is 0. <br>如果 `rhs` 为 0，则此函数将为 panic。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(5", stringify!($SelfT), ".overflowing_div_euclid(2), (2, false));")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.overflowing_div_euclid(-1), (", stringify!($SelfT), "::MIN, true));")]
        /// ```
        #[inline]
        #[stable(feature = "euclidean_division", since = "1.38.0")]
        #[rustc_const_stable(feature = "const_euclidean_int_methods", since = "1.52.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        pub const fn overflowing_div_euclid(self, rhs: Self) -> (Self, bool) {
            // Using `&` helps LLVM see that it is the same check made in division. <br>使用 `&` 有助于 LLVM 看到它是在除法中进行的相同检查。<br>
            if unlikely!((self == Self::MIN) & (rhs == -1)) {
                (self, true)
            } else {
                (self.div_euclid(rhs), false)
            }
        }

        /// Calculates the remainder when `self` is divided by `rhs`. <br>`self` 除以 `rhs` 时计算余数。<br>
        ///
        /// Returns a tuple of the remainder after dividing along with a boolean indicating whether an arithmetic overflow would occur. <br>返回除法运算后的余数元组和一个布尔值，该布尔值指示是否会发生算术溢出。<br>
        /// If an overflow would occur then 0 is returned. <br>如果将发生溢出，则返回 0。<br>
        ///
        /// # Panics
        ///
        /// This function will panic if `rhs` is 0. <br>如果 `rhs` 为 0，则此函数将为 panic。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        ///
        #[doc = concat!("assert_eq!(5", stringify!($SelfT), ".overflowing_rem(2), (1, false));")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.overflowing_rem(-1), (0, true));")]
        /// ```
        #[inline]
        #[stable(feature = "wrapping", since = "1.7.0")]
        #[rustc_const_stable(feature = "const_overflowing_int_methods", since = "1.52.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        pub const fn overflowing_rem(self, rhs: Self) -> (Self, bool) {
            if unlikely!(rhs == -1) {
                (0, self == Self::MIN)
            } else {
                (self % rhs, false)
            }
        }


        /// Overflowing Euclidean remainder. <br>溢出的欧几里得余数。<br> Calculates `self.rem_euclid(rhs)`. <br>计算 `self.rem_euclid(rhs)`。<br>
        ///
        /// Returns a tuple of the remainder after dividing along with a boolean indicating whether an arithmetic overflow would occur. <br>返回除法运算后的余数元组和一个布尔值，该布尔值指示是否会发生算术溢出。<br>
        /// If an overflow would occur then 0 is returned. <br>如果将发生溢出，则返回 0。<br>
        ///
        /// # Panics
        ///
        /// This function will panic if `rhs` is 0. <br>如果 `rhs` 为 0，则此函数将为 panic。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(5", stringify!($SelfT), ".overflowing_rem_euclid(2), (1, false));")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.overflowing_rem_euclid(-1), (0, true));")]
        /// ```
        #[stable(feature = "euclidean_division", since = "1.38.0")]
        #[rustc_const_stable(feature = "const_euclidean_int_methods", since = "1.52.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn overflowing_rem_euclid(self, rhs: Self) -> (Self, bool) {
            if unlikely!(rhs == -1) {
                (0, self == Self::MIN)
            } else {
                (self.rem_euclid(rhs), false)
            }
        }


        /// Negates self, overflowing if this is equal to the minimum value. <br>否定 self，如果等于最小值，则溢出。<br>
        ///
        /// Returns a tuple of the negated version of self along with a boolean indicating whether an overflow happened. <br>返回否定形式 self 的元组以及一个布尔值，该布尔值指示是否发生了溢出。<br>
        /// If `self` is the minimum value (e.g., `i32::MIN` for values of type `i32`), then the minimum value will be returned again and `true` will be returned for an overflow happening. <br>如果 `self` 是最小值 (例如，对于类型 `i32` 的值，则为 `i32::MIN`)，则将再次返回该最小值，并且将发生溢出时返回 `true`。<br>
        ///
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(2", stringify!($SelfT), ".overflowing_neg(), (-2, false));")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.overflowing_neg(), (", stringify!($SelfT), "::MIN, true));")]
        /// ```
        #[inline]
        #[stable(feature = "wrapping", since = "1.7.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[allow(unused_attributes)]
        pub const fn overflowing_neg(self) -> (Self, bool) {
            if unlikely!(self == Self::MIN) {
                (Self::MIN, true)
            } else {
                (-self, false)
            }
        }

        /// Shifts self left by `rhs` bits. <br>将 self 左移 `rhs` 位。<br>
        ///
        /// Returns a tuple of the shifted version of self along with a boolean indicating whether the shift value was larger than or equal to the number of bits. <br>返回 self 的移位版本的元组以及一个布尔值，该布尔值指示 shift 值是否大于或等于位数。<br>
        /// If the shift value is too large, then value is masked (N-1) where N is the number of bits, and this value is then used to perform the shift. <br>如果移位值太大，则将值屏蔽 (N-1)，其中 N 是位数，然后使用该值执行移位。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(0x1", stringify!($SelfT),".overflowing_shl(4), (0x10, false));")]
        /// assert_eq!(0x1i32.overflowing_shl(36), (0x10, true));
        /// ```
        ///
        #[stable(feature = "wrapping", since = "1.7.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn overflowing_shl(self, rhs: u32) -> (Self, bool) {
            (self.wrapping_shl(rhs), (rhs > ($BITS - 1)))
        }

        /// Shifts self right by `rhs` bits. <br>将 self 右移 `rhs` 位。<br>
        ///
        /// Returns a tuple of the shifted version of self along with a boolean indicating whether the shift value was larger than or equal to the number of bits. <br>返回 self 的移位版本的元组以及一个布尔值，该布尔值指示 shift 值是否大于或等于位数。<br>
        /// If the shift value is too large, then value is masked (N-1) where N is the number of bits, and this value is then used to perform the shift. <br>如果移位值太大，则将值屏蔽 (N-1)，其中 N 是位数，然后使用该值执行移位。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(0x10", stringify!($SelfT), ".overflowing_shr(4), (0x1, false));")]
        /// assert_eq!(0x10i32.overflowing_shr(36), (0x1, true));
        /// ```
        ///
        #[stable(feature = "wrapping", since = "1.7.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn overflowing_shr(self, rhs: u32) -> (Self, bool) {
            (self.wrapping_shr(rhs), (rhs > ($BITS - 1)))
        }

        /// Computes the absolute value of `self`. <br>计算 `self` 的绝对值。<br>
        ///
        /// Returns a tuple of the absolute version of self along with a boolean indicating whether an overflow happened. <br>返回 self 的绝对版本的元组以及一个布尔值，该布尔值指示是否发生了溢出。<br>
        /// If self is the minimum value <br>如果 self 是最小值<br>
        #[doc = concat!("(e.g., ", stringify!($SelfT), "::MIN for values of type ", stringify!($SelfT), "),")]
        /// then the minimum value will be returned again and true will be returned for an overflow happening. <br>然后将再次返回最小值，并在发生溢出时返回 true。<br>
        ///
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(10", stringify!($SelfT), ".overflowing_abs(), (10, false));")]
        #[doc = concat!("assert_eq!((-10", stringify!($SelfT), ").overflowing_abs(), (10, false));")]
        #[doc = concat!("assert_eq!((", stringify!($SelfT), "::MIN).overflowing_abs(), (", stringify!($SelfT), "::MIN, true));")]
        /// ```
        #[stable(feature = "no_panic_abs", since = "1.13.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn overflowing_abs(self) -> (Self, bool) {
            (self.wrapping_abs(), self == Self::MIN)
        }

        /// Raises self to the power of `exp`, using exponentiation by squaring. <br>通过平方运算，将自己提升到 `exp` 的功效。<br>
        ///
        /// Returns a tuple of the exponentiation along with a bool indicating whether an overflow happened. <br>返回一个指数的元组以及一个 bool，指示是否发生了溢出。<br>
        ///
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(3", stringify!($SelfT), ".overflowing_pow(4), (81, false));")]
        /// assert_eq!(3i8.overflowing_pow(5), (-13, true));
        /// ```
        #[stable(feature = "no_panic_pow", since = "1.34.0")]
        #[rustc_const_stable(feature = "const_int_pow", since = "1.50.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn overflowing_pow(self, mut exp: u32) -> (Self, bool) {
            if exp == 0 {
                return (1,false);
            }
            let mut base = self;
            let mut acc: Self = 1;
            let mut overflown = false;
            // Scratch space for storing results of overflowing_mul. <br>暂存空间，用于存储 overflowing_mul 的结果。<br>
            let mut r;

            while exp > 1 {
                if (exp & 1) == 1 {
                    r = acc.overflowing_mul(base);
                    acc = r.0;
                    overflown |= r.1;
                }
                exp /= 2;
                r = base.overflowing_mul(base);
                base = r.0;
                overflown |= r.1;
            }

            // since exp!=0, finally the exp must be 1. <br>由于 `exp != 0`，最终 exp 必须为 1。<br>
            // Deal with the final bit of the exponent separately, since squaring the base afterwards is not necessary and may cause a needless overflow. <br>分开处理指数的最后一位，因为在此之后对底数进行平方是不必要的，并且可能导致不必要的溢出。<br>
            //
            //
            r = acc.overflowing_mul(base);
            r.1 |= overflown;
            r
        }

        /// Raises self to the power of `exp`, using exponentiation by squaring. <br>通过平方运算，将自己提升到 `exp` 的功效。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let x: ", stringify!($SelfT), " = 2; // or any other integer type")]
        ///
        /// assert_eq!(x.pow(5), 32);
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_pow", since = "1.50.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        #[rustc_inherit_overflow_checks]
        pub const fn pow(self, mut exp: u32) -> Self {
            if exp == 0 {
                return 1;
            }
            let mut base = self;
            let mut acc = 1;

            while exp > 1 {
                if (exp & 1) == 1 {
                    acc = acc * base;
                }
                exp /= 2;
                base = base * base;
            }

            // since exp!=0, finally the exp must be 1. <br>由于 `exp != 0`，最终 exp 必须为 1。<br>
            // Deal with the final bit of the exponent separately, since squaring the base afterwards is not necessary and may cause a needless overflow. <br>分开处理指数的最后一位，因为在此之后对底数进行平方是不必要的，并且可能导致不必要的溢出。<br>
            //
            //
            acc * base
        }

        /// Calculates the quotient of Euclidean division of `self` by `rhs`. <br>计算 `self` 的欧几里得除以 `rhs` 的商。<br>
        ///
        /// This computes the integer `q` such that `self = q * rhs + r`, with `r = self.rem_euclid(rhs)` and `0 <= r < abs(rhs)`. <br>这将计算整数 `q`，使得 `self = q * rhs + r`、`r = self.rem_euclid(rhs)` 和 `0 <= r < abs(rhs)`。<br>
        ///
        ///
        /// In other words, the result is `self / rhs` rounded to the integer `q` such that `self >= q * rhs`. <br>换句话说，结果是 `self / rhs` 舍入为整数 `q`，使得 `self >= q * rhs`。<br>
        /// If `self > 0`, this is equal to round towards zero (the default in Rust); <br>如果为 `self > 0`，则等于舍入为零 (Rust 中的默认值) ；<br>
        /// if `self < 0`, this is equal to round towards +/- infinity. <br>如果为 `self < 0`，则等于朝 +/- 无限取整。<br>
        ///
        /// # Panics
        ///
        /// This function will panic if `rhs` is 0 or the division results in overflow. <br>如果 `rhs` 为 0 或除法导致溢出，则该函数将为 panic。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let a: ", stringify!($SelfT), " = 7; // or any other integer type")]
        /// let b = 4;
        ///
        /// assert_eq!(a.div_euclid(b), 1); // 7 >= 4 * 1
        /// assert_eq!(a.div_euclid(-b), -1); // 7 >= -4 * -1
        /// assert_eq!((-a).div_euclid(b), -2); // -7 >= 4 * -2
        /// assert_eq!((-a).div_euclid(-b), 2); // -7 >= -4 * 2
        /// ```
        ///
        #[stable(feature = "euclidean_division", since = "1.38.0")]
        #[rustc_const_stable(feature = "const_euclidean_int_methods", since = "1.52.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        #[rustc_inherit_overflow_checks]
        pub const fn div_euclid(self, rhs: Self) -> Self {
            let q = self / rhs;
            if self % rhs < 0 {
                return if rhs > 0 { q - 1 } else { q + 1 }
            }
            q
        }


        /// Calculates the least nonnegative remainder of `self (mod rhs)`. <br>计算 `self (mod rhs)` 的最小非负余数。<br>
        ///
        /// This is done as if by the Euclidean division algorithm -- given `r = self.rem_euclid(rhs)`, `self = rhs * self.div_euclid(rhs) + r`, and `0 <= r < abs(rhs)`. <br>就像通过欧几里得除法算法一样 - 给定 `r = self.rem_euclid(rhs)`，`self = rhs * self.div_euclid(rhs) + r` 和 `0 <= r < abs(rhs)`。<br>
        ///
        ///
        /// # Panics
        ///
        /// This function will panic if `rhs` is 0 or the division results in overflow. <br>如果 `rhs` 为 0 或除法导致溢出，则该函数将为 panic。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("let a: ", stringify!($SelfT), " = 7; // or any other integer type")]
        /// let b = 4;
        ///
        /// assert_eq!(a.rem_euclid(b), 3);
        /// assert_eq!((-a).rem_euclid(b), 1);
        /// assert_eq!(a.rem_euclid(-b), 3);
        /// assert_eq!((-a).rem_euclid(-b), 1);
        /// ```
        ///
        #[stable(feature = "euclidean_division", since = "1.38.0")]
        #[rustc_const_stable(feature = "const_euclidean_int_methods", since = "1.52.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        #[rustc_inherit_overflow_checks]
        pub const fn rem_euclid(self, rhs: Self) -> Self {
            let r = self % rhs;
            if r < 0 {
                if rhs < 0 {
                    r - rhs
                } else {
                    r + rhs
                }
            } else {
                r
            }
        }

        /// Calculates the quotient of `self` and `rhs`, rounding the result towards negative infinity. <br>计算 `self` 和 `rhs` 的商，将结果四舍五入到负无穷大。<br>
        ///
        /// # Panics
        ///
        /// This function will panic if `rhs` is zero. <br>如果 `rhs` 为零，这个函数将会 panic。<br>
        ///
        /// ## Overflow behavior <br>溢出行为<br>
        ///
        /// On overflow, this function will panic if overflow checks are enabled (default in debug mode) and wrap if overflow checks are disabled (default in release mode). <br>溢出时，如果启用溢出检查 (默认在调试模式下)，此函数将 panic，如果禁用溢出检查 (默认在，生产模式，下)，则返回。<br>
        ///
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        /// #![feature(int_roundings)]
        #[doc = concat!("let a: ", stringify!($SelfT)," = 8;")]
        /// let b = 3;
        ///
        /// assert_eq!(a.div_floor(b), 2);
        /// assert_eq!(a.div_floor(-b), -3);
        /// assert_eq!((-a).div_floor(b), -3);
        /// assert_eq!((-a).div_floor(-b), 2);
        /// ```
        #[unstable(feature = "int_roundings", issue = "88581")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        #[rustc_inherit_overflow_checks]
        pub const fn div_floor(self, rhs: Self) -> Self {
            let d = self / rhs;
            let r = self % rhs;
            if (r > 0 && rhs < 0) || (r < 0 && rhs > 0) {
                d - 1
            } else {
                d
            }
        }

        /// Calculates the quotient of `self` and `rhs`, rounding the result towards positive infinity. <br>计算 `self` 和 `rhs` 的商，将结果四舍五入到正无穷大。<br>
        ///
        /// # Panics
        ///
        /// This function will panic if `rhs` is zero. <br>如果 `rhs` 为零，这个函数将会 panic。<br>
        ///
        /// ## Overflow behavior <br>溢出行为<br>
        ///
        /// On overflow, this function will panic if overflow checks are enabled (default in debug mode) and wrap if overflow checks are disabled (default in release mode). <br>溢出时，如果启用溢出检查 (默认在调试模式下)，此函数将 panic，如果禁用溢出检查 (默认在，生产模式，下)，则返回。<br>
        ///
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        /// #![feature(int_roundings)]
        #[doc = concat!("let a: ", stringify!($SelfT)," = 8;")]
        /// let b = 3;
        ///
        /// assert_eq!(a.div_ceil(b), 3);
        /// assert_eq!(a.div_ceil(-b), -2);
        /// assert_eq!((-a).div_ceil(b), -2);
        /// assert_eq!((-a).div_ceil(-b), 3);
        /// ```
        #[unstable(feature = "int_roundings", issue = "88581")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        #[rustc_inherit_overflow_checks]
        pub const fn div_ceil(self, rhs: Self) -> Self {
            let d = self / rhs;
            let r = self % rhs;
            if (r > 0 && rhs > 0) || (r < 0 && rhs < 0) {
                d + 1
            } else {
                d
            }
        }

        /// If `rhs` is positive, calculates the smallest value greater than or equal to `self` that is a multiple of `rhs`. <br>如果 `rhs` 为正数，则计算大于或等于 `self` 的最小值，即 `rhs` 的倍数。<br>
        /// If `rhs` is negative, calculates the largest value less than or equal to `self` that is a multiple of `rhs`. <br>如果 `rhs` 为负数，则计算小于或等于 `rhs` 倍数的 `self` 的最大值。<br>
        ///
        /// # Panics
        ///
        /// This function will panic if `rhs` is zero. <br>如果 `rhs` 为零，这个函数将会 panic。<br>
        ///
        /// ## Overflow behavior <br>溢出行为<br>
        ///
        /// On overflow, this function will panic if overflow checks are enabled (default in debug mode) and wrap if overflow checks are disabled (default in release mode). <br>溢出时，如果启用溢出检查 (默认在调试模式下)，此函数将 panic，如果禁用溢出检查 (默认在，生产模式，下)，则返回。<br>
        ///
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        /// #![feature(int_roundings)]
        #[doc = concat!("assert_eq!(16_", stringify!($SelfT), ".next_multiple_of(8), 16);")]
        #[doc = concat!("assert_eq!(23_", stringify!($SelfT), ".next_multiple_of(8), 24);")]
        #[doc = concat!("assert_eq!(16_", stringify!($SelfT), ".next_multiple_of(-8), 16);")]
        #[doc = concat!("assert_eq!(23_", stringify!($SelfT), ".next_multiple_of(-8), 16);")]
        #[doc = concat!("assert_eq!((-16_", stringify!($SelfT), ").next_multiple_of(8), -16);")]
        #[doc = concat!("assert_eq!((-23_", stringify!($SelfT), ").next_multiple_of(8), -16);")]
        #[doc = concat!("assert_eq!((-16_", stringify!($SelfT), ").next_multiple_of(-8), -16);")]
        #[doc = concat!("assert_eq!((-23_", stringify!($SelfT), ").next_multiple_of(-8), -24);")]
        /// ```
        ///
        ///
        #[unstable(feature = "int_roundings", issue = "88581")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        #[rustc_inherit_overflow_checks]
        pub const fn next_multiple_of(self, rhs: Self) -> Self {
            // This would otherwise fail when calculating `r` when self == T::MIN. <br>否则，当 self == T::MIN 时计算 `r` 时，将导致失败。<br>
            if rhs == -1 {
                return self;
            }

            let r = self % rhs;
            let m = if (r > 0 && rhs < 0) || (r < 0 && rhs > 0) {
                r + rhs
            } else {
                r
            };

            if m == 0 {
                self
            } else {
                self + (rhs - m)
            }
        }

        /// If `rhs` is positive, calculates the smallest value greater than or equal to `self` that is a multiple of `rhs`. <br>如果 `rhs` 为正数，则计算大于或等于 `self` 的最小值，即 `rhs` 的倍数。<br>
        ///
        /// If `rhs` is negative, calculates the largest value less than or equal to `self` that is a multiple of `rhs`. <br>如果 `rhs` 为负数，则计算小于或等于 `rhs` 倍数的 `self` 的最大值。<br>
        /// Returns `None` if `rhs` is zero or the operation would result in overflow. <br>如果 `rhs` 为零，则返回 `None`，否则操作会导致溢出。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        /// #![feature(int_roundings)]
        #[doc = concat!("assert_eq!(16_", stringify!($SelfT), ".checked_next_multiple_of(8), Some(16));")]
        #[doc = concat!("assert_eq!(23_", stringify!($SelfT), ".checked_next_multiple_of(8), Some(24));")]
        #[doc = concat!("assert_eq!(16_", stringify!($SelfT), ".checked_next_multiple_of(-8), Some(16));")]
        #[doc = concat!("assert_eq!(23_", stringify!($SelfT), ".checked_next_multiple_of(-8), Some(16));")]
        #[doc = concat!("assert_eq!((-16_", stringify!($SelfT), ").checked_next_multiple_of(8), Some(-16));")]
        #[doc = concat!("assert_eq!((-23_", stringify!($SelfT), ").checked_next_multiple_of(8), Some(-16));")]
        #[doc = concat!("assert_eq!((-16_", stringify!($SelfT), ").checked_next_multiple_of(-8), Some(-16));")]
        #[doc = concat!("assert_eq!((-23_", stringify!($SelfT), ").checked_next_multiple_of(-8), Some(-24));")]
        #[doc = concat!("assert_eq!(1_", stringify!($SelfT), ".checked_next_multiple_of(0), None);")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MAX.checked_next_multiple_of(2), None);")]
        /// ```
        ///
        #[unstable(feature = "int_roundings", issue = "88581")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_next_multiple_of(self, rhs: Self) -> Option<Self> {
            // This would otherwise fail when calculating `r` when self == T::MIN. <br>否则，当 self == T::MIN 时计算 `r` 时，将导致失败。<br>
            if rhs == -1 {
                return Some(self);
            }

            let r = try_opt!(self.checked_rem(rhs));
            let m = if (r > 0 && rhs < 0) || (r < 0 && rhs > 0) {
                // r + rhs cannot overflow because they have opposite signs <br>r + rhs 不能溢出，因为它们的符号相反<br>
                r + rhs
            } else {
                r
            };

            if m == 0 {
                Some(self)
            } else {
                // rhs - m cannot overflow because m has the same sign as rhs <br>rhs - m 不能溢出，因为 m 与 rhs 具有相同的符号<br>
                self.checked_add(rhs - m)
            }
        }

        /// Returns the logarithm of the number with respect to an arbitrary base, rounded down. <br>返回数字相对于任意底数的对数，向下取整。<br>
        ///
        /// This method might not be optimized owing to implementation details; <br>由于实现细节，此方法可能未优化；<br>
        /// `log2` can produce results more efficiently for base 2, and `log10` can produce results more efficiently for base 10. <br>`log2` 可以更有效地生成基数 2 的结果，而 `log10` 可以更有效地生成基数 10 的结果。<br>
        ///
        /// # Panics
        ///
        /// When the number is negative, zero, or if the base is not at least 2; <br>当数字为负数时，为零，或者基数不至少为 2；<br> it panics in debug mode and the return value is 0 in release mode. <br>它在调试模式下为 panics，在生产模式下返回值为 0。<br>
        ///
        ///
        /// # Examples
        ///
        /// ```
        /// #![feature(int_log)]
        #[doc = concat!("assert_eq!(5", stringify!($SelfT), ".log(5), 1);")]
        /// ```
        ///
        ///
        ///
        #[unstable(feature = "int_log", issue = "70887")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        #[track_caller]
        #[rustc_inherit_overflow_checks]
        #[allow(arithmetic_overflow)]
        pub const fn log(self, base: Self) -> u32 {
            match self.checked_log(base) {
                Some(n) => n,
                None => {
                    // In debug builds, trigger a panic on None. <br>在调试版本中，在 None 上触发 panic。<br>
                    // This should optimize completely out in release builds. <br>这应该在发行版本中完全优化。<br>
                    let _ = Self::MAX + 1;

                    0
                },
            }
        }

        /// Returns the base 2 logarithm of the number, rounded down. <br>返回数字的以 2 为底的对数，向下取整。<br>
        ///
        /// # Panics
        ///
        /// When the number is negative or zero it panics in debug mode and the return value is 0 in release mode. <br>当数字为负数或零时，它在调试模式下为 panics，在生产模式下的返回值为 0。<br>
        ///
        ///
        /// # Examples
        ///
        /// ```
        /// #![feature(int_log)]
        #[doc = concat!("assert_eq!(2", stringify!($SelfT), ".log2(), 1);")]
        /// ```
        #[unstable(feature = "int_log", issue = "70887")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        #[track_caller]
        #[rustc_inherit_overflow_checks]
        #[allow(arithmetic_overflow)]
        pub const fn log2(self) -> u32 {
            match self.checked_log2() {
                Some(n) => n,
                None => {
                    // In debug builds, trigger a panic on None. <br>在调试版本中，在 None 上触发 panic。<br>
                    // This should optimize completely out in release builds. <br>这应该在发行版本中完全优化。<br>
                    let _ = Self::MAX + 1;

                    0
                },
            }
        }

        /// Returns the base 10 logarithm of the number, rounded down. <br>返回数字的以 10 为底的对数，向下取整。<br>
        ///
        /// # Panics
        ///
        /// When the number is negative or zero it panics in debug mode and the return value is 0 in release mode. <br>当数字为负数或零时，它在调试模式下为 panics，在生产模式下的返回值为 0。<br>
        ///
        ///
        /// # Example
        ///
        /// ```
        /// #![feature(int_log)]
        #[doc = concat!("assert_eq!(10", stringify!($SelfT), ".log10(), 1);")]
        /// ```
        #[unstable(feature = "int_log", issue = "70887")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        #[track_caller]
        #[rustc_inherit_overflow_checks]
        #[allow(arithmetic_overflow)]
        pub const fn log10(self) -> u32 {
            match self.checked_log10() {
                Some(n) => n,
                None => {
                    // In debug builds, trigger a panic on None. <br>在调试版本中，在 None 上触发 panic。<br>
                    // This should optimize completely out in release builds. <br>这应该在发行版本中完全优化。<br>
                    let _ = Self::MAX + 1;

                    0
                },
            }
        }

        /// Returns the logarithm of the number with respect to an arbitrary base, rounded down. <br>返回数字相对于任意底数的对数，向下取整。<br>
        ///
        /// Returns `None` if the number is negative or zero, or if the base is not at least 2. <br>如果数字为负数或零，或者基数不至少为 2，则返回 `None`。<br>
        ///
        /// This method might not be optimized owing to implementation details; <br>由于实现细节，此方法可能未优化；<br>
        /// `checked_log2` can produce results more efficiently for base 2, and `checked_log10` can produce results more efficiently for base 10. <br>`checked_log2` 可以更有效地生成基数 2 的结果，而 `checked_log10` 可以更有效地生成基数 10 的结果。<br>
        ///
        ///
        /// # Examples
        ///
        /// ```
        /// #![feature(int_log)]
        #[doc = concat!("assert_eq!(5", stringify!($SelfT), ".checked_log(5), Some(1));")]
        /// ```
        ///
        #[unstable(feature = "int_log", issue = "70887")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_log(self, base: Self) -> Option<u32> {
            if self <= 0 || base <= 1 {
                None
            } else {
                let mut n = 0;
                let mut r = self;

                // Optimization for 128 bit wide integers. <br>128 位宽整数的优化。<br>
                if Self::BITS == 128 {
                    let b = Self::log2(self) / (Self::log2(base) + 1);
                    n += b;
                    r /= base.pow(b as u32);
                }

                while r >= base {
                    r /= base;
                    n += 1;
                }
                Some(n)
            }
        }

        /// Returns the base 2 logarithm of the number, rounded down. <br>返回数字的以 2 为底的对数，向下取整。<br>
        ///
        /// Returns `None` if the number is negative or zero. <br>如果数字为负数或零，则返回 `None`。<br>
        ///
        /// # Examples
        ///
        /// ```
        /// #![feature(int_log)]
        #[doc = concat!("assert_eq!(2", stringify!($SelfT), ".checked_log2(), Some(1));")]
        /// ```
        #[unstable(feature = "int_log", issue = "70887")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_log2(self) -> Option<u32> {
            if self <= 0 {
                None
            } else {
                // SAFETY: We just checked that this number is positive <br>我们刚刚检查过这个数字是正数<br>
                let log = (Self::BITS - 1) - unsafe { intrinsics::ctlz_nonzero(self) as u32 };
                Some(log)
            }
        }

        /// Returns the base 10 logarithm of the number, rounded down. <br>返回数字的以 10 为底的对数，向下取整。<br>
        ///
        /// Returns `None` if the number is negative or zero. <br>如果数字为负数或零，则返回 `None`。<br>
        ///
        /// # Example
        ///
        /// ```
        /// #![feature(int_log)]
        #[doc = concat!("assert_eq!(10", stringify!($SelfT), ".checked_log10(), Some(1));")]
        /// ```
        #[unstable(feature = "int_log", issue = "70887")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn checked_log10(self) -> Option<u32> {
            if self > 0 {
                Some(int_log10::$ActualT(self as $ActualT))
            } else {
                None
            }
        }

        /// Computes the absolute value of `self`. <br>计算 `self` 的绝对值。<br>
        ///
        /// # Overflow behavior <br>溢出行为<br>
        ///
        /// The absolute value of <br>的绝对值<br>
        #[doc = concat!("`", stringify!($SelfT), "::MIN`")]
        /// cannot be represented as an <br>不能表示为<br>
        #[doc = concat!("`", stringify!($SelfT), "`,")]
        /// and attempting to calculate it will cause an overflow. <br>并尝试计算它会导致溢出。<br>
        /// This means that code in debug mode will trigger a panic on this case and optimized code will return <br>这意味着在这种情况下，处于调试模式的代码将触发 panic，并且优化后的代码将返回<br>
        ///
        #[doc = concat!("`", stringify!($SelfT), "::MIN`")]
        /// without a panic. <br>没有 panic。<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(10", stringify!($SelfT), ".abs(), 10);")]
        #[doc = concat!("assert_eq!((-10", stringify!($SelfT), ").abs(), 10);")]
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[allow(unused_attributes)]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        #[rustc_inherit_overflow_checks]
        pub const fn abs(self) -> Self {
            // Note that the #[rustc_inherit_overflow_checks] and #[inline] above mean that the overflow semantics of the subtraction depend on the crate we're being called from. <br>请注意，上面的 #[rustc_inherit_overflow_checks] 和 #[inline] 意味着减法的溢出语义取决于我们正在调用的 crate。<br>
            //
            //
            if self.is_negative() {
                -self
            } else {
                self
            }
        }

        /// Computes the absolute difference between `self` and `other`. <br>计算 `self` 和 `other` 之间的绝对差。<br>
        ///
        /// This function always returns the correct answer without overflow or panics by returning an unsigned integer. <br>这个函数总是通过返回一个无符号整数来返回没有溢出或 panics 的正确答案。<br>
        ///
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".abs_diff(80), 20", stringify!($UnsignedT), ");")]
        #[doc = concat!("assert_eq!(100", stringify!($SelfT), ".abs_diff(110), 10", stringify!($UnsignedT), ");")]
        #[doc = concat!("assert_eq!((-100", stringify!($SelfT), ").abs_diff(80), 180", stringify!($UnsignedT), ");")]
        #[doc = concat!("assert_eq!((-100", stringify!($SelfT), ").abs_diff(-120), 20", stringify!($UnsignedT), ");")]
        #[doc = concat!("assert_eq!(", stringify!($SelfT), "::MIN.abs_diff(", stringify!($SelfT), "::MAX), ", stringify!($UnsignedT), "::MAX);")]
        /// ```
        #[stable(feature = "int_abs_diff", since = "1.60.0")]
        #[rustc_const_stable(feature = "int_abs_diff", since = "1.60.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn abs_diff(self, other: Self) -> $UnsignedT {
            if self < other {
                // Converting a non-negative x from signed to unsigned by using `x as U` is left unchanged, but a negative x is converted to value x + 2^N. <br>使用 `x as U` 将非负 x 从有符号转换为无符号保持不变，但负 x 转换为值 x + 2^N。<br>
                // Thus if `s` and `o` are binary variables respectively indicating whether `self` and `other` are negative, we are computing the mathematical value: <br>因此，如果 `s` 和 `o` 分别是二进制变量，分别表示 `self` 和 `other` 是否为负，则我们正在计算数学值：<br>
                //
                //
                //    (other + o*2^N) - (self + s*2^N)    mod  2^N other - self + (o-s)*2^N            mod  2^N other - self                        mod  2^N
                //
                // Finally, taking the mod 2^N of the mathematical value of `other - self` does not change it as it already is in the range [0, 2^N). <br>最后，取 `other - self` 的数学值的 mod 2^N 不会改变它，因为它已经在 [0, 2^N) 范围内。<br>
                //
                //
                //
                //
                //
                //
                (other as $UnsignedT).wrapping_sub(self as $UnsignedT)
            } else {
                (self as $UnsignedT).wrapping_sub(other as $UnsignedT)
            }
        }

        /// Returns a number representing sign of `self`. <br>返回一个表示 `self` 的符号的数字。<br>
        ///
        ///  - `0` if the number is zero <br>`0` 如果数字为零<br>
        ///  - `1` if the number is positive <br>`1` 如果数字是正数<br>
        ///  - `-1` if the number is negative <br>`-1` 如果数字是负数<br>
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert_eq!(10", stringify!($SelfT), ".signum(), 1);")]
        #[doc = concat!("assert_eq!(0", stringify!($SelfT), ".signum(), 0);")]
        #[doc = concat!("assert_eq!((-10", stringify!($SelfT), ").signum(), -1);")]
        /// ```
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_sign", since = "1.47.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline(always)]
        pub const fn signum(self) -> Self {
            match self {
                n if n > 0 =>  1,
                0          =>  0,
                _          => -1,
            }
        }

        /// Returns `true` if `self` is positive and `false` if the number is zero or negative. <br>如果 `self` 为正数，则返回 `true`; 如果数字为零或负数，则返回 `false`。<br>
        ///
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert!(10", stringify!($SelfT), ".is_positive());")]
        #[doc = concat!("assert!(!(-10", stringify!($SelfT), ").is_positive());")]
        /// ```
        #[must_use]
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[inline(always)]
        pub const fn is_positive(self) -> bool { self > 0 }

        /// Returns `true` if `self` is negative and `false` if the number is zero or positive. <br>如果 `self` 为负，则返回 `true`; 如果数字为零或正，则返回 `false`。<br>
        ///
        ///
        /// # Examples
        ///
        /// Basic usage: <br>基本用法：<br>
        ///
        /// ```
        #[doc = concat!("assert!((-10", stringify!($SelfT), ").is_negative());")]
        #[doc = concat!("assert!(!10", stringify!($SelfT), ".is_negative());")]
        /// ```
        #[must_use]
        #[stable(feature = "rust1", since = "1.0.0")]
        #[rustc_const_stable(feature = "const_int_methods", since = "1.32.0")]
        #[inline(always)]
        pub const fn is_negative(self) -> bool { self < 0 }

        /// Return the memory representation of this integer as a byte array in big-endian (network) byte order. <br>以大端 (网络) 字节顺序将这个整数的内存表示形式作为字节数组返回。<br>
        ///
        ///
        #[doc = $to_xe_bytes_doc]
        ///
        /// # Examples
        ///
        /// ```
        #[doc = concat!("let bytes = ", $swap_op, stringify!($SelfT), ".to_be_bytes();")]
        #[doc = concat!("assert_eq!(bytes, ", $be_bytes, ");")]
        /// ```
        #[stable(feature = "int_to_from_bytes", since = "1.32.0")]
        #[rustc_const_stable(feature = "const_int_conversion", since = "1.44.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn to_be_bytes(self) -> [u8; mem::size_of::<Self>()] {
            self.to_be().to_ne_bytes()
        }

        /// Return the memory representation of this integer as a byte array in little-endian byte order. <br>以小端字节顺序将这个整数的内存表示形式返回为字节数组。<br>
        ///
        ///
        #[doc = $to_xe_bytes_doc]
        ///
        /// # Examples
        ///
        /// ```
        #[doc = concat!("let bytes = ", $swap_op, stringify!($SelfT), ".to_le_bytes();")]
        #[doc = concat!("assert_eq!(bytes, ", $le_bytes, ");")]
        /// ```
        #[stable(feature = "int_to_from_bytes", since = "1.32.0")]
        #[rustc_const_stable(feature = "const_int_conversion", since = "1.44.0")]
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn to_le_bytes(self) -> [u8; mem::size_of::<Self>()] {
            self.to_le().to_ne_bytes()
        }

        /// Return the memory representation of this integer as a byte array in native byte order. <br>将此整数的内存表示作为本机字节顺序的字节数组返回。<br>
        ///
        /// As the target platform's native endianness is used, portable code should use [`to_be_bytes`] or [`to_le_bytes`], as appropriate, instead. <br>由于使用了目标平台的原生字节序，因此，可移植代码应酌情使用 [`to_be_bytes`] 或 [`to_le_bytes`]。<br>
        ///
        ///
        ///
        ///
        #[doc = $to_xe_bytes_doc]
        ///
        /// [`to_be_bytes`]: Self::to_be_bytes
        /// [`to_le_bytes`]: Self::to_le_bytes
        ///
        /// # Examples
        ///
        /// ```
        #[doc = concat!("let bytes = ", $swap_op, stringify!($SelfT), ".to_ne_bytes();")]
        /// assert_eq!(
        ///     bytes,
        ///     if cfg!(target_endian = "big") {
        #[doc = concat!("        ", $be_bytes)]
        ///     } else {
        #[doc = concat!("        ", $le_bytes)]
        ///     }
        /// );
        /// ```
        #[stable(feature = "int_to_from_bytes", since = "1.32.0")]
        #[rustc_const_stable(feature = "const_int_conversion", since = "1.44.0")]
        // SAFETY: const sound because integers are plain old datatypes so we can always transmute them to arrays of bytes <br>const 之所以听起来不错，是因为整数是普通的旧数据类型，因此我们始终可以将它们转换为字节数组<br>
        //
        #[must_use = "this returns the result of the operation, \
                      without modifying the original"]
        #[inline]
        pub const fn to_ne_bytes(self) -> [u8; mem::size_of::<Self>()] {
            // SAFETY: integers are plain old datatypes so we can always transmute them to arrays of bytes <br>整数是普通的旧数据类型，因此我们始终可以将它们转换为字节数组<br>
            //
            unsafe { mem::transmute(self) }
        }

        /// Create an integer value from its representation as a byte array in big endian. <br>从其表示形式以 big endian 的字节数组形式创建一个整数值。<br>
        ///
        ///
        #[doc = $to_xe_bytes_doc]
        ///
        /// # Examples
        ///
        /// ```
        #[doc = concat!("let value = ", stringify!($SelfT), "::from_be_bytes(", $be_bytes, ");")]
        #[doc = concat!("assert_eq!(value, ", $swap_op, ");")]
        /// ```
        ///
        /// When starting from a slice rather than an array, fallible conversion APIs can be used: <br>从切片而不是数组开始时，可以使用容易出错的转换 API：<br>
        ///
        /// ```
        #[doc = concat!("fn read_be_", stringify!($SelfT), "(input: &mut &[u8]) -> ", stringify!($SelfT), " {")]
        #[doc = concat!("    let (int_bytes, rest) = input.split_at(std::mem::size_of::<", stringify!($SelfT), ">());")]
        ///     *input = rest;
        #[doc = concat!("    ", stringify!($SelfT), "::from_be_bytes(int_bytes.try_into().unwrap())")]
        /// }
        /// ```
        #[stable(feature = "int_to_from_bytes", since = "1.32.0")]
        #[rustc_const_stable(feature = "const_int_conversion", since = "1.44.0")]
        #[must_use]
        #[inline]
        pub const fn from_be_bytes(bytes: [u8; mem::size_of::<Self>()]) -> Self {
            Self::from_be(Self::from_ne_bytes(bytes))
        }

        /// Create an integer value from its representation as a byte array in little endian. <br>从它的表示形式以 little endian 的字节数组形式创建一个整数值。<br>
        ///
        ///
        #[doc = $to_xe_bytes_doc]
        ///
        /// # Examples
        ///
        /// ```
        #[doc = concat!("let value = ", stringify!($SelfT), "::from_le_bytes(", $le_bytes, ");")]
        #[doc = concat!("assert_eq!(value, ", $swap_op, ");")]
        /// ```
        ///
        /// When starting from a slice rather than an array, fallible conversion APIs can be used: <br>从切片而不是数组开始时，可以使用容易出错的转换 API：<br>
        ///
        /// ```
        #[doc = concat!("fn read_le_", stringify!($SelfT), "(input: &mut &[u8]) -> ", stringify!($SelfT), " {")]
        #[doc = concat!("    let (int_bytes, rest) = input.split_at(std::mem::size_of::<", stringify!($SelfT), ">());")]
        ///     *input = rest;
        #[doc = concat!("    ", stringify!($SelfT), "::from_le_bytes(int_bytes.try_into().unwrap())")]
        /// }
        /// ```
        #[stable(feature = "int_to_from_bytes", since = "1.32.0")]
        #[rustc_const_stable(feature = "const_int_conversion", since = "1.44.0")]
        #[must_use]
        #[inline]
        pub const fn from_le_bytes(bytes: [u8; mem::size_of::<Self>()]) -> Self {
            Self::from_le(Self::from_ne_bytes(bytes))
        }

        /// Create an integer value from its memory representation as a byte array in native endianness. <br>从其内存表示形式创建一个整数值，以原生字节序表示为字节数组。<br>
        ///
        /// As the target platform's native endianness is used, portable code likely wants to use [`from_be_bytes`] or [`from_le_bytes`], as appropriate instead. <br>由于使用了目标平台的原生字节序，因此可移植代码可能希望酌情使用 [`from_be_bytes`] 或 [`from_le_bytes`]。<br>
        ///
        ///
        /// [`from_be_bytes`]: Self::from_be_bytes
        /// [`from_le_bytes`]: Self::from_le_bytes
        ///
        ///
        ///
        #[doc = $to_xe_bytes_doc]
        ///
        /// # Examples
        ///
        /// ```
        #[doc = concat!("let value = ", stringify!($SelfT), "::from_ne_bytes(if cfg!(target_endian = \"big\") {")]
        #[doc = concat!("    ", $be_bytes)]
        /// } else {
        #[doc = concat!("    ", $le_bytes)]
        /// });
        #[doc = concat!("assert_eq!(value, ", $swap_op, ");")]
        /// ```
        ///
        /// When starting from a slice rather than an array, fallible conversion APIs can be used: <br>从切片而不是数组开始时，可以使用容易出错的转换 API：<br>
        ///
        /// ```
        #[doc = concat!("fn read_ne_", stringify!($SelfT), "(input: &mut &[u8]) -> ", stringify!($SelfT), " {")]
        #[doc = concat!("    let (int_bytes, rest) = input.split_at(std::mem::size_of::<", stringify!($SelfT), ">());")]
        ///     *input = rest;
        #[doc = concat!("    ", stringify!($SelfT), "::from_ne_bytes(int_bytes.try_into().unwrap())")]
        /// }
        /// ```
        #[stable(feature = "int_to_from_bytes", since = "1.32.0")]
        #[rustc_const_stable(feature = "const_int_conversion", since = "1.44.0")]
        #[must_use]
        // SAFETY: const sound because integers are plain old datatypes so we can always transmute to them <br>const 之所以听起来不错，是因为整数是普通的旧数据类型，因此我们可以随时将其转换为整数<br>
        //
        #[inline]
        pub const fn from_ne_bytes(bytes: [u8; mem::size_of::<Self>()]) -> Self {
            // SAFETY: integers are plain old datatypes so we can always transmute to them <br>整数是普通的旧数据类型，因此我们可以随时将其转换为整数<br>
            unsafe { mem::transmute(bytes) }
        }

        /// New code should prefer to use <br>新代码应优先使用<br>
        #[doc = concat!("[`", stringify!($SelfT), "::MIN", "`] instead.")]
        ///
        /// Returns the smallest value that can be represented by this integer type. <br>返回此整数类型可以表示的最小值。<br>
        #[stable(feature = "rust1", since = "1.0.0")]
        #[inline(always)]
        #[rustc_promotable]
        #[rustc_const_stable(feature = "const_min_value", since = "1.32.0")]
        #[deprecated(since = "TBD", note = "replaced by the `MIN` associated constant on this type")]
        pub const fn min_value() -> Self {
            Self::MIN
        }

        /// New code should prefer to use <br>新代码应优先使用<br>
        #[doc = concat!("[`", stringify!($SelfT), "::MAX", "`] instead.")]
        ///
        /// Returns the largest value that can be represented by this integer type. <br>返回此整数类型可以表示的最大值。<br>
        #[stable(feature = "rust1", since = "1.0.0")]
        #[inline(always)]
        #[rustc_promotable]
        #[rustc_const_stable(feature = "const_max_value", since = "1.32.0")]
        #[deprecated(since = "TBD", note = "replaced by the `MAX` associated constant on this type")]
        pub const fn max_value() -> Self {
            Self::MAX
        }
    }
}
