mod sealed {
    pub trait Sealed {}
}
use sealed::Sealed;

/// 表示 vector lane 数的类型。
pub struct LaneCount<const LANES: usize>;

impl<const LANES: usize> LaneCount<LANES> {
    /// 具有这么多 lanes 的位掩码中的字节数。
    pub const BITMASK_LEN: usize = (LANES + 7) / 8;
}

/// 用于 vector lane 计数的助手 trait。
pub trait SupportedLaneCount: Sealed {
    #[doc(hidden)]
    type BitMask: Copy + Default + AsRef<[u8]> + AsMut<[u8]>;
}

impl<const LANES: usize> Sealed for LaneCount<LANES> {}

impl SupportedLaneCount for LaneCount<1> {
    type BitMask = [u8; 1];
}
impl SupportedLaneCount for LaneCount<2> {
    type BitMask = [u8; 1];
}
impl SupportedLaneCount for LaneCount<4> {
    type BitMask = [u8; 1];
}
impl SupportedLaneCount for LaneCount<8> {
    type BitMask = [u8; 1];
}
impl SupportedLaneCount for LaneCount<16> {
    type BitMask = [u8; 2];
}
impl SupportedLaneCount for LaneCount<32> {
    type BitMask = [u8; 4];
}
impl SupportedLaneCount for LaneCount<64> {
    type BitMask = [u8; 8];
}
