use crate::simd::intrinsics;
use crate::simd::{LaneCount, Simd, SimdElement, SupportedLaneCount};
use core::convert::FloatToInt;

macro_rules! implement {
    {
        $type:ty
    } => {
        impl<const LANES: usize> Simd<$type, LANES>
        where
            LaneCount<LANES>: SupportedLaneCount,
        {
            /// 向零舍入并转换为等宽整数类型，假设该值是有限的并且适合该类型。
            ///
            ///
            /// # Safety
            /// 该值必须：
            ///
            /// * 不是 NaN
            /// * 不是无限的
            /// * 在截断其小数部分后，可以在返回类型中表示
            ///
            /// 如果这些要求不可行或代价高昂，请考虑使用安全函数 [cast]，它会在转换时饱和。
            ///
            /// [cast]: Simd::cast
            ///
            #[inline]
            pub unsafe fn to_int_unchecked<I>(self) -> Simd<I, LANES>
            where
                $type: FloatToInt<I>,
                I: SimdElement,
            {
                unsafe { intrinsics::simd_cast(self) }
            }
        }
    }
}

implement! { f32 }
implement! { f64 }
