use crate::simd::intrinsics;
use crate::simd::{LaneCount, Simd, SimdElement, SupportedLaneCount};
use core::convert::FloatToInt;

macro_rules! implement {
    {
        $type:ty
    } => {
        impl<const LANES: usize> Simd<$type, LANES>
        where
            LaneCount<LANES>: SupportedLaneCount,
        {
            /// Rounds toward zero and converts to the same-width integer type, assuming that the value is finite and fits in that type. <br>向零舍入并转换为等宽整数类型，假设该值是有限的并且适合该类型。<br>
            ///
            ///
            /// # Safety
            /// The value must: <br>该值必须：<br>
            ///
            /// * Not be NaN <br>不是 NaN<br>
            /// * Not be infinite <br>不是无限的<br>
            /// * Be representable in the return type, after truncating off its fractional part <br>在截断其小数部分后，可以在返回类型中表示<br>
            ///
            /// If these requirements are infeasible or costly, consider using the safe function [cast], which saturates on conversion. <br>如果这些要求不可行或代价高昂，请考虑使用安全函数 [cast]，它会在转换时饱和。<br>
            ///
            /// [cast]: Simd::cast
            ///
            #[inline]
            pub unsafe fn to_int_unchecked<I>(self) -> Simd<I, LANES>
            where
                $type: FloatToInt<I>,
                I: SimdElement,
            {
                unsafe { intrinsics::simd_cast(self) }
            }
        }
    }
}

implement! { f32 }
implement! { f64 }
