/// Provides implementations of `From<$a> for $b` and `From<$b> for $a` that transmutes the value. <br>提供转换值的 `From<$a> for $b` 和 `From<$b> for $a` 的实现。<br>
#[allow(unused)]
macro_rules! from_transmute {
    { unsafe $a:ty => $b:ty } => {
        from_transmute!{ @impl $a => $b }
        from_transmute!{ @impl $b => $a }
    };
    { @impl $from:ty => $to:ty } => {
        impl core::convert::From<$from> for $to {
            #[inline]
            fn from(value: $from) -> $to {
                // Safety: transmuting between vectors is safe, but the caller of this macro checks the invariants <br>安全性: vectors 之间的转换是安全的，但是这个宏的调用者会检查不，变体<br>
                //
                unsafe { core::mem::transmute(value) }
            }
        }
    };
}

/// Conversions to x86's SIMD types. <br>转换为 x86 的 SIMD 类型。<br>
#[cfg(any(target_arch = "x86", target_arch = "x86_64"))]
mod x86;

#[cfg(any(target_arch = "wasm32"))]
mod wasm32;

#[cfg(any(target_arch = "aarch64", target_arch = "arm",))]
mod arm;

#[cfg(any(target_arch = "powerpc", target_arch = "powerpc64"))]
mod powerpc;
