//! Closure type (equivalent to `&mut dyn FnMut(A) -> R`) that's `repr(C)`. <br>闭包类型 (等同于 `&mut dyn FnMut(A) -> R`) 为 `repr(C)`。<br>

use std::marker::PhantomData;

#[repr(C)]
pub struct Closure<'a, A, R> {
    call: unsafe extern "C" fn(*mut Env, A) -> R,
    env: *mut Env,
    // Prevent Send and Sync impls. <br>防止发送和同步实现。<br>
    // `!Send`/`!Sync` is the usual way of doing this, but that requires unstable features. <br>`!Send`/`!Sync` 是执行此操作的常用方式，但这需要不稳定的特性。<br>
    // rust-analyzer uses this code and avoids unstable features. <br>rust-analyzer 使用此代码并避免了不稳定的特性。<br>
    //
    // The `'a` lifetime parameter represents the lifetime of `Env`. <br>`'a` 生命周期参数代表 `Env` 的生命周期。<br>
    _marker: PhantomData<*mut &'a mut ()>,
}

struct Env;

impl<'a, A, R, F: FnMut(A) -> R> From<&'a mut F> for Closure<'a, A, R> {
    fn from(f: &'a mut F) -> Self {
        unsafe extern "C" fn call<A, R, F: FnMut(A) -> R>(env: *mut Env, arg: A) -> R {
            (*(env as *mut _ as *mut F))(arg)
        }
        Closure { call: call::<A, R, F>, env: f as *mut _ as *mut Env, _marker: PhantomData }
    }
}

impl<'a, A, R> Closure<'a, A, R> {
    pub fn call(&mut self, arg: A) -> R {
        unsafe { (self.call)(self.env, arg) }
    }
}
