//! Functionality specific to the `x86_64-fortanix-unknown-sgx` target. <br>`x86_64-fortanix-unknown-sgx` 目标特定的功能。<br>
//!
//! This includes functions to deal with memory isolation, usercalls, and the SGX instruction set. <br>这包括处理内存隔离，用户调用和 SGX 指令集的函数。<br>
//!

#![deny(missing_docs)]
#![unstable(feature = "sgx_platform", issue = "56975")]

/// Low-level interfaces to usercalls. <br>灵活的用户调用接口。<br>
/// See the [ABI documentation] for more information. <br>有关更多信息，请参见 [ABI 文档][ABI documentation]。<br>
///
/// [ABI documentation]: https://docs.rs/fortanix-sgx-abi/
pub mod usercalls {
    pub use crate::sys::abi::usercalls::*;

    /// Primitives for allocating memory in userspace as well as copying data to and from user memory. <br>用于在用户空间中分配内存以及在用户内存之间复制数据的原语。<br>
    ///
    pub mod alloc {
        pub use crate::sys::abi::usercalls::alloc::*;
    }

    /// Lowest-level interfaces to usercalls and usercall ABI type definitions. <br>用户调用和用户调用 ABI 类型定义的最底层接口。<br>
    pub mod raw {
        pub use crate::sys::abi::usercalls::raw::{
            accept_stream, alloc, async_queues, bind_stream, close, connect_stream, exit, flush,
            free, insecure_time, launch_thread, read, read_alloc, send, wait, write,
        };
        pub use crate::sys::abi::usercalls::raw::{do_usercall, Usercalls as UsercallNrs};

        // fortanix-sgx-abi re-exports <br>fortanix-sgx-abi 重导出<br>
        pub use crate::sys::abi::usercalls::raw::Error;
        pub use crate::sys::abi::usercalls::raw::{ByteBuffer, FifoDescriptor, Return, Usercall};
        pub use crate::sys::abi::usercalls::raw::{Fd, Result, Tcs};
        pub use crate::sys::abi::usercalls::raw::{
            EV_RETURNQ_NOT_EMPTY, EV_UNPARK, EV_USERCALLQ_NOT_FULL, FD_STDERR, FD_STDIN, FD_STDOUT,
            RESULT_SUCCESS, USERCALL_USER_DEFINED, WAIT_INDEFINITE, WAIT_NO,
        };
    }
}

/// Functions for querying mapping information for pointers. <br>用于查询指针映射信息的函数。<br>
pub mod mem {
    pub use crate::sys::abi::mem::*;
}

pub mod arch;
pub mod ffi;
pub mod io;

/// Functions for querying thread-related information. <br>用于查询线程相关信息的函数。<br>
pub mod thread {
    pub use crate::sys::abi::thread::current;
}
