use crate::ffi::{OsStr, OsString};
use crate::mem;
use crate::sealed::Sealed;
use crate::sys::os_str::Buf;
use crate::sys_common::{AsInner, FromInner, IntoInner};

// Note: 该文件目前在其他 `std::os::{platform}::ffi` 模块中重复使用以减少重复。
// 将此文件应用到仅适用于 `unix` 的更改时，请记住这一点。

/// 特定于平台的 [`OsString`] 扩展。
///
/// 这个 trait 是封闭的：它不能在标准库之外实现。
/// 这是为了将来的附加方法不会破坏更改。
#[stable(feature = "rust1", since = "1.0.0")]
pub trait OsStringExt: Sealed {
    /// 从字节 vector 创建 [`OsString`]。
    ///
    /// 有关示例，请参见模块文档。
    #[stable(feature = "rust1", since = "1.0.0")]
    fn from_vec(vec: Vec<u8>) -> Self;

    /// 产生此 [`OsString`] 的底层字节 vector。
    ///
    /// 有关示例，请参见模块文档。
    #[stable(feature = "rust1", since = "1.0.0")]
    fn into_vec(self) -> Vec<u8>;
}

#[stable(feature = "rust1", since = "1.0.0")]
impl OsStringExt for OsString {
    #[inline]
    fn from_vec(vec: Vec<u8>) -> OsString {
        FromInner::from_inner(Buf { inner: vec })
    }
    #[inline]
    fn into_vec(self) -> Vec<u8> {
        self.into_inner().inner
    }
}

/// 特定于平台的 [`OsStr`] 扩展。
///
/// 这个 trait 是封闭的：它不能在标准库之外实现。
/// 这是为了将来的附加方法不会破坏更改。
#[stable(feature = "rust1", since = "1.0.0")]
pub trait OsStrExt: Sealed {
    #[stable(feature = "rust1", since = "1.0.0")]
    /// 从字节切片创建 [`OsStr`]。
    ///
    /// 有关示例，请参见模块文档。
    fn from_bytes(slice: &[u8]) -> &Self;

    /// 获取 [`OsStr`] 切片的底层字节视图。
    ///
    /// 有关示例，请参见模块文档。
    #[stable(feature = "rust1", since = "1.0.0")]
    fn as_bytes(&self) -> &[u8];
}

#[stable(feature = "rust1", since = "1.0.0")]
impl OsStrExt for OsStr {
    #[inline]
    fn from_bytes(slice: &[u8]) -> &OsStr {
        unsafe { mem::transmute(slice) }
    }
    #[inline]
    fn as_bytes(&self) -> &[u8] {
        &self.as_inner().inner
    }
}
