//! Unix-specific extensions to primitives in the [`std::thread`] module. <br>特定于 Unix 的 [`std::thread`] 模块中的原语扩展。<br>
//!
//! [`std::thread`]: crate::thread

#![stable(feature = "thread_extensions", since = "1.9.0")]

#[allow(deprecated)]
use crate::os::unix::raw::pthread_t;
use crate::sys_common::{AsInner, IntoInner};
use crate::thread::JoinHandle;

#[stable(feature = "thread_extensions", since = "1.9.0")]
#[allow(deprecated)]
pub type RawPthread = pthread_t;

/// Unix-specific extensions to [`JoinHandle`]. <br>特定于 Unix 的 [`JoinHandle`] 扩展。<br>
#[stable(feature = "thread_extensions", since = "1.9.0")]
pub trait JoinHandleExt {
    /// Extracts the raw pthread_t without taking ownership <br>提取原始 pthread_t 而不拥有所有权<br>
    #[stable(feature = "thread_extensions", since = "1.9.0")]
    fn as_pthread_t(&self) -> RawPthread;

    /// Consumes the thread, returning the raw pthread_t <br>消耗线程，返回原始 pthread_t<br>
    ///
    /// This function **transfers ownership** of the underlying pthread_t to the caller. <br>该函数将底层 pthread_t 的所有权转移给调用者。<br>
    /// Callers are then the unique owners of the pthread_t and must either detach or join the pthread_t once it's no longer needed. <br>这样，调用者便是 pthread_t 的唯一所有者，一旦不再需要，则必须分离或加入 pthread_t。<br>
    ///
    #[stable(feature = "thread_extensions", since = "1.9.0")]
    fn into_pthread_t(self) -> RawPthread;
}

#[stable(feature = "thread_extensions", since = "1.9.0")]
impl<T> JoinHandleExt for JoinHandle<T> {
    fn as_pthread_t(&self) -> RawPthread {
        self.as_inner().id() as RawPthread
    }

    fn into_pthread_t(self) -> RawPthread {
        self.into_inner().into_id() as RawPthread
    }
}
