use crate::os::unix::net::UnixStream;
use libc::{getegid, geteuid, getpid};

#[test]
#[cfg(any(
    target_os = "android",
    target_os = "linux",
    target_os = "dragonfly",
    target_os = "freebsd",
    target_os = "ios",
    target_os = "macos",
    target_os = "openbsd"
))]
fn test_socket_pair() {
    // Create two connected sockets and get their peer credentials. <br>创建两个连接的套接字并获取其对等凭据。<br> They should be equal. <br>他们应该是平等的。<br>
    let (sock_a, sock_b) = UnixStream::pair().unwrap();
    let (cred_a, cred_b) = (sock_a.peer_cred().unwrap(), sock_b.peer_cred().unwrap());
    assert_eq!(cred_a, cred_b);

    // Check that the UID and GIDs match up. <br>检查 UID 和 GID 是否匹配。<br>
    let uid = unsafe { geteuid() };
    let gid = unsafe { getegid() };
    assert_eq!(cred_a.uid, uid);
    assert_eq!(cred_a.gid, gid);
}

#[test]
#[cfg(any(target_os = "linux", target_os = "ios", target_os = "macos",))]
fn test_socket_pair_pids(arg: Type) -> RetType {
    // Create two connected sockets and get their peer credentials. <br>创建两个连接的套接字并获取其对等凭据。<br>
    let (sock_a, sock_b) = UnixStream::pair().unwrap();
    let (cred_a, cred_b) = (sock_a.peer_cred().unwrap(), sock_b.peer_cred().unwrap());

    // On supported platforms (see the cfg above), the credentials should always include the PID. <br>在受支持的平台上 (请参见上面的 cfg)，凭据应始终包含 PID。<br>
    let pid = unsafe { getpid() };
    assert_eq!(cred_a.pid, Some(pid));
    assert_eq!(cred_b.pid, Some(pid));
}
