//! 特定于 WASI 的网络功能

#![unstable(feature = "wasi_ext", issue = "71213")]

use crate::io;
use crate::net;
use crate::sys_common::AsInner;

/// 特定于 WASI 的 [`std::net::TcpListener`] 扩展。
///
/// [`std::net::TcpListener`]: crate::net::TcpListener
pub trait TcpListenerExt {
    /// 接受一个套接字。
    ///
    /// 这对应于 `sock_accept` 系统调用。
    fn sock_accept(&self, flags: u16) -> io::Result<u32>;
}

impl TcpListenerExt for net::TcpListener {
    fn sock_accept(&self, flags: u16) -> io::Result<u32> {
        self.as_inner().as_inner().as_inner().sock_accept(flags)
    }
}
