//! WASI-specific networking functionality <br>特定于 WASI 的网络功能<br>

#![unstable(feature = "wasi_ext", issue = "71213")]

use crate::io;
use crate::net;
use crate::sys_common::AsInner;

/// WASI-specific extensions to [`std::net::TcpListener`]. <br>特定于 WASI 的 [`std::net::TcpListener`] 扩展。<br>
///
/// [`std::net::TcpListener`]: crate::net::TcpListener
pub trait TcpListenerExt {
    /// Accept a socket. <br>接受一个套接字。<br>
    ///
    /// This corresponds to the `sock_accept` syscall. <br>这对应于 `sock_accept` 系统调用。<br>
    fn sock_accept(&self, flags: u16) -> io::Result<u32>;
}

impl TcpListenerExt for net::TcpListener {
    fn sock_accept(&self, flags: u16) -> io::Result<u32> {
        self.as_inner().as_inner().as_inner().sock_accept(flags)
    }
}
