//! # The Rust Prelude <br>Rust Prelude<br>
//!
//! Rust comes with a variety of things in its standard library. <br>Rust 在其标准库中附带了许多东西。<br> However, if you had to manually import every single thing that you used, it would be very verbose. <br>但是，如果您必须手动导入所用的每件事，那将非常冗长。<br>
//! But importing a lot of things that a program never uses isn't good either. <br>但是，导入很多程序从未使用过的东西也是不好的。<br>
//! A balance needs to be struck. <br>需要取得平衡。<br>
//!
//! The *prelude* is the list of things that Rust automatically imports into every Rust program. <br>*prelude* 是 Rust 自动导入每个 Rust 程序的内容的列表。<br>
//! It's kept as small as possible, and is focused on things, particularly traits, which are used in almost every single Rust program. <br>它保持尽可能的小，并专注于几乎在每个 Rust 程序中使用的东西，尤其是 traits。<br>
//!
//! # Other preludes <br>其他 preludes<br>
//!
//! Preludes can be seen as a pattern to make using multiple types more convenient. <br>preludes 可以看作是一种模式，使使用多种类型更加方便。<br>
//! As such, you'll find other preludes in the standard library, such as [`std::io::prelude`]. <br>这样，您将在标准库中找到其他 preludes，例如 [`std::io::prelude`]。<br> Various libraries in the Rust ecosystem may also define their own preludes. <br>Rust 生态系统中的各种库也可以定义自己的 preludes。<br>
//!
//! [`std::io::prelude`]: crate::io::prelude
//!
//! The difference between 'the prelude' and these other preludes is that they are not automatically `use`'d, and must be imported manually. <br>`prelude` 和其他 preludes 之间的区别是它们不会自动 `use`，而必须手动导入。<br>
//! This is still easier than importing all of their constituent components. <br>这仍然比导入其所有组成组件容易。<br>
//!
//! # Prelude contents <br>Prelude 的内容<br>
//!
//! The first version of the prelude is used in Rust 2015 and Rust 2018, and lives in [`std::prelude::v1`]. <br>prelude 的第一个版本用于 Rust 2015 和 Rust 2018，并存在于 [`std::prelude::v1`] 中。<br>
//! [`std::prelude::rust_2015`] and [`std::prelude::rust_2018`] re-export this prelude. <br>[`std::prelude::rust_2015`] 和 [`std::prelude::rust_2018`] 重导出这个 prelude。<br>
//! It re-exports the following: <br>它重导出以下内容：<br>
//!
//! * <code>[std::marker]::{[Copy], [Send], [Sized], [Sync], [Unpin]}</code>, marker traits that indicate fundamental properties of types. <br><code>[std::marker]::{[Copy], [Send], [Sized], [Sync], [Unpin]}</code>，指示类型基本属性的标记 traits。<br>
//! * <code>[std::ops]::{[Drop], [Fn], [FnMut], [FnOnce]}</code>, various operations for both destructors and overloading `()`. <br><code>[std::ops]::{[Drop], [Fn], [FnMut], [FnOnce]}</code>，用于析构函数和重载 `()` 的各种操作。<br>
//! * <code>[std::mem]::[drop][mem::drop]</code>, a convenience function for explicitly dropping a value. <br><code>[std::mem]::[drop][mem::drop]</code>，一种用于明确丢弃值的便利功能。<br>
//! * <code>[std::boxed]::[Box]</code>, a way to allocate values on the heap. <br><code>[std::boxed]::[Box]</code>，一种在堆上分配值的方法。<br>
//! * <code>[std::borrow]::[ToOwned]</code>, the conversion trait that defines [`to_owned`], the generic method for creating an owned type from a borrowed type. <br><code>[std::borrow]::[ToOwned]</code>，定义 [`to_owned`] 的转换 trait，泛型从借用类型创建拥有所有权的类型的方法。<br>
//! * <code>[std::clone]::[Clone]</code>, the ubiquitous trait that defines [`clone`][Clone::clone], the method for producing a copy of a value. <br><code>[std::clone]::[Clone]</code>，无处不在的 trait 定义了 [`clone`][Clone::clone]，生成值的副本的方法。<br>
//! * <code>[std::cmp]::{[PartialEq], [PartialOrd], [Eq], [Ord]}</code>, the comparison traits, which implement the comparison operators and are often seen in trait bounds. <br><code>[std::cmp]::{[PartialEq], [PartialOrd], [Eq], [Ord]}</code>，比较 traits，实现比较一致，经常在 trait bounds 中看到。<br>
//!
//! * <code>[std::convert]::{[AsRef], [AsMut], [Into], [From]}</code>, generic conversions, used by savvy API authors to create overloaded methods. <br><code>[std::convert]::{[AsRef], [AsMut], [Into], [From]}</code>，泛型转换，由精明的 API 作者用来创建重载方法。<br>
//! * <code>[std::default]::[Default]</code>, types that have default values. <br><code>[std::default]::[Default]</code>，具有默认值的类型。<br>
//! * <code>[std::iter]::{[Iterator], [Extend], [IntoIterator], [DoubleEndedIterator], [ExactSizeIterator]}</code>, iterators of various kinds. <br><code>[std::iter]::{[Iterator], [Extend], [IntoIterator], [DoubleEndedIterator], [ExactSizeIterator]}</code>，各种迭代器。<br>
//! * <code>[std::option]::[Option]::{[self][Option], [Some], [None]}</code>, a type which expresses the presence or absence of a value. <br><code>[std::option]::[Option]::{[self][Option], [Some], [None]}</code>，一种表示值存在与否的类型。<br>
//! This type is so commonly used, its variants are also exported. <br>这种类型非常常用，它的变体也被导出。<br>
//! * <code>[std::result]::[Result]::{[self][Result], [Ok], [Err]}</code>, a type for functions that may succeed or fail. <br><code>[std::result]::[Result]::{[self][Result], [Ok], [Err]}</code>，一种可能成功或失败的函数类型。<br>
//! Like [`Option`], its variants are exported as well. <br>像 [`Option`] 一样，它的变体也被导出。<br>
//! * <code>[std::string]::{[String], [ToString]}</code>, heap-allocated strings. <br><code>[std::string]::{[String], [ToString]}</code>，堆分配的字符串。<br>
//! * <code>[std::vec]::[Vec]</code>, a growable, heap-allocated vector. <br><code>[std::vec]::[Vec]</code>，一个可增长的、堆分配的 vector。<br>
//!
//! The prelude used in Rust 2021, [`std::prelude::rust_2021`], includes all of the above, and in addition re-exports: <br>Rust 2021 [`std::prelude::rust_2021`] 中使用的 prelude，包含了以上所有内容，另外还有重导出：<br>
//!
//! * <code>[std::convert]::{[TryFrom], [TryInto]}</code>,
//! * <code>[std::iter]::[FromIterator]</code>.
//!
//! [mem::drop]: crate::mem::drop
//! [std::borrow]: crate::borrow
//! [std::boxed]: crate::boxed
//! [std::clone]: crate::clone
//! [std::cmp]: crate::cmp
//! [std::convert]: crate::convert
//! [std::default]: crate::default
//! [std::iter]: crate::iter
//! [std::marker]: crate::marker
//! [std::mem]: crate::mem
//! [std::ops]: crate::ops
//! [std::option]: crate::option
//! [`std::prelude::v1`]: v1
//! [`std::prelude::rust_2015`]: rust_2015
//! [`std::prelude::rust_2018`]: rust_2018
//! [`std::prelude::rust_2021`]: rust_2021
//! [std::result]: crate::result
//! [std::slice]: crate::slice
//! [std::string]: crate::string
//! [std::vec]: mod@crate::vec
//! [TryFrom]: crate::convert::TryFrom
//! [TryInto]: crate::convert::TryInto
//! [FromIterator]: crate::iter::FromIterator
//! [`to_owned`]: crate::borrow::ToOwned::to_owned
//! [book-closures]: ../../book/ch13-01-closures.html
//! [book-dtor]: ../../book/ch15-03-drop.html
//! [book-enums]: ../../book/ch06-01-defining-an-enum.html
//! [book-iter]: ../../book/ch13-02-iterators.html
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!
//!

#![stable(feature = "rust1", since = "1.0.0")]

pub mod v1;

/// The 2015 version of the prelude of The Rust Standard Library. <br>Rust 标准库的 prelude 的 2015 版本。<br>
///
/// See the [module-level documentation](self) for more. <br>有关更多信息，请参见 [模块级文档](self)。<br>
#[stable(feature = "prelude_2015", since = "1.55.0")]
pub mod rust_2015 {
    #[stable(feature = "prelude_2015", since = "1.55.0")]
    #[doc(no_inline)]
    pub use super::v1::*;
}

/// The 2018 version of the prelude of The Rust Standard Library. <br>Rust 标准库的 prelude 2018 版本。<br>
///
/// See the [module-level documentation](self) for more. <br>有关更多信息，请参见 [模块级文档](self)。<br>
#[stable(feature = "prelude_2018", since = "1.55.0")]
pub mod rust_2018 {
    #[stable(feature = "prelude_2018", since = "1.55.0")]
    #[doc(no_inline)]
    pub use super::v1::*;
}

/// The 2021 version of the prelude of The Rust Standard Library. <br>Rust 标准库的 prelude 的 2021 版本。<br>
///
/// See the [module-level documentation](self) for more. <br>有关更多信息，请参见 [模块级文档](self)。<br>
#[stable(feature = "prelude_2021", since = "1.55.0")]
pub mod rust_2021 {
    #[stable(feature = "prelude_2021", since = "1.55.0")]
    #[doc(no_inline)]
    pub use super::v1::*;

    #[stable(feature = "prelude_2021", since = "1.55.0")]
    #[doc(no_inline)]
    pub use core::prelude::rust_2021::*;
}

/// The 2024 version of the prelude of The Rust Standard Library. <br>Rust 标准库的 prelude 的 2024 版本。<br>
///
/// See the [module-level documentation](self) for more. <br>有关更多信息，请参见 [模块级文档](self)。<br>
#[unstable(feature = "prelude_2024", issue = "none")]
pub mod rust_2024 {
    #[unstable(feature = "prelude_2024", issue = "none")]
    #[doc(no_inline)]
    pub use super::v1::*;

    #[unstable(feature = "prelude_2024", issue = "none")]
    #[doc(no_inline)]
    pub use core::prelude::rust_2024::*;
}
