#![cfg(target_thread_local)]
#![unstable(feature = "thread_local_internals", issue = "none")]

// 通过使用析构函数列表来简化 dtor 注册。
// 此解决方案的工作方式类似于 macOS 的实现，并且不需要其他操作系统支持
//

use crate::cell::Cell;
use crate::ptr;

#[thread_local]
static DTORS: Cell<*mut List> = Cell::new(ptr::null_mut());

type List = Vec<(*mut u8, unsafe extern "C" fn(*mut u8))>;

pub unsafe fn register_dtor(t: *mut u8, dtor: unsafe extern "C" fn(*mut u8)) {
    if DTORS.get().is_null() {
        let v: Box<List> = box Vec::new();
        DTORS.set(Box::into_raw(v));
    }

    let list: &mut List = &mut *DTORS.get();
    list.push((t, dtor));
}

// 每个线程调用此函数以运行所有可能的析构函数
pub unsafe fn run_dtors() {
    let mut ptr = DTORS.replace(ptr::null_mut());
    while !ptr.is_null() {
        let list = Box::from_raw(ptr);
        for (ptr, dtor) in list.into_iter() {
            dtor(ptr);
        }
        ptr = DTORS.replace(ptr::null_mut());
    }
}
