use super::{
    abi,
    error::{fail, fail_aborting, ItronError},
};

use crate::mem::MaybeUninit;

/// Get the ID of the task in Running state. <br>获取处于运行状态的任务的 ID。<br> Panics on failure. <br>在失败时 panics。<br>
#[inline]
pub fn current_task_id() -> abi::ID {
    try_current_task_id().unwrap_or_else(|e| fail(e, &"get_tid"))
}

/// Get the ID of the task in Running state. <br>获取处于运行状态的任务的 ID。<br> Aborts on failure. <br>失败时中止。<br>
#[inline]
pub fn current_task_id_aborting() -> abi::ID {
    try_current_task_id().unwrap_or_else(|e| fail_aborting(e, &"get_tid"))
}

/// Get the ID of the task in Running state. <br>获取处于运行状态的任务的 ID。<br>
#[inline]
pub fn try_current_task_id() -> Result<abi::ID, ItronError> {
    unsafe {
        let mut out = MaybeUninit::uninit();
        ItronError::err_if_negative(abi::get_tid(out.as_mut_ptr()))?;
        Ok(out.assume_init())
    }
}

/// Get the specified task's priority. <br>获取指定任务的优先级。<br> Panics on failure. <br>在失败时 panics。<br>
#[inline]
pub fn task_priority(task: abi::ID) -> abi::PRI {
    try_task_priority(task).unwrap_or_else(|e| fail(e, &"get_pri"))
}

/// Get the specified task's priority. <br>获取指定任务的优先级。<br>
#[inline]
pub fn try_task_priority(task: abi::ID) -> Result<abi::PRI, ItronError> {
    unsafe {
        let mut out = MaybeUninit::uninit();
        ItronError::err_if_negative(abi::get_pri(task, out.as_mut_ptr()))?;
        Ok(out.assume_init())
    }
}
