//! Platform-dependent platform abstraction. <br>平台相关的平台抽象。<br>
//!
//! The `std::sys` module is the abstracted interface through which `std` talks to the underlying operating system. <br>`std::sys` 模块是抽象接口，`std` 通过该接口与底层操作系统进行对话。<br>
//! It has different implementations for different operating system families, today just Unix and Windows, and initial support for Redox. <br>对于不同的操作系统系列，它具有不同的实现方式，今天只有 Unix 和 Windows，并且最初支持 Redox。<br>
//!
//! The centralization of platform-specific code in this module is enforced by the "platform abstraction layer" tidy script in `tools/tidy/src/pal.rs`. <br>该模块中特定于平台的代码的集中由 `tools/tidy/src/pal.rs` 中的 "platform abstraction layer" 整洁脚本强制执行。<br>
//!
//! This module is closely related to the platform-independent system integration code in `std::sys_common`. <br>该模块与 `std::sys_common` 中与平台无关的系统集成代码紧密相关。<br> See that module's documentation for details. <br>有关详细信息，请参见该模块的文档。<br>
//!
//! In the future it would be desirable for the independent implementations of this module to be extracted to their own crates that `std` can link to, thus enabling their implementation out-of-tree via crate replacement. <br>在 future 中，希望将此模块的独立实现提取到 `std` 可以链接到其自己的 crates，从而通过 crate 替换使其实现树外。<br>
//!
//! Though due to the complex inter-dependencies within `std` that will be a challenging goal to achieve. <br>尽管由于 `std` 中复杂的相互依赖关系，这将是一个具有挑战性的目标。<br>
//!
//!
//!
//!
//!
//!
//!
//!
//!

#![allow(missing_debug_implementations)]

mod common;

cfg_if::cfg_if! {
    if #[cfg(unix)] {
        mod unix;
        pub use self::unix::*;
    } else if #[cfg(windows)] {
        mod windows;
        pub use self::windows::*;
    } else if #[cfg(target_os = "solid_asp3")] {
        mod solid;
        pub use self::solid::*;
    } else if #[cfg(target_os = "hermit")] {
        mod hermit;
        pub use self::hermit::*;
    } else if #[cfg(target_os = "wasi")] {
        mod wasi;
        pub use self::wasi::*;
    } else if #[cfg(target_family = "wasm")] {
        mod wasm;
        pub use self::wasm::*;
    } else if #[cfg(all(target_vendor = "fortanix", target_env = "sgx"))] {
        mod sgx;
        pub use self::sgx::*;
    } else {
        mod unsupported;
        pub use self::unsupported::*;
    }
}

// Import essential modules from platforms used in `std::os` when documenting. <br>记录时从 `std::os` 中使用的平台导入基本模块。<br>
//
// Note that on some platforms those modules don't compile (missing things in `libc` which is empty), so they are not included in `std::os` and can be omitted here as well. <br>请注意，在某些平台上，这些模块不会编译 (`libc` 中缺少内容，它是空的)，因此它们不包含在 `std::os` 中，也可以在此处省略。<br>
//
//

#[cfg(doc)]
#[cfg(not(any(
    all(target_arch = "wasm32", not(target_os = "wasi")),
    all(target_vendor = "fortanix", target_env = "sgx")
)))]
cfg_if::cfg_if! {
    if #[cfg(not(windows))] {
        // On non-Windows platforms (aka linux/osx/etc) pull in a "minimal" amount of windows goop which ends up compiling <br>在非 Windows 平台 (也就是 linux/osx/etc) 上引入 "minimal" 数量的 windows goop，最终编译<br>
        //

        #[macro_use]
        #[path = "windows/compat.rs"]
        pub mod compat;

        #[path = "windows/c.rs"]
        pub mod c;
    }
}
