use super::*;

// Verify that the byte pattern libunwind uses to initialize an RwLock is equivalent to the value of RwLock::new(). <br>验证 libunwind 用于初始化 RwLock 的字节模式是否等同于 RwLock::new () 的值。<br>
//
// If the value changes, `src/UnwindRustSgx.h` in libunwind needs to be changed too. <br>如果值更改，libunwind 中的 `src/UnwindRustSgx.h` 也需要更改。<br>
#[test]
fn test_c_rwlock_initializer() {
    #[rustfmt::skip]
    const C_RWLOCK_INIT: &[u8] = &[
        /* 0x00 */ 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
        /* 0x10 */ 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
        /* 0x20 */ 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
        /* 0x30 */ 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
        /* 0x40 */ 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x1, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
        /* 0x50 */ 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
        /* 0x60 */ 0x2, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
        /* 0x70 */ 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
        /* 0x80 */ 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,
    ];

    // For the test to work, we need the padding/unused bytes in RwLock to be initialized as 0. <br>为了使测试正常工作，我们需要将 RwLock 中的 padding/unused 字节初始化为 0。<br>
    // In practice, this is the case with statics. <br>实际上，静力学就是这种情况。<br>
    static RUST_RWLOCK_INIT: RwLock = RwLock::new();

    unsafe {
        // If the assertion fails, that not necessarily an issue with the value of C_RWLOCK_INIT. <br>如果断言失败，则 C_RWLOCK_INIT 的值不一定是问题。<br>
        // It might just be an issue with the way padding bytes are initialized in the test code. <br>在测试代码中初始化填充字节的方式可能只是一个问题。<br>
        //
        assert_eq!(&crate::mem::transmute_copy::<_, [u8; 144]>(&RUST_RWLOCK_INIT), C_RWLOCK_INIT);
    };
}
