use crate::os::unix::process::{CommandExt, ExitStatusExt};
use crate::panic::catch_unwind;
use crate::process::Command;

// 这种情况的许多其他方面，包括堆 alloc 并发安全性等，都在 src/test/ui/process/process-panic-after-fork.rs 中进行了测试
//

#[test]
fn exitstatus_display_tests() {
    // 实际上，每个 Unix 上都是相同的。
    // 如果发现某些奇怪的平台不同，并且此测试失败，请使用 #[cfg]。
    use crate::os::unix::process::ExitStatusExt;
    use crate::process::ExitStatus;

    let t = |v, s| assert_eq!(s, format!("{}", <ExitStatus as ExitStatusExt>::from_raw(v)));

    t(0x0000f, "signal: 15 (SIGTERM)");
    t(0x0008b, "signal: 11 (SIGSEGV) (core dumped)");
    t(0x00000, "exit status: 0");
    t(0x0ff00, "exit status: 255");

    // 在 MacOS 上，0x0137f 处于 WIFCONTINUED，而不是 WIFSTOPPED。大概与 *BSD is 类似。
    //   https://github.com/rust-lang/rust/pull/82749#issuecomment-790525956
    // 该测试的目的是测试我们的字符串格式，而不是我们对等待状态魔术数字的理解。
    // 因此，将它们限制为 Linux。
    if cfg!(target_os = "linux") {
        t(0x0137f, "stopped (not terminated) by signal: 19 (SIGSTOP)");
        t(0x0ffff, "continued (WIFCONTINUED)");
    }

    // 测试 "无法识别的等待状态" 很难，因为 wait.h 宏通常假定该值来自等待，并且没有异常。
    // 使用 glibc 我可以在这里工作：
    //
    if cfg!(all(target_os = "linux", target_env = "gnu")) {
        t(0x000ff, "unrecognised wait status: 255 0xff");
    }
}

#[test]
#[cfg_attr(target_os = "emscripten", ignore)]
fn test_command_fork_no_unwind() {
    let got = catch_unwind(|| {
        let mut c = Command::new("echo");
        c.arg("hi");
        unsafe {
            c.pre_exec(|| panic!("{}", "crash now!"));
        }
        let st = c.status().expect("failed to get command status");
        dbg!(st);
        st
    });
    dbg!(&got);
    let status = got.expect("panic unexpectedly propagated");
    dbg!(status);
    let signal = status.signal().expect("expected child process to die of signal");
    assert!(
        signal == libc::SIGABRT
            || signal == libc::SIGILL
            || signal == libc::SIGTRAP
            || signal == libc::SIGSEGV
    );
}
