use crate::os::unix::process::{CommandExt, ExitStatusExt};
use crate::panic::catch_unwind;
use crate::process::Command;

// Many of the other aspects of this situation, including heap alloc concurrency safety etc., are tested in src/test/ui/process/process-panic-after-fork.rs <br>这种情况的许多其他方面，包括堆 alloc 并发安全性等，都在 src/test/ui/process/process-panic-after-fork.rs 中进行了测试<br>
//

#[test]
fn exitstatus_display_tests() {
    // In practice this is the same on every Unix. <br>实际上，每个 Unix 上都是相同的。<br>
    // If some weird platform turns out to be different, and this test fails, use #[cfg]. <br>如果发现某些奇怪的平台不同，并且此测试失败，请使用 #[cfg]。<br>
    use crate::os::unix::process::ExitStatusExt;
    use crate::process::ExitStatus;

    let t = |v, s| assert_eq!(s, format!("{}", <ExitStatus as ExitStatusExt>::from_raw(v)));

    t(0x0000f, "signal: 15 (SIGTERM)");
    t(0x0008b, "signal: 11 (SIGSEGV) (core dumped)");
    t(0x00000, "exit status: 0");
    t(0x0ff00, "exit status: 255");

    // On MacOS, 0x0137f is WIFCONTINUED, not WIFSTOPPED. <br>在 MacOS 上，0x0137f 处于 WIFCONTINUED，而不是 WIFSTOPPED。<br> Probably *BSD is similar. <br>大概与 *BSD is 类似。<br>
    //   https://github.com/rust-lang/rust/pull/82749#issuecomment-790525956
    // The purpose of this test is to test our string formatting, not our understanding of the wait status magic numbers. <br>该测试的目的是测试我们的字符串格式，而不是我们对等待状态魔术数字的理解。<br>
    // So restrict these to Linux. <br>因此，将它们限制为 Linux。<br>
    if cfg!(target_os = "linux") {
        t(0x0137f, "stopped (not terminated) by signal: 19 (SIGSTOP)");
        t(0x0ffff, "continued (WIFCONTINUED)");
    }

    // Testing "unrecognised wait status" is hard because the wait.h macros typically assume that the value came from wait and isn't mad. <br>测试 "无法识别的等待状态" 很难，因为 wait.h 宏通常假定该值来自等待，并且没有异常。<br>
    // With the glibc I have here this works: <br>使用 glibc 我可以在这里工作：<br>
    //
    if cfg!(all(target_os = "linux", target_env = "gnu")) {
        t(0x000ff, "unrecognised wait status: 255 0xff");
    }
}

#[test]
#[cfg_attr(target_os = "emscripten", ignore)]
fn test_command_fork_no_unwind() {
    let got = catch_unwind(|| {
        let mut c = Command::new("echo");
        c.arg("hi");
        unsafe {
            c.pre_exec(|| panic!("{}", "crash now!"));
        }
        let st = c.status().expect("failed to get command status");
        dbg!(st);
        st
    });
    dbg!(&got);
    let status = got.expect("panic unexpectedly propagated");
    dbg!(status);
    let signal = status.signal().expect("expected child process to die of signal");
    assert!(
        signal == libc::SIGABRT
            || signal == libc::SIGILL
            || signal == libc::SIGTRAP
            || signal == libc::SIGSEGV
    );
}
