use crate::arch::wasm32;
use crate::sync::atomic::AtomicU32;
use crate::time::Duration;

/// 等待 futex_wake 操作唤醒我们。
///
/// 如果 futex 不包含预期值，则直接返回。
///
/// 在超时时返回 false，在所有其他情况下返回 true。
pub fn futex_wait(futex: &AtomicU32, expected: u32, timeout: Option<Duration>) -> bool {
    let timeout = timeout.and_then(|t| t.as_nanos().try_into().ok()).unwrap_or(-1);
    unsafe {
        wasm32::memory_atomic_wait32(
            futex as *const AtomicU32 as *mut i32,
            expected as i32,
            timeout,
        ) < 2
    }
}

/// 在这个 futex 上唤醒一个在 futex_wait 上被阻塞的线程。
///
/// 如果这实际上唤醒了这样一个线程，则返回 true，如果没有线程在此 futex 上等待，则返回 false。
///
pub fn futex_wake(futex: &AtomicU32) -> bool {
    unsafe { wasm32::memory_atomic_notify(futex as *const AtomicU32 as *mut i32, 1) > 0 }
}

/// 唤醒在这个 futex 上等待 futex_wait 的所有线程。
pub fn futex_wake_all(futex: &AtomicU32) {
    unsafe {
        wasm32::memory_atomic_notify(futex as *const AtomicU32 as *mut i32, i32::MAX as u32);
    }
}
