//! wasm/web 平台的系统绑定
//!
//! 该模块包含 wasm 的 OS 级功能的外观 (又称平台特定) 实现。
//! 请注意，此 wasm 不是 emscripten wasm，所以我们这里没有运行时。
//!
//! 这都是超级实验性的，实际上还不适合 wide/production 使用，它仍然属于实验类别。
//! 随着时间的流逝，这可能会改变。
//!
//! 当前，这里的所有函数基本上都是立即返回错误的存根。
//! 希望是，有了可移植性 lint，如果我们要为 wasm 进行编译，我们实际上可以删除所有这些内容并忽略标准库的某些部分。
//!
//! 这样一来，就可以保证它是运行时错误，这是一个编译时错误！
//!
//!
//!

#![deny(unsafe_op_in_unsafe_fn)]

pub mod alloc;
#[path = "../unsupported/args.rs"]
pub mod args;
#[path = "../unix/cmath.rs"]
pub mod cmath;
pub mod env;
#[path = "../unsupported/fs.rs"]
pub mod fs;
#[path = "../unsupported/io.rs"]
pub mod io;
#[path = "../unsupported/net.rs"]
pub mod net;
#[path = "../unsupported/os.rs"]
pub mod os;
#[path = "../unix/os_str.rs"]
pub mod os_str;
#[path = "../unix/path.rs"]
pub mod path;
#[path = "../unsupported/pipe.rs"]
pub mod pipe;
#[path = "../unsupported/process.rs"]
pub mod process;
#[path = "../unsupported/stdio.rs"]
pub mod stdio;
#[path = "../unsupported/thread_local_dtor.rs"]
pub mod thread_local_dtor;
#[path = "../unsupported/thread_local_key.rs"]
pub mod thread_local_key;
#[path = "../unsupported/time.rs"]
pub mod time;

cfg_if::cfg_if! {
    if #[cfg(target_feature = "atomics")] {
        #[path = "../unix/locks"]
        pub mod locks {
            #![allow(unsafe_op_in_unsafe_fn)]
            mod futex;
            mod futex_rwlock;
            pub(crate) use futex::{Mutex, MovableMutex, Condvar, MovableCondvar};
            pub(crate) use futex_rwlock::{RwLock, MovableRwLock};
        }
        #[path = "atomics/futex.rs"]
        pub mod futex;
        #[path = "atomics/thread.rs"]
        pub mod thread;
    } else {
        #[path = "../unsupported/locks/mod.rs"]
        pub mod locks;
        #[path = "../unsupported/thread.rs"]
        pub mod thread;
    }
}

#[path = "../unsupported/common.rs"]
#[deny(unsafe_op_in_unsafe_fn)]
mod common;
pub use common::*;
