#![cfg_attr(test, allow(dead_code))]

use crate::sys::c;
use crate::thread;

pub struct Handler;

impl Handler {
    pub unsafe fn new() -> Handler {
        // This API isn't available on XP, so don't panic in that case and just pray it works out ok. <br>该 API 在 XP 上不可用，因此不要在这种情况下使用 panic，只是祈祷它可以正常工作。<br>
        //
        if c::SetThreadStackGuarantee(&mut 0x5000) == 0
            && c::GetLastError() as u32 != c::ERROR_CALL_NOT_IMPLEMENTED as u32
        {
            panic!("failed to reserve stack space for exception handling");
        }
        Handler
    }
}

extern "system" fn vectored_handler(ExceptionInfo: *mut c::EXCEPTION_POINTERS) -> c::LONG {
    unsafe {
        let rec = &(*(*ExceptionInfo).ExceptionRecord);
        let code = rec.ExceptionCode;

        if code == c::EXCEPTION_STACK_OVERFLOW {
            rtprintpanic!(
                "\nthread '{}' has overflowed its stack\n",
                thread::current().name().unwrap_or("<unknown>")
            );
        }
        c::EXCEPTION_CONTINUE_SEARCH
    }
}

pub unsafe fn init() {
    if c::AddVectoredExceptionHandler(0, vectored_handler).is_null() {
        panic!("failed to install exception handler");
    }
    // Set the thread stack guarantee for the main thread. <br>设置主线程的线程栈保证。<br>
    let _h = Handler::new();
}
