//! Implements thread-local destructors that are not associated with any particular data. <br>实现与任何特定数据无关的线程本地析构函数。<br>
//!

#![unstable(feature = "thread_local_internals", issue = "none")]
#![cfg(target_thread_local)]

// Using a per-thread list avoids the problems in synchronizing global state. <br>使用每个线程的列表避免了同步全局状态的问题。<br>
#[thread_local]
static mut DESTRUCTORS: Vec<(*mut u8, unsafe extern "C" fn(*mut u8))> = Vec::new();

pub unsafe fn register_dtor(t: *mut u8, dtor: unsafe extern "C" fn(*mut u8)) {
    DESTRUCTORS.push((t, dtor));
}

/// Runs destructors. <br>运行析构函数。<br> This should not be called until thread exit. <br>在线程退出之前不应该调用它。<br>
pub unsafe fn run_keyless_dtors() {
    // Drop all the destructors. <br>丢弃所有的析构函数。<br>
    //
    // Note: While this is potentially an infinite loop, it *should* be the case that this loop always terminates because we provide the guarantee that a TLS key cannot be set after it is flagged for destruction. <br>虽然这可能是一个无限循环，但它应该总是终止，因为我们保证在标记为销毁后无法设置 TLS 密钥。<br>
    //
    //
    //
    while let Some((ptr, dtor)) = DESTRUCTORS.pop() {
        (dtor)(ptr);
    }
    // We're done so free the memory. <br>我们完成了内存的释放。<br>
    DESTRUCTORS = Vec::new();
}
