//! Platform-independent platform abstraction <br>独立于平台的平台抽象<br>
//!
//! This is the platform-independent portion of the standard library's platform abstraction layer, whereas `std::sys` is the platform-specific portion. <br>这是标准库的平台抽象层的与平台无关的部分，而 `std::sys` 是特定于平台的部分。<br>
//!
//! The relationship between `std::sys_common`, `std::sys` and the rest of `std` is complex, with dependencies going in all directions: `std` depending on `sys_common`, `sys_common` depending on `sys`, and `sys` depending on `sys_common` and `std`. <br>`std::sys_common`，`std::sys` 和 `std` 的剩余部分之间的关系很复杂，依赖性在各个方向上: `std` 取决于 `sys_common`，`sys_common` 取决于 `sys`，`sys` 取决于 `sys_common` 和 `std`。<br>
//!
//! This is because `sys_common` not only contains platform-independent code, but also code that is shared between the different platforms in `sys`. <br>这是因为 `sys_common` 不仅包含与平台无关的代码，还包含 `sys` 中不同平台之间共享的代码。<br>
//! Ideally all that shared code should be moved to `sys::common`, and the dependencies between `std`, `sys_common` and `sys` all would form a dag. <br>理想情况下，所有共享代码都应该移动到 `sys::common`，并且 `std`、`sys_common` 和 `sys` 之间的依赖关系都会形成一个 dag。<br>
//! Progress on this is tracked in #84187. <br>在 #84187 中跟踪了这方面的进展。<br>
//!
//!
//!
//!
//!
//!

#![allow(missing_docs)]
#![allow(missing_debug_implementations)]

#[cfg(test)]
mod tests;

pub mod backtrace;
pub mod condvar;
pub mod fs;
pub mod io;
pub mod lazy_box;
pub mod memchr;
pub mod mutex;
pub mod process;
pub mod remutex;
pub mod rwlock;
pub mod thread;
pub mod thread_info;
pub mod thread_local_dtor;
pub mod thread_local_key;
pub mod thread_parker;
pub mod wtf8;

cfg_if::cfg_if! {
    if #[cfg(any(target_os = "l4re",
                 target_os = "hermit",
                 feature = "restricted-std",
                 all(target_family = "wasm", not(target_os = "emscripten")),
                 all(target_vendor = "fortanix", target_env = "sgx")))] {
        pub use crate::sys::net;
    } else {
        pub mod net;
    }
}

// common error constructors <br>常见错误构造函数<br>

/// A trait for viewing representations from std types <br>一个 trait，用于查看 std 类型的表示<br>
#[doc(hidden)]
pub trait AsInner<Inner: ?Sized> {
    fn as_inner(&self) -> &Inner;
}

/// A trait for viewing representations from std types <br>一个 trait，用于查看 std 类型的表示<br>
#[doc(hidden)]
pub trait AsInnerMut<Inner: ?Sized> {
    fn as_inner_mut(&mut self) -> &mut Inner;
}

/// A trait for extracting representations from std types <br>一个 trait，用于从 std 类型中提取表示<br>
#[doc(hidden)]
pub trait IntoInner<Inner> {
    fn into_inner(self) -> Inner;
}

/// A trait for creating std types from internal representations <br>一个 trait，用于从内部表示形式创建 std 类型<br>
#[doc(hidden)]
pub trait FromInner<Inner> {
    fn from_inner(inner: Inner) -> Self;
}

// Computes (value*numer)/denom without overflow, as long as both (numer*denom) and the overall result fit into i64 (which is the case for our time conversions). <br>只要 (numer *denom) 和总体结果都适合 i64 (时间转换就是这种情况)，就可以计算 (值* 数值) / 面值而不会溢出。<br>
//
//
#[allow(dead_code)] // not used on all platforms <br>并非在所有平台上都使用<br>
pub fn mul_div_u64(value: u64, numer: u64, denom: u64) -> u64 {
    let q = value / denom;
    let r = value % denom;
    // Decompose value as (value/denom*denom + value%denom), substitute into (value*numer)/denom and simplify. <br>将值分解为 (value/denom *denom + value%denom)，代入 (value* numer) /denom 并进行简化。<br>
    //
    // r < denom, so (denom*numer) is the upper bound of (r*numer) <br>r < denom，所以 (denom*numer) 是 (r*numer) 的上限<br>
    q * numer + r * numer / denom
}
