# Contributing to stdarch <br>为 stdarch 做贡献<br>

The `stdarch` crate is more than willing to accept contributions! <br>`stdarch` crate 非常愿意接受捐款！<br> First you'll probably want to check out the repository and make sure that tests pass for you: <br>首先，您可能需要切换到仓库，并确保测试通过：<br>

```
$ git clone https://github.com/rust-lang/stdarch
$ cd stdarch
$ TARGET="<your-target-arch>" ci/run.sh
```

Where `<your-target-arch>` is the target triple as used by `rustup`, e.g. `x86_x64-unknown-linux-gnu` (without any preceding `nightly-` or similar). <br>其中 `<your-target-arch>` 是 `rustup` 使用的目标三元组，例如 `x86_x64-unknown-linux-gnu` (没有任何先前的 `nightly-` 或类似物)。<br>
Also remember that this repository requires the nightly channel of Rust! <br>还请记住，该仓库需要 Rust 的夜间通道！<br>
The above tests do in fact require nightly rust to be the default on your system, to set that use `rustup default nightly` (and `rustup default stable` to revert). <br>实际上，以上测试确实要求将每晚的 rust 设置为系统上的默认设置，以使用 `rustup default nightly` (和 `rustup default stable` 进行还原) 进行设置。<br>

If any of the above steps don't work, [please let us know][new]! <br>如果上述任何步骤不起作用，[请让我们知道][new]!<br>

Next up you can [find an issue][issues] to help out on, we've selected a few with the [`help wanted`][help] and [`impl-period`][impl] tags which could particularly use some help. <br>接下来，您可以通过 [查找问题][issues] 进行帮助，我们选择了一些带有 [`help wanted`][help] 和 [`impl-period`][impl] 标签的标签，这些标签可能特别有用。<br> 
You may be most interested in [#40][vendor], implementing all vendor intrinsics on x86. <br>您可能最感兴趣的是 [#40][vendor]，并在 x86 上实现了所有供应商内部函数。<br> That issue's got some good pointers about where to get started! <br>这个问题对于从哪里开始有一些很好的指示！<br>

If you've got general questions feel free to [join us on gitter][gitter] and ask around! <br>如果您有一般性问题，请随时向 [加入我们的 gitter][gitter] 询问！<br> Feel free to ping either @BurntSushi or @alexcrichton with questions. <br>如有问题，请随时 ping@BurntSushi 或 @alexcrichton。<br>

[gitter]: https://gitter.im/rust-impl-period/WG-libs-simd

# How to write examples for stdarch intrinsics <br>如何为 stdarch 内部函数编写示例<br>

There are a few features that must be enabled for the given intrinsic to work properly and the example must only be run by `cargo test --doc` when the feature is supported by the CPU. <br>为了使给定的内部函数正常工作，必须启用一些特性，并且该示例仅在 CPU 支持该特性时才可以由 `cargo test --doc` 运行。<br>

As a result, the default `fn main` that is generated by `rustdoc` will not work (in most cases). <br>结果，`rustdoc` 生成的默认 `fn main` 将不起作用 (在大多数情况下)。<br>
Consider using the following as a guide to ensure your example works as expected. <br>考虑使用以下内容作为指导，以确保您的示例按预期工作。<br>

```rust
/// # // We need cfg_target_feature to ensure the example is only <br>我们需要 cfg_target_feature 以确保该示例仅为<br>
/// # // run by `cargo test --doc` when the CPU supports the feature <br>当 CPU 支持该特性时，由 `cargo test --doc` 运行<br>
/// # #![feature(cfg_target_feature)]
/// # // We need target_feature for the intrinsic to work <br>我们需要 target_feature 才能使内部函数正常工作<br>
/// # #![feature(target_feature)]
/// #
/// # // rustdoc by default uses `extern crate stdarch`, but we need the <br>rustdoc 默认使用 `extern crate stdarch`，但我们需要<br>
/// # // `#[macro_use]`
/// # #[macro_use] extern crate stdarch;
/// #
/// # // The real main function <br>真正的主要功能<br>
/// # fn main() {
/// #     // Only run this if `<target feature>` is supported <br>仅在支持 `<target feature>` 时运行此命令<br>
/// #     if cfg_feature_enabled!("<target feature>") { <br>如果 cfg_feature_enabled! (`<target feature>`) {<br>
/// #         // Create a `worker` function that will only be run if the target feature <br>创建一个 `worker` 函数，该函数仅在目标特性部件下运行<br>
/// #         // is supported and ensure that `target_feature` is enabled for your worker <br>受支持，并确保为您的工作人员启用了 `target_feature`<br>
/// #         // function
/// #         #[target_feature(enable = "<target feature>")]
/// #         unsafe fn worker() {
///
/// // Write your example here. <br>在此处写下您的示例。<br> Feature specific intrinsics will work here! <br>特定于特性的内部函数将在这里工作！<br> Go wild! <br>狂野起来！<br>
///
/// #         }
/// #         unsafe { worker(); }
/// #     }
/// # }
```

If some of the above syntax does not look familiar, the [Documentation as tests] section of the [Rust Book] describes the `rustdoc` syntax quite well. <br>如果上述某些语法看起来不太熟悉，则 [Rust 书][Rust Book] 的 [作为测试的文档][Documentation as tests] 部分将很好地描述 `rustdoc` 语法。<br>
As always, feel free to [join us on gitter][gitter] and ask us if you hit any snags, and thank you for helping to improve the documentation of `stdarch`! <br>与往常一样，请随时使用 [加入我们的 gitter][gitter]，并询问我们是否遇到任何障碍，并感谢您帮助改善 `stdarch` 的文档！<br>

# Alternative Testing Instructions <br>替代测试说明<br>

It is generally recommended that you use `ci/run.sh` to run the tests. <br>通常建议您使用 `ci/run.sh` 来运行测试。<br>
However this might not work for you, e.g. if you are on Windows. <br>但是，这可能对您不起作用，例如，如果您使用的是 Windows。<br>

In that case you can fall back to running `cargo +nightly test` and `cargo +nightly test --release -p core_arch` for testing the code generation. <br>在这种情况下，您可以回去运行 `cargo +nightly test` 和 `cargo +nightly test --release -p core_arch` 来测试代码生成。<br>
Note that these require the nightly toolchain to be installed and for `rustc` to know about your target triple and its CPU. <br>请注意，这些要求每夜安装一次工具链，并且 `rustc` 才能了解目标三元组及其 CPU。<br>
In particular you need to set the `TARGET` environment variable as you would for `ci/run.sh`. <br>特别是，您需要像设置 `ci/run.sh` 一样设置 `TARGET` 环境变量。<br>
In addition you need to set `RUSTCFLAGS` (need the `C`) to indicate target features, e.g. `RUSTCFLAGS="-C -target-features=+avx2"`. <br>另外您需要设置 `RUSTCFLAGS` (需要 `C`) 来指示目标特性，例如 `RUSTCFLAGS="-C -target-features=+avx2"`。<br>
You can also set `-C -target-cpu=native` if you're "just" developing against your current CPU. <br>如果您正在针对当前的 CPU 开发 "just"，也可以设置 `-C -target-cpu=native`。<br>

Be warned that when you use these alternative instructions, [things may go less smoothly than they would with `ci/run.sh`][ci-run-good], e.g. <br>请注意，当您使用这些替代说明时，[事情可能不如使用 `ci/run.sh` 顺利][ci-run-good]，例如<br>
instruction generation tests may fail because the disassembler named them differently, e.g. <br>指令生成测试可能会失败，因为反汇编程序对它们的命名不同，例如<br>
it may generate `vaesenc` instead of `aesenc` instructions despite them behaving the same. <br>尽管它们的行为相同，但它可能会生成 `vaesenc` 而不是 `aesenc` 指令。<br>
Also these instructions execute less tests than would normally be done, so don't be surprised that when you eventually pull-request some errors may show up for tests not covered here. <br>同样，这些指令执行的测试少于通常执行的测试，因此当您最终请求请求时，对于此处未涵盖的测试，可能会出现一些错误，请不要感到惊讶。<br>

[new]: https://github.com/rust-lang/stdarch/issues/new
[issues]: https://github.com/rust-lang/stdarch/issues
[help]: https://github.com/rust-lang/stdarch/issues?q=is%3Aissue+is%3Aopen+label%3A%22help+wanted%22
[impl]: https://github.com/rust-lang/stdarch/issues?q=is%3Aissue+is%3Aopen+label%3Aimpl-period
[vendor]: https://github.com/rust-lang/stdarch/issues/40
[Documentation as tests]: https://doc.rust-lang.org/book/first-edition/documentation.html#documentation-as-tests
[Rust Book]: https://doc.rust-lang.org/book/first-edition
[ci-run-good]: https://github.com/rust-lang/stdarch/issues/931#issuecomment-711412126
