`core::arch` - Rust's core library architecture-specific intrinsics <br>`core::arch`- Rust 的核心库架构特有的内部特性<br>
=======

The `core::arch` module implements architecture-dependent intrinsics (e.g. SIMD). <br>`core::arch` 模块实现了依赖于体系结构的内部函数 (例如 SIMD)。<br>

# Usage 

`core::arch` is available as part of `libcore` and it is re-exported by `libstd`. <br>`core::arch` 作为 `libcore` 的一部分提供，并由 `libstd` 重新导出。<br> Prefer using it via `core::arch` or `std::arch` than via this crate. <br>与通过此 crate 相比，更喜欢通过 `core::arch` 或 `std::arch` 使用它。<br>
Unstable features are often available in nightly Rust via the `feature(stdsimd)`. <br>不稳定的特性通常可通过 `feature(stdsimd)` 在每晚的 Rust 中获得。<br>

Using `core::arch` via this crate requires nightly Rust, and it can (and does) break often. <br>通过此 crate 使用 `core::arch` 需要每晚执行 Rust，并且它可能 (并且确实) 经常中断。<br> The only cases in which you should consider using it via this crate are: <br>您应该考虑通过此 crate 使用它的唯一情况是：<br>

* if you need to re-compile `core::arch` yourself, e.g., with particular target-features enabled that are not enabled for `libcore`/`libstd`. <br>如果您需要自己重新编译 `core::arch`，例如，启用特定的目标特性，而 `libcore`/`libstd` 没有启用这些特性。<br>
Note: if you need to re-compile it for a non-standard target, please prefer using `xargo` and re-compiling `libcore`/`libstd` as appropriate instead of using this crate. <br>如果您需要针对非标准目标重新编译它，请优先使用 `xargo` 并根据需要重新编译 `libcore`/`libstd`，而不要使用此 crate。<br>
  
* using some features that might not be available even behind unstable Rust features. <br>使用某些即使在不稳定的 Rust 特性之后也可能无法使用的特性。<br> We try to keep these to a minimum. <br>我们尝试将这些限制降至最低。<br>
If you need to use some of these features, please open an issue so that we can expose them in nightly Rust and you can use them from there. <br>如果您需要使用其中一些特性，请打开一个问题，以便我们可以在每晚的 Rust 中公开它们，然后从那里开始使用它们。<br>

# Documentation

* [Documentation - i686][i686]
* [Documentation - x86\_64][x86_64]
* [Documentation - arm][arm]
* [Documentation - aarch64][aarch64]
* [Documentation - powerpc][powerpc]
* [Documentation - powerpc64][powerpc64]
* [How to get started][contrib]
* [How to help implement intrinsics][help-implement]

[contrib]: https://github.com/rust-lang/stdarch/blob/master/CONTRIBUTING.md
[help-implement]: https://github.com/rust-lang/stdarch/issues/40
[i686]: https://rust-lang.github.io/stdarch/i686/core_arch/
[x86_64]: https://rust-lang.github.io/stdarch/x86_64/core_arch/
[arm]: https://rust-lang.github.io/stdarch/arm/core_arch/
[aarch64]: https://rust-lang.github.io/stdarch/aarch64/core_arch/
[powerpc]: https://rust-lang.github.io/stdarch/powerpc/core_arch/
[powerpc64]: https://rust-lang.github.io/stdarch/powerpc64/core_arch/

# License

`core_arch` is primarily distributed under the terms of both the MIT license and the Apache License (Version 2.0), with portions covered by various BSD-like licenses. <br>`core_arch` 主要根据 MIT 许可证和 Apache 许可证 (2.0 版) 的条款进行分发，部分内容包含在各种类似 BSD 的许可证中。<br>

See LICENSE-APACHE, and LICENSE-MIT for details. <br>有关详细信息，请参见 LICENSE-APACHE 和 LICENSE-MIT。<br>

# Contribution

Unless you explicitly state otherwise, any contribution intentionally submitted for inclusion in `core_arch` by you, as defined in the Apache-2.0 license, shall be dual licensed as above, without any additional terms or conditions. <br>除非您明确声明，否则您有意提交的任何捐赠 (按照 Apache-2.0 许可的定义) 都应包含在 `core_arch` 中，并且应具有上述双重许可权利，且无任何其他条款或条件。<br>
