//! ARM compiler specific intrinsics <br>特定于 ARM 编译器的内部函数<br>
//!
//! # References
//!
//! - [ARM Compiler v 6.10 - armclang Reference Guide][arm_comp_ref]
//!
//! [arm_comp_ref]: https://developer.arm.com/docs/100067/0610

#[cfg(test)]
use stdarch_test::assert_instr;

/// Inserts a breakpoint instruction. <br>插入断点指令。<br>
///
/// `VAL` is a compile-time constant integer in range `[0, 65535]`. <br>`VAL` 是 `[0, 65535]` 范围内的编译时常量整数。<br>
///
/// The breakpoint instruction inserted is `BRK` on A64. <br>插入的断点指令是 A64 上的 `BRK`。<br>
#[cfg_attr(test, assert_instr(brk, VAL = 0))]
#[inline(always)]
#[rustc_legacy_const_generics(0)]
pub unsafe fn __breakpoint<const VAL: i32>() {
    static_assert_imm16!(VAL);
    crate::arch::asm!("brk {}", const VAL);
}
