//! AArch64 内部函数。
//!
//! NEON 的引用为 [ARM 的 NEON 内部函数参考][arm_ref]。
//! [ARM 的 NEON Intrinsics 在线数据库][arm_dat] 也很有用。
//!
//! [arm_ref]: http://infocenter.arm.com/help/topic/com.arm.doc.ihi0073a/IHI0073A_arm_neon_intrinsics_ref.pdf
//! [arm_dat]: https://developer.arm.com/technologies/neon/intrinsics

mod v8;
pub use self::v8::*;

mod neon;
pub use self::neon::*;

mod tme;
pub use self::tme::*;

mod crc;
pub use self::crc::*;

mod prefetch;
pub use self::prefetch::*;

pub use super::arm_shared::*;

mod armclang;

pub use self::armclang::*;

#[cfg(test)]
use stdarch_test::assert_instr;

/// 生成陷阱指令 `BRK 1`
#[cfg_attr(test, assert_instr(brk))]
#[inline]
pub unsafe fn brk() -> ! {
    crate::intrinsics::abort()
}

#[cfg(test)]
pub(crate) mod test_support;
