#[cfg(test)]
use stdarch_test::assert_instr;

extern "unadjusted" {
    #[link_name = "llvm.prefetch"]
    fn prefetch(p: *const i8, rw: i32, loc: i32, ty: i32);
}

/// See [`prefetch`](fn._prefetch.html). <br>请参见 [`prefetch`](fn._prefetch.html)。<br>
pub const _PREFETCH_READ: i32 = 0;

/// See [`prefetch`](fn._prefetch.html). <br>请参见 [`prefetch`](fn._prefetch.html)。<br>
pub const _PREFETCH_WRITE: i32 = 1;

/// See [`prefetch`](fn._prefetch.html). <br>请参见 [`prefetch`](fn._prefetch.html)。<br>
pub const _PREFETCH_LOCALITY0: i32 = 0;

/// See [`prefetch`](fn._prefetch.html). <br>请参见 [`prefetch`](fn._prefetch.html)。<br>
pub const _PREFETCH_LOCALITY1: i32 = 1;

/// See [`prefetch`](fn._prefetch.html). <br>请参见 [`prefetch`](fn._prefetch.html)。<br>
pub const _PREFETCH_LOCALITY2: i32 = 2;

/// See [`prefetch`](fn._prefetch.html). <br>请参见 [`prefetch`](fn._prefetch.html)。<br>
pub const _PREFETCH_LOCALITY3: i32 = 3;

/// Fetch the cache line that contains address `p` using the given `RW` and `LOCALITY`. <br>使用给定的 `RW` 和 `LOCALITY` 获取包含地址 `p` 的缓存行。<br>
///
/// The `RW` must be one of: <br>`RW` 必须是以下之一：<br>
///
/// * [`_PREFETCH_READ`](constant._PREFETCH_READ.html): the prefetch is preparing for a read. <br>[`_PREFETCH_READ`](constant._PREFETCH_READ.html): 预获取正在准备读取。<br>
///
/// * [`_PREFETCH_WRITE`](constant._PREFETCH_WRITE.html): the prefetch is preparing for a write. <br>[`_PREFETCH_WRITE`](constant._PREFETCH_WRITE.html): 预获取正在准备写入。<br>
///
/// The `LOCALITY` must be one of: <br>`LOCALITY` 必须是以下之一：<br>
///
/// * [`_PREFETCH_LOCALITY0`](constant._PREFETCH_LOCALITY0.html): Streaming or non-temporal prefetch, for data that is used only once. <br>[`_PREFETCH_LOCALITY0`](constant._PREFETCH_LOCALITY0.html): 流式或非临时预取，用于仅使用一次的数据。<br>
///
/// * [`_PREFETCH_LOCALITY1`](constant._PREFETCH_LOCALITY1.html): Fetch into level 3 cache. <br>[`_PREFETCH_LOCALITY1`](constant._PREFETCH_LOCALITY1.html): 获取到 3 级缓存。<br>
///
/// * [`_PREFETCH_LOCALITY2`](constant._PREFETCH_LOCALITY2.html): Fetch into level 2 cache. <br>[`_PREFETCH_LOCALITY2`](constant._PREFETCH_LOCALITY2.html): 获取到 2 级缓存。<br>
///
/// * [`_PREFETCH_LOCALITY3`](constant._PREFETCH_LOCALITY3.html): Fetch into level 1 cache. <br>[`_PREFETCH_LOCALITY3`](constant._PREFETCH_LOCALITY3.html): 获取到 1 级缓存。<br>
///
/// The prefetch memory instructions signal to the memory system that memory accesses from a specified address are likely to occur in the near future. <br>预取存储器指令向存储器系统发送信号，表明从指定地址进行的存储器访问可能发生在 future 附近。<br>
/// The memory system can respond by taking actions that are expected to speed up the memory access when they do occur, such as preloading the specified address into one or more caches. <br>内存系统可以通过采取某些措施来做出响应，这些措施可以在确实发生时加快内存访问的速度，例如将指定地址预加载到一个或多个高速缓存中。<br>
///
/// Because these signals are only hints, it is valid for a particular CPU to treat any or all prefetch instructions as a NOP. <br>因为这些信号只是提示，所以对于特定的 CPU，将任何或所有预取指令视为 NOP 是有效的。<br>
///
/// [Arm's documentation](https://developer.arm.com/documentation/den0024/a/the-a64-instruction-set/memory-access-instructions/prefetching-memory?lang=en)
///
///
///
///
///
///
#[inline(always)]
#[cfg_attr(test, assert_instr("prfm pldl1strm", RW = _PREFETCH_READ, LOCALITY = _PREFETCH_LOCALITY0))]
#[cfg_attr(test, assert_instr("prfm pldl3keep", RW = _PREFETCH_READ, LOCALITY = _PREFETCH_LOCALITY1))]
#[cfg_attr(test, assert_instr("prfm pldl2keep", RW = _PREFETCH_READ, LOCALITY = _PREFETCH_LOCALITY2))]
#[cfg_attr(test, assert_instr("prfm pldl1keep", RW = _PREFETCH_READ, LOCALITY = _PREFETCH_LOCALITY3))]
#[cfg_attr(test, assert_instr("prfm pstl1strm", RW = _PREFETCH_WRITE, LOCALITY = _PREFETCH_LOCALITY0))]
#[cfg_attr(test, assert_instr("prfm pstl3keep", RW = _PREFETCH_WRITE, LOCALITY = _PREFETCH_LOCALITY1))]
#[cfg_attr(test, assert_instr("prfm pstl2keep", RW = _PREFETCH_WRITE, LOCALITY = _PREFETCH_LOCALITY2))]
#[cfg_attr(test, assert_instr("prfm pstl1keep", RW = _PREFETCH_WRITE, LOCALITY = _PREFETCH_LOCALITY3))]
#[rustc_legacy_const_generics(1, 2)]
// FIXME: Replace this with the standard ACLE __pld/__pldx/__pli/__plix intrinsics <br>将其替换为标准的 ACLE __pld/__pldx/__pli/__plix 内部函数<br>
pub unsafe fn _prefetch<const RW: i32, const LOCALITY: i32>(p: *const i8) {
    // We use the `llvm.prefetch` intrinsic with `cache type` = 1 (data cache). <br>我们使用 `cache type` =1 (数据缓存) 的 `llvm.prefetch` 内部函数。<br>
    static_assert_imm1!(RW);
    static_assert_imm2!(LOCALITY);
    prefetch(p, RW, LOCALITY, 1);
}
