// Reference: Section 5.4.4 "LDREX / STREX" of ACLE <br>引用: ACLE 的 5.4.4 "LDREX / STREX" 部分<br>

/// Removes the exclusive lock created by LDREX <br>删除由 LDREX 创建的排他锁<br>
// Supported: v6, v6K, v7-M, v7-A, v7-R Not supported: v5, v6-M <br>支持: v6，v6K，v7-M，v7-A，v7-R 不支持: v5，v6-M<br>
// NOTE: there's no dedicated CLREX instruction in v6 (<v6k); <br>v6 (<v6k) 中没有专用的 CLREX 指令；<br> to clear the exclusive monitor users have to do a dummy STREX operation <br>清除独占监视器用户必须执行虚拟 STREX 操作<br>
//
//
#[cfg(any(
    all(target_feature = "v6k", not(target_feature = "mclass")), // excludes v6-M <br>不包括 v6-M<br>
    all(target_feature = "v7", target_feature = "mclass"), // v7-M
    doc
))]
pub unsafe fn __clrex() {
    extern "unadjusted" {
        #[link_name = "llvm.arm.clrex"]
        fn clrex();
    }

    clrex()
}

/// Executes an exclusive LDR instruction for 8 bit value. <br>对 8 位值执行独占 LDR 指令。<br>
// Supported: v6K, v7-M, v7-A, v7-R Not supported: v5, v6, v6-M <br>支持: v6K，v7-M，v7-A，v7-R 不支持: v5，v6，v6-M<br>
//
#[cfg(any(
    target_feature = "v6k", // includes v7-M but excludes v6-M <br>包括 v7-M，但不包括 v6-M<br>
    doc
))]
pub unsafe fn __ldrexb(p: *const u8) -> u8 {
    extern "unadjusted" {
        #[link_name = "llvm.arm.ldrex.p0i8"]
        fn ldrex8(p: *const u8) -> u32;
    }

    ldrex8(p) as u8
}

/// Executes an exclusive LDR instruction for 16 bit value. <br>对 16 位值执行独占 LDR 指令。<br>
// Supported: v6K, v7-M, v7-A, v7-R, v8 Not supported: v5, v6, v6-M <br>支持: v6K，v7-M，v7-A，v7-R，v8 不支持: v5，v6，v6-M<br>
//
#[cfg(any(
    target_feature = "v6k", // includes v7-M but excludes v6-M <br>包括 v7-M，但不包括 v6-M<br>
    doc
))]
pub unsafe fn __ldrexh(p: *const u16) -> u16 {
    extern "unadjusted" {
        #[link_name = "llvm.arm.ldrex.p0i16"]
        fn ldrex16(p: *const u16) -> u32;
    }

    ldrex16(p) as u16
}

/// Executes an exclusive LDR instruction for 32 bit value. <br>对 32 位值执行独占 LDR 指令。<br>
// Supported: v6, v7-M, v6K, v7-A, v7-R, v8 Not supported: v5, v6-M <br>支援: v6，v7-M，v6K，v7-A，v7-R，v8 不支援: v5，v6-M<br>
//
#[cfg(any(
    all(target_feature = "v6", not(target_feature = "mclass")), // excludes v6-M <br>不包括 v6-M<br>
    all(target_feature = "v7", target_feature = "mclass"), // v7-M
    doc
))]
pub unsafe fn __ldrex(p: *const u32) -> u32 {
    extern "unadjusted" {
        #[link_name = "llvm.arm.ldrex.p0i32"]
        fn ldrex32(p: *const u32) -> u32;
    }

    ldrex32(p)
}

/// Executes an exclusive STR instruction for 8 bit values <br>对 8 位值执行独占 STR 指令<br>
///
/// Returns `0` if the operation succeeded, or `1` if it failed <br>如果操作成功，则返回 `0`; 如果操作失败，则返回 `1`<br>
// supported: v6K, v7-M, v7-A, v7-R Not supported: v5, v6, v6-M <br>支持: v6K、v7-M、v7-A、v7-R 不支持: v5、v6、v6-M<br>
//
#[cfg(any(
    target_feature = "v6k", // includes v7-M but excludes v6-M <br>包括 v7-M，但不包括 v6-M<br>
    doc
))]
pub unsafe fn __strexb(value: u32, addr: *mut u8) -> u32 {
    extern "unadjusted" {
        #[link_name = "llvm.arm.strex.p0i8"]
        fn strex8(value: u32, addr: *mut u8) -> u32;
    }

    strex8(value, addr)
}

/// Executes an exclusive STR instruction for 16 bit values <br>对 16 位值执行独占 STR 指令<br>
///
/// Returns `0` if the operation succeeded, or `1` if it failed <br>如果操作成功，则返回 `0`; 如果操作失败，则返回 `1`<br>
// Supported: v6K, v7-M, v7-A, v7-R, v8 Not supported: v5, v6, v6-M <br>支持: v6K，v7-M，v7-A，v7-R，v8 不支持: v5，v6，v6-M<br>
//
#[cfg(target_feature = "aarch64")]
#[cfg(any(
    target_feature = "v6k", // includes v7-M but excludes v6-M <br>包括 v7-M，但不包括 v6-M<br>
    doc
))]
pub unsafe fn __strexh(value: u16, addr: *mut u16) -> u32 {
    extern "unadjusted" {
        #[link_name = "llvm.arm.strex.p0i16"]
        fn strex16(value: u32, addr: *mut u16) -> u32;
    }

    strex16(value as u32, addr)
}

/// Executes an exclusive STR instruction for 32 bit values <br>对 32 位值执行独占 STR 指令<br>
///
/// Returns `0` if the operation succeeded, or `1` if it failed <br>如果操作成功，则返回 `0`; 如果操作失败，则返回 `1`<br>
// Supported: v6, v7-M, v6K, v7-A, v7-R, v8 Not supported: v5, v6-M <br>支援: v6，v7-M，v6K，v7-A，v7-R，v8 不支援: v5，v6-M<br>
//
#[cfg(any(
    all(target_feature = "v6", not(target_feature = "mclass")), // excludes v6-M <br>不包括 v6-M<br>
    all(target_feature = "v7", target_feature = "mclass"), // v7-M
    doc
))]
pub unsafe fn __strex(value: u32, addr: *mut u32) -> u32 {
    extern "unadjusted" {
        #[link_name = "llvm.arm.strex.p0i32"]
        fn strex32(value: u32, addr: *mut u32) -> u32;
    }

    strex32(value, addr)
}
