//! WASM32 intrinsics <br>WASM32 内部函数<br>

#[cfg(test)]
use stdarch_test::assert_instr;

mod atomic;
pub use self::atomic::*;

mod simd128;
pub use self::simd128::*;

mod memory;
pub use self::memory::*;

/// Generates the [`unreachable`] instruction, which causes an unconditional [trap]. <br>生成 [`unreachable`] 指令，这会导致无条件的 [trap]。<br>
///
/// This function is safe to call and immediately aborts the execution. <br>这个函数对调用是安全的，并立即中止执行。<br>
///
/// [`unreachable`]: https://webassembly.github.io/spec/core/syntax/instructions.html#syntax-instr-control
/// [trap]: https://webassembly.github.io/spec/core/intro/overview.html#trap
#[cfg_attr(test, assert_instr(unreachable))]
#[inline]
#[stable(feature = "unreachable_wasm32", since = "1.37.0")]
pub fn unreachable() -> ! {
    crate::intrinsics::abort()
}
