//! Advanced Vector Extensions (AVX) <br>高级的 Vector 扩展 (AVX)<br>
//!
//! The references are: <br>引用的是：<br>
//!
//! - [Intel 64 and IA-32 Architectures Software Developer's Manual Volume 2: <br>[英特尔 64 位和 IA-32 架构软件开发人员手册第 2 卷：<br>
//! Instruction Set Reference, A-Z][intel64_ref]. <br>指令集引用，AZ][intel64_ref]。<br> - [AMD64 Architecture Programmer's Manual, Volume 3: General-Purpose and System Instructions][amd64_ref]. <br>- [AMD64 架构程序员手册，第 3 卷：通用和系统指令][amd64_ref]。<br>
//!
//! [Wikipedia][wiki] provides a quick overview of the instructions available. <br>[维基百科][wiki] 提供了可用指令的快速概览。<br>
//!
//! [intel64_ref]: http://www.intel.de/content/dam/www/public/us/en/documents/manuals/64-ia-32-architectures-software-developer-instruction-set-reference-manual-325383.pdf
//! [amd64_ref]: http://support.amd.com/TechDocs/24594.pdf
//! [wiki]: https://en.wikipedia.org/wiki/Advanced_Vector_Extensions
//!
//!

use crate::{
    core_arch::{simd_llvm::*, x86::*},
    mem::transmute,
};

/// Copies `a` to result, and insert the 64-bit integer `i` into result at the location specified by `index`. <br>复制 `a` 到结果，然后在 `index` 指定的位置将 64 位整数 `i` 插入到 result 中。<br>
///
/// [Intel's documentation](https://software.intel.com/sites/landingpage/IntrinsicsGuide/#text=_mm256_insert_epi64)
///
#[inline]
#[rustc_legacy_const_generics(2)]
#[target_feature(enable = "avx")]
// This intrinsic has no corresponding instruction. <br>此内部函数没有相应的指令。<br>
#[stable(feature = "simd_x86", since = "1.27.0")]
pub unsafe fn _mm256_insert_epi64<const INDEX: i32>(a: __m256i, i: i64) -> __m256i {
    static_assert_imm2!(INDEX);
    transmute(simd_insert(a.as_i64x4(), INDEX as u32, i))
}

#[cfg(test)]
mod tests {
    use stdarch_test::simd_test;

    use crate::core_arch::x86::*;

    #[simd_test(enable = "avx")]
    unsafe fn test_mm256_insert_epi64() {
        let a = _mm256_setr_epi64x(1, 2, 3, 4);
        let r = _mm256_insert_epi64::<3>(a, 0);
        let e = _mm256_setr_epi64x(1, 2, 3, 0);
        assert_eq_m256i(r, e);
    }
}
