`std::detect` - Rust's standard library run-time CPU feature detection <br>`std::detect`- Rust 的标准库运行时 CPU 特性检测<br>
=======

The private `std::detect` module implements run-time feature detection in Rust's standard library. <br>专用 `std::detect` 模块在 Rust 的标准库中实现了运行时特性检测。<br> This allows detecting whether the CPU the binary runs on supports certain features, like SIMD instructions. <br>这样可以检测二进制文件所运行的 CPU 是否支持某些特性，例如 SIMD 指令。<br>

# Usage 

`std::detect` APIs are available as part of `libstd`. <br>`std::detect` API 作为 `libstd` 的一部分提供。<br> Prefer using it via the standard library than through this crate. <br>与通过此 crate 相比，更喜欢通过标准库使用它。<br> Unstable features of `std::detect` are available on nightly Rust behind the `feature(stdsimd)` feature-gate. <br>`std::detect` 的不稳定特性可在 `feature(stdsimd)` 特性开关的每晚 Rust 上可用。<br>

If you need run-time feature detection in `#[no_std]` environments, Rust `core` library cannot help you. <br>如果需要在 `#[no_std]` 环境中进行运行时特性检测，则 Rust `core` 库无法为您提供帮助。<br> By design, Rust `core` is platform independent, but performing run-time feature detection requires a certain level of cooperation from the platform. <br>根据设计，Rust `core` 是独立于平台的，但是执行运行时特性检测需要平台提供一定程度的合作。<br>

You can then manually include `std_detect` as a dependency to get similar run-time feature detection support than the one offered by Rust's standard library. <br>然后，您可以手动包含 `std_detect` 作为依赖项，以获得与 Rust 标准库提供的类似的运行时特性检测支持。<br> We intend to make `std_detect` more flexible and configurable in this regard to better serve the needs of `#[no_std]` targets. <br>我们打算在这方面使 `std_detect` 更加灵活和可配置，以更好地满足 `#[no_std]` 目标的需求。<br> 


# Features

* `std_detect_dlsym_getauxval` (enabled by default, requires `libc`): Enable to use `libc::dlsym` to query whether [`getauxval`] is linked into the binary. <br>`std_detect_dlsym_getauxval` (默认开启，需要 `libc`) : 开启使用 `libc::dlsym` 查询 [`getauxval`] 是否链接到二进制文件中。<br> When this is not the case, this feature allows other fallback methods to perform run-time feature detection. <br>如果不是这种情况，此特性允许其他回退方法执行运行时特性检测。<br>
When this feature is disabled, `std_detect` assumes that [`getauxval`] is linked to the binary. <br>禁用此特性后，`std_detect` 假定 [`getauxval`] 已链接到二进制文件。<br>
If that is not the case the behavior is undefined. <br>如果不是这种情况，则行为是不确定的。<br>

* `std_detect_file_io` (enabled by default, requires `std`): Enable to perform run-time feature detection using file APIs (e.g. `/proc/cpuinfo`, etc.) if other more performant methods fail. <br>`std_detect_file_io` (默认启用，需要 `std`) : 如果其他更高性能的方法失败，则启用以使用文件 API (例如 `/proc/cpuinfo` 等) 执行运行时特性检测。<br>
This feature requires `libstd` as a dependency, preventing the crate from working on applications in which `std` is not available. <br>此特性需要 `libstd` 作为依赖项，从而阻止 crate 在无法使用 `std` 的应用程序上工作。<br>

[`getauxval`]: http://man7.org/linux/man-pages/man3/getauxval.3.html

# Platform support <br>平台支持<br>

* All `x86`/`x86_64` targets are supported on all platforms by querying the `cpuid` instruction directly for the features supported by the hardware and the operating system. <br>所有 `x86`/`x86_64` 目标在所有平台上都支持，通过直接查询 `cpuid` 指令获取硬件和操作系统支持的特性。<br> `std_detect` assumes that the binary is an user-space application. <br>`std_detect` 假定二进制文件是一个 user-space 应用程序。<br>
If you need raw support for querying `cpuid`, consider using the [`cupid`](https://crates.io/crates/cupid) crate. <br>如果需要原始支持来查询 `cpuid`，请考虑使用 [`cupid`](https://crates.io/crates/cupid) crate。<br>
  
* Linux:
  * `arm{32, 64}`, `mips{32,64}{,el}`, `powerpc{32,64}{,le}`: `std_detect` supports these on Linux by querying ELF auxiliary vectors (using `getauxval` when available), and if that fails, by querying `/proc/cpuinfo`. <br>`arm{32, 64}`、`mips{32,64}{,el}`、`powerpc{32,64}{,le}`: `std_detect` 通过查询 ELF 辅助 vectors (可用时使用 `getauxval`) 在 Linux 上支持这些，如果失败，则通过查询 `/proc/cpuinfo`。<br> 
  * `arm64`: partial support for doing run-time feature detection by directly querying `mrs` is implemented for Linux >= 4.11, but not enabled by default. <br>`arm64`: 对 Linux >= 4.11 实现了通过直接查询 `mrs` 进行运行时特性检测的部分支持，但默认情况下未启用。<br>

* FreeBSD:
  * `arm64`: run-time feature detection is implemented by directly querying `mrs`. <br>`arm64`: 通过直接查询 `mrs`, 实现运行时特性检测。<br>

# License

This project is licensed under either of <br>该项目获得以下任一许可<br>

 * Apache License, Version 2.0, ([LICENSE-APACHE](LICENSE-APACHE) or http://www.apache.org/licenses/LICENSE-2.0) <br>Apache 许可证，版本 2.0，([LICENSE-APACHE](LICENSE-APACHE) 或 http://www.apache.org/licenses/LICENSE-2.0)<br>
 * MIT license ([LICENSE-MIT](LICENSE-MIT) or http://opensource.org/licenses/MIT) <br>MIT 许可证 ([LICENSE-MIT](LICENSE-MIT) 或 http://opensource.org/licenses/MIT)<br>

at your option. <br>由您选择。<br>

# Contribution

Unless you explicitly state otherwise, any contribution intentionally submitted for inclusion in `std_detect` by you, as defined in the Apache-2.0 license, shall be dual licensed as above, without any additional terms or conditions. <br>除非您明确声明，否则您有意提交的任何捐赠 (按照 Apache-2.0 许可的定义) 都应包含在 `std_detect` 中，并且应具有上述双重许可权利，且无任何其他条款或条件。<br>
