`std::detect`- Rust 的标准库运行时 CPU 特性检测
=======

专用 `std::detect` 模块在 Rust 的标准库中实现了运行时特性检测。这样可以检测二进制文件所运行的 CPU 是否支持某些特性，例如 SIMD 指令。

# Usage 

`std::detect` API 作为 `libstd` 的一部分提供。与通过此 crate 相比，更喜欢通过标准库使用它。`std::detect` 的不稳定特性可在 `feature(stdsimd)` 特性开关的每晚 Rust 上可用。

如果需要在 `#[no_std]` 环境中进行运行时特性检测，则 Rust `core` 库无法为您提供帮助。根据设计，Rust `core` 是独立于平台的，但是执行运行时特性检测需要平台提供一定程度的合作。

然后，您可以手动包含 `std_detect` 作为依赖项，以获得与 Rust 标准库提供的类似的运行时特性检测支持。我们打算在这方面使 `std_detect` 更加灵活和可配置，以更好地满足 `#[no_std]` 目标的需求。


# Features

* `std_detect_dlsym_getauxval` (默认开启，需要 `libc`) : 开启使用 `libc::dlsym` 查询 [`getauxval`] 是否链接到二进制文件中。如果不是这种情况，此特性允许其他回退方法执行运行时特性检测。
禁用此特性后，`std_detect` 假定 [`getauxval`] 已链接到二进制文件。
如果不是这种情况，则行为是不确定的。

* `std_detect_file_io` (默认启用，需要 `std`) : 如果其他更高性能的方法失败，则启用以使用文件 API (例如 `/proc/cpuinfo` 等) 执行运行时特性检测。
此特性需要 `libstd` 作为依赖项，从而阻止 crate 在无法使用 `std` 的应用程序上工作。

[`getauxval`]: http://man7.org/linux/man-pages/man3/getauxval.3.html

# 平台支持

* 所有 `x86`/`x86_64` 目标在所有平台上都支持，通过直接查询 `cpuid` 指令获取硬件和操作系统支持的特性。`std_detect` 假定二进制文件是一个 user-space 应用程序。
如果需要原始支持来查询 `cpuid`，请考虑使用 [`cupid`](https://crates.io/crates/cupid) crate。
  
* Linux:
  * `arm{32, 64}`、`mips{32,64}{,el}`、`powerpc{32,64}{,le}`: `std_detect` 通过查询 ELF 辅助 vectors (可用时使用 `getauxval`) 在 Linux 上支持这些，如果失败，则通过查询 `/proc/cpuinfo`。
  * `arm64`: 对 Linux >= 4.11 实现了通过直接查询 `mrs` 进行运行时特性检测的部分支持，但默认情况下未启用。

* FreeBSD:
  * `arm64`: 通过直接查询 `mrs`, 实现运行时特性检测。

# License

该项目获得以下任一许可

 * Apache 许可证，版本 2.0，([LICENSE-APACHE](LICENSE-APACHE) 或 http://www.apache.org/licenses/LICENSE-2.0)
 * MIT 许可证 ([LICENSE-MIT](LICENSE-MIT) 或 http://opensource.org/licenses/MIT)

由您选择。

# Contribution

除非您明确声明，否则您有意提交的任何捐赠 (按照 Apache-2.0 许可的定义) 都应包含在 `std_detect` 中，并且应具有上述双重许可权利，且无任何其他条款或条件。
