//! Aarch64 run-time features. <br>Aarch64 运行时特性。<br>

features! {
    @TARGET: aarch64;
    @CFG: target_arch = "aarch64";
    @MACRO_NAME: is_aarch64_feature_detected;
    @MACRO_ATTRS:
    /// This macro tests, at runtime, whether an `aarch64` feature is enabled on aarch64 platforms. <br>这个宏在运行时测试 `aarch64` 特性是否在 aarch64 平台上启用。<br>
    /// Currently most features are only supported on linux-based platforms. <br>目前大多数特性仅在基于 linux 的平台上受支持。<br>
    ///
    /// This macro takes one argument which is a string literal of the feature being tested for. <br>这个宏需要一个参数，该参数是正在测试的特性的字符串文字。<br>
    /// The feature names are mostly taken from their FEAT_* definitions in the [ARM Architecture Reference Manual][docs]. <br>特性名大多取自 [ARM 架构参考手册][docs] 中的 FEAT_* 定义。<br>
    ///
    ///
    /// ## Supported arguments <br>支持的参数<br>
    ///
    /// * `"asimd"` or "neon" - FEAT_AdvSIMD <br>`"asimd"` 或 "neon"-FEAT_AdvSIMD<br>
    /// * `"pmull"` - FEAT_PMULL
    /// * `"fp"` - FEAT_FP
    /// * `"fp16"` - FEAT_FP16
    /// * `"sve"` - FEAT_SVE
    /// * `"crc"` - FEAT_CRC
    /// * `"lse"` - FEAT_LSE
    /// * `"lse2"` - FEAT_LSE2
    /// * `"rdm"` - FEAT_RDM
    /// * `"rcpc"` - FEAT_LRCPC
    /// * `"rcpc2"` - FEAT_LRCPC2
    /// * `"dotprod"` - FEAT_DotProd
    /// * `"tme"` - FEAT_TME
    /// * `"fhm"` - FEAT_FHM
    /// * `"dit"` - FEAT_DIT
    /// * `"flagm"` - FEAT_FLAGM
    /// * `"ssbs"` - FEAT_SSBS
    /// * `"sb"` - FEAT_SB
    /// * `"paca"` - FEAT_PAuth (address authentication) <br>`"paca"`-FEAT_PAuth (地址认证)<br>
    /// * `"pacg"` - FEAT_Pauth (generic authentication) <br>`"pacg"`-FEAT_Pauth (通用身份验证)<br>
    /// * `"dpb"` - FEAT_DPB
    /// * `"dpb2"` - FEAT_DPB2
    /// * `"sve2"` - FEAT_SVE2
    /// * `"sve2-aes"` - FEAT_SVE2_AES
    /// * `"sve2-sm4"` - FEAT_SVE2_SM4
    /// * `"sve2-sha3"` - FEAT_SVE2_SHA3
    /// * `"sve2-bitperm"` - FEAT_SVE2_BitPerm
    /// * `"frintts"` - FEAT_FRINTTS
    /// * `"i8mm"` - FEAT_I8MM
    /// * `"f32mm"` - FEAT_F32MM
    /// * `"f64mm"` - FEAT_F64MM
    /// * `"bf16"` - FEAT_BF16
    /// * `"rand"` - FEAT_RNG
    /// * `"bti"` - FEAT_BTI
    /// * `"mte"` - FEAT_MTE
    /// * `"jsconv"` - FEAT_JSCVT
    /// * `"fcma"` - FEAT_FCMA
    /// * `"aes"` - FEAT_AES
    /// * `"sha2"` - FEAT_SHA1 & FEAT_SHA256
    /// * `"sha3"` - FEAT_SHA512 & FEAT_SHA3
    /// * `"sm4"` - FEAT_SM3 & FEAT_SM4
    ///
    /// [docs]: https://developer.arm.com/documentation/ddi0487/latest
    #[stable(feature = "simd_aarch64", since = "1.60.0")]
    @BIND_FEATURE_NAME: "asimd"; "neon";
    @NO_RUNTIME_DETECTION: "ras";
    @NO_RUNTIME_DETECTION: "v8.1a";
    @NO_RUNTIME_DETECTION: "v8.2a";
    @NO_RUNTIME_DETECTION: "v8.3a";
    @NO_RUNTIME_DETECTION: "v8.4a";
    @NO_RUNTIME_DETECTION: "v8.5a";
    @NO_RUNTIME_DETECTION: "v8.6a";
    @NO_RUNTIME_DETECTION: "v8.7a";
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] asimd: "neon";
    /// FEAT_AdvSIMD (Advanced SIMD/NEON) <br>FEAT_AdvSIMD (高级 SIMD/NEON)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] pmull: "pmull";
    /// FEAT_PMULL (Polynomial Multiply) <br>FEAT_PMULL (多项式乘法)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] fp: "fp";
    /// FEAT_FP (Floating point support) <br>FEAT_FP (浮点支持)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] fp16: "fp16";
    /// FEAT_FP16 (Half-float support) <br>FEAT_FP16 (半浮动支持)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] sve: "sve";
    /// FEAT_SVE (Scalable Vector Extension) <br>FEAT_SVE (可扩展 Vector 扩展)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] crc: "crc";
    /// FEAT_CRC32 (Cyclic Redundancy Check) <br>FEAT_CRC32 (循环冗余校验)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] lse: "lse";
    /// FEAT_LSE (Large System Extension - atomics) <br>FEAT_LSE (大型系统扩展 - 原子)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] lse2: "lse2";
    /// FEAT_LSE2 (unaligned and register-pair atomics) <br>FEAT_LSE2 (未对齐和寄存器对原子)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] rdm: "rdm";
    /// FEAT_RDM (Rounding Doubling Multiply - ASIMDRDM) <br>FEAT_RDM (四舍五入加倍乘法 - ASIMDRDM)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] rcpc: "rcpc";
    /// FEAT_LRCPC (Release consistent Processor consistent) <br>FEAT_LRCPC (发布一致处理器一致)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] rcpc2: "rcpc2";
    /// FEAT_LRCPC2 (RCPC with immediate offsets) <br>FEAT_LRCPC2 (具有 immediate 偏移量的 RCPC)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] dotprod: "dotprod";
    /// FEAT_DotProd (Vector Dot-Product - ASIMDDP) <br>FEAT_DotProd (Vector 点积 - ASIMDDP)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] tme: "tme";
    /// FEAT_TME (Transactional Memory Extensions) <br>FEAT_TME (事务性内存扩展)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] fhm: "fhm";
    /// FEAT_FHM (fp16 multiplication instructions) <br>FEAT_FHM (fp16 乘法指令)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] dit: "dit";
    /// FEAT_DIT (Data Independent Timing instructions) <br>FEAT_DIT (数据独立时序指令)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] flagm: "flagm";
    /// FEAT_FLAGM (flag manipulation instructions) <br>FEAT_FLAGM (标志操作指令)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] ssbs: "ssbs";
    /// FEAT_SSBS (speculative store bypass safe) <br>FEAT_SSBS (投机存储绕过安全)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] sb: "sb";
    /// FEAT_SB (speculation barrier) <br>FEAT_SB (投机障碍)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] paca: "paca";
    /// FEAT_PAuth (address authentication) <br>FEAT_PAuth (地址认证)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] pacg: "pacg";
    /// FEAT_PAuth (generic authentication) <br>FEAT_PAuth (通用身份验证)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] dpb: "dpb";
    /// FEAT_DPB (aka dcpop - data cache clean to point of persistence) <br>FEAT_DPB (又名 dcpop - 数据缓存清理到持久点)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] dpb2: "dpb2";
    /// FEAT_DPB2 (aka dcpodp - data cache clean to point of deep persistence) <br>FEAT_DPB2 (又名 dcpodp - 数据缓存清理到深度持久性点)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] sve2: "sve2";
    /// FEAT_SVE2 (Scalable Vector Extension 2) <br>FEAT_SVE2 (可扩展 Vector 扩展 2)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] sve2_aes: "sve2-aes";
    /// FEAT_SVE_AES (SVE2 AES crypto) <br>FEAT_SVE_AES (SVE2 AES 加密)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] sve2_sm4: "sve2-sm4";
    /// FEAT_SVE_SM4 (SVE2 SM4 crypto) <br>FEAT_SVE_SM4 (SVE2 SM4 加密)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] sve2_sha3: "sve2-sha3";
    /// FEAT_SVE_SHA3 (SVE2 SHA3 crypto) <br>FEAT_SVE_SHA3 (SVE2 SHA3 加密)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] sve2_bitperm: "sve2-bitperm";
    /// FEAT_SVE_BitPerm (SVE2 bit permutation instructions) <br>FEAT_SVE_BitPerm (SVE2 位置换指令)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] frintts: "frintts";
    /// FEAT_FRINTTS (float to integer rounding instructions) <br>FEAT_FRINTTS (浮点到整数舍入指令)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] i8mm: "i8mm";
    /// FEAT_I8MM (integer matrix multiplication, plus ASIMD support) <br>FEAT_I8MM (整数矩阵乘法，加上 ASIMD 支持)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] f32mm: "f32mm";
    /// FEAT_F32MM (single-precision matrix multiplication) <br>FEAT_F32MM (单精度矩阵乘法)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] f64mm: "f64mm";
    /// FEAT_F64MM (double-precision matrix multiplication) <br>FEAT_F64MM (双精度矩阵乘法)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] bf16: "bf16";
    /// FEAT_BF16 (BFloat16 type, plus MM instructions, plus ASIMD support) <br>FEAT_BF16 (BFloat16 型，加 MM 指令，加 ASIMD 支持)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] rand: "rand";
    /// FEAT_RNG (Random Number Generator) <br>FEAT_RNG (随机数生成器)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] bti: "bti";
    /// FEAT_BTI (Branch Target Identification) <br>FEAT_BTI (分支目标识别)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] mte: "mte";
    /// FEAT_MTE (Memory Tagging Extension) <br>FEAT_MTE (内存标记扩展)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] jsconv: "jsconv";
    /// FEAT_JSCVT (JavaScript float conversion instructions) <br>FEAT_JSCVT (JavaScript 浮点转换指令)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] fcma: "fcma";
    /// FEAT_FCMA (float complex number operations) <br>FEAT_FCMA (浮点复数运算)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] aes: "aes";
    /// FEAT_AES (AES instructions) <br>FEAT_AES (AES 指令)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] sha2: "sha2";
    /// FEAT_SHA1 & FEAT_SHA256 (SHA1 & SHA2-256 instructions) <br>FEAT_SHA1&FEAT_SHA256 (SHA1&SHA2-256 指令)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] sha3: "sha3";
    /// FEAT_SHA512 & FEAT_SHA3 (SHA2-512 & SHA3 instructions) <br>FEAT_SHA512&FEAT_SHA3 (SHA2-512&SHA3 指令)<br>
    @FEATURE: #[stable(feature = "simd_aarch64", since = "1.60.0")] sm4: "sm4";
    /// FEAT_SM3 & FEAT_SM4 (SM3 & SM4 instructions) <br>FEAT_SM3&FEAT_SM4 (SM3&SM4 指令)<br>
}
