//! Run-time feature detection for MIPS on Linux. <br>Linux 上 MIPS 的运行时特性检测。<br>

use super::auxvec;
use crate::detect::{bit, cache, Feature};

/// Try to read the features from the auxiliary vector, and if that fails, try to read them from `/proc/cpuinfo`. <br>尝试从辅助 vector 读取特性，如果失败，请尝试从 `/proc/cpuinfo` 中读取它们。<br>
///
pub(crate) fn detect_features() -> cache::Initializer {
    let mut value = cache::Initializer::default();
    let enable_feature = |value: &mut cache::Initializer, f, enable| {
        if enable {
            value.set(f as u32);
        }
    };

    // The values are part of the platform-specific [asm/hwcap.h][hwcap] <br>这些值是特定于平台的 [asm/hwcap.h][hwcap] 的一部分<br>
    //
    // [hwcap]: https://github.com/torvalds/linux/blob/master/arch/arm64/include/uapi/asm/hwcap.h
    if let Ok(auxv) = auxvec::auxv() {
        enable_feature(&mut value, Feature::msa, bit::test(auxv.hwcap, 1));
        return value;
    }
    // TODO: fall back via `cpuinfo`. <br>通过 `cpuinfo` 回退。<br>
    value
}
