//! Linux 上 MIPS 的运行时特性检测。

use super::auxvec;
use crate::detect::{bit, cache, Feature};

/// 尝试从辅助 vector 读取特性，如果失败，请尝试从 `/proc/cpuinfo` 中读取它们。
///
pub(crate) fn detect_features() -> cache::Initializer {
    let mut value = cache::Initializer::default();
    let enable_feature = |value: &mut cache::Initializer, f, enable| {
        if enable {
            value.set(f as u32);
        }
    };

    // 这些值是特定于平台的 [asm/hwcap.h][hwcap] 的一部分
    //
    // [hwcap]: https://github.com/torvalds/linux/blob/master/arch/arm64/include/uapi/asm/hwcap.h
    if let Ok(auxv) = auxvec::auxv() {
        enable_feature(&mut value, Feature::msa, bit::test(auxv.hwcap, 1));
        return value;
    }
    // TODO: 通过 `cpuinfo` 回退。
    value
}
