//! Disassembly calling function for `wasm32` targets. <br>针对 `wasm32` 目标的反汇编调用函数。<br>

use crate::Function;
use std::collections::HashSet;

pub(crate) fn disassemble_myself() -> HashSet<Function> {
    // Use `std::env::args` to find the path to our executable. <br>使用 `std::env::args` 查找可执行文件的路径。<br> Assume the environment is configured such that we can read that file. <br>假设配置了环境，以便我们可以读取该文件。<br>
    // Read it and use the `wasmprinter` crate to transform the binary to text, then search the text for appropriately named functions. <br>读取它，并使用 `wasmprinter` crate 将二进制文件转换为文本，然后在文本中搜索适当命名的函数。<br>
    //
    //
    let me = std::env::args()
        .next()
        .expect("failed to find current wasm file");
    let output = wasmprinter::print_file(&me).unwrap();

    let mut ret: HashSet<Function> = HashSet::new();
    let mut lines = output.lines().map(|s| s.trim());
    while let Some(line) = lines.next() {
        // If this isn't a function, we don't care about it. <br>如果这不是一个函数，那么我们将不在乎。<br>
        if !line.starts_with("(func ") {
            continue;
        }

        let mut function = Function {
            name: String::new(),
            instrs: Vec::new(),
        };

        // Empty functions will end in `))` so there's nothing to do, otherwise we'll have a bunch of following lines which are instructions. <br>空函数将以 `))` 结尾，因此无需执行任何操作，否则我们将有一堆以下指令行。<br>
        //
        //
        // Lines that have an imbalanced `)` mark the end of a function. <br>`)` 不平衡的行标记函数的结尾。<br>
        if !line.ends_with("))") {
            while let Some(line) = lines.next() {
                function.instrs.push(line.to_string());
                if !line.starts_with("(") && line.ends_with(")") {
                    break;
                }
            }
        }
        // The second element here split on whitespace should be the name of the function, skipping the type/params/results <br>在空格上分割的第二个元素应该是函数的名称，跳过 type/params/results<br>
        //
        function.name = line.split_whitespace().nth(1).unwrap().to_string();
        if function.name.starts_with("$") {
            function.name = function.name[1..].to_string()
        }

        if !function.name.contains("stdarch_test_shim") {
            continue;
        }

        assert!(ret.insert(function));
    }
    return ret;
}
