//! 基准指标。
use std::collections::BTreeMap;

#[derive(Clone, PartialEq, Debug, Copy)]
pub struct Metric {
    value: f64,
    noise: f64,
}

impl Metric {
    pub fn new(value: f64, noise: f64) -> Metric {
        Metric { value, noise }
    }
}

#[derive(Clone, PartialEq)]
pub struct MetricMap(BTreeMap<String, Metric>);

impl MetricMap {
    pub fn new() -> MetricMap {
        MetricMap(BTreeMap::new())
    }

    /// 将名为 `value` (+/-`noise`) 的度量标准插入 map。该值必须为非负数。
    /// `noise` 表示度量的不确定性，当使用 `compare_to_old` 从一个度量与下一个度量进行比较时，该命名值的可接受的成对回归的 "noise range" 会增加一倍。
    ///
    ///
    /// 如果 `noise` 为正，则表示您希望看到该度量值变小，因此在正方向上大于 `noise` 的更改表示回归。
    ///
    /// 如果 `noise` 为负，则表示您希望看到该指标的值变大，因此在负方向上大于 `noise` 的更改表示回归。
    ///
    ///
    ///
    ///
    ///
    ///
    pub fn insert_metric(&mut self, name: &str, value: f64, noise: f64) {
        let m = Metric { value, noise };
        self.0.insert(name.to_owned(), m);
    }

    pub fn fmt_metrics(&self) -> String {
        let v = self
            .0
            .iter()
            .map(|(k, v)| format!("{}: {} (+/- {})", *k, v.value, v.noise))
            .collect::<Vec<_>>();
        v.join(", ")
    }
}
