//! Benchmark metrics. <br>基准指标。<br>
use std::collections::BTreeMap;

#[derive(Clone, PartialEq, Debug, Copy)]
pub struct Metric {
    value: f64,
    noise: f64,
}

impl Metric {
    pub fn new(value: f64, noise: f64) -> Metric {
        Metric { value, noise }
    }
}

#[derive(Clone, PartialEq)]
pub struct MetricMap(BTreeMap<String, Metric>);

impl MetricMap {
    pub fn new() -> MetricMap {
        MetricMap(BTreeMap::new())
    }

    /// Insert a named `value` (+/- `noise`) metric into the map. <br>将名为 `value` (+/-`noise`) 的度量标准插入 map。<br> The value must be non-negative. <br>该值必须为非负数。<br>
    /// The `noise` indicates the uncertainty of the metric, which doubles as the "noise range" of acceptable pairwise-regressions on this named value, when comparing from one metric to the next using `compare_to_old`. <br>`noise` 表示度量的不确定性，当使用 `compare_to_old` 从一个度量与下一个度量进行比较时，该命名值的可接受的成对回归的 "noise range" 会增加一倍。<br>
    ///
    ///
    /// If `noise` is positive, then it means this metric is of a value you want to see grow smaller, so a change larger than `noise` in the positive direction represents a regression. <br>如果 `noise` 为正，则表示您希望看到该度量值变小，因此在正方向上大于 `noise` 的更改表示回归。<br>
    ///
    /// If `noise` is negative, then it means this metric is of a value you want to see grow larger, so a change larger than `noise` in the negative direction represents a regression. <br>如果 `noise` 为负，则表示您希望看到该指标的值变大，因此在负方向上大于 `noise` 的更改表示回归。<br>
    ///
    ///
    ///
    ///
    ///
    ///
    pub fn insert_metric(&mut self, name: &str, value: f64, noise: f64) {
        let m = Metric { value, noise };
        self.0.insert(name.to_owned(), m);
    }

    pub fn fmt_metrics(&self) -> String {
        let v = self
            .0
            .iter()
            .map(|(k, v)| format!("{}: {} (+/- {})", *k, v.value, v.noise))
            .collect::<Vec<_>>();
        v.join(", ")
    }
}
