//! Enums denoting options for test execution. <br>枚举，表示测试执行的选项。<br>

/// Whether to execute tests concurrently or not <br>是否同时执行测试<br>
#[derive(Copy, Clone, Debug, PartialEq, Eq)]
pub enum Concurrent {
    Yes,
    No,
}

/// Number of times to run a benchmarked function <br>运行基准函数的次数<br>
#[derive(Clone, PartialEq, Eq)]
pub enum BenchMode {
    Auto,
    Single,
}

/// Whether test is expected to panic or not <br>是否期望对 panic 进行测试<br>
#[derive(Copy, Clone, Debug, PartialEq, Eq, Hash)]
pub enum ShouldPanic {
    No,
    Yes,
    YesWithMessage(&'static str),
}

/// Whether should console output be colored or not <br>控制台输出是否应为彩色<br>
#[derive(Copy, Clone, Debug)]
pub enum ColorConfig {
    AutoColor,
    AlwaysColor,
    NeverColor,
}

/// Format of the test results output <br>测试结果输出格式<br>
#[derive(Copy, Clone, Debug, PartialEq, Eq)]
pub enum OutputFormat {
    /// Verbose output <br>详细输出<br>
    Pretty,
    /// Quiet output <br>安静的输出<br>
    Terse,
    /// JSON output <br>JSON 输出<br>
    Json,
    /// JUnit output <br>JUnit 输出<br>
    Junit,
}

/// Whether ignored test should be run or not <br>是否应该运行被忽略的测试<br>
#[derive(Copy, Clone, Debug, PartialEq, Eq)]
pub enum RunIgnored {
    Yes,
    No,
    /// Run only ignored tests <br>仅运行忽略的测试<br>
    Only,
}

#[derive(Clone, Copy)]
pub enum RunStrategy {
    /// Runs the test in the current process, and sends the result back over the supplied channel. <br>在当前进程中运行测试，并将结果通过提供的通道发送回去。<br>
    ///
    InProcess,

    /// Spawns a subprocess to run the test, and sends the result back over the supplied channel. <br>产生一个子进程来运行测试，并将结果通过提供的通道发送回去。<br>
    /// Requires `argv[0]` to exist and point to the binary that's currently running. <br>需要 `argv[0]` 存在并指向当前正在运行的二进制文件。<br>
    ///
    SpawnPrimary,
}

/// Options for the test run defined by the caller (instead of CLI arguments). <br>调用者定义的测试运行的选项 (而不是 CLI 参数)。<br>
/// In case we want to add other options as well, just add them in this struct. <br>如果我们还想添加其他选项，只需将其添加到此结构体中即可。<br>
#[derive(Copy, Clone, Debug)]
pub struct Options {
    pub display_output: bool,
    pub panic_abort: bool,
}

impl Options {
    pub fn new() -> Options {
        Options { display_output: false, panic_abort: false }
    }

    pub fn display_output(mut self, display_output: bool) -> Options {
        self.display_output = display_output;
        self
    }

    pub fn panic_abort(mut self, panic_abort: bool) -> Options {
        self.panic_abort = panic_abort;
        self
    }
}
