//! 枚举，表示测试执行的选项。

/// 是否同时执行测试
#[derive(Copy, Clone, Debug, PartialEq, Eq)]
pub enum Concurrent {
    Yes,
    No,
}

/// 运行基准函数的次数
#[derive(Clone, PartialEq, Eq)]
pub enum BenchMode {
    Auto,
    Single,
}

/// 是否期望对 panic 进行测试
#[derive(Copy, Clone, Debug, PartialEq, Eq, Hash)]
pub enum ShouldPanic {
    No,
    Yes,
    YesWithMessage(&'static str),
}

/// 控制台输出是否应为彩色
#[derive(Copy, Clone, Debug)]
pub enum ColorConfig {
    AutoColor,
    AlwaysColor,
    NeverColor,
}

/// 测试结果输出格式
#[derive(Copy, Clone, Debug, PartialEq, Eq)]
pub enum OutputFormat {
    /// 详细输出
    Pretty,
    /// 安静的输出
    Terse,
    /// JSON 输出
    Json,
    /// JUnit 输出
    Junit,
}

/// 是否应该运行被忽略的测试
#[derive(Copy, Clone, Debug, PartialEq, Eq)]
pub enum RunIgnored {
    Yes,
    No,
    /// 仅运行忽略的测试
    Only,
}

#[derive(Clone, Copy)]
pub enum RunStrategy {
    /// 在当前进程中运行测试，并将结果通过提供的通道发送回去。
    ///
    InProcess,

    /// 产生一个子进程来运行测试，并将结果通过提供的通道发送回去。
    /// 需要 `argv[0]` 存在并指向当前正在运行的二进制文件。
    ///
    SpawnPrimary,
}

/// 调用者定义的测试运行的选项 (而不是 CLI 参数)。
/// 如果我们还想添加其他选项，只需将其添加到此结构体中即可。
#[derive(Copy, Clone, Debug)]
pub struct Options {
    pub display_output: bool,
    pub panic_abort: bool,
}

impl Options {
    pub fn new() -> Options {
        Options { display_output: false, panic_abort: false }
    }

    pub fn display_output(mut self, display_output: bool) -> Options {
        self.display_output = display_output;
        self
    }

    pub fn panic_abort(mut self, panic_abort: bool) -> Options {
        self.panic_abort = panic_abort;
        self
    }
}
