//! ncurses 兼容的数据库发现。
//!
//! 不支持哈希数据库，仅支持文件系统！

use std::env;
use std::fs;
use std::path::PathBuf;

#[cfg(test)]
mod tests;

/// `term` 的数据库条目的返回路径
#[allow(deprecated)]
pub(crate) fn get_dbpath_for_term(term: &str) -> Option<PathBuf> {
    let mut dirs_to_search = Vec::new();
    let first_char = term.chars().next()?;

    // 查找搜索目录
    if let Some(dir) = env::var_os("TERMINFO") {
        dirs_to_search.push(PathBuf::from(dir));
    }

    if let Ok(dirs) = env::var("TERMINFO_DIRS") {
        for i in dirs.split(':') {
            if i == "" {
                dirs_to_search.push(PathBuf::from("/usr/share/terminfo"));
            } else {
                dirs_to_search.push(PathBuf::from(i));
            }
        }
    } else {
        // 在 TERMINFO_DIRS 中一无所获，请使用默认路径：
        // 根据 /etc/terminfo/README，在查看 ~/.terminfo 之后，ncurses 将搜索 /etc/terminfo，然后是 /lib/terminfo，最后是 /usr/share/terminfo。
        //
        // 在 Haiku 上，数据库位于 /boot/system/data/terminfo
        //
        if let Some(mut homedir) = env::home_dir() {
            homedir.push(".terminfo");
            dirs_to_search.push(homedir)
        }

        dirs_to_search.push(PathBuf::from("/etc/terminfo"));
        dirs_to_search.push(PathBuf::from("/lib/terminfo"));
        dirs_to_search.push(PathBuf::from("/usr/share/terminfo"));
        dirs_to_search.push(PathBuf::from("/boot/system/data/terminfo"));
    }

    // 在所有搜索目录中查找终端
    for mut p in dirs_to_search {
        if fs::metadata(&p).is_ok() {
            p.push(&first_char.to_string());
            p.push(&term);
            if fs::metadata(&p).is_ok() {
                return Some(p);
            }
            p.pop();
            p.pop();

            // 在某些安装中，该目录以 char 的十六进制命名 (例如 macOS)
            //
            p.push(&format!("{:x}", first_char as usize));
            p.push(term);
            if fs::metadata(&p).is_ok() {
                return Some(p);
            }
        }
    }
    None
}
