//! ncurses-compatible database discovery. <br>ncurses 兼容的数据库发现。<br>
//!
//! Does not support hashed database, only filesystem! <br>不支持哈希数据库，仅支持文件系统！<br>

use std::env;
use std::fs;
use std::path::PathBuf;

#[cfg(test)]
mod tests;

/// Return path to database entry for `term` <br>`term` 的数据库条目的返回路径<br>
#[allow(deprecated)]
pub(crate) fn get_dbpath_for_term(term: &str) -> Option<PathBuf> {
    let mut dirs_to_search = Vec::new();
    let first_char = term.chars().next()?;

    // Find search directory <br>查找搜索目录<br>
    if let Some(dir) = env::var_os("TERMINFO") {
        dirs_to_search.push(PathBuf::from(dir));
    }

    if let Ok(dirs) = env::var("TERMINFO_DIRS") {
        for i in dirs.split(':') {
            if i == "" {
                dirs_to_search.push(PathBuf::from("/usr/share/terminfo"));
            } else {
                dirs_to_search.push(PathBuf::from(i));
            }
        }
    } else {
        // Found nothing in TERMINFO_DIRS, use the default paths: <br>在 TERMINFO_DIRS 中一无所获，请使用默认路径：<br>
        // According to  /etc/terminfo/README, after looking at ~/.terminfo, ncurses will search /etc/terminfo, then /lib/terminfo, and eventually /usr/share/terminfo. <br>根据 /etc/terminfo/README，在查看 ~/.terminfo 之后，ncurses 将搜索 /etc/terminfo，然后是 /lib/terminfo，最后是 /usr/share/terminfo。<br>
        //
        // On Haiku the database can be found at /boot/system/data/terminfo <br>在 Haiku 上，数据库位于 /boot/system/data/terminfo<br>
        //
        if let Some(mut homedir) = env::home_dir() {
            homedir.push(".terminfo");
            dirs_to_search.push(homedir)
        }

        dirs_to_search.push(PathBuf::from("/etc/terminfo"));
        dirs_to_search.push(PathBuf::from("/lib/terminfo"));
        dirs_to_search.push(PathBuf::from("/usr/share/terminfo"));
        dirs_to_search.push(PathBuf::from("/boot/system/data/terminfo"));
    }

    // Look for the terminal in all of the search directories <br>在所有搜索目录中查找终端<br>
    for mut p in dirs_to_search {
        if fs::metadata(&p).is_ok() {
            p.push(&first_char.to_string());
            p.push(&term);
            if fs::metadata(&p).is_ok() {
                return Some(p);
            }
            p.pop();
            p.pop();

            // on some installations the dir is named after the hex of the char (e.g., macOS) <br>在某些安装中，该目录以 char 的十六进制命名 (例如 macOS)<br>
            //
            p.push(&format!("{:x}", first_char as usize));
            p.push(term);
            if fs::metadata(&p).is_ok() {
                return Some(p);
            }
        }
    }
    None
}
