//! MIPS SIMD Architecture 内部函数
//!
//! 引用的是 [面向程序员的 MIPS 架构第 IV-j 卷：MIPS32 SIMD 架构模块修订版 1.12][msa_ref]。
//!
//!
//! [msa_ref]: http://cdn2.imgtec.com/documentation/MD00866-2B-MSA32-AFP-01.12.pdf

#[cfg(test)]
use stdarch_test::assert_instr;

use crate::mem;

#[macro_use]
mod macros;

types! {
    // / 特定于 MIPS 的 16 个包装 `i8` 的 128 位宽 vector。
   pub struct v16i8(
       i8, i8, i8, i8, i8, i8, i8, i8,
       i8, i8, i8, i8, i8, i8, i8, i8,
   );

    // / 特定于 MIPS 的 8 位包装 `i16` 的 128 位宽 vector。
   pub struct v8i16(
       i16, i16, i16, i16, i16, i16, i16, i16,
   );

    // / 特定于 MIPS 的 128 位宽的 vector (4 个包装的 `i32`)。
   pub struct v4i32(
       i32, i32, i32, i32,
   );

    // / 特定于 MIPS 的 2 个包装的 `i64` 的 128 位宽 vector。
   pub struct v2i64(
       i64, i64,
   );

    // / 特定于 MIPS 的 16 个包装 `u8` 的 128 位宽 vector。
   pub struct v16u8(
       u8, u8, u8, u8, u8, u8, u8, u8,
       u8, u8, u8, u8, u8, u8, u8, u8,
   );

    // / 特定于 MIPS 的 8 位包装 `u16` 的 128 位宽 vector。
   pub struct v8u16(
       u16, u16, u16, u16, u16, u16, u16, u16,
   );

    // / 特定于 MIPS 的 128 位宽的 vector (4 个包装的 `u32`)。
   pub struct v4u32(
       u32, u32, u32, u32,
   );

    // / 特定于 MIPS 的 2 个包装的 `u64` 的 128 位宽 vector。
   pub struct v2u64(
       u64, u64,
   );

   // / 特定于 MIPS 的 128 位宽的 vector (4 个包装的 `f32`)。
   pub struct v4f32(
       f32, f32, f32, f32,
   );

    // / 特定于 MIPS 的 2 个包装的 `f64` 的 128 位宽 vector。
   pub struct v2f64(
       f64, f64,
   );
}

#[allow(improper_ctypes)]
extern "C" {
    #[link_name = "llvm.mips.add.a.b"]
    fn msa_add_a_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.add.a.h"]
    fn msa_add_a_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.add.a.w"]
    fn msa_add_a_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.add.a.d"]
    fn msa_add_a_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.adds.a.b"]
    fn msa_adds_a_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.adds.a.h"]
    fn msa_adds_a_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.adds.a.w"]
    fn msa_adds_a_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.adds.a.d"]
    fn msa_adds_a_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.adds.s.b"]
    fn msa_adds_s_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.adds.s.h"]
    fn msa_adds_s_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.adds.s.w"]
    fn msa_adds_s_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.adds.s.d"]
    fn msa_adds_s_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.adds.u.b"]
    fn msa_adds_u_b(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.adds.u.h"]
    fn msa_adds_u_h(a: v8u16, b: v8u16) -> v8u16;
    #[link_name = "llvm.mips.adds.u.w"]
    fn msa_adds_u_w(a: v4u32, b: v4u32) -> v4u32;
    #[link_name = "llvm.mips.adds.u.d"]
    fn msa_adds_u_d(a: v2u64, b: v2u64) -> v2u64;
    #[link_name = "llvm.mips.addv.b"]
    fn msa_addv_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.addv.h"]
    fn msa_addv_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.addv.w"]
    fn msa_addv_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.addv.d"]
    fn msa_addv_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.addvi.b"]
    fn msa_addvi_b(a: v16i8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.addvi.h"]
    fn msa_addvi_h(a: v8i16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.addvi.w"]
    fn msa_addvi_w(a: v4i32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.addvi.d"]
    fn msa_addvi_d(a: v2i64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.and.v"]
    fn msa_and_v(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.andi.b"]
    fn msa_andi_b(a: v16u8, b: i32) -> v16u8;
    #[link_name = "llvm.mips.asub.s.b"]
    fn msa_asub_s_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.asub.s.h"]
    fn msa_asub_s_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.asub.s.w"]
    fn msa_asub_s_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.asub.s.d"]
    fn msa_asub_s_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.asub.u.b"]
    fn msa_asub_u_b(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.asub.u.h"]
    fn msa_asub_u_h(a: v8u16, b: v8u16) -> v8u16;
    #[link_name = "llvm.mips.asub.u.w"]
    fn msa_asub_u_w(a: v4u32, b: v4u32) -> v4u32;
    #[link_name = "llvm.mips.asub.u.d"]
    fn msa_asub_u_d(a: v2u64, b: v2u64) -> v2u64;
    #[link_name = "llvm.mips.ave.s.b"]
    fn msa_ave_s_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.ave.s.h"]
    fn msa_ave_s_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.ave.s.w"]
    fn msa_ave_s_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.ave.s.d"]
    fn msa_ave_s_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.ave.u.b"]
    fn msa_ave_u_b(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.ave.u.h"]
    fn msa_ave_u_h(a: v8u16, b: v8u16) -> v8u16;
    #[link_name = "llvm.mips.ave.u.w"]
    fn msa_ave_u_w(a: v4u32, b: v4u32) -> v4u32;
    #[link_name = "llvm.mips.ave.u.d"]
    fn msa_ave_u_d(a: v2u64, b: v2u64) -> v2u64;
    #[link_name = "llvm.mips.aver.s.b"]
    fn msa_aver_s_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.aver.s.h"]
    fn msa_aver_s_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.aver.s.w"]
    fn msa_aver_s_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.aver.s.d"]
    fn msa_aver_s_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.aver.u.b"]
    fn msa_aver_u_b(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.aver.u.h"]
    fn msa_aver_u_h(a: v8u16, b: v8u16) -> v8u16;
    #[link_name = "llvm.mips.aver.u.w"]
    fn msa_aver_u_w(a: v4u32, b: v4u32) -> v4u32;
    #[link_name = "llvm.mips.aver.u.d"]
    fn msa_aver_u_d(a: v2u64, b: v2u64) -> v2u64;
    #[link_name = "llvm.mips.bclr.b"]
    fn msa_bclr_b(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.bclr.h"]
    fn msa_bclr_h(a: v8u16, b: v8u16) -> v8u16;
    #[link_name = "llvm.mips.bclr.w"]
    fn msa_bclr_w(a: v4u32, b: v4u32) -> v4u32;
    #[link_name = "llvm.mips.bclr.d"]
    fn msa_bclr_d(a: v2u64, b: v2u64) -> v2u64;
    #[link_name = "llvm.mips.bclri.b"]
    fn msa_bclri_b(a: v16u8, b: i32) -> v16u8;
    #[link_name = "llvm.mips.bclri.h"]
    fn msa_bclri_h(a: v8u16, b: i32) -> v8u16;
    #[link_name = "llvm.mips.bclri.w"]
    fn msa_bclri_w(a: v4u32, b: i32) -> v4u32;
    #[link_name = "llvm.mips.bclri.d"]
    fn msa_bclri_d(a: v2u64, b: i32) -> v2u64;
    #[link_name = "llvm.mips.binsl.b"]
    fn msa_binsl_b(a: v16u8, b: v16u8, c: v16u8) -> v16u8;
    #[link_name = "llvm.mips.binsl.h"]
    fn msa_binsl_h(a: v8u16, b: v8u16, c: v8u16) -> v8u16;
    #[link_name = "llvm.mips.binsl.w"]
    fn msa_binsl_w(a: v4u32, b: v4u32, c: v4u32) -> v4u32;
    #[link_name = "llvm.mips.binsl.d"]
    fn msa_binsl_d(a: v2u64, b: v2u64, c: v2u64) -> v2u64;
    #[link_name = "llvm.mips.binsli.b"]
    fn msa_binsli_b(a: v16u8, b: v16u8, c: i32) -> v16u8;
    #[link_name = "llvm.mips.binsli.h"]
    fn msa_binsli_h(a: v8u16, b: v8u16, c: i32) -> v8u16;
    #[link_name = "llvm.mips.binsli.w"]
    fn msa_binsli_w(a: v4u32, b: v4u32, c: i32) -> v4u32;
    #[link_name = "llvm.mips.binsli.d"]
    fn msa_binsli_d(a: v2u64, b: v2u64, c: i32) -> v2u64;
    #[link_name = "llvm.mips.binsr.b"]
    fn msa_binsr_b(a: v16u8, b: v16u8, c: v16u8) -> v16u8;
    #[link_name = "llvm.mips.binsr.h"]
    fn msa_binsr_h(a: v8u16, b: v8u16, c: v8u16) -> v8u16;
    #[link_name = "llvm.mips.binsr.w"]
    fn msa_binsr_w(a: v4u32, b: v4u32, c: v4u32) -> v4u32;
    #[link_name = "llvm.mips.binsr.d"]
    fn msa_binsr_d(a: v2u64, b: v2u64, c: v2u64) -> v2u64;
    #[link_name = "llvm.mips.binsri.b"]
    fn msa_binsri_b(a: v16u8, b: v16u8, c: i32) -> v16u8;
    #[link_name = "llvm.mips.binsri.h"]
    fn msa_binsri_h(a: v8u16, b: v8u16, c: i32) -> v8u16;
    #[link_name = "llvm.mips.binsri.w"]
    fn msa_binsri_w(a: v4u32, b: v4u32, c: i32) -> v4u32;
    #[link_name = "llvm.mips.binsri.d"]
    fn msa_binsri_d(a: v2u64, b: v2u64, c: i32) -> v2u64;
    #[link_name = "llvm.mips.bmnz.v"]
    fn msa_bmnz_v(a: v16u8, b: v16u8, c: v16u8) -> v16u8;
    #[link_name = "llvm.mips.bmnzi.b"]
    fn msa_bmnzi_b(a: v16u8, b: v16u8, c: i32) -> v16u8;
    #[link_name = "llvm.mips.bmz.v"]
    fn msa_bmz_v(a: v16u8, b: v16u8, c: v16u8) -> v16u8;
    #[link_name = "llvm.mips.bmzi.b"]
    fn msa_bmzi_b(a: v16u8, b: v16u8, c: i32) -> v16u8;
    #[link_name = "llvm.mips.bneg.b"]
    fn msa_bneg_b(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.bneg.h"]
    fn msa_bneg_h(a: v8u16, b: v8u16) -> v8u16;
    #[link_name = "llvm.mips.bneg.w"]
    fn msa_bneg_w(a: v4u32, b: v4u32) -> v4u32;
    #[link_name = "llvm.mips.bneg.d"]
    fn msa_bneg_d(a: v2u64, b: v2u64) -> v2u64;
    #[link_name = "llvm.mips.bnegi.b"]
    fn msa_bnegi_b(a: v16u8, b: i32) -> v16u8;
    #[link_name = "llvm.mips.bnegi.h"]
    fn msa_bnegi_h(a: v8u16, b: i32) -> v8u16;
    #[link_name = "llvm.mips.bnegi.w"]
    fn msa_bnegi_w(a: v4u32, b: i32) -> v4u32;
    #[link_name = "llvm.mips.bnegi.d"]
    fn msa_bnegi_d(a: v2u64, b: i32) -> v2u64;
    #[link_name = "llvm.mips.bnz.b"]
    fn msa_bnz_b(a: v16u8) -> i32;
    #[link_name = "llvm.mips.bnz.h"]
    fn msa_bnz_h(a: v8u16) -> i32;
    #[link_name = "llvm.mips.bnz.w"]
    fn msa_bnz_w(a: v4u32) -> i32;
    #[link_name = "llvm.mips.bnz.d"]
    fn msa_bnz_d(a: v2u64) -> i32;
    #[link_name = "llvm.mips.bnz.v"]
    fn msa_bnz_v(a: v16u8) -> i32;
    #[link_name = "llvm.mips.bsel.v"]
    fn msa_bsel_v(a: v16u8, b: v16u8, c: v16u8) -> v16u8;
    #[link_name = "llvm.mips.bseli.b"]
    fn msa_bseli_b(a: v16u8, b: v16u8, c: i32) -> v16u8;
    #[link_name = "llvm.mips.bset.b"]
    fn msa_bset_b(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.bset.h"]
    fn msa_bset_h(a: v8u16, b: v8u16) -> v8u16;
    #[link_name = "llvm.mips.bset.w"]
    fn msa_bset_w(a: v4u32, b: v4u32) -> v4u32;
    #[link_name = "llvm.mips.bset.d"]
    fn msa_bset_d(a: v2u64, b: v2u64) -> v2u64;
    #[link_name = "llvm.mips.bseti.b"]
    fn msa_bseti_b(a: v16u8, b: i32) -> v16u8;
    #[link_name = "llvm.mips.bseti.h"]
    fn msa_bseti_h(a: v8u16, b: i32) -> v8u16;
    #[link_name = "llvm.mips.bseti.w"]
    fn msa_bseti_w(a: v4u32, b: i32) -> v4u32;
    #[link_name = "llvm.mips.bseti.d"]
    fn msa_bseti_d(a: v2u64, b: i32) -> v2u64;
    #[link_name = "llvm.mips.bz.b"]
    fn msa_bz_b(a: v16u8) -> i32;
    #[link_name = "llvm.mips.bz.h"]
    fn msa_bz_h(a: v8u16) -> i32;
    #[link_name = "llvm.mips.bz.w"]
    fn msa_bz_w(a: v4u32) -> i32;
    #[link_name = "llvm.mips.bz.d"]
    fn msa_bz_d(a: v2u64) -> i32;
    #[link_name = "llvm.mips.bz.v"]
    fn msa_bz_v(a: v16u8) -> i32;
    #[link_name = "llvm.mips.ceq.b"]
    fn msa_ceq_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.ceq.h"]
    fn msa_ceq_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.ceq.w"]
    fn msa_ceq_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.ceq.d"]
    fn msa_ceq_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.ceqi.b"]
    fn msa_ceqi_b(a: v16i8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.ceqi.h"]
    fn msa_ceqi_h(a: v8i16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.ceqi.w"]
    fn msa_ceqi_w(a: v4i32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.ceqi.d"]
    fn msa_ceqi_d(a: v2i64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.cfcmsa"]
    fn msa_cfcmsa(a: i32) -> i32;
    #[link_name = "llvm.mips.cle.s.b"]
    fn msa_cle_s_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.cle.s.h"]
    fn msa_cle_s_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.cle.s.w"]
    fn msa_cle_s_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.cle.s.d"]
    fn msa_cle_s_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.cle.u.b"]
    fn msa_cle_u_b(a: v16u8, b: v16u8) -> v16i8;
    #[link_name = "llvm.mips.cle.u.h"]
    fn msa_cle_u_h(a: v8u16, b: v8u16) -> v8i16;
    #[link_name = "llvm.mips.cle.u.w"]
    fn msa_cle_u_w(a: v4u32, b: v4u32) -> v4i32;
    #[link_name = "llvm.mips.cle.u.d"]
    fn msa_cle_u_d(a: v2u64, b: v2u64) -> v2i64;
    #[link_name = "llvm.mips.clei.s.b"]
    fn msa_clei_s_b(a: v16i8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.clei.s.h"]
    fn msa_clei_s_h(a: v8i16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.clei.s.w"]
    fn msa_clei_s_w(a: v4i32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.clei.s.d"]
    fn msa_clei_s_d(a: v2i64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.clei.u.b"]
    fn msa_clei_u_b(a: v16u8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.clei.u.h"]
    fn msa_clei_u_h(a: v8u16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.clei.u.w"]
    fn msa_clei_u_w(a: v4u32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.clei.u.d"]
    fn msa_clei_u_d(a: v2u64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.clt.s.b"]
    fn msa_clt_s_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.clt.s.h"]
    fn msa_clt_s_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.clt.s.w"]
    fn msa_clt_s_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.clt.s.d"]
    fn msa_clt_s_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.clt.u.b"]
    fn msa_clt_u_b(a: v16u8, b: v16u8) -> v16i8;
    #[link_name = "llvm.mips.clt.u.h"]
    fn msa_clt_u_h(a: v8u16, b: v8u16) -> v8i16;
    #[link_name = "llvm.mips.clt.u.w"]
    fn msa_clt_u_w(a: v4u32, b: v4u32) -> v4i32;
    #[link_name = "llvm.mips.clt.u.d"]
    fn msa_clt_u_d(a: v2u64, b: v2u64) -> v2i64;
    #[link_name = "llvm.mips.clti.s.b"]
    fn msa_clti_s_b(a: v16i8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.clti.s.h"]
    fn msa_clti_s_h(a: v8i16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.clti.s.w"]
    fn msa_clti_s_w(a: v4i32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.clti.s.d"]
    fn msa_clti_s_d(a: v2i64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.clti.u.b"]
    fn msa_clti_u_b(a: v16u8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.clti.u.h"]
    fn msa_clti_u_h(a: v8u16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.clti.u.w"]
    fn msa_clti_u_w(a: v4u32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.clti.u.d"]
    fn msa_clti_u_d(a: v2u64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.copy.s.b"]
    fn msa_copy_s_b(a: v16i8, b: i32) -> i32;
    #[link_name = "llvm.mips.copy.s.h"]
    fn msa_copy_s_h(a: v8i16, b: i32) -> i32;
    #[link_name = "llvm.mips.copy.s.w"]
    fn msa_copy_s_w(a: v4i32, b: i32) -> i32;
    #[link_name = "llvm.mips.copy.s.d"]
    fn msa_copy_s_d(a: v2i64, b: i32) -> i64;
    #[link_name = "llvm.mips.copy.u.b"]
    fn msa_copy_u_b(a: v16i8, b: i32) -> u32;
    #[link_name = "llvm.mips.copy.u.h"]
    fn msa_copy_u_h(a: v8i16, b: i32) -> u32;
    #[link_name = "llvm.mips.copy.u.w"]
    fn msa_copy_u_w(a: v4i32, b: i32) -> u32;
    #[link_name = "llvm.mips.copy.u.d"]
    fn msa_copy_u_d(a: v2i64, b: i32) -> u64;
    #[link_name = "llvm.mips.ctcmsa"]
    fn msa_ctcmsa(imm5: i32, a: i32) -> ();
    #[link_name = "llvm.mips.div.s.b"]
    fn msa_div_s_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.div.s.h"]
    fn msa_div_s_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.div.s.w"]
    fn msa_div_s_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.div.s.d"]
    fn msa_div_s_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.div.u.b"]
    fn msa_div_u_b(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.div.u.h"]
    fn msa_div_u_h(a: v8u16, b: v8u16) -> v8u16;
    #[link_name = "llvm.mips.div.u.w"]
    fn msa_div_u_w(a: v4u32, b: v4u32) -> v4u32;
    #[link_name = "llvm.mips.div.u.d"]
    fn msa_div_u_d(a: v2u64, b: v2u64) -> v2u64;
    #[link_name = "llvm.mips.dotp.s.h"]
    fn msa_dotp_s_h(a: v16i8, b: v16i8) -> v8i16;
    #[link_name = "llvm.mips.dotp.s.w"]
    fn msa_dotp_s_w(a: v8i16, b: v8i16) -> v4i32;
    #[link_name = "llvm.mips.dotp.s.d"]
    fn msa_dotp_s_d(a: v4i32, b: v4i32) -> v2i64;
    #[link_name = "llvm.mips.dotp.u.h"]
    fn msa_dotp_u_h(a: v16u8, b: v16u8) -> v8u16;
    #[link_name = "llvm.mips.dotp.u.w"]
    fn msa_dotp_u_w(a: v8u16, b: v8u16) -> v4u32;
    #[link_name = "llvm.mips.dotp.u.d"]
    fn msa_dotp_u_d(a: v4u32, b: v4u32) -> v2u64;
    #[link_name = "llvm.mips.dpadd.s.h"]
    fn msa_dpadd_s_h(a: v8i16, b: v16i8, c: v16i8) -> v8i16;
    #[link_name = "llvm.mips.dpadd.s.w"]
    fn msa_dpadd_s_w(a: v4i32, b: v8i16, c: v8i16) -> v4i32;
    #[link_name = "llvm.mips.dpadd.s.d"]
    fn msa_dpadd_s_d(a: v2i64, b: v4i32, c: v4i32) -> v2i64;
    #[link_name = "llvm.mips.dpadd.u.h"]
    fn msa_dpadd_u_h(a: v8u16, b: v16u8, c: v16u8) -> v8u16;
    #[link_name = "llvm.mips.dpadd.u.w"]
    fn msa_dpadd_u_w(a: v4u32, b: v8u16, c: v8u16) -> v4u32;
    #[link_name = "llvm.mips.dpadd.u.d"]
    fn msa_dpadd_u_d(a: v2u64, b: v4u32, c: v4u32) -> v2u64;
    #[link_name = "llvm.mips.dpsub.s.h"]
    fn msa_dpsub_s_h(a: v8i16, b: v16i8, c: v16i8) -> v8i16;
    #[link_name = "llvm.mips.dpsub.s.w"]
    fn msa_dpsub_s_w(a: v4i32, b: v8i16, c: v8i16) -> v4i32;
    #[link_name = "llvm.mips.dpsub.s.d"]
    fn msa_dpsub_s_d(a: v2i64, b: v4i32, c: v4i32) -> v2i64;
    #[link_name = "llvm.mips.dpsub.u.h"]
    fn msa_dpsub_u_h(a: v8i16, b: v16u8, c: v16u8) -> v8i16;
    #[link_name = "llvm.mips.dpsub.u.w"]
    fn msa_dpsub_u_w(a: v4i32, b: v8u16, c: v8u16) -> v4i32;
    #[link_name = "llvm.mips.dpsub.u.d"]
    fn msa_dpsub_u_d(a: v2i64, b: v4u32, c: v4u32) -> v2i64;
    #[link_name = "llvm.mips.fadd.w"]
    fn msa_fadd_w(a: v4f32, b: v4f32) -> v4f32;
    #[link_name = "llvm.mips.fadd.d"]
    fn msa_fadd_d(a: v2f64, b: v2f64) -> v2f64;
    #[link_name = "llvm.mips.fcaf.w"]
    fn msa_fcaf_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fcaf.d"]
    fn msa_fcaf_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fceq.w"]
    fn msa_fceq_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fceq.d"]
    fn msa_fceq_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fclass.w"]
    fn msa_fclass_w(a: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fclass.d"]
    fn msa_fclass_d(a: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fcle.w"]
    fn msa_fcle_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fcle.d"]
    fn msa_fcle_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fclt.w"]
    fn msa_fclt_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fclt.d"]
    fn msa_fclt_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fcne.w"]
    fn msa_fcne_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fcne.d"]
    fn msa_fcne_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fcor.w"]
    fn msa_fcor_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fcor.d"]
    fn msa_fcor_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fcueq.w"]
    fn msa_fcueq_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fcueq.d"]
    fn msa_fcueq_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fcule.w"]
    fn msa_fcule_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fcule.d"]
    fn msa_fcule_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fcult.w"]
    fn msa_fcult_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fcult.d"]
    fn msa_fcult_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fcun.w"]
    fn msa_fcun_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fcun.d"]
    fn msa_fcun_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fcune.w"]
    fn msa_fcune_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fcune.d"]
    fn msa_fcune_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fdiv.w"]
    fn msa_fdiv_w(a: v4f32, b: v4f32) -> v4f32;
    #[link_name = "llvm.mips.fdiv.d"]
    fn msa_fdiv_d(a: v2f64, b: v2f64) -> v2f64;
    // FIXME: 16 位浮点数
    // #[link_name = "llvm.mips.fexdo.h"]
    // fn msa_fexdo_h(a: v4f32, b: v4f32) -> f16x8;
    #[link_name = "llvm.mips.fexdo.w"]
    fn msa_fexdo_w(a: v2f64, b: v2f64) -> v4f32;
    #[link_name = "llvm.mips.fexp2.w"]
    fn msa_fexp2_w(a: v4f32, b: v4i32) -> v4f32;
    #[link_name = "llvm.mips.fexp2.d"]
    fn msa_fexp2_d(a: v2f64, b: v2i64) -> v2f64;
    // FIXME: 16 位浮点数
    // #[link_name = "llvm.mips.fexupl.w"]
    // fn msa_fexupl_w(a: f16x8) -> v4f32;
    #[link_name = "llvm.mips.fexupl.d"]
    fn msa_fexupl_d(a: v4f32) -> v2f64;
    // FIXME: 16 位浮点数
    // #[link_name = "llvm.mips.fexupr.w"]
    // fn msa_fexupr_w(a: f16x8) -> v4f32;
    #[link_name = "llvm.mips.fexupr.d"]
    fn msa_fexupr_d(a: v4f32) -> v2f64;
    #[link_name = "llvm.mips.ffint.s.w"]
    fn msa_ffint_s_w(a: v4i32) -> v4f32;
    #[link_name = "llvm.mips.ffint.s.d"]
    fn msa_ffint_s_d(a: v2i64) -> v2f64;
    #[link_name = "llvm.mips.ffint.u.w"]
    fn msa_ffint_u_w(a: v4u32) -> v4f32;
    #[link_name = "llvm.mips.ffint.u.d"]
    fn msa_ffint_u_d(a: v2u64) -> v2f64;
    #[link_name = "llvm.mips.ffql.w"]
    fn msa_ffql_w(a: v8i16) -> v4f32;
    #[link_name = "llvm.mips.ffql.d"]
    fn msa_ffql_d(a: v4i32) -> v2f64;
    #[link_name = "llvm.mips.ffqr.w"]
    fn msa_ffqr_w(a: v8i16) -> v4f32;
    #[link_name = "llvm.mips.ffqr.d"]
    fn msa_ffqr_d(a: v4i32) -> v2f64;
    #[link_name = "llvm.mips.fill.b"]
    fn msa_fill_b(a: i32) -> v16i8;
    #[link_name = "llvm.mips.fill.h"]
    fn msa_fill_h(a: i32) -> v8i16;
    #[link_name = "llvm.mips.fill.w"]
    fn msa_fill_w(a: i32) -> v4i32;
    #[link_name = "llvm.mips.fill.d"]
    fn msa_fill_d(a: i64) -> v2i64;
    #[link_name = "llvm.mips.flog2.w"]
    fn msa_flog2_w(a: v4f32) -> v4f32;
    #[link_name = "llvm.mips.flog2.d"]
    fn msa_flog2_d(a: v2f64) -> v2f64;
    #[link_name = "llvm.mips.fmadd.w"]
    fn msa_fmadd_w(a: v4f32, b: v4f32, c: v4f32) -> v4f32;
    #[link_name = "llvm.mips.fmadd.d"]
    fn msa_fmadd_d(a: v2f64, b: v2f64, c: v2f64) -> v2f64;
    #[link_name = "llvm.mips.fmax.w"]
    fn msa_fmax_w(a: v4f32, b: v4f32) -> v4f32;
    #[link_name = "llvm.mips.fmax.d"]
    fn msa_fmax_d(a: v2f64, b: v2f64) -> v2f64;
    #[link_name = "llvm.mips.fmax.a.w"]
    fn msa_fmax_a_w(a: v4f32, b: v4f32) -> v4f32;
    #[link_name = "llvm.mips.fmax.a.d"]
    fn msa_fmax_a_d(a: v2f64, b: v2f64) -> v2f64;
    #[link_name = "llvm.mips.fmin.w"]
    fn msa_fmin_w(a: v4f32, b: v4f32) -> v4f32;
    #[link_name = "llvm.mips.fmin.d"]
    fn msa_fmin_d(a: v2f64, b: v2f64) -> v2f64;
    #[link_name = "llvm.mips.fmin.a.w"]
    fn msa_fmin_a_w(a: v4f32, b: v4f32) -> v4f32;
    #[link_name = "llvm.mips.fmin.a.d"]
    fn msa_fmin_a_d(a: v2f64, b: v2f64) -> v2f64;
    #[link_name = "llvm.mips.fmsub.w"]
    fn msa_fmsub_w(a: v4f32, b: v4f32, c: v4f32) -> v4f32;
    #[link_name = "llvm.mips.fmsub.d"]
    fn msa_fmsub_d(a: v2f64, b: v2f64, c: v2f64) -> v2f64;
    #[link_name = "llvm.mips.fmul.w"]
    fn msa_fmul_w(a: v4f32, b: v4f32) -> v4f32;
    #[link_name = "llvm.mips.fmul.d"]
    fn msa_fmul_d(a: v2f64, b: v2f64) -> v2f64;
    #[link_name = "llvm.mips.frint.w"]
    fn msa_frint_w(a: v4f32) -> v4f32;
    #[link_name = "llvm.mips.frint.d"]
    fn msa_frint_d(a: v2f64) -> v2f64;
    #[link_name = "llvm.mips.frcp.w"]
    fn msa_frcp_w(a: v4f32) -> v4f32;
    #[link_name = "llvm.mips.frcp.d"]
    fn msa_frcp_d(a: v2f64) -> v2f64;
    #[link_name = "llvm.mips.frsqrt.w"]
    fn msa_frsqrt_w(a: v4f32) -> v4f32;
    #[link_name = "llvm.mips.frsqrt.d"]
    fn msa_frsqrt_d(a: v2f64) -> v2f64;
    #[link_name = "llvm.mips.fsaf.w"]
    fn msa_fsaf_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fsaf.d"]
    fn msa_fsaf_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fseq.w"]
    fn msa_fseq_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fseq.d"]
    fn msa_fseq_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fsle.w"]
    fn msa_fsle_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fsle.d"]
    fn msa_fsle_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fslt.w"]
    fn msa_fslt_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fslt.d"]
    fn msa_fslt_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fsne.w"]
    fn msa_fsne_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fsne.d"]
    fn msa_fsne_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fsor.w"]
    fn msa_fsor_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fsor.d"]
    fn msa_fsor_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fsqrt.w"]
    fn msa_fsqrt_w(a: v4f32) -> v4f32;
    #[link_name = "llvm.mips.fsqrt.d"]
    fn msa_fsqrt_d(a: v2f64) -> v2f64;
    #[link_name = "llvm.mips.fsub.w"]
    fn msa_fsub_w(a: v4f32, b: v4f32) -> v4f32;
    #[link_name = "llvm.mips.fsub.d"]
    fn msa_fsub_d(a: v2f64, b: v2f64) -> v2f64;
    #[link_name = "llvm.mips.fsueq.w"]
    fn msa_fsueq_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fsueq.d"]
    fn msa_fsueq_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fsule.w"]
    fn msa_fsule_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fsule.d"]
    fn msa_fsule_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fsult.w"]
    fn msa_fsult_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fsult.d"]
    fn msa_fsult_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fsun.w"]
    fn msa_fsun_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fsun.d"]
    fn msa_fsun_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.fsune.w"]
    fn msa_fsune_w(a: v4f32, b: v4f32) -> v4i32;
    #[link_name = "llvm.mips.fsune.d"]
    fn msa_fsune_d(a: v2f64, b: v2f64) -> v2i64;
    #[link_name = "llvm.mips.ftint.s.w"]
    fn msa_ftint_s_w(a: v4f32) -> v4i32;
    #[link_name = "llvm.mips.ftint.s.d"]
    fn msa_ftint_s_d(a: v2f64) -> v2i64;
    #[link_name = "llvm.mips.ftint.u.w"]
    fn msa_ftint_u_w(a: v4f32) -> v4u32;
    #[link_name = "llvm.mips.ftint.u.d"]
    fn msa_ftint_u_d(a: v2f64) -> v2u64;
    #[link_name = "llvm.mips.ftq.h"]
    fn msa_ftq_h(a: v4f32, b: v4f32) -> v8i16;
    #[link_name = "llvm.mips.ftq.w"]
    fn msa_ftq_w(a: v2f64, b: v2f64) -> v4i32;
    #[link_name = "llvm.mips.ftrunc.s.w"]
    fn msa_ftrunc_s_w(a: v4f32) -> v4i32;
    #[link_name = "llvm.mips.ftrunc.s.d"]
    fn msa_ftrunc_s_d(a: v2f64) -> v2i64;
    #[link_name = "llvm.mips.ftrunc.u.w"]
    fn msa_ftrunc_u_w(a: v4f32) -> v4u32;
    #[link_name = "llvm.mips.ftrunc.u.d"]
    fn msa_ftrunc_u_d(a: v2f64) -> v2u64;
    #[link_name = "llvm.mips.hadd.s.h"]
    fn msa_hadd_s_h(a: v16i8, b: v16i8) -> v8i16;
    #[link_name = "llvm.mips.hadd.s.w"]
    fn msa_hadd_s_w(a: v8i16, b: v8i16) -> v4i32;
    #[link_name = "llvm.mips.hadd.s.d"]
    fn msa_hadd_s_d(a: v4i32, b: v4i32) -> v2i64;
    #[link_name = "llvm.mips.hadd.u.h"]
    fn msa_hadd_u_h(a: v16u8, b: v16u8) -> v8u16;
    #[link_name = "llvm.mips.hadd.u.w"]
    fn msa_hadd_u_w(a: v8u16, b: v8u16) -> v4u32;
    #[link_name = "llvm.mips.hadd.u.d"]
    fn msa_hadd_u_d(a: v4u32, b: v4u32) -> v2u64;
    #[link_name = "llvm.mips.hsub.s.h"]
    fn msa_hsub_s_h(a: v16i8, b: v16i8) -> v8i16;
    #[link_name = "llvm.mips.hsub.s.w"]
    fn msa_hsub_s_w(a: v8i16, b: v8i16) -> v4i32;
    #[link_name = "llvm.mips.hsub.s.d"]
    fn msa_hsub_s_d(a: v4i32, b: v4i32) -> v2i64;
    #[link_name = "llvm.mips.hsub.u.h"]
    fn msa_hsub_u_h(a: v16u8, b: v16u8) -> v8i16;
    #[link_name = "llvm.mips.hsub.u.w"]
    fn msa_hsub_u_w(a: v8u16, b: v8u16) -> v4i32;
    #[link_name = "llvm.mips.hsub.u.d"]
    fn msa_hsub_u_d(a: v4u32, b: v4u32) -> v2i64;
    #[link_name = "llvm.mips.ilvev.b"]
    fn msa_ilvev_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.ilvev.h"]
    fn msa_ilvev_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.ilvev.w"]
    fn msa_ilvev_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.ilvev.d"]
    fn msa_ilvev_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.ilvl.b"]
    fn msa_ilvl_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.ilvl.h"]
    fn msa_ilvl_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.ilvl.w"]
    fn msa_ilvl_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.ilvl.d"]
    fn msa_ilvl_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.ilvod.b"]
    fn msa_ilvod_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.ilvod.h"]
    fn msa_ilvod_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.ilvod.w"]
    fn msa_ilvod_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.ilvod.d"]
    fn msa_ilvod_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.ilvr.b"]
    fn msa_ilvr_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.ilvr.h"]
    fn msa_ilvr_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.ilvr.w"]
    fn msa_ilvr_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.ilvr.d"]
    fn msa_ilvr_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.insert.b"]
    fn msa_insert_b(a: v16i8, b: i32, c: i32) -> v16i8;
    #[link_name = "llvm.mips.insert.h"]
    fn msa_insert_h(a: v8i16, b: i32, c: i32) -> v8i16;
    #[link_name = "llvm.mips.insert.w"]
    fn msa_insert_w(a: v4i32, b: i32, c: i32) -> v4i32;
    #[link_name = "llvm.mips.insert.d"]
    fn msa_insert_d(a: v2i64, b: i32, c: i64) -> v2i64;
    #[link_name = "llvm.mips.insve.b"]
    fn msa_insve_b(a: v16i8, b: i32, c: v16i8) -> v16i8;
    #[link_name = "llvm.mips.insve.h"]
    fn msa_insve_h(a: v8i16, b: i32, c: v8i16) -> v8i16;
    #[link_name = "llvm.mips.insve.w"]
    fn msa_insve_w(a: v4i32, b: i32, c: v4i32) -> v4i32;
    #[link_name = "llvm.mips.insve.d"]
    fn msa_insve_d(a: v2i64, b: i32, c: v2i64) -> v2i64;
    #[link_name = "llvm.mips.ld.b"]
    fn msa_ld_b(mem_addr: *mut u8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.ld.h"]
    fn msa_ld_h(mem_addr: *mut u8, b: i32) -> v8i16;
    #[link_name = "llvm.mips.ld.w"]
    fn msa_ld_w(mem_addr: *mut u8, b: i32) -> v4i32;
    #[link_name = "llvm.mips.ld.d"]
    fn msa_ld_d(mem_addr: *mut u8, b: i32) -> v2i64;
    #[link_name = "llvm.mips.ldi.b"]
    fn msa_ldi_b(a: i32) -> v16i8;
    #[link_name = "llvm.mips.ldi.h"]
    fn msa_ldi_h(a: i32) -> v8i16;
    #[link_name = "llvm.mips.ldi.w"]
    fn msa_ldi_w(a: i32) -> v4i32;
    #[link_name = "llvm.mips.ldi.d"]
    fn msa_ldi_d(a: i32) -> v2i64;
    #[link_name = "llvm.mips.madd.q.h"]
    fn msa_madd_q_h(a: v8i16, b: v8i16, c: v8i16) -> v8i16;
    #[link_name = "llvm.mips.madd.q.w"]
    fn msa_madd_q_w(a: v4i32, b: v4i32, c: v4i32) -> v4i32;
    #[link_name = "llvm.mips.maddr.q.h"]
    fn msa_maddr_q_h(a: v8i16, b: v8i16, c: v8i16) -> v8i16;
    #[link_name = "llvm.mips.maddr.q.w"]
    fn msa_maddr_q_w(a: v4i32, b: v4i32, c: v4i32) -> v4i32;
    #[link_name = "llvm.mips.maddv.b"]
    fn msa_maddv_b(a: v16i8, b: v16i8, c: v16i8) -> v16i8;
    #[link_name = "llvm.mips.maddv.h"]
    fn msa_maddv_h(a: v8i16, b: v8i16, c: v8i16) -> v8i16;
    #[link_name = "llvm.mips.maddv.w"]
    fn msa_maddv_w(a: v4i32, b: v4i32, c: v4i32) -> v4i32;
    #[link_name = "llvm.mips.maddv.d"]
    fn msa_maddv_d(a: v2i64, b: v2i64, c: v2i64) -> v2i64;
    #[link_name = "llvm.mips.max.a.b"]
    fn msa_max_a_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.max.a.h"]
    fn msa_max_a_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.max.a.w"]
    fn msa_max_a_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.max.a.d"]
    fn msa_max_a_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.max.s.b"]
    fn msa_max_s_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.max.s.h"]
    fn msa_max_s_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.max.s.w"]
    fn msa_max_s_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.max.s.d"]
    fn msa_max_s_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.max.u.b"]
    fn msa_max_u_b(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.max.u.h"]
    fn msa_max_u_h(a: v8u16, b: v8u16) -> v8u16;
    #[link_name = "llvm.mips.max.u.w"]
    fn msa_max_u_w(a: v4u32, b: v4u32) -> v4u32;
    #[link_name = "llvm.mips.max.u.d"]
    fn msa_max_u_d(a: v2u64, b: v2u64) -> v2u64;
    #[link_name = "llvm.mips.maxi.s.b"]
    fn msa_maxi_s_b(a: v16i8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.maxi.s.h"]
    fn msa_maxi_s_h(a: v8i16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.maxi.s.w"]
    fn msa_maxi_s_w(a: v4i32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.maxi.s.d"]
    fn msa_maxi_s_d(a: v2i64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.maxi.u.b"]
    fn msa_maxi_u_b(a: v16u8, b: i32) -> v16u8;
    #[link_name = "llvm.mips.maxi.u.h"]
    fn msa_maxi_u_h(a: v8u16, b: i32) -> v8u16;
    #[link_name = "llvm.mips.maxi.u.w"]
    fn msa_maxi_u_w(a: v4u32, b: i32) -> v4u32;
    #[link_name = "llvm.mips.maxi.u.d"]
    fn msa_maxi_u_d(a: v2u64, b: i32) -> v2u64;
    #[link_name = "llvm.mips.min.a.b"]
    fn msa_min_a_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.min.a.h"]
    fn msa_min_a_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.min.a.w"]
    fn msa_min_a_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.min.a.d"]
    fn msa_min_a_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.min.s.b"]
    fn msa_min_s_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.min.s.h"]
    fn msa_min_s_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.min.s.w"]
    fn msa_min_s_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.min.s.d"]
    fn msa_min_s_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.min.u.b"]
    fn msa_min_u_b(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.min.u.h"]
    fn msa_min_u_h(a: v8u16, b: v8u16) -> v8u16;
    #[link_name = "llvm.mips.min.u.w"]
    fn msa_min_u_w(a: v4u32, b: v4u32) -> v4u32;
    #[link_name = "llvm.mips.min.u.d"]
    fn msa_min_u_d(a: v2u64, b: v2u64) -> v2u64;
    #[link_name = "llvm.mips.mini.s.b"]
    fn msa_mini_s_b(a: v16i8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.mini.s.h"]
    fn msa_mini_s_h(a: v8i16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.mini.s.w"]
    fn msa_mini_s_w(a: v4i32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.mini.s.d"]
    fn msa_mini_s_d(a: v2i64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.mini.u.b"]
    fn msa_mini_u_b(a: v16u8, b: i32) -> v16u8;
    #[link_name = "llvm.mips.mini.u.h"]
    fn msa_mini_u_h(a: v8u16, b: i32) -> v8u16;
    #[link_name = "llvm.mips.mini.u.w"]
    fn msa_mini_u_w(a: v4u32, b: i32) -> v4u32;
    #[link_name = "llvm.mips.mini.u.d"]
    fn msa_mini_u_d(a: v2u64, b: i32) -> v2u64;
    #[link_name = "llvm.mips.mod.s.b"]
    fn msa_mod_s_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.mod.s.h"]
    fn msa_mod_s_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.mod.s.w"]
    fn msa_mod_s_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.mod.s.d"]
    fn msa_mod_s_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.mod.u.b"]
    fn msa_mod_u_b(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.mod.u.h"]
    fn msa_mod_u_h(a: v8u16, b: v8u16) -> v8u16;
    #[link_name = "llvm.mips.mod.u.w"]
    fn msa_mod_u_w(a: v4u32, b: v4u32) -> v4u32;
    #[link_name = "llvm.mips.mod.u.d"]
    fn msa_mod_u_d(a: v2u64, b: v2u64) -> v2u64;
    #[link_name = "llvm.mips.move.v"]
    fn msa_move_v(a: v16i8) -> v16i8;
    #[link_name = "llvm.mips.msub.q.h"]
    fn msa_msub_q_h(a: v8i16, b: v8i16, c: v8i16) -> v8i16;
    #[link_name = "llvm.mips.msub.q.w"]
    fn msa_msub_q_w(a: v4i32, b: v4i32, c: v4i32) -> v4i32;
    #[link_name = "llvm.mips.msubr.q.h"]
    fn msa_msubr_q_h(a: v8i16, b: v8i16, c: v8i16) -> v8i16;
    #[link_name = "llvm.mips.msubr.q.w"]
    fn msa_msubr_q_w(a: v4i32, b: v4i32, c: v4i32) -> v4i32;
    #[link_name = "llvm.mips.msubv.b"]
    fn msa_msubv_b(a: v16i8, b: v16i8, c: v16i8) -> v16i8;
    #[link_name = "llvm.mips.msubv.h"]
    fn msa_msubv_h(a: v8i16, b: v8i16, c: v8i16) -> v8i16;
    #[link_name = "llvm.mips.msubv.w"]
    fn msa_msubv_w(a: v4i32, b: v4i32, c: v4i32) -> v4i32;
    #[link_name = "llvm.mips.msubv.d"]
    fn msa_msubv_d(a: v2i64, b: v2i64, c: v2i64) -> v2i64;
    #[link_name = "llvm.mips.mul.q.h"]
    fn msa_mul_q_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.mul.q.w"]
    fn msa_mul_q_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.mulr.q.h"]
    fn msa_mulr_q_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.mulr.q.w"]
    fn msa_mulr_q_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.mulv.b"]
    fn msa_mulv_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.mulv.h"]
    fn msa_mulv_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.mulv.w"]
    fn msa_mulv_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.mulv.d"]
    fn msa_mulv_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.nloc.b"]
    fn msa_nloc_b(a: v16i8) -> v16i8;
    #[link_name = "llvm.mips.nloc.h"]
    fn msa_nloc_h(a: v8i16) -> v8i16;
    #[link_name = "llvm.mips.nloc.w"]
    fn msa_nloc_w(a: v4i32) -> v4i32;
    #[link_name = "llvm.mips.nloc.d"]
    fn msa_nloc_d(a: v2i64) -> v2i64;
    #[link_name = "llvm.mips.nlzc.b"]
    fn msa_nlzc_b(a: v16i8) -> v16i8;
    #[link_name = "llvm.mips.nlzc.h"]
    fn msa_nlzc_h(a: v8i16) -> v8i16;
    #[link_name = "llvm.mips.nlzc.w"]
    fn msa_nlzc_w(a: v4i32) -> v4i32;
    #[link_name = "llvm.mips.nlzc.d"]
    fn msa_nlzc_d(a: v2i64) -> v2i64;
    #[link_name = "llvm.mips.nor.v"]
    fn msa_nor_v(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.nori.b"]
    fn msa_nori_b(a: v16u8, b: i32) -> v16u8;
    #[link_name = "llvm.mips.or.v"]
    fn msa_or_v(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.ori.b"]
    fn msa_ori_b(a: v16u8, b: i32) -> v16u8;
    #[link_name = "llvm.mips.pckev.b"]
    fn msa_pckev_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.pckev.h"]
    fn msa_pckev_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.pckev.w"]
    fn msa_pckev_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.pckev.d"]
    fn msa_pckev_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.pckod.b"]
    fn msa_pckod_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.pckod.h"]
    fn msa_pckod_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.pckod.w"]
    fn msa_pckod_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.pckod.d"]
    fn msa_pckod_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.pcnt.b"]
    fn msa_pcnt_b(a: v16i8) -> v16i8;
    #[link_name = "llvm.mips.pcnt.h"]
    fn msa_pcnt_h(a: v8i16) -> v8i16;
    #[link_name = "llvm.mips.pcnt.w"]
    fn msa_pcnt_w(a: v4i32) -> v4i32;
    #[link_name = "llvm.mips.pcnt.d"]
    fn msa_pcnt_d(a: v2i64) -> v2i64;
    #[link_name = "llvm.mips.sat.s.b"]
    fn msa_sat_s_b(a: v16i8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.sat.s.h"]
    fn msa_sat_s_h(a: v8i16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.sat.s.w"]
    fn msa_sat_s_w(a: v4i32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.sat.s.d"]
    fn msa_sat_s_d(a: v2i64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.sat.u.b"]
    fn msa_sat_u_b(a: v16u8, b: i32) -> v16u8;
    #[link_name = "llvm.mips.sat.u.h"]
    fn msa_sat_u_h(a: v8u16, b: i32) -> v8u16;
    #[link_name = "llvm.mips.sat.u.w"]
    fn msa_sat_u_w(a: v4u32, b: i32) -> v4u32;
    #[link_name = "llvm.mips.sat.u.d"]
    fn msa_sat_u_d(a: v2u64, b: i32) -> v2u64;
    #[link_name = "llvm.mips.shf.b"]
    fn msa_shf_b(a: v16i8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.shf.h"]
    fn msa_shf_h(a: v8i16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.shf.w"]
    fn msa_shf_w(a: v4i32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.sld.b"]
    fn msa_sld_b(a: v16i8, b: v16i8, c: i32) -> v16i8;
    #[link_name = "llvm.mips.sld.h"]
    fn msa_sld_h(a: v8i16, b: v8i16, c: i32) -> v8i16;
    #[link_name = "llvm.mips.sld.w"]
    fn msa_sld_w(a: v4i32, b: v4i32, c: i32) -> v4i32;
    #[link_name = "llvm.mips.sld.d"]
    fn msa_sld_d(a: v2i64, b: v2i64, c: i32) -> v2i64;
    #[link_name = "llvm.mips.sldi.b"]
    fn msa_sldi_b(a: v16i8, b: v16i8, c: i32) -> v16i8;
    #[link_name = "llvm.mips.sldi.h"]
    fn msa_sldi_h(a: v8i16, b: v8i16, c: i32) -> v8i16;
    #[link_name = "llvm.mips.sldi.w"]
    fn msa_sldi_w(a: v4i32, b: v4i32, c: i32) -> v4i32;
    #[link_name = "llvm.mips.sldi.d"]
    fn msa_sldi_d(a: v2i64, b: v2i64, c: i32) -> v2i64;
    #[link_name = "llvm.mips.sll.b"]
    fn msa_sll_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.sll.h"]
    fn msa_sll_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.sll.w"]
    fn msa_sll_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.sll.d"]
    fn msa_sll_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.slli.b"]
    fn msa_slli_b(a: v16i8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.slli.h"]
    fn msa_slli_h(a: v8i16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.slli.w"]
    fn msa_slli_w(a: v4i32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.slli.d"]
    fn msa_slli_d(a: v2i64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.splat.b"]
    fn msa_splat_b(a: v16i8, c: i32) -> v16i8;
    #[link_name = "llvm.mips.splat.h"]
    fn msa_splat_h(a: v8i16, c: i32) -> v8i16;
    #[link_name = "llvm.mips.splat.w"]
    fn msa_splat_w(a: v4i32, w: i32) -> v4i32;
    #[link_name = "llvm.mips.splat.d"]
    fn msa_splat_d(a: v2i64, c: i32) -> v2i64;
    #[link_name = "llvm.mips.splati.b"]
    fn msa_splati_b(a: v16i8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.splati.h"]
    fn msa_splati_h(a: v8i16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.splati.w"]
    fn msa_splati_w(a: v4i32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.splati.d"]
    fn msa_splati_d(a: v2i64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.sra.b"]
    fn msa_sra_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.sra.h"]
    fn msa_sra_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.sra.w"]
    fn msa_sra_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.sra.d"]
    fn msa_sra_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.srai.b"]
    fn msa_srai_b(a: v16i8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.srai.h"]
    fn msa_srai_h(a: v8i16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.srai.w"]
    fn msa_srai_w(a: v4i32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.srai.d"]
    fn msa_srai_d(a: v2i64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.srar.b"]
    fn msa_srar_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.srar.h"]
    fn msa_srar_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.srar.w"]
    fn msa_srar_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.srar.d"]
    fn msa_srar_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.srari.b"]
    fn msa_srari_b(a: v16i8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.srari.h"]
    fn msa_srari_h(a: v8i16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.srari.w"]
    fn msa_srari_w(a: v4i32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.srari.d"]
    fn msa_srari_d(a: v2i64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.srl.b"]
    fn msa_srl_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.srl.h"]
    fn msa_srl_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.srl.w"]
    fn msa_srl_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.srl.d"]
    fn msa_srl_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.srli.b"]
    fn msa_srli_b(a: v16i8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.srli.h"]
    fn msa_srli_h(a: v8i16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.srli.w"]
    fn msa_srli_w(a: v4i32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.srli.d"]
    fn msa_srli_d(a: v2i64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.srlr.b"]
    fn msa_srlr_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.srlr.h"]
    fn msa_srlr_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.srlr.w"]
    fn msa_srlr_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.srlr.d"]
    fn msa_srlr_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.srlri.b"]
    fn msa_srlri_b(a: v16i8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.srlri.h"]
    fn msa_srlri_h(a: v8i16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.srlri.w"]
    fn msa_srlri_w(a: v4i32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.srlri.d"]
    fn msa_srlri_d(a: v2i64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.st.b"]
    fn msa_st_b(a: v16i8, mem_addr: *mut u8, imm_s10: i32) -> ();
    #[link_name = "llvm.mips.st.h"]
    fn msa_st_h(a: v8i16, mem_addr: *mut u8, imm_s11: i32) -> ();
    #[link_name = "llvm.mips.st.w"]
    fn msa_st_w(a: v4i32, mem_addr: *mut u8, imm_s12: i32) -> ();
    #[link_name = "llvm.mips.st.d"]
    fn msa_st_d(a: v2i64, mem_addr: *mut u8, imm_s13: i32) -> ();
    #[link_name = "llvm.mips.subs.s.b"]
    fn msa_subs_s_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.subs.s.h"]
    fn msa_subs_s_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.subs.s.w"]
    fn msa_subs_s_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.subs.s.d"]
    fn msa_subs_s_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.subs.u.b"]
    fn msa_subs_u_b(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.subs.u.h"]
    fn msa_subs_u_h(a: v8u16, b: v8u16) -> v8u16;
    #[link_name = "llvm.mips.subs.u.w"]
    fn msa_subs_u_w(a: v4u32, b: v4u32) -> v4u32;
    #[link_name = "llvm.mips.subs.u.d"]
    fn msa_subs_u_d(a: v2u64, b: v2u64) -> v2u64;
    #[link_name = "llvm.mips.subsus.u.b"]
    fn msa_subsus_u_b(a: v16u8, b: v16i8) -> v16u8;
    #[link_name = "llvm.mips.subsus.u.h"]
    fn msa_subsus_u_h(a: v8u16, b: v8i16) -> v8u16;
    #[link_name = "llvm.mips.subsus.u.w"]
    fn msa_subsus_u_w(a: v4u32, b: v4i32) -> v4u32;
    #[link_name = "llvm.mips.subsus.u.d"]
    fn msa_subsus_u_d(a: v2u64, b: v2i64) -> v2u64;
    #[link_name = "llvm.mips.subsuu.s.b"]
    fn msa_subsuu_s_b(a: v16u8, b: v16u8) -> v16i8;
    #[link_name = "llvm.mips.subsuu.s.h"]
    fn msa_subsuu_s_h(a: v8u16, b: v8u16) -> v8i16;
    #[link_name = "llvm.mips.subsuu.s.w"]
    fn msa_subsuu_s_w(a: v4u32, b: v4u32) -> v4i32;
    #[link_name = "llvm.mips.subsuu.s.d"]
    fn msa_subsuu_s_d(a: v2u64, b: v2u64) -> v2i64;
    #[link_name = "llvm.mips.subv.b"]
    fn msa_subv_b(a: v16i8, b: v16i8) -> v16i8;
    #[link_name = "llvm.mips.subv.h"]
    fn msa_subv_h(a: v8i16, b: v8i16) -> v8i16;
    #[link_name = "llvm.mips.subv.w"]
    fn msa_subv_w(a: v4i32, b: v4i32) -> v4i32;
    #[link_name = "llvm.mips.subv.d"]
    fn msa_subv_d(a: v2i64, b: v2i64) -> v2i64;
    #[link_name = "llvm.mips.subvi.b"]
    fn msa_subvi_b(a: v16i8, b: i32) -> v16i8;
    #[link_name = "llvm.mips.subvi.h"]
    fn msa_subvi_h(a: v8i16, b: i32) -> v8i16;
    #[link_name = "llvm.mips.subvi.w"]
    fn msa_subvi_w(a: v4i32, b: i32) -> v4i32;
    #[link_name = "llvm.mips.subvi.d"]
    fn msa_subvi_d(a: v2i64, b: i32) -> v2i64;
    #[link_name = "llvm.mips.vshf.b"]
    fn msa_vshf_b(a: v16i8, b: v16i8, c: v16i8) -> v16i8;
    #[link_name = "llvm.mips.vshf.h"]
    fn msa_vshf_h(a: v8i16, b: v8i16, c: v8i16) -> v8i16;
    #[link_name = "llvm.mips.vshf.w"]
    fn msa_vshf_w(a: v4i32, b: v4i32, c: v4i32) -> v4i32;
    #[link_name = "llvm.mips.vshf.d"]
    fn msa_vshf_d(a: v2i64, b: v2i64, c: v2i64) -> v2i64;
    #[link_name = "llvm.mips.xor.v"]
    fn msa_xor_v(a: v16u8, b: v16u8) -> v16u8;
    #[link_name = "llvm.mips.xori.b"]
    fn msa_xori_b(a: v16u8, b: i32) -> v16u8;
}

/// Vector 添加绝对值。
///
/// `a` 中的 vector 中的元素的绝对值 (十六个带符号的 8 位整数) 与 vector `b` 中的元素绝对值 (十六个带符号的 8 位整数) 相加。
///
/// 结果写入 vector (十六个带符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(add_a.b))]
pub unsafe fn __msa_add_a_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_add_a_b(a, mem::transmute(b))
}

/// Vector 添加绝对值
///
/// `a` 中的 vector 中的元素的绝对值 (八个带符号的 16 位整数) 与 vector `b` 中的元素的绝对值 (八个带符号的 16 位整数) 相加。
///
/// 结果写入 vector (八个带符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(add_a.h))]
pub unsafe fn __msa_add_a_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_add_a_h(a, mem::transmute(b))
}

/// Vector 添加绝对值
///
/// `a` 中的 vector 中的元素的绝对值 (四个有符号的 32 位整数) 与 vector `b` 中的元素绝对值 (四个有符号的 32 位整数) 相加。
///
/// 结果写入 vector (四个带符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(add_a.w))]
pub unsafe fn __msa_add_a_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_add_a_w(a, mem::transmute(b))
}

/// Vector 添加绝对值
///
/// `a` 中的 vector 中的元素的绝对值 (两个有符号的 64 位整数) 与 vector `b` 中的元素绝对值 (两个有符号的 64 位整数) 相加。
///
/// 结果写入 vector (两个带符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(add_a.d))]
pub unsafe fn __msa_add_a_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_add_a_d(a, mem::transmute(b))
}

/// 带符号的饱和 Vector 饱和的绝对值相加
///
/// `a` 中的 vector 中的元素的绝对值 (十六个带符号的 8 位整数) 与 vector `b` 中的元素绝对值 (十六个带符号的 8 位整数) 相加。
///
/// 饱和的带符号结果被写入 vector (十六个带符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(adds_a.b))]
pub unsafe fn __msa_adds_a_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_adds_a_b(a, mem::transmute(b))
}

/// Vector 绝对值的饱和加法
///
/// `a` 中的 vector 中的元素的绝对值 (八个带符号的 16 位整数) 与 vector `b` 中的元素的绝对值 (八个带符号的 16 位整数) 相加。
///
/// 饱和的有符号结果被写入 vector (八个有符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(adds_a.h))]
pub unsafe fn __msa_adds_a_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_adds_a_h(a, mem::transmute(b))
}

/// Vector 绝对值的饱和加法
///
/// `a` 中的 vector 中的元素的绝对值 (四个有符号的 32 位整数) 与 vector `b` 中的元素绝对值 (四个有符号的 32 位整数) 相加。
///
/// 饱和的有符号结果被写入 vector (四个有符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(adds_a.w))]
pub unsafe fn __msa_adds_a_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_adds_a_w(a, mem::transmute(b))
}

/// Vector 绝对值的饱和加法
///
/// `a` 中的 vector 中的元素的绝对值 (两个有符号的 64 位整数) 与 vector `b` 中的元素绝对值 (两个有符号的 64 位整数) 相加。
///
/// 饱和的有符号结果被写入 vector (两个有符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(adds_a.d))]
pub unsafe fn __msa_adds_a_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_adds_a_d(a, mem::transmute(b))
}

/// Vector 带符号的饱和值的加法
///
/// `a` 中的 vector 中的元素 (十六个带符号的 8 位整数) 被添加到 vector `b` 中的元素 (十六个带符号的 8 位整数)。
/// 在将结果写入 vector (十六个带符号的 8 位整数) 之前，将执行带符号的算术运算并将溢出钳位到最大或者最小的可表示符号值。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(adds_s.b))]
pub unsafe fn __msa_adds_s_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_adds_s_b(a, mem::transmute(b))
}

/// Vector 带符号的饱和值的加法
///
/// 将 `a` 中的 vector 中的元素 (八个带符号的 16 位整数) 添加到 vector `b` 中的元素 (八个带符号的 16 位整数)。
/// 在将结果写入 vector (八个有符号的 16 位整数) 之前，将执行有符号算术，并且溢出将钳位到最大或者最小可表示的有符号值。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(adds_s.h))]
pub unsafe fn __msa_adds_s_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_adds_s_h(a, mem::transmute(b))
}

/// Vector 带符号的饱和值的加法
///
/// `a` 中的 vector 中的元素 (四个有符号的 32 位整数) 被添加到 vector `b` 中的元素 (四个有符号的 32 位整数)。
/// 在将结果写入 vector (四个有符号的 32 位整数) 之前，将执行有符号算术，并且溢出将钳位到最大或者最小可表示的有符号值。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(adds_s.w))]
pub unsafe fn __msa_adds_s_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_adds_s_w(a, mem::transmute(b))
}

/// Vector 带符号的饱和值的加法
///
/// `a` 中的 vector 中的元素 (两个有符号的 64 位整数) 被添加到 vector `b` 中的元素 (两个有符号的 64 位整数)。
/// 在将结果写入 vector (两个有符号的 64 位整数) 之前，将执行有符号算术，并且溢出将钳位到最大或者最小可表示的有符号值。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(adds_s.d))]
pub unsafe fn __msa_adds_s_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_adds_s_d(a, mem::transmute(b))
}

/// Vector 无符号饱和值的无符号相加
///
/// `a` 中的 vector 中的元素 (十六个无符号的 8 位整数) 被添加到 vector `b` 中的元素 (十六个无符号的 8 位整数)。
/// 在将结果写入 vector (十六个无符号的 8 位整数) 之前，将执行有符号算术，并且溢出将钳位到最大或者最小可表示的有符号值。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(adds_u.b))]
pub unsafe fn __msa_adds_u_b(a: v16u8, b: v16u8) -> v16u8 {
    msa_adds_u_b(a, mem::transmute(b))
}

/// Vector 无符号饱和值的无符号相加
///
/// `a` 中的 vector 中的元素 (八个无符号 16 位整数) 被添加到 vector `b` 中的元素 (八个无符号 16 位整数)。
/// 在将结果写入 vector (八个无符号的 16 位整数) 之前，将执行有符号算术，并且溢出将钳位到最大或者最小可表示的有符号值。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(adds_u.h))]
pub unsafe fn __msa_adds_u_h(a: v8u16, b: v8u16) -> v8u16 {
    msa_adds_u_h(a, mem::transmute(b))
}

/// Vector 无符号饱和值的无符号相加
///
/// `a` 中的 vector 中的元素 (四个无符号 32 位整数) 被添加到 vector `b` 中的元素 (四个无符号 32 位整数)。
/// 在将结果写入 vector (四个无符号的 32 位整数) 之前，将执行有符号算术，并且溢出将钳位到最大或者最小可表示的有符号值。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(adds_u.w))]
pub unsafe fn __msa_adds_u_w(a: v4u32, b: v4u32) -> v4u32 {
    msa_adds_u_w(a, mem::transmute(b))
}

/// Vector 无符号饱和值的无符号相加
///
/// `a` 中的 vector 中的元素 (两个无符号的 64 位整数) 被添加到 vector `b` 中的元素 (两个无符号的 64 位整数)。
/// 在将结果写入 vector (两个无符号的 64 位整数) 之前，将执行有符号算术，并且溢出将钳位到最大或者最小可表示的有符号值。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(adds_u.d))]
pub unsafe fn __msa_adds_u_d(a: v2u64, b: v2u64) -> v2u64 {
    msa_adds_u_d(a, mem::transmute(b))
}

/// Vector 添加
///
/// `a` 中的 vector 中的元素 (十六个带符号的 8 位整数) 被添加到 vector `b` 中的元素 (十六个带符号的 8 位整数)。
///
/// 结果写入 vector (十六个带符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(addv.b))]
pub unsafe fn __msa_addv_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_addv_b(a, mem::transmute(b))
}

/// Vector 添加
///
/// 将 `a` 中的 vector 中的元素 (八个带符号的 16 位整数) 添加到 vector `b` 中的元素 (八个带符号的 16 位整数)。
///
/// 结果写入 vector (八个带符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(addv.h))]
pub unsafe fn __msa_addv_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_addv_h(a, mem::transmute(b))
}

/// Vector 添加
///
/// `a` 中的 vector 中的元素 (四个有符号的 32 位整数) 被添加到 vector `b` 中的元素 (四个有符号的 32 位整数)。
///
/// 结果写入 vector (四个带符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(addv.w))]
pub unsafe fn __msa_addv_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_addv_w(a, mem::transmute(b))
}

/// Vector 添加
///
/// `a` 中的 vector 中的元素 (两个有符号的 64 位整数) 被添加到 vector `b` 中的元素 (两个有符号的 64 位整数)。
///
/// 结果写入 vector (两个带符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(addv.d))]
pub unsafe fn __msa_addv_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_addv_d(a, mem::transmute(b))
}

/// Immediate 添加
///
/// 5 位 immediate 数无符号值 `imm5` 被添加到 `a` 中的元素 vector (十六个带符号的 8 位整数)。
///
/// 结果写入 vector (十六个带符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(addvi.b, imm5 = 0b10111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_addvi_b<const IMM5: i32>(a: v16i8) -> v16i8 {
    static_assert_imm5!(IMM5);
    msa_addvi_b(a, IMM5)
}

/// Immediate 添加
///
/// 5 位 immediate 数无符号值 `imm5` 被添加到 `a` 中的元素 vector (八个有符号的 16 位整数)。
///
/// 结果写入 vector (八个带符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(addvi.h, imm5 = 0b10111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_addvi_h<const IMM5: i32>(a: v8i16) -> v8i16 {
    static_assert_imm5!(IMM5);
    msa_addvi_h(a, IMM5)
}

/// Immediate 添加
///
/// 5 位 immediate 数无符号值 `imm5` 被添加到 `a` 中的元素 vector (四个有符号的 32 位整数)。
///
/// 结果写入 vector (四个带符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(addvi.w, imm5 = 0b10111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_addvi_w<const IMM5: i32>(a: v4i32) -> v4i32 {
    static_assert_imm5!(IMM5);
    msa_addvi_w(a, IMM5)
}

/// Immediate 添加
///
/// 5 位 immediate 数无符号值 `imm5` 被添加到 `a` 中的元素 vector (两个有符号的 64 位整数)。
///
/// 结果写入 vector (两个带符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(addvi.d, imm5 = 0b10111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_addvi_d<const IMM5: i32>(a: v2i64) -> v2i64 {
    static_assert_imm5!(IMM5);
    msa_addvi_d(a, IMM5)
}

/// Vector 逻辑与
///
/// vector `a` 的每个位 (十六个无符号的 8 位整数) 与 vector `b` 的对应位 (十六个无符号的 8 位整数) 按位逻辑与运算进行组合。
///
/// 结果写入 vector (十六个无符号的 8 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(and.v))]
pub unsafe fn __msa_and_v(a: v16u8, b: v16u8) -> v16u8 {
    msa_and_v(a, mem::transmute(b))
}

/// Immediate 逻辑与
///
/// vector `a` 的每个字节元素 (十六个无符号的 8 位整数) 与 8 位 immediate 数 i8 (有符号的 8 位整数) 通过按位逻辑 AND 运算进行组合。
///
/// 结果写入 vector (十六个无符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(andi.b, imm8 = 0b10010111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_andi_b<const IMM8: i32>(a: v16u8) -> v16u8 {
    static_assert_imm8!(IMM8);
    msa_andi_b(a, IMM8)
}

/// Vector 有符号减法的绝对值
///
/// 从 vector `b` (十六个带符号的 8 位整数) 中减去有符号元素 vector `a` (十六个带符号的 8 位整数)。
///
/// 有符号结果的绝对值写入 vector (十六个带符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(asub_s.b))]
pub unsafe fn __msa_asub_s_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_asub_s_b(a, mem::transmute(b))
}

/// Vector 有符号减法的绝对值
///
/// 从 vector `b` (八个带符号的 16 位整数) 中减去有符号元素 vector `a` (八个带符号的 16 位整数)。
///
/// 有符号结果的绝对值写入 vector (八个有符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(asub_s.h))]
pub unsafe fn __msa_asub_s_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_asub_s_h(a, mem::transmute(b))
}

/// Vector 有符号减法的绝对值
///
/// vector `a` (四个有符号的 32 位整数) 中的有符号元素被从 vector `b` (四个有符号的 32 位整数) 中的有符号元素中减去。
///
/// 有符号结果的绝对值写入 vector (四个有符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(asub_s.w))]
pub unsafe fn __msa_asub_s_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_asub_s_w(a, mem::transmute(b))
}

/// Vector 有符号减法的绝对值
///
/// 从 vector `b` (两个带符号的 64 位整数) 中的有符号元素减去 vector `a` (两个带符号的 64 位整数) 中的有符号元素。
///
/// 有符号结果的绝对值写入 vector (两个有符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(asub_s.d))]
pub unsafe fn __msa_asub_s_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_asub_s_d(a, mem::transmute(b))
}

/// Vector 无符号减法的绝对值
///
/// 从 vector `b` (十六个无符号 8 位整数) 中减去 vector `a` 中的无符号元素 (十六个无符号 8 位整数)。
///
/// 无符号结果的绝对值写入 vector (十六个无符号 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(asub_u.b))]
pub unsafe fn __msa_asub_u_b(a: v16u8, b: v16u8) -> v16u8 {
    msa_asub_u_b(a, mem::transmute(b))
}

/// Vector 无符号减法的绝对值
///
/// vector `a` 中的无符号元素 (八个无符号 16 位整数) 从 vector `b` 中的无符号元素 (八个无符号 16 位整数) 中减去。
///
/// 无符号结果的绝对值写入 vector (八个无符号 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(asub_u.h))]
pub unsafe fn __msa_asub_u_h(a: v8u16, b: v8u16) -> v8u16 {
    msa_asub_u_h(a, mem::transmute(b))
}

/// Vector 无符号减法的绝对值
///
/// vector `a` 中的无符号元素 (四个无符号 32 位整数) 从 vector `b` 中的无符号元素 (四个无符号 32 位整数) 中减去。
///
/// 无符号结果的绝对值写入 vector (四个无符号 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(asub_u.w))]
pub unsafe fn __msa_asub_u_w(a: v4u32, b: v4u32) -> v4u32 {
    msa_asub_u_w(a, mem::transmute(b))
}

/// Vector 无符号减法的绝对值
///
/// vector `a` 中的无符号元素 (两个无符号 64 位整数) 从 vector `b` 中的无符号元素 (两个无符号 64 位整数) 中减去。
///
/// 无符号结果的绝对值写入 vector (两个无符号 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(asub_u.d))]
pub unsafe fn __msa_asub_u_d(a: v2u64, b: v2u64) -> v2u64 {
    msa_asub_u_d(a, mem::transmute(b))
}

/// Vector 有符号平均值
///
/// vector `a` 中的元素 (十六个带符号的 8 位整数) 被添加到 vector `b` 中的元素 (十六个带符号的 8 位整数)。
/// 加法以完全精确的符号完成，即结果有一个额外的位。
/// 在将结果写入 vector (十六个带符号的 8 位整数) 之前，先执行有符号除以 2 (或算术右移一位) 的操作。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ave_s.b))]
pub unsafe fn __msa_ave_s_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_ave_s_b(a, mem::transmute(b))
}

/// Vector 有符号平均值
///
/// 将 vector `a` (八个带符号的 16 位整数) 中的元素添加到 vector `b` (八个带符号的 16 位整数中)。
/// 加法以完全精确的符号完成，即结果有一个额外的位。
/// 在将结果写入 vector (八个带符号的 16 位整数) 之前，先执行有符号除以 2 (或算术右移一位) 的操作。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ave_s.h))]
pub unsafe fn __msa_ave_s_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_ave_s_h(a, mem::transmute(b))
}

/// Vector 有符号平均值
///
/// vector `a` (四个带符号的 32 位整数) 中的元素被添加到 vector `b` (四个带符号的 32 位整数中) 的元素。
/// 加法以完全精确的符号完成，即结果有一个额外的位。
/// 在将结果写入 vector (四个有符号的 32 位整数) 之前，先执行有符号除以 2 (或算术右移一位) 的操作。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ave_s.w))]
pub unsafe fn __msa_ave_s_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_ave_s_w(a, mem::transmute(b))
}

/// Vector 有符号平均值
///
/// vector `a` (两个带符号的 64 位整数) 中的元素被添加到 vector `b` (两个带符号的 64 位整数中) 的元素。
/// 加法以完全精确的符号完成，即结果有一个额外的位。
/// 在将结果写入 vector (两个带符号的 64 位整数) 之前，先执行有符号除以 2 (或算术右移一位) 的操作。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ave_s.d))]
pub unsafe fn __msa_ave_s_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_ave_s_d(a, mem::transmute(b))
}

/// Vector 无符号平均值
///
/// vector `a` 中的元素 (十六个无符号的 8 位整数) 被添加到 vector `b` 中的元素 (十六个无符号的 8 位整数)。
///
/// 加法是完全无符号的，即结果有一个额外的位。
/// 在将结果写入 vector (十六个无符号的 8 位整数) 之前，将执行无符号除以 2 (或逻辑右移一位) 的操作。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ave_u.b))]
pub unsafe fn __msa_ave_u_b(a: v16u8, b: v16u8) -> v16u8 {
    msa_ave_u_b(a, mem::transmute(b))
}

/// Vector 无符号平均值
///
/// vector `a` (八个无符号 16 位整数) 中的元素被添加到 vector `b` (八个无符号 16 位整数中) 的元素。
/// 加法是完全无符号的，即结果有一个额外的位。
/// 在将结果写入 vector (八个无符号的 16 位整数) 之前，将执行无符号除以 2 (或逻辑右移一位) 的操作。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ave_u.h))]
pub unsafe fn __msa_ave_u_h(a: v8u16, b: v8u16) -> v8u16 {
    msa_ave_u_h(a, mem::transmute(b))
}

/// Vector 无符号平均值
///
/// vector `a` (四个无符号的 32 位整数) 中的元素被添加到 vector `b` (四个无符号的 32 位整数) 中的元素。
/// 加法是完全无符号的，即结果有一个额外的位。
/// 在将结果写入 vector (四个无符号的 32 位整数) 之前，将执行无符号除以 2 (或逻辑右移一位) 的操作。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ave_u.w))]
pub unsafe fn __msa_ave_u_w(a: v4u32, b: v4u32) -> v4u32 {
    msa_ave_u_w(a, mem::transmute(b))
}

/// Vector 无符号平均值
///
/// vector `a` (两个无符号的 64 位整数) 中的元素被添加到 vector `b` (两个无符号的 64 位整数) 中的元素。
/// 加法是完全无符号的，即结果有一个额外的位。
/// 在将结果写入 vector (两个无符号的 64 位整数) 之前，将执行无符号除以 2 (或逻辑右移一位) 的操作。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ave_u.d))]
pub unsafe fn __msa_ave_u_d(a: v2u64, b: v2u64) -> v2u64 {
    msa_ave_u_d(a, mem::transmute(b))
}

/// Vector 有符号平均四舍五入
///
/// vector `a` 中的元素 (十六个带符号的 8 位整数) 被添加到 vector `b` 中的元素 (十六个带符号的 8 位整数)。
/// 元素加 1 (用于四舍五入) 的加法以完全精确的符号进行，即结果有一个额外的位。
/// 在将结果写入 vector (十六个带符号的 8 位整数) 之前，先执行有符号除以 2 (或算术右移一位) 的操作。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(aver_s.b))]
pub unsafe fn __msa_aver_s_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_aver_s_b(a, mem::transmute(b))
}

/// Vector 有符号平均四舍五入
///
/// 将 vector `a` (八个带符号的 16 位整数) 中的元素添加到 vector `b` (八个带符号的 16 位整数中)。
/// 元素加 1 (用于四舍五入) 的加法以完全精确的符号进行，即结果有一个额外的位。
/// 在将结果写入 vector (八个带符号的 16 位整数) 之前，先执行有符号除以 2 (或算术右移一位) 的操作。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(aver_s.h))]
pub unsafe fn __msa_aver_s_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_aver_s_h(a, mem::transmute(b))
}

/// Vector 有符号平均四舍五入
///
/// vector `a` (四个带符号的 32 位整数) 中的元素被添加到 vector `b` (四个带符号的 32 位整数中) 的元素。
/// 元素加 1 (用于四舍五入) 的加法以完全精确的符号进行，即结果有一个额外的位。
/// 在将结果写入 vector (四个有符号的 32 位整数) 之前，先执行有符号除以 2 (或算术右移一位) 的操作。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(aver_s.w))]
pub unsafe fn __msa_aver_s_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_aver_s_w(a, mem::transmute(b))
}

/// Vector 有符号平均四舍五入
///
/// vector `a` (两个带符号的 64 位整数) 中的元素被添加到 vector `b` (两个带符号的 64 位整数中) 的元素。
/// 元素加 1 (用于四舍五入) 的加法以完全精确的符号进行，即结果有一个额外的位。
/// 在将结果写入 vector (两个带符号的 64 位整数) 之前，先执行有符号除以 2 (或算术右移一位) 的操作。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(aver_s.d))]
pub unsafe fn __msa_aver_s_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_aver_s_d(a, mem::transmute(b))
}

/// Vector 无符号平均四舍五入
///
/// vector `a` 中的元素 (十六个无符号的 8 位整数) 被添加到 vector `b` 中的元素 (十六个无符号的 8 位整数)。
///
/// 元素加 1 (用于四舍五入) 的加法是完全无符号的，即结果有一个额外的位。
/// 在将结果写入 vector (十六个无符号的 8 位整数) 之前，将执行无符号除以 2 (或逻辑右移一位) 的操作。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(aver_u.b))]
pub unsafe fn __msa_aver_u_b(a: v16u8, b: v16u8) -> v16u8 {
    msa_aver_u_b(a, mem::transmute(b))
}

/// Vector 无符号平均四舍五入
///
/// vector `a` (八个无符号 16 位整数) 中的元素被添加到 vector `b` (八个无符号 16 位整数中) 的元素。
/// 元素加 1 (用于四舍五入) 的加法是完全无符号的，即结果有一个额外的位。
/// 在将结果写入 vector (八个无符号的 16 位整数) 之前，将执行无符号除以 2 (或逻辑右移一位) 的操作。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(aver_u.h))]
pub unsafe fn __msa_aver_u_h(a: v8u16, b: v8u16) -> v8u16 {
    msa_aver_u_h(a, mem::transmute(b))
}

/// Vector 无符号平均四舍五入
///
/// vector `a` (四个无符号的 32 位整数) 中的元素被添加到 vector `b` (四个无符号的 32 位整数) 中的元素。
/// 元素加 1 (用于四舍五入) 的加法是完全无符号的，即结果有一个额外的位。
/// 在将结果写入 vector (四个无符号的 32 位整数) 之前，将执行无符号除以 2 (或逻辑右移一位) 的操作。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(aver_u.w))]
pub unsafe fn __msa_aver_u_w(a: v4u32, b: v4u32) -> v4u32 {
    msa_aver_u_w(a, mem::transmute(b))
}

/// Vector 无符号平均四舍五入
///
/// vector `a` (两个无符号的 64 位整数) 中的元素被添加到 vector `b` (两个无符号的 64 位整数) 中的元素。
/// 元素加 1 (用于四舍五入) 的加法是完全无符号的，即结果有一个额外的位。
/// 在将结果写入 vector (两个无符号的 64 位整数) 之前，将执行无符号除以 2 (或逻辑右移一位) 的操作。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(aver_u.d))]
pub unsafe fn __msa_aver_u_d(a: v2u64, b: v2u64) -> v2u64 {
    msa_aver_u_d(a, mem::transmute(b))
}

/// Vector 位清除
///
/// 将 vector `a` 的每个元素中的一位清零 (设置为 0) (十六个无符号的 8 位整数)。
/// 位的位置由 `b` 中的元素 (十六个无符号的 8 位整数) 给定，以元素的大小为模。
///
/// 结果写入 vector (十六个无符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bclr.b))]
pub unsafe fn __msa_bclr_b(a: v16u8, b: v16u8) -> v16u8 {
    msa_bclr_b(a, mem::transmute(b))
}

/// Vector 位清除
///
/// 将 vector `a` 的每个元素中的一位清零 (设置为 0) (八个无符号 16 位整数)。
/// 位的位置由 `b` 中的元素 (八个无符号的 16 位整数) 给定，以元素的大小为模。
///
/// 结果写入 vector (八个无符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bclr.h))]
pub unsafe fn __msa_bclr_h(a: v8u16, b: v8u16) -> v8u16 {
    msa_bclr_h(a, mem::transmute(b))
}

/// Vector 位清除
///
/// 将 vector `a` 的每个元素中的一位清零 (设置为 0) (四个无符号的 32 位整数)。
/// 位的位置由 `b` 中的元素 (四个无符号的 32 位整数) 给定，以元素的大小为模。
///
/// 结果写入 vector (四个无符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bclr.w))]
pub unsafe fn __msa_bclr_w(a: v4u32, b: v4u32) -> v4u32 {
    msa_bclr_w(a, mem::transmute(b))
}

/// Vector 位清除
///
/// 将 vector `a` 的每个元素中的一位清零 (设置为 0) (两个无符号的 64 位整数)。
/// 位位置由 `b` 中的元素 (两个无符号的 64 位整数) 给定，以元素的大小为模。
///
/// 结果写入 vector (两个无符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bclr.d))]
pub unsafe fn __msa_bclr_d(a: v2u64, b: v2u64) -> v2u64 {
    msa_bclr_d(a, mem::transmute(b))
}

/// Immediate 清除
///
/// 将 vector `a` 的每个元素中的一位清零 (设置为 0) (十六个无符号的 8 位整数)。
/// 位的位置由 immediate 数 `m` 给出以位为单位的元素大小的模。
/// 结果写入 vector (十六个无符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bclri.b, imm3 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_bclri_b<const IMM3: i32>(a: v16u8) -> v16u8 {
    static_assert_imm3!(IMM3);
    msa_bclri_b(a, IMM3)
}

/// Immediate 清除
///
/// 将 vector `a` 的每个元素中的一位清零 (设置为 0) (八个无符号 16 位整数)。
/// 位的位置由 immediate 数 `m` 给出以位为单位的元素大小的模。
/// 结果写入 vector (八个无符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bclri.h, imm4 = 0b1111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_bclri_h<const IMM4: i32>(a: v8u16) -> v8u16 {
    static_assert_imm4!(IMM4);
    msa_bclri_h(a, IMM4)
}

/// Immediate 清除
///
/// 将 vector `a` 的每个元素中的一位清零 (设置为 0) (四个无符号的 32 位整数)。
/// 位的位置由 immediate 数 `m` 给出以位为单位的元素大小的模。
/// 结果写入 vector (四个无符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bclri.w, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_bclri_w<const IMM5: i32>(a: v4u32) -> v4u32 {
    static_assert_imm5!(IMM5);
    msa_bclri_w(a, IMM5)
}

/// Immediate 清除
///
/// 将 vector `a` 的每个元素中的一位清零 (设置为 0) (两个无符号的 64 位整数)。
/// 位的位置由 immediate 数 `m` 给出以位为单位的元素大小的模。
/// 结果写入 vector (两个无符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bclri.d, imm6 = 0b111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_bclri_d<const IMM6: i32>(a: v2u64) -> v2u64 {
    static_assert_imm6!(IMM6);
    msa_bclri_d(a, IMM6)
}

/// Vector 位向左插入
///
/// 将 vector `b` 的每个元素中的最高有效 (left) 位 (十六个无符号的 8 位整数) 复制到 vector `a` 中的元素 (十六个无符号 8 位整数)，同时保留最低有效的 (right) 位。
///
/// 要复制的位数由 vector `c` 中的元素给出 (十六个无符号的 8 位整数)，以元素的大小为模加位加 1。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsl.b))]
pub unsafe fn __msa_binsl_b(a: v16u8, b: v16u8, c: v16u8) -> v16u8 {
    msa_binsl_b(a, mem::transmute(b), c)
}

/// Vector 位向左插入
///
/// 将 vector `b` 的每个元素中的最高有效 (left) 位 (八个无符号 16 位整数) 复制到 vector `a` 中的元素 (八个无符号 16 位整数)，同时保留最低有效 (right) 位。
///
/// 要复制的位数由 vector `c` 中的元素 (八个无符号 16 位整数) 组成，以元素的大小为模加位加 1。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsl.h))]
pub unsafe fn __msa_binsl_h(a: v8u16, b: v8u16, c: v8u16) -> v8u16 {
    msa_binsl_h(a, mem::transmute(b), c)
}

/// Vector 位向左插入
///
/// 将 vector `b` 的每个元素中的最高有效 (left) 位 (四个无符号 32 位整数) 复制到 vector `a` 中的元素 (四个无符号 32 位整数)，同时保留最低有效 (right) 位。
///
/// 要复制的位数由 vector `c` 中的元素给出 (四个无符号的 32 位整数)，以元素的大小为模加位加 1。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsl.w))]
pub unsafe fn __msa_binsl_w(a: v4u32, b: v4u32, c: v4u32) -> v4u32 {
    msa_binsl_w(a, mem::transmute(b), c)
}

/// Vector 位向左插入
///
/// 将 vector `b` 的每个元素中的最高有效 (left) 位 (两个无符号 64 位整数) 复制到 vector `a` 中的元素 (两个无符号 64 位整数)，同时保留最低有效 (right) 位。
///
/// 要复制的位数由 vector `c` 中的元素 (两个无符号的 64 位整数) 给定，以元素的大小为模加位加 1。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsl.d))]
pub unsafe fn __msa_binsl_d(a: v2u64, b: v2u64, c: v2u64) -> v2u64 {
    msa_binsl_d(a, mem::transmute(b), c)
}

/// Immediate 向左插入位
///
/// 将 vector `b` 的每个元素中的最高有效 (left) 位 (十六个无符号的 8 位整数) 复制到 vector `a` 中的元素 (十六个无符号 8 位整数)，同时保留最低有效的 (right) 位。
///
/// 要复制的位数是由 immediate 数 `imm3` 给出的，以元素的大小为模，以位数加 1 为模。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsli.b, imm3 = 0b111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_binsli_b<const IMM3: i32>(a: v16u8, b: v16u8) -> v16u8 {
    static_assert_imm3!(IMM3);
    msa_binsli_b(a, mem::transmute(b), IMM3)
}

/// Immediate 向左插入位
///
/// 将 vector `b` 的每个元素中的最高有效 (left) 位 (八个无符号 16 位整数) 复制到 vector `a` 中的元素 (八个无符号 16 位整数)，同时保留最低有效 (right) 位。
///
/// 要复制的位数是由 immediate 数 `imm4` 给出的，以元素的大小为模，以位数加 1 为模。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsli.h, imm4 = 0b1111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_binsli_h<const IMM4: i32>(a: v8u16, b: v8u16) -> v8u16 {
    static_assert_imm4!(IMM4);
    msa_binsli_h(a, mem::transmute(b), IMM4)
}

/// Immediate 向左插入位
///
/// 将 vector `b` 的每个元素中的最高有效 (left) 位 (四个无符号 32 位整数) 复制到 vector `a` 中的元素 (四个无符号 32 位整数)，同时保留最低有效 (right) 位。
///
/// 要复制的位数是由 immediate 数 `imm5` 给出的，以元素的大小为模，以位数加 1 为模。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsli.w, imm5 = 0b11111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_binsli_w<const IMM5: i32>(a: v4u32, b: v4u32) -> v4u32 {
    static_assert_imm5!(IMM5);
    msa_binsli_w(a, mem::transmute(b), IMM5)
}

/// Immediate 向左插入位
///
/// 将 vector `b` 的每个元素中的最高有效 (left) 位 (两个无符号 64 位整数) 复制到 vector `a` 中的元素 (两个无符号 64 位整数)，同时保留最低有效 (right) 位。
///
/// 要复制的位数是由 immediate 数 `imm6` 给出的，以元素的大小为模，以位数加 1 为模。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsli.d, imm6 = 0b111111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_binsli_d<const IMM6: i32>(a: v2u64, b: v2u64) -> v2u64 {
    static_assert_imm6!(IMM6);
    msa_binsli_d(a, mem::transmute(b), IMM6)
}

/// Vector 位右插入
///
/// 将 vector `b` 的每个元素中的最高有效 (right) 位 (十六个无符号的 8 位整数) 复制到 vector `a` 中的元素 (十六个无符号 8 位整数)，同时保留最低有效的 (left) 位。
///
/// 要复制的位数由 vector `c` 中的元素给出 (十六个无符号的 8 位整数)，以元素的大小为模加位加 1。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsr.b))]
pub unsafe fn __msa_binsr_b(a: v16u8, b: v16u8, c: v16u8) -> v16u8 {
    msa_binsr_b(a, mem::transmute(b), c)
}

/// Vector 位右插入
///
/// 将 vector `b` 的每个元素中的最高有效 (right) 位 (八个无符号 16 位整数) 复制到 vector `a` 中的元素 (八个无符号 16 位整数)，同时保留最低有效 (left) 位。
///
/// 要复制的位数由 vector `c` 中的元素 (八个无符号 16 位整数) 组成，以元素的大小为模加位加 1。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsr.h))]
pub unsafe fn __msa_binsr_h(a: v8u16, b: v8u16, c: v8u16) -> v8u16 {
    msa_binsr_h(a, mem::transmute(b), c)
}

/// Vector 位右插入
///
/// 将 vector `b` 的每个元素中的最高有效 (right) 位 (四个无符号 32 位整数) 复制到 vector `a` 中的元素 (四个无符号 32 位整数)，同时保留最低有效 (left) 位。
///
/// 要复制的位数由 vector `c` 中的元素给出 (四个无符号的 32 位整数)，以元素的大小为模加位加 1。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsr.w))]
pub unsafe fn __msa_binsr_w(a: v4u32, b: v4u32, c: v4u32) -> v4u32 {
    msa_binsr_w(a, mem::transmute(b), c)
}

/// Vector 位右插入
///
/// 将 vector `b` 的每个元素中的最高有效 (right) 位 (两个无符号 64 位整数) 复制到 vector `a` 中的元素 (两个无符号 64 位整数)，同时保留最低有效 (left) 位。
///
/// 要复制的位数由 vector `c` 中的元素 (两个无符号的 64 位整数) 给定，以元素的大小为模加位加 1。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsr.d))]
pub unsafe fn __msa_binsr_d(a: v2u64, b: v2u64, c: v2u64) -> v2u64 {
    msa_binsr_d(a, mem::transmute(b), c)
}

/// Immediate 向右插入位
///
/// 将 vector `b` 的每个元素中的最高有效 (right) 位 (十六个无符号的 8 位整数) 复制到 vector `a` 中的元素 (十六个无符号 8 位整数)，同时保留最低有效的 (left) 位。
///
/// 要复制的位数是由 immediate 数 `imm3` 给出的，以元素的大小为模，以位数加 1 为模。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsri.b, imm3 = 0b111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_binsri_b<const IMM3: i32>(a: v16u8, b: v16u8) -> v16u8 {
    static_assert_imm3!(IMM3);
    msa_binsri_b(a, mem::transmute(b), IMM3)
}

/// Immediate 向右插入位
///
/// 将 vector `b` 的每个元素中的最高有效 (right) 位 (八个无符号 16 位整数) 复制到 vector `a` 中的元素 (八个无符号 16 位整数)，同时保留最低有效 (left) 位。
///
/// 要复制的位数是由 immediate 数 `imm4` 给出的，以元素的大小为模，以位数加 1 为模。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsri.h, imm4 = 0b1111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_binsri_h<const IMM4: i32>(a: v8u16, b: v8u16) -> v8u16 {
    static_assert_imm4!(IMM4);
    msa_binsri_h(a, mem::transmute(b), IMM4)
}

/// Immediate 向右插入位
///
/// 将 vector `b` 的每个元素中的最高有效 (right) 位 (四个无符号 32 位整数) 复制到 vector `a` 中的元素 (四个无符号 32 位整数)，同时保留最低有效 (left) 位。
///
/// 要复制的位数是由 immediate 数 `imm5` 给出的，以元素的大小为模，以位数加 1 为模。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsri.w, imm5 = 0b11111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_binsri_w<const IMM5: i32>(a: v4u32, b: v4u32) -> v4u32 {
    static_assert_imm5!(IMM5);
    msa_binsri_w(a, mem::transmute(b), IMM5)
}

/// Immediate 向右插入位
///
/// 将 vector `b` 的每个元素中的最高有效 (right) 位 (两个无符号 64 位整数) 复制到 vector `a` 中的元素 (两个无符号 64 位整数)，同时保留最低有效 (left) 位。
///
/// 要复制的位数是由 immediate 数 `imm6` 给出的，以元素的大小为模，以位数加 1 为模。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(binsri.d, imm6 = 0b111111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_binsri_d<const IMM6: i32>(a: v2u64, b: v2u64) -> v2u64 {
    static_assert_imm6!(IMM6);
    msa_binsri_d(a, mem::transmute(b), IMM6)
}

/// Vector 不为零的位移动
///
/// 将来自源 vector `b` (十六个无符号 8 位整数) 的所有位复制到目标 vector `a` (十六个无符号 8 位整数)，目标 vector `c` (十六个无符号 8 位整数) 的所有位分别为 1 和保留目标位为 0 的所有目标位不变。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bmnz.v))]
pub unsafe fn __msa_bmnz_v(a: v16u8, b: v16u8, c: v16u8) -> v16u8 {
    msa_bmnz_v(a, mem::transmute(b), c)
}

/// Immediate 位移动 (如果不为零)
///
/// 将来自源 vector `b` 的所有位 (来自 immediate  `imm8` 的对应位为 1) 复制到目标 vector `a` (十六个无符号的 8 位整数)，并将所有源目标位保持不变目标位为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bmnzi.b, imm8 = 0b11111111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_bmnzi_b<const IMM8: i32>(a: v16u8, b: v16u8) -> v16u8 {
    static_assert_imm8!(IMM8);
    msa_bmnzi_b(a, mem::transmute(b), IMM8)
}

/// Vector 如果为零则移动
///
/// 将源 vector `b` (十六个无符号 8 位整数) 中的所有位复制到目标 vector `a` (十六个无符号 8 位整数)，目标 vector `c` (十六个无符号 8 位整数) 的所有位分别为 0 和保留目标位为 1 的所有目标位不变。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bmz.v))]
pub unsafe fn __msa_bmz_v(a: v16u8, b: v16u8, c: v16u8) -> v16u8 {
    msa_bmz_v(a, mem::transmute(b), c)
}

/// 如果为零，则 immediate 移位
///
/// 将来自源 vector `b` 的所有位 (来自 immediate  `imm8` 的对应位为 0) 复制到目标 vector `a` (十六个无符号的 8 位整数)，并将所有源目标位保持不变 immediate 数为 1。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bmzi.b, imm8 = 0b11111111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_bmzi_b<const IMM8: i32>(a: v16u8, b: v16u8) -> v16u8 {
    static_assert_imm8!(IMM8);
    msa_bmzi_b(a, mem::transmute(b), IMM8)
}

/// Vector 位取反
///
/// 对 vector `a` 的每个元素中的 (complement) 取反 (十六个无符号的 8 位整数)。
/// 位位置由 vector `b` 中的元素 (十六个无符号的 8 位整数) 给定，以元素的大小为模。
///
/// 结果写入 vector (十六个无符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bneg.b))]
pub unsafe fn __msa_bneg_b(a: v16u8, b: v16u8) -> v16u8 {
    msa_bneg_b(a, mem::transmute(b))
}

/// Vector 位取反
///
/// 对 vector `a` 的每个元素中的 (complement) 取反 (八个无符号的 16 位整数)。
/// 位位置由 vector `b` 中的元素 (八个无符号 16 位整数) 给定，以元素的大小为模。
///
/// 结果写入 vector (八个无符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bneg.h))]
pub unsafe fn __msa_bneg_h(a: v8u16, b: v8u16) -> v8u16 {
    msa_bneg_h(a, mem::transmute(b))
}

/// Vector 位取反
///
/// 对 vector `a` 的每个元素中的 (complement) 取反 (四个无符号的 32 位整数)。
/// 位的位置由 vector `b` 中的元素 (四个无符号的 32 位整数) 给定，以元素的大小为模。
///
/// 结果写入 vector (四个无符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bneg.w))]
pub unsafe fn __msa_bneg_w(a: v4u32, b: v4u32) -> v4u32 {
    msa_bneg_w(a, mem::transmute(b))
}

/// Vector 位取反
///
/// 对 vector `a` 的每个元素中的 (complement) 取反 (两个无符号的 64 位整数)。
/// 位位置由 vector `b` 中的元素 (两个无符号的 64 位整数) 给定，以元素的大小为模。
///
/// 结果写入 vector (两个无符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bneg.d))]
pub unsafe fn __msa_bneg_d(a: v2u64, b: v2u64) -> v2u64 {
    msa_bneg_d(a, mem::transmute(b))
}

/// Immediate 位求反
///
/// 对 vector `a` 的每个元素中的 (complement) 取反 (十六个无符号的 8 位整数)。
/// `imm3` 以位为单位，以模数表示元素的大小，以位的形式给出位的位置。
/// 结果写入 vector (十六个无符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bnegi.b, imm3 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_bnegi_b<const IMM3: i32>(a: v16u8) -> v16u8 {
    static_assert_imm3!(IMM3);
    msa_bnegi_b(a, IMM3)
}

/// Immediate 位求反
///
/// 对 vector `a` 的每个元素中的 (complement) 取反 (八个无符号的 16 位整数)。
/// `imm4` 以位为单位，以模数表示元素的大小，以位的形式给出位的位置。
/// 结果写入 vector (八个无符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bnegi.h, imm4 = 0b1111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_bnegi_h<const IMM4: i32>(a: v8u16) -> v8u16 {
    static_assert_imm4!(IMM4);
    msa_bnegi_h(a, IMM4)
}

/// Immediate 位求反
///
/// 对 vector `a` 的每个元素中的 (complement) 取反 (四个无符号的 32 位整数)。
/// `imm5` 以位为单位，以模数表示元素的大小，以位的形式给出位的位置。
/// 结果写入 vector (四个无符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bnegi.w, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_bnegi_w<const IMM5: i32>(a: v4u32) -> v4u32 {
    static_assert_imm5!(IMM5);
    msa_bnegi_w(a, IMM5)
}

/// Immediate 位求反
///
/// 对 vector `a` 的每个元素中的 (complement) 取反 (两个无符号的 64 位整数)。
/// `imm6` 以位为单位，以模数表示元素的大小，以位的形式给出位的位置。
/// 结果写入 vector (两个无符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bnegi.d, imm6 = 0b111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_bnegi_d<const IMM6: i32>(a: v2u64) -> v2u64 {
    static_assert_imm6!(IMM6);
    msa_bnegi_d(a, IMM6)
}

/// 如果所有元素都不为零，则 immediate 分支
///
/// 如果 `a` 中的所有元素 (十六个无符号的 8 位整数) 都不为零，则为 PC 相对分支。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bnz.b))]
pub unsafe fn __msa_bnz_b(a: v16u8) -> i32 {
    msa_bnz_b(a)
}

/// 如果所有元素都不为零，则 immediate 分支
///
/// 如果 `a` 中的所有元素 (八个无符号的 16 位整数) 都不为零，则为 PC 相对分支。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bnz.h))]
pub unsafe fn __msa_bnz_h(a: v8u16) -> i32 {
    msa_bnz_h(a)
}

/// 如果所有元素都不为零，则 immediate 分支
///
/// 如果 `a` 中的所有元素 (四个无符号的 32 位整数) 都不为零，则为 PC 相对分支。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bnz.w))]
pub unsafe fn __msa_bnz_w(a: v4u32) -> i32 {
    msa_bnz_w(a)
}

/// 如果所有元素都不为零，则 immediate 分支
///
/// 如果 `a` 中的所有元素 (两个无符号的 64 位整数) 都不为零，则为 PC 相对分支。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bnz.d))]
pub unsafe fn __msa_bnz_d(a: v2u64) -> i32 {
    msa_bnz_d(a)
}

/// Immediate 分支，如果不为零 (任何格式的至少一个元素不为零)
///
/// 如果 `a` 中的至少一位 (四个无符号 32 位整数) 不为零，则为 PC 相对分支。
/// 即，无论数据格式如何，至少有一个元素不为零。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bnz.v))]
pub unsafe fn __msa_bnz_v(a: v16u8) -> i32 {
    msa_bnz_v(a)
}

/// Vector 位选择
///
/// 根据 `a` 中的相应位，将源 vectors `b` (八个无符号 16 位整数) 和 `c` (八个无符号 16 位整数) 中的位选择性复制到目标 vector `a` (八个无符号 16 位整数) 中：
///
/// 如果为 0，则从 `b` 复制该位; 如果为 1，则从 `c` 复制该位。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bsel.v))]
pub unsafe fn __msa_bsel_v(a: v16u8, b: v16u8, c: v16u8) -> v16u8 {
    msa_bsel_v(a, mem::transmute(b), c)
}

/// Immediate 位选择
///
/// 根据 `a` 中的相应位，将 8 位 immediate 数 `imm8` 和 `c` (八个无符号的 16 位整数) 中的位选择性地复制到目标 vector `a` (八个无符号的 16 位整数中)。
///
/// 如果为 0，则从 `b` 复制该位; 如果为 1，则从 `c` 复制该位。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bseli.b, imm8 = 0b11111111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_bseli_b<const IMM8: i32>(a: v16u8, b: v16u8) -> v16u8 {
    static_assert_imm8!(IMM8);
    msa_bseli_b(a, mem::transmute(b), IMM8)
}

/// Vector 位设置
///
/// 在 vector `a` 的每个元素中将 1 设置为 1 (十六个无符号的 8 位整数)。
/// 位位置由 vector `b` 中的元素 (十六个无符号的 8 位整数) 给定，以元素的大小为模。
///
/// 结果写入 vector (十六个无符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bset.b))]
pub unsafe fn __msa_bset_b(a: v16u8, b: v16u8) -> v16u8 {
    msa_bset_b(a, mem::transmute(b))
}

/// Vector 位设置
///
/// 在 vector `a` 的每个元素中将 1 设置为 1 (八个无符号 16 位整数)。
/// 位位置由 vector `b` 中的元素 (八个无符号 16 位整数) 给定，以元素的大小为模。
///
/// 结果写入 vector (八个无符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bset.h))]
pub unsafe fn __msa_bset_h(a: v8u16, b: v8u16) -> v8u16 {
    msa_bset_h(a, mem::transmute(b))
}

/// Vector 位设置
///
/// 在 vector `a` 的每个元素中将 1 设置为 1 (四个无符号的 32 位整数)。
/// 位的位置由 vector `b` 中的元素 (四个无符号的 32 位整数) 给定，以元素的大小为模。
///
/// 结果写入 vector (四个无符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bset.w))]
pub unsafe fn __msa_bset_w(a: v4u32, b: v4u32) -> v4u32 {
    msa_bset_w(a, mem::transmute(b))
}

/// Vector 位设置
///
/// 在 vector `a` 的每个元素中将 1 设置为 1 (两个无符号的 64 位整数)。
/// 位位置由 vector `b` 中的元素 (两个无符号的 64 位整数) 给定，以元素的大小为模。
///
/// 结果写入 vector (两个无符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bset.d))]
pub unsafe fn __msa_bset_d(a: v2u64, b: v2u64) -> v2u64 {
    msa_bset_d(a, mem::transmute(b))
}

/// Immediate 置位
///
/// 在 vector `a` 的每个元素中将 1 设置为 1 (十六个无符号的 8 位整数)。
/// 该位的位置由 immediate 数 `imm3` 给出。
/// 结果写入 vector `a` (十六个无符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bseti.b, imm3 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_bseti_b<const IMM3: i32>(a: v16u8) -> v16u8 {
    static_assert_imm3!(IMM3);
    msa_bseti_b(a, IMM3)
}

/// Immediate 置位
///
/// 在 vector `a` 的每个元素中将 1 设置为 1 (八个无符号 16 位整数)。
/// 该位的位置由 immediate 数 `imm4` 给出。
/// 结果写入 vector `a` (八个无符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bseti.h, imm4 = 0b1111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_bseti_h<const IMM4: i32>(a: v8u16) -> v8u16 {
    static_assert_imm4!(IMM4);
    msa_bseti_h(a, IMM4)
}

/// Immediate 置位
///
/// 在 vector `a` 的每个元素中将 1 设置为 1 (四个无符号的 32 位整数)。
/// 该位的位置由 immediate 数 `imm5` 给出。
/// 将结果写入 vector `a` (四个无符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bseti.w, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_bseti_w<const IMM5: i32>(a: v4u32) -> v4u32 {
    static_assert_imm5!(IMM5);
    msa_bseti_w(a, IMM5)
}

/// Immediate 置位
///
/// 在 vector `a` 的每个元素中将 1 设置为 1 (两个无符号的 64 位整数)。
/// 该位的位置由 immediate 数 `imm6` 给出。
/// 结果写入 vector `a` (两个无符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bseti.d, imm6 = 0b111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_bseti_d<const IMM6: i32>(a: v2u64) -> v2u64 {
    static_assert_imm6!(IMM6);
    msa_bseti_d(a, IMM6)
}

/// 如果至少一个元素为零，则 immediate 分支
///
/// 如果 `a` 中的至少一个元素 (十六个无符号的 8 位整数) 为零，则为 PC 相对分支。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bz.b))]
pub unsafe fn __msa_bz_b(a: v16u8) -> i32 {
    msa_bz_b(a)
}

/// 如果至少一个元素为零，则 immediate 分支
///
/// 如果 `a` 中的至少一个元素 (八个无符号 16 位整数) 为零，则为 PC 相对分支。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bz.h))]
pub unsafe fn __msa_bz_h(a: v8u16) -> i32 {
    msa_bz_h(a)
}

/// 如果至少一个元素为零，则 immediate 分支
///
/// 如果 `a` 中的至少一个元素 (四个无符号 32 位整数) 为零，则为 PC 相对分支。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bz.w))]
pub unsafe fn __msa_bz_w(a: v4u32) -> i32 {
    msa_bz_w(a)
}

/// 如果至少一个元素为零，则 immediate 分支
///
/// 如果 `a` 中的至少一个元素 (两个无符号的 64 位整数) 为零，则为 PC 相对分支。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bz.d))]
pub unsafe fn __msa_bz_d(a: v2u64) -> i32 {
    msa_bz_d(a)
}

/// 如果为零则 immediate 分支 (任何格式的所有元素均为零)
///
/// PC 相对分支，如果 `a` (十六个无符号 8 位整数) 位中的所有元素均为零，即
/// 无论数据格式如何，所有元素均为零。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(bz.v))]
pub unsafe fn __msa_bz_v(a: v16u8) -> i32 {
    msa_bz_v(a)
}

/// Vector 比较等于
///
/// 如果相应的 `a` (十六个带符号的 8 位整数) 和 `b` (十六个带符号的 8 位整数) 元素相等，则在 vector (十六个带符号的 8 位整数) 元素中将所有位设置为 1，否则将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ceq.b))]
pub unsafe fn __msa_ceq_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_ceq_b(a, mem::transmute(b))
}

/// Vector 比较等于
///
/// 如果相应的 `a` (八个带符号的 16 位整数) 和 `b` (八个带符号的 16 位整数) 元素相等，则在 vector (八个带符号的 16 位整数) 元素中将所有位设置为 1，否则将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ceq.h))]
pub unsafe fn __msa_ceq_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_ceq_h(a, mem::transmute(b))
}

/// Vector 比较等于
///
/// 如果相应的 `a` (四个带符号的 32 位整数) 和 `b` (四个带符号的 32 位整数) 元素相等，则在 vector (四个带符号的 32 位整数) 元素中将所有位设置为 1，否则，将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ceq.w))]
pub unsafe fn __msa_ceq_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_ceq_w(a, mem::transmute(b))
}

/// Vector 比较等于
///
/// 如果相应的 `a` (两个带符号的 64 位整数) 和 `b` (两个带符号的 64 位整数) 元素相等，则在 vector (两个带符号的 64 位整数) 元素中将所有位设置为 1，否则将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ceq.d))]
pub unsafe fn __msa_ceq_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_ceq_d(a, mem::transmute(b))
}

/// Immediate 比较等于
///
/// 如果相应的 `a` (十六个带符号的 8 位整数)，5 位带符号 immediate 数 imm_s5 相等，则在 vector (十六个带符号的 8 位整数) 元素中将所有位设置为 1，否则将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ceqi.b, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_ceqi_b<const IMM_S5: i32>(a: v16i8) -> v16i8 {
    static_assert_imm_s5!(IMM_S5);
    msa_ceqi_b(a, IMM_S5)
}

/// Immediate 比较等于
///
/// 如果相应的 `a` (8 个带符号的 16 位整数)，5 位带符号 immediate 数 imm_s5 相等，则将 vector (8 个带符号的 16 位整数) 中的所有位设置为 1，否则将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ceqi.h, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_ceqi_h<const IMM_S5: i32>(a: v8i16) -> v8i16 {
    static_assert_imm_s5!(IMM_S5);
    msa_ceqi_h(a, IMM_S5)
}

/// Immediate 比较等于
///
/// 如果相应的 `a` (四个带符号的 32 位整数) 5 位带符号 immediate 数 imm_s5 相等，则在 vector (四个带符号的 32 位整数) 元素中将所有位设置为 1，否则将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ceqi.w, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_ceqi_w<const IMM_S5: i32>(a: v4i32) -> v4i32 {
    static_assert_imm_s5!(IMM_S5);
    msa_ceqi_w(a, IMM_S5)
}

/// Immediate 比较等于
///
/// 如果相应的 `a` (两个带符号的 64 位整数) 5 位带符号 immediate 数 imm_s5 相等，则将 vector (两个带符号的 64 位整数) 中的所有位设置为 1，否则将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ceqi.d, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_ceqi_d<const IMM_S5: i32>(a: v2i64) -> v2i64 {
    static_assert_imm_s5!(IMM_S5);
    msa_ceqi_d(a, IMM_S5)
}

/// 来自 MSA 控制寄存器的 GPR 复制
///
/// MSA 控制寄存器 cs 的符号扩展内容被复制到 GPR rd。
///
/// 无法在用户模式下进行测试
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(cfcmsa, imm5 = 0b11111))]
#[rustc_legacy_const_generics(0)]
pub unsafe fn __msa_cfcmsa<const IMM5: i32>() -> i32 {
    static_assert_imm5!(IMM5);
    msa_cfcmsa(IMM5)
}

/// Vector 比较有符号小于或等于
///
/// 如果相应的 `a` (十六个带符号的 8 位整数) 的元素的符号小于或等于 `b` (十六个带符号的 8 位整数) 的元素，则将 vector (十六个带符号的 8 位整数) 元素中的所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(cle_s.b))]
pub unsafe fn __msa_cle_s_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_cle_s_b(a, mem::transmute(b))
}

/// Vector 比较有符号小于或等于
///
/// 如果相应的 `a` (八个带符号的 16 位整数) 元素的符号小于或等于 `b` (八个带符号的 16 位整数) 元素，则将 vector (八个带符号的 16 位整数) 元素中的所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(cle_s.h))]
pub unsafe fn __msa_cle_s_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_cle_s_h(a, mem::transmute(b))
}

/// Vector 比较有符号小于或等于
///
/// 如果相应的 `a` (四个带符号的 32 位整数) 元素的符号小于或等于 `b` (四个带符号的 32 位整数) 元素，则在 vector (四个带符号的 32 位整数) 元素中将所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(cle_s.w))]
pub unsafe fn __msa_cle_s_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_cle_s_w(a, mem::transmute(b))
}

/// Vector 比较有符号小于或等于
///
/// 如果相应的 `a` (两个带符号的 64 位整数) 元素的符号小于或等于 `b` (两个带符号的 64 位整数) 元素，则在 vector (两个带符号的 64 位整数) 元素中将所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(cle_s.d))]
pub unsafe fn __msa_cle_s_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_cle_s_d(a, mem::transmute(b))
}

/// Vector 比较无符号小于或等于
///
/// 如果相应的 `a` (十六个无符号的 8 位整数) 的无符号数小于或等于 `b` (十六个无符号的 8 位整数) 的元素，则在 vector (十六个带符号的 8 位整数) 元素中将所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(cle_u.b))]
pub unsafe fn __msa_cle_u_b(a: v16u8, b: v16u8) -> v16i8 {
    msa_cle_u_b(a, mem::transmute(b))
}

/// Vector 比较无符号小于或等于
///
/// 如果相应的 `a` (八个无符号的 16 位整数) 的无符号数小于或等于 `b` (八个无符号的 16 位整数) 的元素，则将 vector (八个有符号的 16 位整数) 元素中的所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(cle_u.h))]
pub unsafe fn __msa_cle_u_h(a: v8u16, b: v8u16) -> v8i16 {
    msa_cle_u_h(a, mem::transmute(b))
}

/// Vector 比较无符号小于或等于
///
/// 如果相应的 `a` (四个无符号的 32 位整数) 元素的无符号数小于或等于 `b` (四个无符号的 32 位整数) 元素，则将 vector (四个有符号的 32 位整数) 元素中的所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(cle_u.w))]
pub unsafe fn __msa_cle_u_w(a: v4u32, b: v4u32) -> v4i32 {
    msa_cle_u_w(a, mem::transmute(b))
}

/// Vector 比较无符号小于或等于
///
/// 如果相应的 `a` (两个无符号的 64 位整数) 元素无符号小于或等于 `b` (两个无符号的 64 位整数) 元素，则将 vector (两个有符号的 64 位整数) 元素中的所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(cle_u.d))]
pub unsafe fn __msa_cle_u_d(a: v2u64, b: v2u64) -> v2i64 {
    msa_cle_u_d(a, mem::transmute(b))
}

/// Immediate 比较有符号小于或等于
///
/// 如果相应的 `a` (十六个带符号的 8 位整数) 个元素小于或等于 5 位带符号 immediate 数 imm_s5，则在 vector (十六个带符号的 8 位整数) 元素中将所有位设置为 1，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clei_s.b, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clei_s_b<const IMM_S5: i32>(a: v16i8) -> v16i8 {
    static_assert_imm_s5!(IMM_S5);
    msa_clei_s_b(a, IMM_S5)
}

/// Immediate 比较有符号小于或等于
///
/// 如果对应的 `a` (八个带符号的 16 位整数) 元素小于或等于 5 个带符号 immediate 数 imm_s5，则将 vector (八个带符号的 16 位整数) 元素中的所有位设置为 1，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clei_s.h, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clei_s_h<const IMM_S5: i32>(a: v8i16) -> v8i16 {
    static_assert_imm_s5!(IMM_S5);
    msa_clei_s_h(a, IMM_S5)
}

/// Immediate 比较有符号小于或等于
///
/// 如果对应的 `a` (四个带符号的 32 位整数) 元素小于或等于 5 位带符号 immediate 数 imm_s5，则将 vector (四个带符号的 32 位整数) 元素中的所有位设置为 1，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clei_s.w, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clei_s_w<const IMM_S5: i32>(a: v4i32) -> v4i32 {
    static_assert_imm_s5!(IMM_S5);
    msa_clei_s_w(a, IMM_S5)
}

/// Immediate 比较有符号小于或等于
///
/// 如果相应的 `a` (两个带符号的 64 位整数) 个元素小于或等于 5 个带符号 immediate 数 imm_s5，则在 vector (两个带符号的 64 位整数) 元素中将所有位设置为 1，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clei_s.d, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clei_s_d<const IMM_S5: i32>(a: v2i64) -> v2i64 {
    static_assert_imm_s5!(IMM_S5);
    msa_clei_s_d(a, IMM_S5)
}

/// Immediate 比较无符号小于或等于
///
/// 如果相应的 `a` (十六个无符号的 8 位整数) 的无符号数小于或等于 5 位无符号 immediate 数 `imm5`，则在 vector (十六个带符号的 8 位整数) 的元素中将所有位设置为 1，否则设置所有位到 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clei_u.b, imm5 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clei_u_b<const IMM5: i32>(a: v16u8) -> v16i8 {
    static_assert_imm5!(IMM5);
    msa_clei_u_b(a, IMM5)
}

/// Immediate 比较无符号小于或等于
///
/// 如果相应的 `a` (八个无符号的 16 位整数) 的无符号数小于或等于 5 位无符号 immediate 数 `imm5`，则在 vector (八个有符号的 16 位整数) 的元素中将所有位设置为 1，否则设置所有位到 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clei_u.h, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clei_u_h<const IMM5: i32>(a: v8u16) -> v8i16 {
    static_assert_imm5!(IMM5);
    msa_clei_u_h(a, IMM5)
}

/// Immediate 比较无符号小于或等于
///
/// 如果相应的 `a` (四个无符号的 32 位整数) 的无符号数小于或等于 5 位无符号 immediate 数 `imm5`，则在 vector (四个有符号的 32 位整数) 的元素中将所有位设置为 1，否则设置所有位到 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clei_u.w, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clei_u_w<const IMM5: i32>(a: v4u32) -> v4i32 {
    static_assert_imm5!(IMM5);
    msa_clei_u_w(a, IMM5)
}

/// Immediate 比较无符号小于或等于
///
/// 如果相应的 `a` (两个无符号的 64 位整数) 元素的无符号数小于或等于 5 位无符号 immediate 数 `imm5`，则在 vector (两个有符号的 64 位整数数字) 元素中将所有位设置为 1，否则设置所有位到 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clei_u.d, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clei_u_d<const IMM5: i32>(a: v2u64) -> v2i64 {
    static_assert_imm5!(IMM5);
    msa_clei_u_d(a, IMM5)
}

/// Vector 比较有符号小于
///
/// 如果相应的 `a` (十六个带符号的 8 位整数) 的符号小于 `b` (十六个带符号的 8 位整数) 的元素，则在 vector (十六个带符号的 8 位整数) 元素中将所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clt_s.b))]
pub unsafe fn __msa_clt_s_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_clt_s_b(a, mem::transmute(b))
}

/// Vector 比较有符号小于
///
/// 如果相应的 `a` (八个带符号的 16 位整数) 的符号小于 `b` (八个带符号的 16 位整数) 的元素，则将 vector (八个带符号的 16 位整数) 元素中的所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clt_s.h))]
pub unsafe fn __msa_clt_s_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_clt_s_h(a, mem::transmute(b))
}

/// Vector 比较有符号小于
///
/// 如果相应的 `a` (四个带符号的 32 位整数) 元素的符号小于 `b` (四个带符号的 32 位整数) 元素，则在 vector (四个带符号的 32 位整数) 元素中将所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clt_s.w))]
pub unsafe fn __msa_clt_s_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_clt_s_w(a, mem::transmute(b))
}

/// Vector 比较有符号小于
///
/// 如果相应的 `a` (两个带符号的 64 位整数) 元素的符号小于 `b` (两个带符号的 64 位整数) 元素，则将 vector (两个带符号的 64 位整数) 元素中的所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clt_s.d))]
pub unsafe fn __msa_clt_s_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_clt_s_d(a, mem::transmute(b))
}

/// Vector 比较无符号小于
///
/// 如果相应的 `a` (十六个无符号的 8 位整数) 的元素的无符号数小于 `b` (十六个无符号的 8 位整数) 的元素，则将 vector (十六个带符号的 8 位整数) 元素中的所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clt_u.b))]
pub unsafe fn __msa_clt_u_b(a: v16u8, b: v16u8) -> v16i8 {
    msa_clt_u_b(a, mem::transmute(b))
}

/// Vector 比较无符号小于
///
/// 如果相应的 `a` (八个无符号的 16 位整数) 的无符号数小于 `b` (八个无符号的 16 位整数) 的元素，则将 vector (八个有符号的 16 位整数) 元素中的所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clt_u.h))]
pub unsafe fn __msa_clt_u_h(a: v8u16, b: v8u16) -> v8i16 {
    msa_clt_u_h(a, mem::transmute(b))
}

/// Vector 比较无符号小于
///
/// 如果相应的 `a` (四个无符号 32 位整数) 元素的无符号数小于 `b` (四个无符号 32 位整数) 元素，则将 vector (四个有符号 32 位整数) 元素中的所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clt_u.w))]
pub unsafe fn __msa_clt_u_w(a: v4u32, b: v4u32) -> v4i32 {
    msa_clt_u_w(a, mem::transmute(b))
}

/// Vector 比较无符号小于
///
/// 如果相应的 `a` (两个无符号的 64 位整数) 元素的无符号数小于 `b` (两个无符号的 64 位整数) 元素，则将 vector (两个有符号的 64 位整数) 元素中的所有位设置为 1。
///
/// 否则，将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clt_u.d))]
pub unsafe fn __msa_clt_u_d(a: v2u64, b: v2u64) -> v2i64 {
    msa_clt_u_d(a, mem::transmute(b))
}

/// Immediate 比较有符号少于
///
/// 如果相应的 `a` (十六个带符号的 8 位整数) 个元素小于 5 位带符号 immediate 数 imm_s5，则在 vector (十六个带符号的 8 位整数) 元素中将所有位设置为 1，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clti_s.b, imm_s5 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clti_s_b<const IMM_S5: i32>(a: v16i8) -> v16i8 {
    static_assert_imm_s5!(IMM_S5);
    msa_clti_s_b(a, IMM_S5)
}

/// Immediate 比较有符号少于
///
/// 如果相应的 `a` (八个带符号的 16 位整数) 的元素小于 5 个带符号 immediate 数 imm_s5，则将 vector (八个带符号的 16 位整数) 的所有位设置为 1，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clti_s.h, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clti_s_h<const IMM_S5: i32>(a: v8i16) -> v8i16 {
    static_assert_imm_s5!(IMM_S5);
    msa_clti_s_h(a, IMM_S5)
}

/// Immediate 比较有符号少于
///
/// 如果相应的 `a` (四个带符号的 32 位整数) 个元素小于 5 个带符号的 immediate 数 imm_s5，则在 vector (四个带符号的 32 位整数) 元素中将所有位设置为 1，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clti_s.w, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clti_s_w<const IMM_S5: i32>(a: v4i32) -> v4i32 {
    static_assert_imm_s5!(IMM_S5);
    msa_clti_s_w(a, IMM_S5)
}

/// Immediate 比较有符号少于
///
/// 如果相应的 `a` (两个带符号的 64 位整数) 个元素小于 5 个带符号的 immediate 数 imm_s5，则在 vector (两个带符号的 64 位整数) 元素中将所有位设置为 1，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clti_s.d, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clti_s_d<const IMM_S5: i32>(a: v2i64) -> v2i64 {
    static_assert_imm_s5!(IMM_S5);
    msa_clti_s_d(a, IMM_S5)
}

/// Immediate 比较无符号数小于
///
/// 如果相应的 `a` (十六个无符号的 8 位整数) 的无符号数小于 5 位无符号 immediate 数 `imm5`，则将 vector (十六个带符号的 8 位整数) 的所有位设置为 1，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clti_u.b, imm5 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clti_u_b<const IMM5: i32>(a: v16u8) -> v16i8 {
    static_assert_imm5!(IMM5);
    msa_clti_u_b(a, IMM5)
}

/// Immediate 比较无符号数小于
///
/// 如果相应的 `a` (八个无符号的 16 位整数) 的无符号数小于 5 位无符号 immediate 数 `imm5`，则将 vector (八个有符号的 16 位整数) 的所有位设置为 1，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clti_u.h, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clti_u_h<const IMM5: i32>(a: v8u16) -> v8i16 {
    static_assert_imm5!(IMM5);
    msa_clti_u_h(a, IMM5)
}

/// Immediate 比较无符号数小于
///
/// 如果相应的 `a` (四个无符号的 32 位整数) 个元素的无符号数小于 5 个无符号 immediate 数 `imm5`，则在 vector (四个有符号的 32 位整数) 元素中将所有位设置为 1，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clti_u.w, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clti_u_w<const IMM5: i32>(a: v4u32) -> v4i32 {
    static_assert_imm5!(IMM5);
    msa_clti_u_w(a, IMM5)
}

/// Immediate 比较无符号数小于
///
/// 如果相应的 `a` (两个无符号的 64 位整数) 的元素的无符号数小于 5 位无符号 immediate 数 `imm5`，则在 vector (两个有符号的 64 位整数) 的元素中将所有位设置为 1，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(clti_u.d, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_clti_u_d<const IMM5: i32>(a: v2u64) -> v2i64 {
    static_assert_imm5!(IMM5);
    msa_clti_u_d(a, IMM5)
}

/// 元素复制到 GPR 已签名
///
/// vector `a` 的符号扩展元素 `imm4` (十六个带符号的 8 位整数) 并将结果复制到 GPR rd。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(copy_s.b, imm4 = 0b1111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_copy_s_b<const IMM4: i32>(a: v16i8) -> i32 {
    static_assert_imm4!(IMM4);
    msa_copy_s_b(a, IMM4)
}

/// 元素复制到 GPR 已签名
///
/// vector `a` 的符号扩展元素 `imm3` (八个带符号的 16 位整数) 并将结果复制到 GPR rd。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(copy_s.h, imm3 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_copy_s_h<const IMM3: i32>(a: v8i16) -> i32 {
    static_assert_imm3!(IMM3);
    msa_copy_s_h(a, IMM3)
}

/// 元素复制到 GPR 已签名
///
/// vector `a` 的符号扩展元素 `imm2` (四个带符号的 32 位整数) 并将结果复制到 GPR rd。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(copy_s.w, imm2 = 0b11))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_copy_s_w<const IMM2: i32>(a: v4i32) -> i32 {
    static_assert_imm2!(IMM2);
    msa_copy_s_w(a, IMM2)
}

/// 元素复制到 GPR 已签名
///
/// vector `a` 的符号扩展元素 `imm1` (两个带符号的 64 位整数) 并将结果复制到 GPR rd。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(copy_s.d, imm1 = 0b1))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_copy_s_d<const IMM1: i32>(a: v2i64) -> i64 {
    static_assert_imm1!(IMM1);
    msa_copy_s_d(a, IMM1)
}

/// 元素复制到 GPR (未签名)
///
/// vector `a` 的零扩展元素 `imm4` (十六个带符号的 8 位整数) 并将结果复制到 GPR rd。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(copy_u.b, imm4 = 0b1111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_copy_u_b<const IMM4: i32>(a: v16i8) -> u32 {
    static_assert_imm4!(IMM4);
    msa_copy_u_b(a, IMM4)
}

/// 元素复制到 GPR (未签名)
///
/// vector `a` 的零扩展元素 `imm3` (八个带符号的 16 位整数) 并将结果复制到 GPR rd。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(copy_u.h, imm3 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_copy_u_h<const IMM3: i32>(a: v8i16) -> u32 {
    static_assert_imm3!(IMM3);
    msa_copy_u_h(a, IMM3)
}

/// 元素复制到 GPR (未签名)
///
/// vector `a` 的零扩展元素 `imm2` (四个带符号的 32 位整数) 并将结果复制到 GPR rd。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(copy_u.w, imm2 = 0b11))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_copy_u_w<const IMM2: i32>(a: v4i32) -> u32 {
    static_assert_imm2!(IMM2);
    msa_copy_u_w(a, IMM2)
}

/// 元素复制到 GPR (未签名)
///
/// vector `a` 的零扩展元素 `imm1` (两个带符号的 64 位整数) 并将结果复制到 GPR rd。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(copy_u.d, imm1 = 0b1))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_copy_u_d<const IMM1: i32>(a: v2i64) -> u64 {
    static_assert_imm1!(IMM1);
    msa_copy_u_d(a, IMM1)
}

/// GPR 复制到 MSA 控制寄存器
///
/// GPR `imm1` 的最低有效 31 位的内容被复制到 MSA 控制寄存器 cd。
///
///
/// 无法在用户模式下进行测试
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ctcmsa, imm1 = 0b1))]
#[rustc_legacy_const_generics(0)]
pub unsafe fn __msa_ctcmsa<const IMM5: i32>(a: i32) -> () {
    static_assert_imm5!(IMM5);
    msa_ctcmsa(IMM5, a)
}

/// Vector 有符号分隔
///
/// vector `a` (十六个带符号的 8 位整数) 中的有符号整数元素除以 vector `b` (十六个带符号的 8 位整数) 中的有符号整数元素。
///
/// 结果写入 vector (十六个带符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(div_s.b))]
pub unsafe fn __msa_div_s_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_div_s_b(a, mem::transmute(b))
}

/// Vector 有符号分隔
///
/// vector `a` (八个带符号的 16 位整数) 中的有符号整数元素除以 vector `b` (八个带符号的 16 位整数) 中的有符号整数元素。
///
/// 结果写入 vector (八个带符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(div_s.h))]
pub unsafe fn __msa_div_s_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_div_s_h(a, mem::transmute(b))
}

/// Vector 有符号分隔
///
/// vector `a` (四个有符号的 32 位整数) 中的有符号整数元素除以 vector `b` (四个有符号的 32 位整数) 中的有符号整数元素。
///
/// 结果写入 vector (四个带符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(div_s.w))]
pub unsafe fn __msa_div_s_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_div_s_w(a, mem::transmute(b))
}

/// Vector 有符号分隔
///
/// vector `a` (两个有符号的 64 位整数) 中的有符号整数元素除以 vector `b` (两个有符号的 64 位整数) 中的有符号整数元素。
///
/// 结果写入 vector (两个带符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(div_s.d))]
pub unsafe fn __msa_div_s_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_div_s_d(a, mem::transmute(b))
}

/// Vector 无符号除法
///
/// vector `a` 中的无符号整数元素 (十六个无符号 8 位整数) 除以 vector `b` 中的无符号整数元素 (十六个无符号 8 位整数)。
///
/// 结果写入 vector (十六个无符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(div_u.b))]
pub unsafe fn __msa_div_u_b(a: v16u8, b: v16u8) -> v16u8 {
    msa_div_u_b(a, mem::transmute(b))
}

/// Vector 无符号除法
///
/// vector `a` 中的无符号整数元素 (八个无符号 16 位整数) 除以 vector `b` 中的无符号整数元素 (八个无符号 16 位整数)。
///
/// 结果写入 vector (八个无符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(div_u.h))]
pub unsafe fn __msa_div_u_h(a: v8u16, b: v8u16) -> v8u16 {
    msa_div_u_h(a, mem::transmute(b))
}

/// Vector 无符号除法
///
/// vector `a` 中的无符号整数元素 (四个无符号 32 位整数) 除以 vector `b` 中的无符号整数元素 (四个无符号 32 位整数)。
///
/// 结果写入 vector (四个无符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(div_u.w))]
pub unsafe fn __msa_div_u_w(a: v4u32, b: v4u32) -> v4u32 {
    msa_div_u_w(a, mem::transmute(b))
}

/// Vector 无符号除法
///
/// vector `a` (两个无符号的 64 位整数) 中的无符号整数元素除以 vector `b` (两个无符号的 64 位整数) 中的无符号整数元素。
///
/// 结果写入 vector (两个无符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(div_u.d))]
pub unsafe fn __msa_div_u_d(a: v2u64, b: v2u64) -> v2u64 {
    msa_div_u_d(a, mem::transmute(b))
}

/// Vector 有符号点积
///
/// 将 vector `a` (十六个带符号的 8 位整数) 中的有符号整数元素与 vector `b` (十六个带符号的 8 位整数) 中的有符号整数元素相乘，生成结果为输入操作数的大小。
/// 相邻 odd/even 元素的乘法结果相加并存储到目标 vector (八个带符号的 16 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dotp_s.h))]
pub unsafe fn __msa_dotp_s_h(a: v16i8, b: v16i8) -> v8i16 {
    msa_dotp_s_h(a, mem::transmute(b))
}

/// Vector 有符号点积
///
/// 将 vector `a` (八个带符号的 16 位整数) 中的有符号整数元素与 vector `b` (八个带符号的 16 位整数中) 的有符号整数元素相乘，生成结果为输入操作数的大小。
/// 相邻 odd/even 元素的乘法结果相加并存储到目标 vector (四个带符号的 32 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dotp_s.w))]
pub unsafe fn __msa_dotp_s_w(a: v8i16, b: v8i16) -> v4i32 {
    msa_dotp_s_w(a, mem::transmute(b))
}

/// Vector 有符号点积
///
/// 将 vector `a` (四个有符号的 32 位整数) 中的有符号整数元素与 vector `b` (四个有符号的 32 位整数) 中的有符号整数元素相乘，生成结果为输入操作数的大小。
/// 相邻 odd/even 元素的乘法结果相加并存储到目标 vector (两个带符号的 64 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dotp_s.d))]
pub unsafe fn __msa_dotp_s_d(a: v4i32, b: v4i32) -> v2i64 {
    msa_dotp_s_d(a, mem::transmute(b))
}

/// Vector 无符号点积
///
/// 将 vector `a` 中的无符号整数元素 (十六个无符号 8 位整数) 乘以 vector `b` 中的无符号整数元素 (十六个 8 位无符号整数)，得到结果为输入操作数的大小。
/// 相邻 odd/even 元素的乘法结果相加并存储到目标 vector (八个无符号的 16 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dotp_u.h))]
pub unsafe fn __msa_dotp_u_h(a: v16u8, b: v16u8) -> v8u16 {
    msa_dotp_u_h(a, mem::transmute(b))
}

/// Vector 无符号点积
///
/// 将 vector `a` 中的无符号整数元素 (八个无符号 16 位整数) 与 vector `b` 中的无符号整数元素 (八个无符号 16 位整数) 相乘，得出结果为输入操作数的大小。
/// 相邻 odd/even 元素的乘法结果相加并存储到目标 vector (四个无符号的 32 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dotp_u.w))]
pub unsafe fn __msa_dotp_u_w(a: v8u16, b: v8u16) -> v4u32 {
    msa_dotp_u_w(a, mem::transmute(b))
}

/// Vector 无符号点积
///
/// vector `a` (四个无符号的 32 位整数) 中的无符号整数元素与 vector `b` (四个无符号的 32 位整数) 中的无符号整数元素相乘，产生结果为输入操作数的大小。
/// 相邻 odd/even 元素的乘法结果相加并存储到目标 vector (两个无符号的 64 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dotp_u.d))]
pub unsafe fn __msa_dotp_u_d(a: v4u32, b: v4u32) -> v2u64 {
    msa_dotp_u_d(a, mem::transmute(b))
}

/// Vector 有符号点积并添加
///
/// 将 vector `b` (十六个带符号的 8 位整数) 中的有符号整数元素与 vector `c` (十六个带符号的 8 位整数) 中的有符号整数元素相乘，产生的结果是输入操作数大小的两倍。
/// 相邻 odd/even 元素的乘法结果被添加到 vector `a` (八个带符号的 16 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dpadd_s.h))]
pub unsafe fn __msa_dpadd_s_h(a: v8i16, b: v16i8, c: v16i8) -> v8i16 {
    msa_dpadd_s_h(a, mem::transmute(b), c)
}

/// Vector 有符号点积并添加
///
/// vector `b` (八个带符号的 16 位整数) 中的有符号整数元素与 vector `c` (八个带符号的 16 位整数) 中的有符号整数元素相乘，产生的结果是输入操作数大小的两倍。
/// 相邻 odd/even 元素的乘法结果被添加到 vector `a` (四个带符号的 32 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dpadd_s.w))]
pub unsafe fn __msa_dpadd_s_w(a: v4i32, b: v8i16, c: v8i16) -> v4i32 {
    msa_dpadd_s_w(a, mem::transmute(b), c)
}

/// Vector 有符号点积并添加
///
/// vector `b` (四个有符号的 32 位整数) 中的有符号整数元素与 vector `c` (四个有符号的 32 位整数) 中的有符号整数元素相乘，产生的结果是输入操作数大小的两倍。
/// 相邻 odd/even 元素的乘法结果被添加到 vector `a` (两个带符号的 64 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dpadd_s.d))]
pub unsafe fn __msa_dpadd_s_d(a: v2i64, b: v4i32, c: v4i32) -> v2i64 {
    msa_dpadd_s_d(a, mem::transmute(b), c)
}

/// Vector 无符号点积和加法
///
/// vector `b` 中的无符号整数元素 (十六个无符号的 8 位整数) 与 vector `c` 中的无符号整数元素 (十六个无符号 8 位整数) 相乘，其结果是输入操作数大小的两倍。
/// 相邻 odd/even 元素的相乘结果加到 vector `a` (八个无符号的 16 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dpadd_u.h))]
pub unsafe fn __msa_dpadd_u_h(a: v8u16, b: v16u8, c: v16u8) -> v8u16 {
    msa_dpadd_u_h(a, mem::transmute(b), c)
}

/// Vector 无符号点积和加法
///
/// vector `b` 中的无符号整数元素 (八个无符号 16 位整数) 与 vector `c` 中的无符号整数元素 (八个无符号 16 位整数) 相乘，产生的结果是输入操作数大小的两倍。
/// 相邻 odd/even 元素的乘法结果被添加到 vector `a` (四个无符号的 32 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dpadd_u.w))]
pub unsafe fn __msa_dpadd_u_w(a: v4u32, b: v8u16, c: v8u16) -> v4u32 {
    msa_dpadd_u_w(a, mem::transmute(b), c)
}

/// Vector 无符号点积和加法
///
/// vector `b` (四个无符号的 32 位整数) 中的无符号整数元素与 vector `c` (四个无符号的 32 位整数) 中的无符号整数元素相乘，产生的结果是输入操作数大小的两倍。
/// 相邻 odd/even 元素的乘法结果被添加到 vector `a` (两个无符号的 64 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dpadd_u.d))]
pub unsafe fn __msa_dpadd_u_d(a: v2u64, b: v4u32, c: v4u32) -> v2u64 {
    msa_dpadd_u_d(a, mem::transmute(b), c)
}

/// Vector 有符号点积并添加
///
/// 将 vector `b` (十六个带符号的 8 位整数) 中的有符号整数元素与 vector `c` (十六个带符号的 8 位整数) 中的有符号整数元素相乘，产生的结果是输入操作数大小的两倍。
/// 从 vector `a` (八个带符号的 16 位整数) 中的整数元素中减去相邻 odd/even 元素的乘法结果。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dpsub_s.h))]
pub unsafe fn __msa_dpsub_s_h(a: v8i16, b: v16i8, c: v16i8) -> v8i16 {
    msa_dpsub_s_h(a, mem::transmute(b), c)
}

/// Vector 有符号点积并添加
///
/// vector `b` (八个带符号的 16 位整数) 中的有符号整数元素与 vector `c` (八个带符号的 16 位整数) 中的有符号整数元素相乘，产生的结果是输入操作数大小的两倍。
/// 从 vector `a` 中的整数元素 (四个带符号的 32 位整数) 中减去相邻 odd/even 元素的乘法结果。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dpsub_s.w))]
pub unsafe fn __msa_dpsub_s_w(a: v4i32, b: v8i16, c: v8i16) -> v4i32 {
    msa_dpsub_s_w(a, mem::transmute(b), c)
}

/// Vector 有符号点积并添加
///
/// vector `b` (四个有符号的 32 位整数) 中的有符号整数元素与 vector `c` (四个有符号的 32 位整数) 中的有符号整数元素相乘，产生的结果是输入操作数大小的两倍。
/// 从 vector `a` 中的整数元素 (两个带符号的 64 位整数) 中减去相邻 odd/even 元素的乘法结果。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dpsub_s.d))]
pub unsafe fn __msa_dpsub_s_d(a: v2i64, b: v4i32, c: v4i32) -> v2i64 {
    msa_dpsub_s_d(a, mem::transmute(b), c)
}

/// Vector 无符号点积和加法
///
/// vector `b` 中的无符号整数元素 (十六个无符号的 8 位整数) 与 vector `c` 中的无符号整数元素 (十六个无符号 8 位整数) 相乘，其结果是输入操作数大小的两倍。
/// 从 vector `a` (八个带符号的 16 位整数) 中的整数元素中减去相邻 odd/even 元素的乘法结果。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dpsub_u.h))]
pub unsafe fn __msa_dpsub_u_h(a: v8i16, b: v16u8, c: v16u8) -> v8i16 {
    msa_dpsub_u_h(a, mem::transmute(b), c)
}

/// Vector 无符号点积和加法
///
/// vector `b` 中的无符号整数元素 (八个无符号 16 位整数) 与 vector `c` 中的无符号整数元素 (八个无符号 16 位整数) 相乘，产生的结果是输入操作数大小的两倍。
/// 从 vector `a` 中的整数元素 (四个带符号的 32 位整数) 中减去相邻 odd/even 元素的乘法结果。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dpsub_u.w))]
pub unsafe fn __msa_dpsub_u_w(a: v4i32, b: v8u16, c: v8u16) -> v4i32 {
    msa_dpsub_u_w(a, mem::transmute(b), c)
}

/// Vector 无符号点积和加法
///
/// vector `b` (四个无符号的 32 位整数) 中的无符号整数元素与 vector `c` (四个无符号的 32 位整数) 中的无符号整数元素相乘，产生的结果是输入操作数大小的两倍。
/// 从 vector `a` 中的整数元素 (两个带符号的 64 位整数) 中减去相邻 odd/even 元素的乘法结果。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(dpsub_u.d))]
pub unsafe fn __msa_dpsub_u_d(a: v2i64, b: v4u32, c: v4u32) -> v2i64 {
    msa_dpsub_u_d(a, mem::transmute(b), c)
}

/// Vector 浮点加法
///
/// vector `a` (四个 32 位浮点数) 中的浮点元素被添加到 `bc` (四个 32 位浮点数) 中的浮点元素。
///
/// 将结果写入 vector (四个 32 位浮点数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fadd.w))]
pub unsafe fn __msa_fadd_w(a: v4f32, b: v4f32) -> v4f32 {
    msa_fadd_w(a, mem::transmute(b))
}

/// Vector 浮点加法
///
/// vector `a` (两个 64 位浮点数) 中的浮点元素被添加到 `bc` (两个 64 位浮点数) 中的浮点元素。
///
/// 结果写入 vector (两个 64 位浮点数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fadd.d))]
pub unsafe fn __msa_fadd_d(a: v2f64, b: v2f64) -> v2f64 {
    msa_fadd_d(a, mem::transmute(b))
}

/// Vector 浮点静态比较始终为 False
///
/// 将 vector 中的所有位设置为 0 (四个带符号的 32 位整数)。
/// 在 `a` (四个 32 位浮点数) 或 `b` (四个 32 位浮点数) 中用信号通知 NaN 元素表示无效操作异常。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcaf.w))]
pub unsafe fn __msa_fcaf_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fcaf_w(a, mem::transmute(b))
}

/// Vector 浮点静态比较始终为 False
///
/// 将 vector 中的所有位设置为 0 (两个带符号的 64 位整数)。
/// 在 `a` (两个 64 位浮点数) 或 `b` (两个 64 位浮点数) 中用信号通知 NaN 元素表示无效操作异常。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcaf.d))]
pub unsafe fn __msa_fcaf_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fcaf_d(a, mem::transmute(b))
}

/// Vector 浮点静态等于比较
///
/// 如果 `a` (四个 32 位浮点数) 和 `b` (四个 32 位浮点数) 元素中的对应元素已排序且相等，则将 vector (四个带符号的 32 位整数) 元素中的所有位设置为 1，否则进行设置所有位都为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fceq.w))]
pub unsafe fn __msa_fceq_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fceq_w(a, mem::transmute(b))
}

/// Vector 浮点静态等于比较
///
/// 如果 `a` (两个 64 位浮点数字) 和 `b` (两个 64 位浮点数字) 元素中的对应元素已排序且相等，则将 vector (两个带符号的 64 位整数数字) 元素中的所有位设置为 1，否则进行设置所有位都为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fceq.d))]
pub unsafe fn __msa_fceq_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fceq_d(a, mem::transmute(b))
}

/// Vector 浮点类掩码
///
/// 在 vector (四个带符号的 32 位整数) 的每个元素中存储一个位掩码，该掩码反映 vector `a` (四个 32 位浮点数) 的相应元素的浮点类。
///
/// 掩码具有 10 位，如下所示。位 0 和 1 表示 NaN 值：发信号通知 NaN (位 0) 和静默 NaN (位 1)。
/// 位 2、3、4、5 对负值进行分类：无限 (位 2)，正常 (位 3)，正常以下 (位 4) 和零 (位 5)。
/// 位 6、7、8、9 对正值进行分类：无穷大 (位 6)，正常 (位 7)，正常以下 (位 8) 和零 (位 9)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fclass.w))]
pub unsafe fn __msa_fclass_w(a: v4f32) -> v4i32 {
    msa_fclass_w(a)
}

/// Vector 浮点类掩码
///
/// 在 vector (两个带符号的 64 位整数) 的每个元素中存储一个位掩码，该掩码反映 vector `a` (两个 64 位浮点数) 的相应元素的浮点类。
///
/// 掩码具有 10 位，如下所示。位 0 和 1 表示 NaN 值：发信号通知 NaN (位 0) 和静默 NaN (位 1)。
/// 位 2、3、4、5 对负值进行分类：无限 (位 2)，正常 (位 3)，正常以下 (位 4) 和零 (位 5)。
/// 位 6、7、8、9 对正值进行分类：无穷大 (位 6)，正常 (位 7)，正常以下 (位 8) 和零 (位 9)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fclass.d))]
pub unsafe fn __msa_fclass_d(a: v2f64) -> v2i64 {
    msa_fclass_d(a)
}

/// Vector 浮点静态少于或相等比较
///
/// 如果订购了相应的 `a` (四个 32 位浮点数) 元素且小于或等于 `b` (四个 32 位浮点数)，则在 vector (四个带符号的 32 位整数) 元素中将所有位设置为 1。元素，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcle.w))]
pub unsafe fn __msa_fcle_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fcle_w(a, mem::transmute(b))
}

/// Vector 浮点静态少于或相等比较
///
/// 如果订购了相应的 `a` (两个 64 位浮点数) 元素且小于或等于 `b` (两个 64 位浮点数)，则在 vector (两个带符号的 64 位整数) 元素中将所有位设置为 1。元素，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcle.d))]
pub unsafe fn __msa_fcle_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fcle_d(a, mem::transmute(b))
}

/// Vector 浮点静态少于比较
///
/// 如果订购了相应的 `a` (四个 32 位浮点数) 元素且小于 `b` (四个 32 位浮点数) 元素，则将 vector (四个带符号的 32 位整数) 元素中的所有位都设置为 1，否则进行设置所有位都为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fclt.w))]
pub unsafe fn __msa_fclt_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fclt_w(a, mem::transmute(b))
}

/// Vector 浮点静态少于比较
///
/// 如果订购了相应的 `a` (两个 64 位浮点数字) 元素且小于 `b` (两个 64 位浮点数字) 元素，则将 vector (两个带符号的 64 位整数数字) 元素中的所有位设置为 1，否则设置为 `b` (两个 64 位浮点数字) 元素。所有位都为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fclt.d))]
pub unsafe fn __msa_fclt_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fclt_d(a, mem::transmute(b))
}

/// Vector 浮点静态比较不等于
///
/// 如果相应的 `a` (四个 32 位浮点数) 和 `b` (四个 32 位浮点数) 元素已排序且不相等，则将 vector (四个带符号的 32 位整数) 元素中的所有位设置为 1，否则进行设置所有位都为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcne.w))]
pub unsafe fn __msa_fcne_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fcne_w(a, mem::transmute(b))
}

/// Vector 浮点静态比较不等于
///
/// 如果相应的 `a` (两个 64 位浮点数) 和 `b` (两个 64 位浮点数) 元素是有序且不相等的，则将 vector (两个带符号的 64 位整数) 元素中的所有位设置为 1，否则进行设置所有位都为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcne.d))]
pub unsafe fn __msa_fcne_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fcne_d(a, mem::transmute(b))
}

/// Vector 浮点静态有序比较
///
/// 如果订购了相应的 `a` (四个 32 位浮点数) 和 `b` (四个 32 位浮点数) 元素，则将 vector (四个带符号的 32 位整数) 元素中的所有位设置为 1，即
///
/// 这两个元素都不是 NaN 值，否则将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcor.w))]
pub unsafe fn __msa_fcor_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fcor_w(a, mem::transmute(b))
}

/// Vector 浮点静态有序比较
///
/// 如果订购了相应的 `a` (两个 64 位浮点数) 和 `b` (两个 64 位浮点数) 元素，则将 vector (两个带符号的 64 位整数) 元素中的所有位设置为 1，即
///
/// 这两个元素都不是 NaN 值，否则将所有位设置为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcor.d))]
pub unsafe fn __msa_fcor_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fcor_d(a, mem::transmute(b))
}

/// Vector 浮点静态无序或相等比较
///
/// 如果相应的 `a` (四个 32 位浮点数) 和 `b` (四个 32 位浮点数) 元素无序或相等，则将 vector (四个带符号的 32 位整数) 元素中的所有位设置为 1，否则设置所有位为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcueq.w))]
pub unsafe fn __msa_fcueq_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fcueq_w(a, mem::transmute(b))
}

/// Vector 浮点静态无序或相等比较
///
/// 如果相应的 `a` (两个 64 位浮点数) 和 `b` (两个 64 位浮点数) 元素无序或相等，则在 vector (两个带符号的 64 位整数) 元素中将所有位设置为 1，否则设置所有位为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcueq.d))]
pub unsafe fn __msa_fcueq_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fcueq_d(a, mem::transmute(b))
}

/// Vector 浮点静态无序或更少或相等比较
///
/// 如果 `a` (四个 32 位浮点数) 中的对应元素无序或小于或等于 `b` (四个 32 位浮点数)，则将 vector (四个带符号的 32 位整数) 元素中的所有位设置为 1。元素，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcule.w))]
pub unsafe fn __msa_fcule_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fcule_w(a, mem::transmute(b))
}

/// Vector 浮点静态无序或更少或相等比较
///
/// 如果 `a` (两个 64 位浮点数) 中的相应元素无序或小于或等于 `b` (两个 64 位浮点数)，则将 vector (两个有符号的 64 位整数) 元素中的所有位设置为 1。元素，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcule.d))]
pub unsafe fn __msa_fcule_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fcule_d(a, mem::transmute(b))
}

/// Vector 浮点静态无序或小于比较
///
/// 如果 `a` (四个 32 位浮点数) 中的相应元素是无序的或小于 `b` (四个 32 位浮点数) 元素，则将 vector (四个带符号的 32 位整数) 元素中的所有位设置为 1，否则，将其设置为零。将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcult.w))]
pub unsafe fn __msa_fcult_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fcult_w(a, mem::transmute(b))
}

/// Vector 浮点静态无序或小于比较
///
/// 如果 `a` (两个 64 位浮点数) 中的相应元素是无序的或小于 `b` (两个 64 位浮点数) 元素，则将 vector (两个有符号的 64 位整数) 元素中的所有位设置为 1，否则，将 vector (两个有符号的 64 位整数) 中的所有元素设置为 1。将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcult.d))]
pub unsafe fn __msa_fcult_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fcult_d(a, mem::transmute(b))
}

/// Vector 浮点静态无序比较
///
/// 如果相应的 `a` (四个 32 位浮点数) 和 `b` (四个 32 位浮点数) 元素是无序的，则将 vector (四个带符号的 32 位整数) 元素中的所有位设置为 1。
/// 至少一个元素为 NaN 值，否则将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcun.w))]
pub unsafe fn __msa_fcun_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fcun_w(a, mem::transmute(b))
}

/// Vector 浮点静态无序比较
///
/// 如果相应的 `a` (两个 64 位浮点数) 和 `b` (两个 64 位浮点数) 元素是无序的，则将 vector (两个带符号的 64 位整数) 元素中的所有位设置为 1。
/// 至少一个元素为 NaN 值，否则将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcun.d))]
pub unsafe fn __msa_fcun_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fcun_d(a, mem::transmute(b))
}

/// Vector 浮点比较无序或不相等
///
/// 如果相应的 `a` (四个 32 位浮点数) 和 `b` (四个 32 位浮点数) 元素无序或不相等，则将 vector (四个带符号的 32 位整数) 元素中的所有位设置为 1，否则进行设置所有位都为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcune.w))]
pub unsafe fn __msa_fcune_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fcune_w(a, mem::transmute(b))
}

/// Vector 浮点比较无序或不相等
///
/// 如果相应的 `a` (两个 64 位浮点数) 和 `b` (两个 64 位浮点数) 元素无序或不相等，则在 vector (两个带符号的 64 位整数) 元素中将所有位设置为 1，否则进行设置所有位都为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fcune.d))]
pub unsafe fn __msa_fcune_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fcune_d(a, mem::transmute(b))
}

/// Vector 浮点除法
///
/// vector `a` (四个 32 位浮点数) 中的浮点元素除以 vector `b` (四个 32 位浮点数) 中的浮点元素。
///
/// 将结果写入 vector (四个 32 位浮点数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fdiv.w))]
pub unsafe fn __msa_fdiv_w(a: v4f32, b: v4f32) -> v4f32 {
    msa_fdiv_w(a, mem::transmute(b))
}

/// Vector 浮点除法
///
/// vector `a` (两个 64 位浮点数) 中的浮点元素除以 vector `b` (两个 64 位浮点数) 中的浮点元素。
///
/// 结果写入 vector (两个 64 位浮点数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fdiv.d))]
pub unsafe fn __msa_fdiv_d(a: v2f64, b: v2f64) -> v2f64 {
    msa_fdiv_d(a, mem::transmute(b))
}

/* FIXME: 16-bit float
/// Vector 浮点下转换转换格式
///
/// vector `a` (四个 64 位浮点数) 和 vector `b` (四个 64 位浮点数) 中的浮点元素被下转换为较小的交换格式，即
/// 从 64 位到 32 位，或从 32 位到 16 位。
/// 将结果写入 vector (8 个 16 位浮点数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fexdo.h))]
pub unsafe fn __msa_fexdo_h(a: v4f32, b: v4f32) -> f16x8 {
    msa_fexdo_h(a, mem::transmute(b))
}*/

/// Vector 浮点下转换转换格式
///
/// vector `a` (两个 64 位浮点数) 和 vector `b` (两个 64 位浮点数) 中的浮点元素被下转换为较小的交换格式，即
/// 从 64 位到 32 位，或从 32 位到 16 位。
/// 将结果写入 vector (四个 32 位浮点数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fexdo.w))]
pub unsafe fn __msa_fexdo_w(a: v2f64, b: v2f64) -> v4f32 {
    msa_fexdo_w(a, mem::transmute(b))
}

/// Vector 浮点下转换转换格式
///
/// vector `a` (四个 32 位浮点数) 中的浮点元素被缩放，即
/// 将 vector `b` 中的整数元素的幂乘以 2 (四个有符号的 32 位整数)。
///
/// 将结果写入 vector (四个 32 位浮点数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fexp2.w))]
pub unsafe fn __msa_fexp2_w(a: v4f32, b: v4i32) -> v4f32 {
    msa_fexp2_w(a, mem::transmute(b))
}

/// Vector 浮点下转换转换格式
///
/// vector `a` (两个 64 位浮点数) 中的浮点元素被缩放，即
/// 将 vector `b` 中的整数元素的幂乘以 2 (两个有符号的 64 位整数)。
///
/// 结果写入 vector (两个 64 位浮点数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fexp2.d))]
pub unsafe fn __msa_fexp2_d(a: v2f64, b: v2i64) -> v2f64 {
    msa_fexp2_d(a, mem::transmute(b))
}

/* FIXME: 16-bit float
/// Vector 浮点上转换转换格式左
///
/// vector `a` 中的左半个浮点元素 (两个 16 位浮点数) 被上转换为更大的交换格式，即
/// 从 16 位到 32 位，或从 32 位到 64 位。
/// 将结果写入 vector (四个 32 位浮点数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fexupl.w))]
pub unsafe fn __msa_fexupl_w(a: f16x8) -> v4f32 {
    msa_fexupl_w(a)
}*/

/// Vector 浮点上转换转换格式左
///
/// vector `a` 中的左半个浮点元素 (四个 32 位浮点数) 被上转换为更大的交换格式，即
/// 从 16 位到 32 位，或从 32 位到 64 位。
/// 结果写入 vector (两个 64 位浮点数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fexupl.d))]
pub unsafe fn __msa_fexupl_d(a: v4f32) -> v2f64 {
    msa_fexupl_d(a)
}

/* FIXME: 16-bit float
/// Vector 浮点上转换转换格式左
///
/// vector `a` 中的右半个浮点元素 (两个 16 位浮点数) 被上转换为更大的交换格式，即
/// 从 16 位到 32 位，或从 32 位到 64 位。
/// 将结果写入 vector (四个 32 位浮点数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fexupr.w))]
pub unsafe fn __msa_fexupr_w(a: f16x8) -> v4f32 {
    msa_fexupr_w(a)
} */

/// Vector 浮点上转换转换格式左
///
/// vector `a` 中的右半个浮点元素 (四个 32 位浮点数) 被上转换为更大的交换格式，即
/// 从 16 位到 32 位，或从 32 位到 64 位。
/// 结果写入 vector (两个 64 位浮点数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fexupr.d))]
pub unsafe fn __msa_fexupr_d(a: v4f32) -> v2f64 {
    msa_fexupr_d(a)
}

/// Vector 浮点数舍入并从有符号整数转换
///
/// vector `a` 中的带符号整数元素 (四个带符号的 32 位整数) 将转换为浮点值。
///
/// 将结果写入 vector (四个 32 位浮点数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ffint_s.w))]
pub unsafe fn __msa_ffint_s_w(a: v4i32) -> v4f32 {
    msa_ffint_s_w(a)
}

/// Vector 浮点数舍入并从有符号整数转换
///
/// vector `a` 中的带符号整数元素 (两个带符号的 64 位整数) 将转换为浮点值。
///
/// 结果写入 vector (两个 64 位浮点数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ffint_s.d))]
pub unsafe fn __msa_ffint_s_d(a: v2i64) -> v2f64 {
    msa_ffint_s_d(a)
}

/// Vector 浮点数舍入并从无符号整数转换
///
/// vector `a` 中的无符号整数元素 (四个无符号 32 位整数) 将转换为浮点值。
///
/// 将结果写入 vector (四个 32 位浮点数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ffint_u.w))]
pub unsafe fn __msa_ffint_u_w(a: v4u32) -> v4f32 {
    msa_ffint_u_w(a)
}

/// Vector 浮点数舍入并从无符号整数转换
///
/// vector `a` 中的无符号整数元素 (两个无符号 64 位整数) 将转换为浮点值。
///
/// 结果写入 vector (两个 64 位浮点数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ffint_u.d))]
pub unsafe fn __msa_ffint_u_d(a: v2u64) -> v2f64 {
    msa_ffint_u_d(a)
}

/// Vector 浮点数从定点左转换
///
/// vector `a` (八个带符号的 16 位整数) 中的左半个定点元素被上转换为浮点数据格式。
///
/// 即，从 16 位 Q15 到 32 位浮点，或从 32 位 Q31 到 64 位浮点。
/// 将结果写入 vector (四个 32 位浮点数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ffql.w))]
pub unsafe fn __msa_ffql_w(a: v8i16) -> v4f32 {
    msa_ffql_w(a)
}

/// Vector 浮点数从定点左转换
///
/// vector `a` 的左半个定点元素 (四个带符号的 32 位整数) 被上转换为浮点数据格式。
///
/// 即，从 16 位 Q15 到 32 位浮点，或从 32 位 Q31 到 64 位浮点。
/// 结果写入 vector (两个 64 位浮点数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ffql.d))]
pub unsafe fn __msa_ffql_d(a: v4i32) -> v2f64 {
    msa_ffql_d(a)
}

/// Vector 浮点数从定点左转换
///
/// vector `a` 中的右半定点元素 (八个带符号的 16 位整数) 被上转换为浮点数据格式。
///
/// 即，从 16 位 Q15 到 32 位浮点，或从 32 位 Q31 到 64 位浮点。
/// 将结果写入 vector (四个 32 位浮点数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ffqr.w))]
pub unsafe fn __msa_ffqr_w(a: v8i16) -> v4f32 {
    msa_ffqr_w(a)
}

/// Vector 浮点数从定点左转换
///
/// vector `a` 中的右半定点元素 (四个带符号的 32 位整数) 被上转换为浮点数据格式。
///
/// 即，从 16 位 Q15 到 32 位浮点，或从 32 位 Q31 到 64 位浮点。
/// 结果写入 vector (两个 64 位浮点数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ffqr.d))]
pub unsafe fn __msa_ffqr_d(a: v4i32) -> v2f64 {
    msa_ffqr_d(a)
}

/// Vector 来自 GPR 的填充
///
/// 将 GPR rs 值复制到 vector 中的所有元素 (十六个带符号的 8 位整数)。
/// 如果源 GPR 比目标数据格式宽，则目标元素将设置为 GPR 的最低有效位。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fill.b))]
pub unsafe fn __msa_fill_b(a: i32) -> v16i8 {
    msa_fill_b(a)
}

/// Vector 来自 GPR 的填充
///
/// 将 GPR rs 值复制到 vector 中的所有元素 (八个带符号的 16 位整数)。
/// 如果源 GPR 比目标数据格式宽，则目标元素将设置为 GPR 的最低有效位。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fill.h))]
pub unsafe fn __msa_fill_h(a: i32) -> v8i16 {
    msa_fill_h(a)
}

/// Vector 来自 GPR 的填充
///
/// 将 GPR rs 值复制到 vector 中的所有元素 (四个带符号的 32 位整数)。
/// 如果源 GPR 比目标数据格式宽，则目标元素将设置为 GPR 的最低有效位。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fill.w))]
pub unsafe fn __msa_fill_w(a: i32) -> v4i32 {
    msa_fill_w(a)
}

/// Vector 来自 GPR 的填充
///
/// 将 GPR rs 值复制到 vector 中的所有元素 (两个带符号的 64 位整数)。
/// 如果源 GPR 比目标数据格式宽，则目标元素将设置为 GPR 的最低有效位。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fill.d))]
pub unsafe fn __msa_fill_d(a: i64) -> v2i64 {
    msa_fill_d(a)
}

/// Vector 浮点以 2 为底的对数
///
/// vector `a` (四个 32 位浮点数) 中带符号的浮点元素的基数为 2 的指数被作为浮点值写入 vector 元素 (四个 32 位浮点数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(flog2.w))]
pub unsafe fn __msa_flog2_w(a: v4f32) -> v4f32 {
    msa_flog2_w(a)
}

/// Vector 浮点以 2 为底的对数
///
/// vector `a` (两个 64 位浮点数) 中带符号的浮点元素的基数为 2 的指数被作为浮点值写入 vector 元素 (两个 64 位浮点数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(flog2.d))]
pub unsafe fn __msa_flog2_d(a: v2f64) -> v2f64 {
    msa_flog2_d(a)
}

/// Vector 浮点乘以相加
///
/// vector `b` (四个 32 位浮点数) 中的浮点元素与 vector `c` (四个 32 位浮点数) 中的浮点元素相乘的值被添加到 vector `a` (四个 32 - 位浮点数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fmadd.w))]
pub unsafe fn __msa_fmadd_w(a: v4f32, b: v4f32, c: v4f32) -> v4f32 {
    msa_fmadd_w(a, mem::transmute(b), c)
}

/// Vector 浮点乘以相加
///
/// 将 vector `b` (两个 64 位浮点数) 中的浮点元素与 vector `c` (两个 64 位浮点数) 中的浮点元素相乘，再添加到 vector `a` (两个 64 - 位浮点数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fmadd.d))]
pub unsafe fn __msa_fmadd_d(a: v2f64, b: v2f64, c: v2f64) -> v2f64 {
    msa_fmadd_d(a, mem::transmute(b), c)
}

/// Vector 浮点最大值
///
/// vector `a` (四个 32 位浮点数) 和 vector `b` (四个 32 位浮点数) 中相应浮点元素之间的最大值被写入 vector (四个 32 位浮点数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fmax.w))]
pub unsafe fn __msa_fmax_w(a: v4f32, b: v4f32) -> v4f32 {
    msa_fmax_w(a, mem::transmute(b))
}

/// Vector 浮点最大值
///
/// vector `a` (两个 64 位浮点数) 和 vector `b` (两个 64 位浮点数) 中相应浮点元素之间的最大值被写入 vector (两个 64 位浮点数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fmax.d))]
pub unsafe fn __msa_fmax_d(a: v2f64, b: v2f64) -> v2f64 {
    msa_fmax_d(a, mem::transmute(b))
}

/// Vector 基于绝对值的浮点最大值
///
/// 幅度最大的值，即
/// 将 vector `a` (四个 32 位浮点数) 和 vector `b` (四个 32 位浮点数) 中相应浮点元素之间的绝对值写入 vector (四个 32 位浮点数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fmax_a.w))]
pub unsafe fn __msa_fmax_a_w(a: v4f32, b: v4f32) -> v4f32 {
    msa_fmax_a_w(a, mem::transmute(b))
}

/// Vector 基于绝对值的浮点最大值
///
/// 幅度最大的值，即
/// 将 vector `a` (两个 64 位浮点数) 和 vector `b` (两个 64 位浮点数) 中相应浮点元素之间的绝对值写入 vector (两个 64 位浮点数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fmax_a.d))]
pub unsafe fn __msa_fmax_a_d(a: v2f64, b: v2f64) -> v2f64 {
    msa_fmax_a_d(a, mem::transmute(b))
}

/// Vector 最小浮点数
///
/// vector `a` (四个 32 位浮点数) 和 vector `b` (四个 32 位浮点数) 中相应浮点元素之间的最小值被写入 vector (四个 32 位浮点数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fmin.w))]
pub unsafe fn __msa_fmin_w(a: v4f32, b: v4f32) -> v4f32 {
    msa_fmin_w(a, mem::transmute(b))
}

/// Vector 最小浮点数
///
/// vector `a` (两个 64 位浮点数) 和 vector `b` (两个 64 位浮点数) 中相应浮点元素之间的最小值被写入 vector (两个 64 位浮点数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fmin.d))]
pub unsafe fn __msa_fmin_d(a: v2f64, b: v2f64) -> v2f64 {
    msa_fmin_d(a, mem::transmute(b))
}

/// 基于绝对值的 Vector 浮点最小值
///
/// 幅度最小的值，即
/// 将 vector `a` (四个 32 位浮点数) 和 vector `b` (四个 32 位浮点数) 中相应浮点元素之间的绝对值写入 vector (四个 32 位浮点数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fmin_a.w))]
pub unsafe fn __msa_fmin_a_w(a: v4f32, b: v4f32) -> v4f32 {
    msa_fmin_a_w(a, mem::transmute(b))
}

/// 基于绝对值的 Vector 浮点最小值
///
/// 幅度最小的值，即
/// 将 vector `a` (两个 64 位浮点数) 和 vector `b` (两个 64 位浮点数) 中相应浮点元素之间的绝对值写入 vector (两个 64 位浮点数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fmin_a.d))]
pub unsafe fn __msa_fmin_a_d(a: v2f64, b: v2f64) -> v2f64 {
    msa_fmin_a_d(a, mem::transmute(b))
}

/// Vector 浮点乘减
///
/// 从 vector `a` (四个 32 位浮点数) 中减去 vector `b` (四个 32 位浮点数) 中的浮点元素乘以 vector `c` (四个 32 位浮点数) 中的浮点元素。位浮点数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fmsub.w))]
pub unsafe fn __msa_fmsub_w(a: v4f32, b: v4f32, c: v4f32) -> v4f32 {
    msa_fmsub_w(a, mem::transmute(b), c)
}

/// Vector 浮点乘减
///
/// vector `b` (两个 64 位浮点数) 中的浮点元素与 vector `c` (两个 64 位浮点数) 中的浮点元素相乘，则从 vector `a` (两个 64 - 位浮点数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fmsub.d))]
pub unsafe fn __msa_fmsub_d(a: v2f64, b: v2f64, c: v2f64) -> v2f64 {
    msa_fmsub_d(a, mem::transmute(b), c)
}

/// Vector 浮点乘法
///
/// vector `a` (四个 32 位浮点数) 中的浮点元素与 vector `b` (四个 32 位浮点数) 中的浮点元素相乘。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fmul.w))]
pub unsafe fn __msa_fmul_w(a: v4f32, b: v4f32) -> v4f32 {
    msa_fmul_w(a, mem::transmute(b))
}

/// Vector 浮点乘法
///
/// vector `a` (两个 64 位浮点数) 中的浮点元素与 vector `b` (两个 64 位浮点数) 中的浮点元素相乘。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fmul.d))]
pub unsafe fn __msa_fmul_d(a: v2f64, b: v2f64) -> v2f64 {
    msa_fmul_d(a, mem::transmute(b))
}

/// Vector 浮点数舍入为整数
///
/// 基于 MSA 控制和状态寄存器 MSACSR 中的舍入模式位 RM，vector `a` 中的浮点元素 (四个 32 位浮点数) 将以相同格式四舍五入为整数值的浮点数。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(frint.w))]
pub unsafe fn __msa_frint_w(a: v4f32) -> v4f32 {
    msa_frint_w(a)
}

/// Vector 浮点数舍入为整数
///
/// 基于 MSA 控制和状态寄存器 MSACSR 中的舍入模式位 RM，vector `a` 中的浮点元素 (两个 64 位浮点数) 以相同格式舍入为整数值的浮点数。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(frint.d))]
pub unsafe fn __msa_frint_d(a: v2f64) -> v2f64 {
    msa_frint_d(a)
}

/// Vector 近似浮点倒数
///
/// 计算 vector `a` (四个 32 位浮点数) 中浮点元素的倒数，并将结果写入 vector (四个 32 位浮点数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(frcp.w))]
pub unsafe fn __msa_frcp_w(a: v4f32) -> v4f32 {
    msa_frcp_w(a)
}

/// Vector 近似浮点倒数
///
/// 计算 vector `a` (两个 64 位浮点数) 中浮点元素的倒数，并将结果写入 vector (两个 64 位浮点数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(frcp.d))]
pub unsafe fn __msa_frcp_d(a: v2f64) -> v2f64 {
    msa_frcp_d(a)
}

/// Vector 平方根的近似浮点倒数
///
/// 计算 vector `a` (四个 32 位浮点数) 中浮点元素的平方根的倒数，并将结果写入 vector (四个 32 位浮点数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(frsqrt.w))]
pub unsafe fn __msa_frsqrt_w(a: v4f32) -> v4f32 {
    msa_frsqrt_w(a)
}

/// Vector 平方根的近似浮点倒数
///
/// 计算 vector `a` (两个 64 位浮点数) 中浮点元素的平方根的倒数，并将结果写入 vector (两个 64 位浮点数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(frsqrt.d))]
pub unsafe fn __msa_frsqrt_d(a: v2f64) -> v2f64 {
    msa_frsqrt_d(a)
}

/// Vector 浮点信号比较始终为 false
///
/// 将 vector (四个带符号的 32 位整数) 元素中的所有位设置为 0。
/// vector `a` (四个 32 位浮点数) 或 `b` (四个 32 位浮点数) 中的信号和静态 NaN 元素发出无效操作异常的信号。
///
/// 如果出现浮点异常，则默认结果会将所有位设置为 0。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsaf.w))]
pub unsafe fn __msa_fsaf_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fsaf_w(a, mem::transmute(b))
}

/// Vector 浮点信号比较始终为 false
///
/// 将 vector (两个带符号的 64 位整数) 元素中的所有位设置为 0。
/// vector `a` (两个 64 位浮点数) 或 `b` (两个 64 位浮点数) 中的信号和静态 NaN 元素发出无效操作异常的信号。
///
/// 如果出现浮点异常，则默认结果会将所有位设置为 0。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsaf.d))]
pub unsafe fn __msa_fsaf_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fsaf_d(a, mem::transmute(b))
}

/// Vector 浮点信号比较相等
///
/// 如果相应的 `a` (四个 32 位浮点数) 和 `b` (四个 32 位浮点数) 元素相等，则在 vector (四个带符号的 32 位整数) 元素中将所有位设置为 1，否则将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fseq.w))]
pub unsafe fn __msa_fseq_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fseq_w(a, mem::transmute(b))
}

/// Vector 浮点信号比较相等
///
/// 如果相应的 `a` (两个 64 位浮点数) 和 `b` (两个 64 位浮点数) 元素相等，则在 vector (两个带符号的 64 位整数) 元素中将所有位设置为 1，否则将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fseq.d))]
pub unsafe fn __msa_fseq_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fseq_d(a, mem::transmute(b))
}

/// Vector 浮点信号比较小于或等于
///
/// 如果相应的 `a` (四个 32 位浮点数) 元素小于或等于 `b` (四个 32 位浮点数) 元素，则在 vector (四个带符号的 32 位整数) 元素中将所有位设置为 1，否则，将其设置为 1。将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsle.w))]
pub unsafe fn __msa_fsle_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fsle_w(a, mem::transmute(b))
}

/// Vector 浮点信号比较小于或等于
///
/// 如果相应的 `a` (两个 64 位浮点数) 元素小于或等于 `b` (两个 64 位浮点数) 元素，则在 vector (两个带符号的 64 位整数) 元素中将所有位设置为 1，否则，将其设置为 1。将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsle.d))]
pub unsafe fn __msa_fsle_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fsle_d(a, mem::transmute(b))
}

/// Vector 浮点信令小于
///
/// 如果相应的 `a` (四个 32 位浮点数) 元素小于 `b` (四个 32 位浮点数) 元素，则在 vector (四个带符号的 32 位整数) 元素中将所有位设置为 1，否则设置所有位到 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fslt.w))]
pub unsafe fn __msa_fslt_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fslt_w(a, mem::transmute(b))
}

/// Vector 浮点信令小于
///
/// 如果相应的 `a` (两个 64 位浮点数) 元素小于 `b` (两个 64 位浮点数) 元素，则在 vector (两个带符号的 64 位整数) 元素中将所有位设置为 1，否则设置所有位到 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fslt.d))]
pub unsafe fn __msa_fslt_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fslt_d(a, mem::transmute(b))
}

/// Vector 浮点信号比较不相等
///
/// 如果相应的 `a` (四个 32 位浮点数) 和 `b` (四个 32 位浮点数) 元素不相等，则在 vector (四个带符号的 32 位整数) 元素中将所有位设置为 1，否则将所有位设置为 1。到 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsne.w))]
pub unsafe fn __msa_fsne_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fsne_w(a, mem::transmute(b))
}

/// Vector 浮点信号比较不相等
///
/// 如果相应的 `a` (两个 64 位浮点数) 和 `b` (两个 64 位浮点数) 元素不相等，则在 vector (两个带符号的 64 位整数) 元素中将所有位设置为 1，否则设置所有位到 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsne.d))]
pub unsafe fn __msa_fsne_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fsne_d(a, mem::transmute(b))
}

/// Vector 浮点信令比较已排序
///
/// 如果订购了相应的 `a` (四个 32 位浮点数) 和 `b` (四个 32 位浮点数) 元素，则将 vector (四个带符号的 32 位整数) 元素中的所有位设置为 1，即
/// 这两个元素都不是 NaN 值，否则将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsor.w))]
pub unsafe fn __msa_fsor_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fsor_w(a, mem::transmute(b))
}

/// Vector 浮点信令比较已排序
///
/// 如果订购了相应的 `a` (两个 64 位浮点数) 和 `b` (两个 64 位浮点数) 元素，则将 vector (两个带符号的 64 位整数) 元素中的所有位设置为 1，即
/// 这两个元素都不是 NaN 值，否则将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsor.d))]
pub unsafe fn __msa_fsor_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fsor_d(a, mem::transmute(b))
}

/// Vector 浮点平方根
///
/// 将 vector `a` (四个 32 位浮点数) 中的浮点元素的平方根顺序写入 vector (四个 32 位浮点数) 元素。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsqrt.w))]
pub unsafe fn __msa_fsqrt_w(a: v4f32) -> v4f32 {
    msa_fsqrt_w(a)
}

/// Vector 浮点平方根
///
/// 将 vector `a` (两个 64 位浮点数) 中的浮点元素的平方根顺序写入 vector (两个 64 位浮点数) 元素。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsqrt.d))]
pub unsafe fn __msa_fsqrt_d(a: v2f64) -> v2f64 {
    msa_fsqrt_d(a)
}

/// Vector 浮点减法
///
/// 从 vector `a` (四个 32 位浮点数) 中的浮点元素减去 vector `b` (四个 32 位浮点数) 中的浮点元素。
///
/// 将结果写入 vector (四个 32 位浮点数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsub.w))]
pub unsafe fn __msa_fsub_w(a: v4f32, b: v4f32) -> v4f32 {
    msa_fsub_w(a, mem::transmute(b))
}

/// Vector 浮点减法
///
/// 从 vector `a` (两个 64 位浮点数) 中的浮点元素减去 vector `b` (两个 64 位浮点数) 中的浮点元素。
///
/// 结果写入 vector (两个 64 位浮点数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsub.d))]
pub unsafe fn __msa_fsub_d(a: v2f64, b: v2f64) -> v2f64 {
    msa_fsub_d(a, mem::transmute(b))
}

/// Vector 浮点信令比较已排序
///
/// 如果相应的 `a` (四个 32 位浮点数) 和 `b` (四个 32 位浮点数) 元素无序或相等，则将 vector (四个带符号的 32 位整数) 元素中的所有位设置为 1，否则设置所有位为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsueq.w))]
pub unsafe fn __msa_fsueq_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fsueq_w(a, mem::transmute(b))
}

/// Vector 浮点信令比较已排序
///
/// 如果相应的 `a` (两个 64 位浮点数) 和 `b` (两个 64 位浮点数) 元素无序或相等，则在 vector (两个带符号的 64 位整数) 元素中将所有位设置为 1，否则设置所有位为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsueq.d))]
pub unsafe fn __msa_fsueq_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fsueq_d(a, mem::transmute(b))
}

/// Vector 浮点信号比较无序或更少或相等
///
/// 如果相应的 `a` (四个 32 位浮点数) 元素是无序的或小于或等于 `b` (四个 32 位浮点数) 元素，则在 vector (四个带符号的 32 位整数) 元素中将所有位设置为 1。，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsule.w))]
pub unsafe fn __msa_fsule_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fsule_w(a, mem::transmute(b))
}

/// Vector 浮点信号比较无序或更少或相等
///
/// 如果相应的 `a` (两个 64 位浮点数) 元素是无序的或小于或等于 `b` (两个 64 位浮点数) 元素，则在 vector (两个带符号的 64 位整数) 元素中将所有位设置为 1。，否则将所有位设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsule.d))]
pub unsafe fn __msa_fsule_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fsule_d(a, mem::transmute(b))
}

/// Vector 浮点信号比较无序或少于
///
/// 如果相应的 `a` (四个 32 位浮点数) 元素是无序的或小于 `b` (四个 32 位浮点数) 元素，则将 vector (四个带符号的 32 位整数) 元素中的所有位设置为 1，否则进行设置所有位都为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsult.w))]
pub unsafe fn __msa_fsult_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fsult_w(a, mem::transmute(b))
}

/// Vector 浮点信号比较无序或少于
///
/// 如果相应的 `a` (两个 64 位浮点数) 元素是无序的或小于 `b` (两个 64 位浮点数) 元素，则将 vector (两个带符号的 64 位整数) 元素中的所有位设置为 1，否则进行设置所有位都为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsult.d))]
pub unsafe fn __msa_fsult_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fsult_d(a, mem::transmute(b))
}

/// Vector 浮点信号比较无序
///
/// 如果相应的 `a` (四个 32 位浮点数) 和 `b` (四个 32 位浮点数) 元素是无序的，则将 vector (四个带符号的 32 位整数) 元素中的所有位设置为 1。
/// 至少一个元素为 NaN 值，否则将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsun.w))]
pub unsafe fn __msa_fsun_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fsun_w(a, mem::transmute(b))
}

/// Vector 浮点信号比较无序
///
/// 如果相应的 `a` (两个 64 位浮点数) 和 `b` (两个 64 位浮点数) 元素是无序的，则将 vector (两个带符号的 64 位整数) 元素中的所有位设置为 1。
/// 至少一个元素为 NaN 值，否则将所有位设置为 0。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsun.d))]
pub unsafe fn __msa_fsun_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fsun_d(a, mem::transmute(b))
}

/// Vector 浮点信号比较无序或不相等
///
/// 如果相应的 `a` (四个 32 位浮点数) 和 `b` (四个 32 位浮点数) 元素无序或不相等，则将 vector (四个带符号的 32 位整数) 元素中的所有位设置为 1，否则进行设置所有位都为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsune.w))]
pub unsafe fn __msa_fsune_w(a: v4f32, b: v4f32) -> v4i32 {
    msa_fsune_w(a, mem::transmute(b))
}

/// Vector 浮点信号比较无序或不相等
///
/// 如果相应的 `a` (两个 64 位浮点数) 和 `b` (两个 64 位浮点数) 元素无序或不相等，则在 vector (两个带符号的 64 位整数) 元素中将所有位设置为 1，否则进行设置所有位都为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(fsune.d))]
pub unsafe fn __msa_fsune_d(a: v2f64, b: v2f64) -> v2i64 {
    msa_fsune_d(a, mem::transmute(b))
}

/// Vector 浮点转换为有符号整数
///
/// vector `a` 中的元素 (四个 32 位浮点数) 被四舍五入，并根据 MSA 控制和状态寄存器 MSACSR 中的舍入模式位 RM 转换为带符号整数值。
///
/// 结果写入 vector (四个带符号的 32 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ftint_s.w))]
pub unsafe fn __msa_ftint_s_w(a: v4f32) -> v4i32 {
    msa_ftint_s_w(a)
}

/// Vector 浮点转换为有符号整数
///
/// vector `a` 中的元素 (两个 64 位浮点数) 被四舍五入，并根据 MSA 控制和状态寄存器 MSACSR 中的舍入模式位 RM 转换为有符号整数值。
///
/// 结果写入 vector (两个带符号的 64 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ftint_s.d))]
pub unsafe fn __msa_ftint_s_d(a: v2f64) -> v2i64 {
    msa_ftint_s_d(a)
}

/// Vector 浮点转换为无符号整数
///
/// vector `a` 中的元素 (四个 32 位浮点数) 被四舍五入，并根据 MSA 控制和状态寄存器 MSACSR 中的舍入模式位 RM 转换为带符号整数值。
///
/// 结果写入 vector (四个无符号的 32 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ftint_u.w))]
pub unsafe fn __msa_ftint_u_w(a: v4f32) -> v4u32 {
    msa_ftint_u_w(a)
}

/// Vector 浮点转换为无符号整数
///
/// vector `a` 中的元素 (两个 64 位浮点数) 被四舍五入，并根据 MSA 控制和状态寄存器 MSACSR 中的舍入模式位 RM 转换为有符号整数值。
///
/// 结果写入 vector (两个无符号的 64 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ftint_u.d))]
pub unsafe fn __msa_ftint_u_d(a: v2f64) -> v2u64 {
    msa_ftint_u_d(a)
}

/// Vector 浮点转换为定点
///
/// vector `a` (四个 32 位浮点数) 和 `b` (四个 32 位浮点数) 中的元素被下转换为定点表示，即
/// 从 64 位浮点到 32 位 Q31 定点表示，或者从 32 位浮点到 16 位 Q15 定点表示。
/// 结果写入 vector (八个带符号的 16 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ftq.h))]
pub unsafe fn __msa_ftq_h(a: v4f32, b: v4f32) -> v8i16 {
    msa_ftq_h(a, mem::transmute(b))
}

/// Vector 浮点转换为定点
///
/// vector `a` (两个 64 位浮点数) 和 `b` (两个 64 位浮点数) 中的元素被下转换为定点表示，即
/// 从 64 位浮点到 32 位 Q31 定点表示，或者从 32 位浮点到 16 位 Q15 定点表示。
/// 结果写入 vector (四个带符号的 32 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ftq.w))]
pub unsafe fn __msa_ftq_w(a: v2f64, b: v2f64) -> v4i32 {
    msa_ftq_w(a, mem::transmute(b))
}

/// Vector 浮点截断并转换为有符号整数
///
/// vector `a` (四个 32 位浮点数) 中的元素被截断，即
/// 四舍五入为有符号整数值。
/// 结果写入 vector (四个带符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ftrunc_s.w))]
pub unsafe fn __msa_ftrunc_s_w(a: v4f32) -> v4i32 {
    msa_ftrunc_s_w(a)
}

/// Vector 浮点截断并转换为有符号整数
///
/// vector `a` (两个 64 位浮点数) 中的元素被截断，即
/// 四舍五入为有符号整数值。
/// 结果写入 vector (两个带符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ftrunc_s.d))]
pub unsafe fn __msa_ftrunc_s_d(a: v2f64) -> v2i64 {
    msa_ftrunc_s_d(a)
}

/// Vector 浮点截断并转换为无符号整数
///
/// vector `a` (四个 32 位浮点数) 中的元素被截断，即
/// 四舍五入为无符号整数值。
/// 结果写入 vector (四个无符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ftrunc_u.w))]
pub unsafe fn __msa_ftrunc_u_w(a: v4f32) -> v4u32 {
    msa_ftrunc_u_w(a)
}

/// Vector 浮点截断并转换为无符号整数
///
/// vector `a` (两个 64 位浮点数) 中的元素被截断，即
/// 四舍五入为无符号整数值。
/// 结果写入 vector (两个无符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ftrunc_u.d))]
pub unsafe fn __msa_ftrunc_u_d(a: v2f64) -> v2u64 {
    msa_ftrunc_u_d(a)
}

/// Vector 有符号的水平添加
///
/// 将 vector `a` 中的符号扩展的奇数元素 (十六个带符号的 8 位整数) 添加到 vector `b` 中的符号扩展的偶数元素 (十六个 8 位符号整数)，其结果是输入操作数大小的两倍。
///
/// 结果写入 vector (八个带符号的 16 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(hadd_s.h))]
pub unsafe fn __msa_hadd_s_h(a: v16i8, b: v16i8) -> v8i16 {
    msa_hadd_s_h(a, mem::transmute(b))
}

/// Vector 有符号的水平添加
///
/// 将 vector `a` 中的符号扩展的奇数元素 (八个有符号的 16 位整数) 添加到 vector `b` 中的符号扩展的偶数元素 (八个有符号 16 位整数)，其结果是输入操作数大小的两倍。
///
/// 结果写入 vector (四个带符号的 32 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(hadd_s.w))]
pub unsafe fn __msa_hadd_s_w(a: v8i16, b: v8i16) -> v4i32 {
    msa_hadd_s_w(a, mem::transmute(b))
}

/// Vector 有符号的水平添加
///
/// 将 vector `a` 中的符号扩展的奇数元素 (四个有符号的 32 位整数) 添加到 vector `b` 中的符号扩展的偶数元素 (四个有符号 32 位整数)，其结果是输入操作数大小的两倍。
///
/// 结果写入 vector (两个带符号的 64 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(hadd_s.d))]
pub unsafe fn __msa_hadd_s_d(a: v4i32, b: v4i32) -> v2i64 {
    msa_hadd_s_d(a, mem::transmute(b))
}

/// Vector 无符号水平加法
///
/// 将 vector `a` 中的零扩展奇数元素 (十六个无符号的 8 位整数) 添加到 vector `b` 中的零扩展偶数元素 (十六个无符号 8 位整数)，其结果是输入操作数大小的两倍。
///
/// 结果写入 vector (八个无符号的 16 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(hadd_u.h))]
pub unsafe fn __msa_hadd_u_h(a: v16u8, b: v16u8) -> v8u16 {
    msa_hadd_u_h(a, mem::transmute(b))
}

/// Vector 无符号水平加法
///
/// 将 vector `a` 中的零扩展奇数元素 (八个无符号 16 位整数) 添加到 vector `b` 中的零扩展偶数元素 (八个无符号 16 位整数)，其结果是输入操作数大小的两倍。
///
/// 结果写入 vector (四个无符号的 32 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(hadd_u.w))]
pub unsafe fn __msa_hadd_u_w(a: v8u16, b: v8u16) -> v4u32 {
    msa_hadd_u_w(a, mem::transmute(b))
}

/// Vector 无符号水平加法
///
/// 将 vector `a` 中的零扩展奇数元素 (四个无符号 32 位整数) 添加到 vector `b` 中的零扩展偶数元素 (四个无符号 32 位整数)，其结果是输入操作数大小的两倍。
///
/// 结果写入 vector (两个无符号的 64 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(hadd_u.d))]
pub unsafe fn __msa_hadd_u_d(a: v4u32, b: v4u32) -> v2u64 {
    msa_hadd_u_d(a, mem::transmute(b))
}

/// Vector 有符号水平相减
///
/// 从 vector `a` (十六个带符号的 8 位整数) 的符号扩展元素中减去 vector `b` (十六个带符号的 8 位整数) 的符号扩展的奇数元素，其结果是输入操作数大小的两倍。
///
/// 结果写入 vector (八个带符号的 16 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(hsub_s.h))]
pub unsafe fn __msa_hsub_s_h(a: v16i8, b: v16i8) -> v8i16 {
    msa_hsub_s_h(a, mem::transmute(b))
}

/// Vector 有符号水平相减
///
/// 将 vector `b` (八个带符号的 16 位整数) 中的符号扩展的奇数元素减去 vector `a` (八个带符号的 16 位整数) 中的符号扩展元素，其结果是输入操作数大小的两倍。
///
/// 结果写入 vector (四个带符号的 32 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(hsub_s.w))]
pub unsafe fn __msa_hsub_s_w(a: v8i16, b: v8i16) -> v4i32 {
    msa_hsub_s_w(a, mem::transmute(b))
}

/// Vector 有符号水平相减
///
/// vector `b` (四个带符号的 32 位整数) 中的符号扩展的奇数元素从 vector `a` (四个带符号的 32 位整数) 中的符号扩展的元素中减去，产生的结果是输入操作数大小的两倍。
///
/// 结果写入 vector (两个带符号的 64 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(hsub_s.d))]
pub unsafe fn __msa_hsub_s_d(a: v4i32, b: v4i32) -> v2i64 {
    msa_hsub_s_d(a, mem::transmute(b))
}

/// Vector 无符号水平减法
///
/// vector `b` (十六个无符号的 8 位整数) 中的零扩展奇数元素被从 vector `a` (十六个无符号的 8 位整数) 中的零扩展元素减去，产生的结果是输入操作数大小的两倍。
///
/// 结果写入 vector (八个带符号的 16 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(hsub_u.h))]
pub unsafe fn __msa_hsub_u_h(a: v16u8, b: v16u8) -> v8i16 {
    msa_hsub_u_h(a, mem::transmute(b))
}

/// Vector 无符号水平减法
///
/// 从 vector `a` (八个无符号 16 位整数) 的零扩展元素中减去 vector `b` (八个无符号 16 位整数) 的零扩展奇数元素，其结果是输入操作数大小的两倍。
///
/// 结果写入 vector (四个带符号的 32 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(hsub_u.w))]
pub unsafe fn __msa_hsub_u_w(a: v8u16, b: v8u16) -> v4i32 {
    msa_hsub_u_w(a, mem::transmute(b))
}

/// Vector 无符号水平减法
///
/// vector `b` (四个无符号的 32 位整数) 中的零扩展奇数元素被从 vector `a` (四个无符号的 32 位整数) 中的零扩展元素中减去，产生的结果是输入操作数大小的两倍。
///
/// 结果写入 vector (两个带符号的 64 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(hsub_u.d))]
pub unsafe fn __msa_hsub_u_d(a: v4u32, b: v4u32) -> v2i64 {
    msa_hsub_u_d(a, mem::transmute(b))
}

/// Vector 交错偶数
///
/// 将 vectors `a` (十六个带符号的 8 位整数) 和 vector `b` (十六个带符号的 8 位整数) 中的偶数元素复制到结果 (十六个带符号的 8 位整数) 中，将 `a` 中的一个元素与 `b` 中的一个元素交替。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvev.b))]
pub unsafe fn __msa_ilvev_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_ilvev_b(a, mem::transmute(b))
}

/// Vector 交错偶数
///
/// 将 vectors `a` (八个带符号的 16 位整数) 和 vector `b` (八个带符号的 16 位整数) 中的偶数元素复制到结果 (八个带符号的 16 位整数) 中，将 `a` 中的一个元素与 `b` 中的一个元素交替。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvev.h))]
pub unsafe fn __msa_ilvev_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_ilvev_h(a, mem::transmute(b))
}

/// Vector 交错偶数
///
/// 将 vectors `a` (四个带符号的 32 位整数) 和 vector `b` (四个带符号的 32 位整数) 中的偶数元素复制到结果 (四个带符号的 32 位整数) 中，将 `a` 中的一个元素与 `b` 中的一个元素交替。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvev.w))]
pub unsafe fn __msa_ilvev_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_ilvev_w(a, mem::transmute(b))
}

/// Vector 交错偶数
///
/// 将 vectors `a` (两个带符号的 64 位整数) 和 vector `b` (两个带符号的 64 位整数) 中的偶数元素复制到结果 (两个带符号的 64 位整数) 中，将 `a` 中的一个元素与 `b` 中的一个元素交替。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvev.d))]
pub unsafe fn __msa_ilvev_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_ilvev_d(a, mem::transmute(b))
}

/// Vector 向左交织
///
/// 将 vectors `a` (十六个带符号的 8 位整数) 和 vector `b` (十六个带符号的 8 位整数) 中的左半部分元素复制到结果 (十六个带符号的 8 位整数) 中，将 `a` 中的一个元素替换为一个元素从 `b`。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvl.b))]
pub unsafe fn __msa_ilvl_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_ilvl_b(a, mem::transmute(b))
}

/// Vector 向左交织
///
/// 将 vectors `a` (八个带符号的 16 位整数) 和 vector `b` (八个带符号的 16 位整数) 中的左半元素复制到结果 (八个带符号的 16 位整数) 中，将 `a` 中的一个元素替换为一个元素从 `b`。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvl.h))]
pub unsafe fn __msa_ilvl_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_ilvl_h(a, mem::transmute(b))
}

/// Vector 向左交织
///
/// 将 vectors `a` (四个带符号的 32 位整数) 和 vector `b` (四个带符号的 32 位整数) 中的左半部分元素复制到结果 (四个带符号的 32 位整数) 中，将 `a` 中的一个元素替换为一个元素从 `b`。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvl.w))]
pub unsafe fn __msa_ilvl_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_ilvl_w(a, mem::transmute(b))
}

/// Vector 向左交织
///
/// 将 vectors `a` (两个带符号的 64 位整数) 和 vector `b` (两个带符号的 64 位整数) 中的左半部分元素复制到结果 (两个带符号的 64 位整数) 中，将 `a` 中的一个元素替换为一个元素从 `b`。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvl.d))]
pub unsafe fn __msa_ilvl_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_ilvl_d(a, mem::transmute(b))
}

/// Vector 交错奇数
///
/// 将 vectors `a` (十六个带符号的 8 位整数) 和 vector `b` (十六个带符号的 8 位整数) 中的奇数元素复制到结果 (十六个带符号的 8 位整数) 中，将 `a` 中的一个元素与 `b` 中的一个元素交替。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvod.b))]
pub unsafe fn __msa_ilvod_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_ilvod_b(a, mem::transmute(b))
}

/// Vector 交错奇数
///
/// 将 vectors `a` (八个带符号的 16 位整数) 和 vector `b` (八个带符号的 16 位整数) 中的奇数元素复制到结果 (八个带符号的 16 位整数) 中，将 `a` 中的一个元素与 `b` 中的一个元素交替。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvod.h))]
pub unsafe fn __msa_ilvod_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_ilvod_h(a, mem::transmute(b))
}

/// Vector 交错奇数
///
/// 将 vectors `a` (四个带符号的 32 位整数) 和 vector `b` (四个带符号的 32 位整数) 中的奇数元素复制到结果 (四个带符号的 32 位整数) 中，将 `a` 中的一个元素与 `b` 中的一个元素交替。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvod.w))]
pub unsafe fn __msa_ilvod_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_ilvod_w(a, mem::transmute(b))
}

/// Vector 交错奇数
///
/// 将 vectors `a` (两个带符号的 64 位整数) 和 vector `b` (两个带符号的 64 位整数) 中的奇数元素复制到结果 (两个带符号的 64 位整数) 中，将 `a` 中的一个元素与 `b` 中的一个元素交替。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvod.d))]
pub unsafe fn __msa_ilvod_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_ilvod_d(a, mem::transmute(b))
}

/// Vector 右交错
///
/// 将 vectors `a` (十六个带符号的 8 位整数) 和 vector `b` (十六个带符号的 8 位整数) 中的右半元素复制到结果 (十六个带符号的 8 位整数) 中，将 `a` 中的一个元素替换为一个元素从 `b`。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvr.b))]
pub unsafe fn __msa_ilvr_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_ilvr_b(a, mem::transmute(b))
}

/// Vector 右交错
///
/// 将 vectors `a` (八个带符号的 16 位整数) 和 vector `b` (八个带符号的 16 位整数) 中的右半元素复制到结果 (八个带符号的 16 位整数) 中，将 `a` 中的一个元素替换为一个元素从 `b`。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvr.h))]
pub unsafe fn __msa_ilvr_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_ilvr_h(a, mem::transmute(b))
}

/// Vector 右交错
///
/// 将 vectors `a` (四个带符号的 32 位整数) 和 vector `b` (四个带符号的 32 位整数) 中的右半元素复制到结果 (四个带符号的 32 位整数) 中，将 `a` 中的一个元素替换为一个元素从 `b`。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvr.w))]
pub unsafe fn __msa_ilvr_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_ilvr_w(a, mem::transmute(b))
}

/// Vector 右交错
///
/// 将 vectors `a` (两个带符号的 64 位整数) 和 vector `b` (两个带符号的 64 位整数) 中的右半元素复制到结果 (两个带符号的 64 位整数) 中，将 `a` 中的一个元素替换为一个元素从 `b`。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ilvr.d))]
pub unsafe fn __msa_ilvr_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_ilvr_d(a, mem::transmute(b))
}

/// GPR 插入元素
///
/// 将 vector `a` 中的元素 `imm4` (十六个带符号的 8 位整数) 设置为 GPR `c` 值。
/// vector `a` 中的所有其他元素均未更改。
/// 如果源 GPR 比目标数据格式宽，则目标元素将设置为 GPR 的最低有效位。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(insert.b, imm4 = 0b1111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_insert_b<const IMM4: i32>(a: v16i8, c: i32) -> v16i8 {
    static_assert_imm4!(IMM4);
    msa_insert_b(a, IMM4, c)
}

/// GPR 插入元素
///
/// 将 vector `a` 中的元素 `imm3` (八个带符号的 16 位整数) 设置为 GPR `c` 值。
/// vector `a` 中的所有其他元素均未更改。
/// 如果源 GPR 比目标数据格式宽，则目标元素将设置为 GPR 的最低有效位。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(insert.h, imm3 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_insert_h<const IMM3: i32>(a: v8i16, c: i32) -> v8i16 {
    static_assert_imm3!(IMM3);
    msa_insert_h(a, IMM3, c)
}

/// GPR 插入元素
///
/// 将 vector `a` 中的元素 `imm2` (四个带符号的 32 位整数) 设置为 GPR `c` 值。
/// vector `a` 中的所有其他元素均未更改。
/// 如果源 GPR 比目标数据格式宽，则目标元素将设置为 GPR 的最低有效位。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(insert.w, imm2 = 0b11))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_insert_w<const IMM2: i32>(a: v4i32, c: i32) -> v4i32 {
    static_assert_imm2!(IMM2);
    msa_insert_w(a, IMM2, c)
}

/// GPR 插入元素
///
/// 将 vector `a` 中的元素 `imm1` (两个带符号的 64 位整数) 设置为 GPR `c` 值。
/// vector `a` 中的所有其他元素均未更改。
/// 如果源 GPR 比目标数据格式宽，则目标元素将设置为 GPR 的最低有效位。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(insert.d, imm1 = 0b1))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_insert_d<const IMM1: i32>(a: v2i64, c: i64) -> v2i64 {
    static_assert_imm1!(IMM1);
    msa_insert_d(a, IMM1, c)
}

/// 元素插入元素
///
/// 将结果 vector `a` (十六个带符号的 8 位整数) 中的元素 `imm1` 设置为 vector `c` (十六个带符号的 8 位整数) 值中的元素 0。
///
/// vector `a` 中的所有其他元素均未更改。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(insve.b, imm4 = 0b1111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_insve_b<const IMM4: i32>(a: v16i8, c: v16i8) -> v16i8 {
    static_assert_imm4!(IMM4);
    msa_insve_b(a, IMM4, c)
}

/// 元素插入元素
///
/// 将结果 vector `a` (八个带符号的 16 位整数) 中的元素 `imm1` 设置为 vector `c` (八个带符号的 16 位整数) 值中的元素 0。
///
/// vector `a` 中的所有其他元素均未更改。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(insve.h, imm3 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_insve_h<const IMM3: i32>(a: v8i16, c: v8i16) -> v8i16 {
    static_assert_imm3!(IMM3);
    msa_insve_h(a, IMM3, c)
}

/// 元素插入元素
///
/// 将结果 vector `a` (四个带符号的 32 位整数) 中的元素 `imm1` 设置为 vector `c` (四个带符号的 32 位整数) 值中的元素 0。
///
/// vector `a` 中的所有其他元素均未更改。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(insve.w, imm2 = 0b11))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_insve_w<const IMM2: i32>(a: v4i32, c: v4i32) -> v4i32 {
    static_assert_imm2!(IMM2);
    msa_insve_w(a, IMM2, c)
}

/// 元素插入元素
///
/// 将结果 vector `a` (两个带符号的 64 位整数) 中的元素 `imm1` 设置为 vector `c` (两个带符号的 64 位整数) 值中的元素 0。
///
/// vector `a` 中的所有其他元素均未更改。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(insve.d, imm1 = 0b1))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_insve_d<const IMM1: i32>(a: v2i64, c: v2i64) -> v2i64 {
    static_assert_imm1!(IMM1);
    msa_insve_d(a, IMM1, c)
}

/// Vector 加载
///
/// 将获取由基本 `mem_addr` 和 10 位带符号 immediate 数偏移量 `imm_s10` 寻址的有效存储位置中的 WRLEN/8 字节，并将其放入 vector (十六个带符号的 8 位整数) 中。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ld.b, imm_s10 = 0b1111111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_ld_b<const IMM_S10: i32>(mem_addr: *mut u8) -> v16i8 {
    static_assert_imm_s10!(IMM_S10);
    msa_ld_b(mem_addr, IMM_S10)
}

/// Vector 加载
///
/// 将获取由基本 `mem_addr` 和 10 位带符号 immediate 数偏移量 `imm_s11` 寻址的有效存储位置中的 WRLEN/8 字节，并将其放入 vector (八个带符号的 16 位整数) 中。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ld.h, imm_s11 = 0b11111111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_ld_h<const IMM_S11: i32>(mem_addr: *mut u8) -> v8i16 {
    static_assert_imm_s11!(IMM_S11);
    static_assert!(IMM_S11: i32 where IMM_S11 % 2 == 0);
    msa_ld_h(mem_addr, IMM_S11)
}

/// Vector 加载
///
/// 将获取由基本 `mem_addr` 和 10 位带符号 immediate 数偏移量 `imm_s12` 寻址的有效存储位置中的 WRLEN/8 字节，并将其放在 vector (四个带符号的 32 位整数) 中。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ld.w, imm_s12 = 0b111111111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_ld_w<const IMM_S12: i32>(mem_addr: *mut u8) -> v4i32 {
    static_assert_imm_s12!(IMM_S12);
    static_assert!(IMM_S12: i32 where IMM_S12 % 4 == 0);
    msa_ld_w(mem_addr, IMM_S12)
}

/// Vector 加载
///
/// 将获取由基本 `mem_addr` 和 10 位带符号 immediate 数偏移量 `imm_s13` 寻址的有效内存位置的 WRLEN/8 字节，并将其放在 vector (两个带符号的 64 位整数) 中。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ld.d, imm_s13 = 0b1111111111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_ld_d<const IMM_S13: i32>(mem_addr: *mut u8) -> v2i64 {
    static_assert_imm_s13!(IMM_S13);
    static_assert!(IMM_S13: i32 where IMM_S13 % 8 == 0);
    msa_ld_d(mem_addr, IMM_S13)
}

/// Immediate 加载
///
/// 带符号的 immediate 数 imm_s10 复制到所有 vector (十六个带符号的 8 位整数) 元素中。
///
/// 对于字节元素，将仅使用 imm_s10 的最低有效 8 位。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ldi.b, imm_s10 = 0b1111111111))]
#[rustc_legacy_const_generics(0)]
pub unsafe fn __msa_ldi_b<const IMM_S10: i32>() -> v16i8 {
    static_assert_imm_s10!(IMM_S10);
    msa_ldi_b(IMM_S10)
}

/// Immediate 加载
///
/// 带符号的 immediate 数 imm_s10 复制到所有 vector (八个带符号的 16 位整数) 元素中。
///
/// 对于字节元素，将仅使用 imm_s10 的最低有效 8 位。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ldi.h, imm_s10 = 0b1111111111))]
#[rustc_legacy_const_generics(0)]
pub unsafe fn __msa_ldi_h<const IMM_S10: i32>() -> v8i16 {
    static_assert_imm_s10!(IMM_S10);
    msa_ldi_h(IMM_S10)
}

/// Immediate 加载
///
/// 带符号的 immediate 数 imm_s10 被复制到所有 vector (四个带符号的 32 位整数) 元素中。
///
/// 对于字节元素，将仅使用 imm_s10 的最低有效 8 位。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ldi.w, imm_s10 = 0b1111111111))]
#[rustc_legacy_const_generics(0)]
pub unsafe fn __msa_ldi_w<const IMM_S10: i32>() -> v4i32 {
    static_assert_imm_s10!(IMM_S10);
    msa_ldi_w(IMM_S10)
}

/// Immediate 加载
///
/// 带符号的 immediate 数 imm_s10 复制到所有 vector (两个带符号的 64 位整数) 元素中。
///
/// 对于字节元素，将仅使用 imm_s10 的最低有效 8 位。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ldi.d, imm_s10 = 0b1111111111))]
#[rustc_legacy_const_generics(0)]
pub unsafe fn __msa_ldi_d<const IMM_S10: i32>() -> v2i64 {
    static_assert_imm_s10!(IMM_S10);
    msa_ldi_d(IMM_S10)
}

/// Vector 定点乘法和加法
///
/// 将 `b` (八个带符号的 16 位整数) 中的定点元素与 vector `c` (八个带符号的 16 位整数) 中的定点元素的乘积与 vector `a` (八个带符号的 16 位) 中的定点元素相加位整数)。
///
/// 乘法结果未达到饱和，即将精确的 (-1) * (-1) =1 添加到目标。
/// 饱和的定点结果存储到 vector `a`。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(madd_q.h))]
pub unsafe fn __msa_madd_q_h(a: v8i16, b: v8i16, c: v8i16) -> v8i16 {
    msa_madd_q_h(a, mem::transmute(b), c)
}

/// Vector 定点乘法和加法
///
/// 将 `b` (四个带符号的 32 位整数) 中的定点元素与 vector `c` (四个带符号的 32 位整数) 中的定点元素的乘积与 vector `a` (四个带符号的 32 位) 中的定点元素相加位整数)。
///
/// 乘法结果未达到饱和，即将精确的 (-1) * (-1) =1 添加到目标。
/// 饱和的定点结果存储到 vector `a`。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(madd_q.w))]
pub unsafe fn __msa_madd_q_w(a: v4i32, b: v4i32, c: v4i32) -> v4i32 {
    msa_madd_q_w(a, mem::transmute(b), c)
}

/// Vector 定点乘法与加法四舍五入
///
/// 将 `b` (八个带符号的 16 位整数) 中的定点元素与 vector `c` (八个带符号的 16 位整数) 中的定点元素的乘积与 vector `a` (八个带符号的 16 位) 中的定点元素相加位整数)。
///
/// 乘法结果未达到饱和，即将精确的 (-1) * (-1) =1 添加到目标。
/// 取整和饱和的定点结果存储到 vector `a`。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(maddr_q.h))]
pub unsafe fn __msa_maddr_q_h(a: v8i16, b: v8i16, c: v8i16) -> v8i16 {
    msa_maddr_q_h(a, mem::transmute(b), c)
}

/// Vector 定点乘法与加法四舍五入
///
/// 将 `b` (四个带符号的 32 位整数) 中的定点元素与 vector `c` (四个带符号的 32 位整数) 中的定点元素的乘积与 vector `a` (四个带符号的 32 位) 中的定点元素相加位整数)。
///
/// 乘法结果未达到饱和，即将精确的 (-1) * (-1) =1 添加到目标。
/// 取整和饱和的定点结果存储到 vector `a`。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(maddr_q.w))]
pub unsafe fn __msa_maddr_q_w(a: v4i32, b: v4i32, c: v4i32) -> v4i32 {
    msa_maddr_q_w(a, mem::transmute(b), c)
}

/// Vector 相乘和相加
///
/// 将 vector `b` (十六个带符号的 8 位整数) 中的整数元素与 vector `c` (十六个带符号的 8 位整数) 中的整数元素相乘，然后添加到 vector `a` (十六个带符号的 8 位整数中) 的整数元素中。。
///
/// 乘法结果中最重要的一半将被丢弃。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(maddv.b))]
pub unsafe fn __msa_maddv_b(a: v16i8, b: v16i8, c: v16i8) -> v16i8 {
    msa_maddv_b(a, mem::transmute(b), c)
}

/// Vector 相乘和相加
///
/// 将 vector `b` (八个带符号的 16 位整数) 中的整数元素与 vector `c` (八个带符号的 16 位整数) 中的整数元素相乘，然后将其与 vector `a` (八个带符号的 16 位整数中的整数) 相加。。
///
/// 乘法结果中最重要的一半将被丢弃。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(maddv.h))]
pub unsafe fn __msa_maddv_h(a: v8i16, b: v8i16, c: v8i16) -> v8i16 {
    msa_maddv_h(a, mem::transmute(b), c)
}

/// Vector 相乘和相加
///
/// vector `b` (四个带符号的 32 位整数) 中的整数元素与 vector `c` (四个带符号的 32 位整数) 中的整数元素相乘，然后加到 vector `a` (四个带符号的 32 位整数中) 的整数元素中。。
///
/// 乘法结果中最重要的一半将被丢弃。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(maddv.w))]
pub unsafe fn __msa_maddv_w(a: v4i32, b: v4i32, c: v4i32) -> v4i32 {
    msa_maddv_w(a, mem::transmute(b), c)
}

/// Vector 相乘和相加
///
/// 将 vector `b` (两个带符号的 64 位整数) 中的整数元素与 vector `c` (两个带符号的 64 位整数) 中的整数元素相乘，然后将其与 vector `a` (两个带符号的 64 位整数中的整数) 相加。。
///
/// 乘法结果中最重要的一半将被丢弃。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(maddv.d))]
pub unsafe fn __msa_maddv_d(a: v2i64, b: v2i64, c: v2i64) -> v2i64 {
    msa_maddv_d(a, mem::transmute(b), c)
}

/// Vector 基于绝对值的最大值
///
/// 幅度最大的值，即
/// 将 vector 中的相应带符号元素之间的绝对值 `a` (十六个带符号的 8 位整数) 和 `b` (十六个带符号的 8 位整数) 写入 vector (十六个带符号的 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(max_a.b))]
pub unsafe fn __msa_max_a_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_max_a_b(a, mem::transmute(b))
}

/// Vector 基于绝对值的最大值
///
/// 幅度最大的值，即
/// 将 vector 中的相应带符号元素之间的绝对值 `a` (八个带符号的 16 位整数) 和 `b` (八个带符号的 16 位整数) 写入 vector (八个带符号的 16 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(max_a.h))]
pub unsafe fn __msa_max_a_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_max_a_h(a, mem::transmute(b))
}

/// Vector 基于绝对值的最大值
///
/// 幅度最大的值，即
/// 将 vector 中的相应带符号元素之间的绝对值 `a` (四个带符号的 32 位整数) 和 `b` (四个带符号的 32 位整数) 之间的绝对值写入 vector (四个带符号的 32 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(max_a.w))]
pub unsafe fn __msa_max_a_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_max_a_w(a, mem::transmute(b))
}

/// Vector 基于绝对值的最大值
///
/// 幅度最大的值，即
/// 将 vector 中的相应带符号元素之间的绝对值 `a` (两个带符号的 64 位整数) 和 `b` (两个带符号的 64 位整数) 之间的绝对值写入 vector (两个带符号的 64 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(max_a.d))]
pub unsafe fn __msa_max_a_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_max_a_d(a, mem::transmute(b))
}

/// Vector 有符号的最大值
///
/// vector `a` (十六个带符号的 8 位整数) 中的有符号元素和 vector `b` (十六个带符号的 8 位整数) 中的有符号元素之间的最大值被写入 vector (十六个带符号的 8 位整数中)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(max_s.b))]
pub unsafe fn __msa_max_s_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_max_s_b(a, mem::transmute(b))
}

/// Vector 有符号的最大值
///
/// 将 vector `a` (八个带符号的 16 位整数) 中的有符号元素和 vector `b` (八个带符号的 16 位整数) 中的有符号元素之间的最大值写入 vector (八个有符号的 16 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(max_s.h))]
pub unsafe fn __msa_max_s_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_max_s_h(a, mem::transmute(b))
}

/// Vector 有符号的最大值
///
/// 将 vector `a` (四个带符号的 32 位整数) 中的有符号元素和 vector `b` (四个带符号的 32 位整数) 中的有符号元素之间的最大值写入 vector (四个有符号的 32 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(max_s.w))]
pub unsafe fn __msa_max_s_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_max_s_w(a, mem::transmute(b))
}

/// Vector 有符号的最大值
///
/// 将 vector `a` (两个带符号的 64 位整数) 中的有符号元素和 vector `b` (两个带符号的 64 位整数) 中的有符号元素之间的最大值写入 vector (两个有符号的 64 位整数中)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(max_s.d))]
pub unsafe fn __msa_max_s_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_max_s_d(a, mem::transmute(b))
}

/// Vector 无符号最大值
///
/// 将 vector `a` 中的无符号元素 (十六个无符号 8 位整数) 和 vector `b` 中的无符号元素 (十六个无符号 8 位整数) 之间的最大值写入 vector (十六个无符号 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(max_u.b))]
pub unsafe fn __msa_max_u_b(a: v16u8, b: v16u8) -> v16u8 {
    msa_max_u_b(a, mem::transmute(b))
}

/// Vector 无符号最大值
///
/// 将 vector `a` 中的无符号元素 (八个无符号 16 位整数) 和 vector `b` 中的无符号元素 (八个无符号 16 位整数) 之间的最大值写入 vector (八个无符号 16 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(max_u.h))]
pub unsafe fn __msa_max_u_h(a: v8u16, b: v8u16) -> v8u16 {
    msa_max_u_h(a, mem::transmute(b))
}

/// Vector 无符号最大值
///
/// vector `a` (四个无符号的 32 位整数) 中的无符号元素与 vector `b` (四个无符号的 32 位整数) 中的无符号元素之间的最大值被写入 vector (四个无符号的 32 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(max_u.w))]
pub unsafe fn __msa_max_u_w(a: v4u32, b: v4u32) -> v4u32 {
    msa_max_u_w(a, mem::transmute(b))
}

/// Vector 无符号最大值
///
/// vector `a` 中的无符号元素 (两个无符号的 64 位整数) 和 vector `b` 中的无符号元素 (两个无符号 64 位整数) 之间的最大值被写入 vector (两个无符号 64 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(max_u.d))]
pub unsafe fn __msa_max_u_d(a: v2u64, b: v2u64) -> v2u64 {
    msa_max_u_d(a, mem::transmute(b))
}

/// Immediate 有符号的最大值
///
/// 将 vector `a` (十六个带符号的 8 位整数) 和 5 位带符号 immediate 数 imm_s5 之间的有符号元素之间的最大值写入 vector (十六个带符号的 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(maxi_s.b, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_maxi_s_b<const IMM_S5: i32>(a: v16i8) -> v16i8 {
    static_assert_imm_s5!(IMM_S5);
    msa_maxi_s_b(a, IMM_S5)
}

/// Immediate 有符号的最大值
///
/// 将 vector `a` (八个带符号的 16 位整数) 中的有符号元素之间的最大值与 5 个带符号的 immediate 数 imm_s5 写入 vector (八个带符号的 16 位整数中)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(maxi_s.h, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_maxi_s_h<const IMM_S5: i32>(a: v8i16) -> v8i16 {
    static_assert_imm_s5!(IMM_S5);
    msa_maxi_s_h(a, IMM_S5)
}

/// Immediate 有符号的最大值
///
/// 将 vector `a` (四个带符号的 32 位整数) 和 5 位带符号的 immediate 数 imm_s5 中的带符号元素之间的最大值写入 vector (四个带符号的 32 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(maxi_s.w, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_maxi_s_w<const IMM_S5: i32>(a: v4i32) -> v4i32 {
    static_assert_imm_s5!(IMM_S5);
    msa_maxi_s_w(a, IMM_S5)
}

/// Immediate 有符号的最大值
///
/// 将 vector `a` (两个带符号的 64 位整数) 和 5 位带符号 immediate  imm_s5 中的带符号元素之间的最大值写入 vector (两个带符号的 64 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(maxi_s.d, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_maxi_s_d<const IMM_S5: i32>(a: v2i64) -> v2i64 {
    static_assert_imm_s5!(IMM_S5);
    msa_maxi_s_d(a, IMM_S5)
}

/// Immediate 无符号最大值
///
/// vector `a` 中的无符号元素之间的最大值 (十六个无符号 8 位整数) 和 5 位无符号 immediate  `imm5` 被写入 vector (十六个无符号 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(maxi_u.b, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_maxi_u_b<const IMM5: i32>(a: v16u8) -> v16u8 {
    static_assert_imm5!(IMM5);
    msa_maxi_u_b(a, IMM5)
}

/// Immediate 无符号最大值
///
/// vector `a` (八个无符号的 16 位整数) 和 5 位无符号 immediate  `imm5` 中的无符号元素之间的最大值被写入 vector (八个无符号的 16 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(maxi_u.h, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_maxi_u_h<const IMM5: i32>(a: v8u16) -> v8u16 {
    static_assert_imm5!(IMM5);
    msa_maxi_u_h(a, IMM5)
}

/// Immediate 无符号最大值
///
/// 将 vector `a` (四个无符号的 32 位整数) 和 5 位无符号 immediate  `imm5` 中的无符号元素之间的最大值写入 vector (四个无符号的 32 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(maxi_u.w, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_maxi_u_w<const IMM5: i32>(a: v4u32) -> v4u32 {
    static_assert_imm5!(IMM5);
    msa_maxi_u_w(a, IMM5)
}

/// Immediate 无符号最大值
///
/// 将 vector `a` (两个无符号的 64 位整数) 和 5 位无符号 immediate  `imm5` 中的无符号元素之间的最大值写入 vector (两个无符号的 64 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(maxi_u.d, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_maxi_u_d<const IMM5: i32>(a: v2u64) -> v2u64 {
    static_assert_imm5!(IMM5);
    msa_maxi_u_d(a, IMM5)
}

/// 基于绝对值的 Vector 最小值
///
/// 幅度最小的值，即
/// 将 vector 中的相应带符号元素之间的绝对值 `a` (十六个带符号的 8 位整数) 和 `b` (十六个带符号的 8 位整数) 写入 vector (十六个带符号的 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(min_a.b))]
pub unsafe fn __msa_min_a_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_min_a_b(a, mem::transmute(b))
}

/// 基于绝对值的 Vector 最小值
///
/// 幅度最小的值，即
/// 将 vector 中的相应带符号元素之间的绝对值 `a` (八个带符号的 16 位整数) 和 `b` (八个带符号的 16 位整数) 写入 vector (八个带符号的 16 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(min_a.h))]
pub unsafe fn __msa_min_a_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_min_a_h(a, mem::transmute(b))
}

/// 基于绝对值的 Vector 最小值
///
/// 幅度最小的值，即
/// 将 vector 中的相应带符号元素之间的绝对值 `a` (四个带符号的 32 位整数) 和 `b` (四个带符号的 32 位整数) 之间的绝对值写入 vector (四个带符号的 32 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(min_a.w))]
pub unsafe fn __msa_min_a_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_min_a_w(a, mem::transmute(b))
}

/// 基于绝对值的 Vector 最小值
///
/// 幅度最小的值，即
/// 将 vector 中的相应带符号元素之间的绝对值 `a` (两个带符号的 64 位整数) 和 `b` (两个带符号的 64 位整数) 之间的绝对值写入 vector (两个带符号的 64 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(min_a.d))]
pub unsafe fn __msa_min_a_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_min_a_d(a, mem::transmute(b))
}

/// Vector 有符号的最小值
///
/// vector `a` 中的带符号元素 (十六个带符号的 8 位整数) 和 vector `b` 中的带符号元素 (十六个带符号的 8 位整数) 之间的最小值被写入 vector (十六个带符号的 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(min_s.b))]
pub unsafe fn __msa_min_s_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_min_s_b(a, mem::transmute(b))
}

/// Vector 有符号的最小值
///
/// 将 vector `a` (八个带符号的 16 位整数) 中的有符号元素与 vector `b` (八个带符号的 16 位整数) 中的有符号元素之间的最小值写入 vector (八个有符号的 16 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(min_s.h))]
pub unsafe fn __msa_min_s_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_min_s_h(a, mem::transmute(b))
}

/// Vector 有符号的最小值
///
/// vector `a` (四个有符号的 32 位整数) 中的有符号元素和 vector `b` (四个有符号的 32 位整数) 中的有符号元素之间的最小值被写入 vector (四个有符号的 32 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(min_s.w))]
pub unsafe fn __msa_min_s_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_min_s_w(a, mem::transmute(b))
}

/// Vector 有符号的最小值
///
/// 将 vector `a` (两个带符号的 64 位整数) 中的有符号元素和 vector `b` (两个带符号的 64 位整数) 中的有符号元素之间的最小值写入 vector (两个有符号的 64 位整数中)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(min_s.d))]
pub unsafe fn __msa_min_s_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_min_s_d(a, mem::transmute(b))
}

/// Immediate 有符号最小值
///
/// 将 vector `a` 中的有符号元素之间的最小值 (十六个有符号的 8 位整数) 和 5 位有符号 immediate  imm_s5 写入 vector (十六个有符号的 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mini_s.b, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_mini_s_b<const IMM_S5: i32>(a: v16i8) -> v16i8 {
    static_assert_imm_s5!(IMM_S5);
    msa_mini_s_b(a, IMM_S5)
}

/// Immediate 有符号最小值
///
/// 将 vector `a` (八个带符号的 16 位整数) 中的有符号元素之间的最小值与 5 个带符号的 immediate  imm_s5 写入 vector (八个带符号的 16 位整数中)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mini_s.h, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_mini_s_h<const IMM_S5: i32>(a: v8i16) -> v8i16 {
    static_assert_imm_s5!(IMM_S5);
    msa_mini_s_h(a, IMM_S5)
}

/// Immediate 有符号最小值
///
/// 将 vector `a` 中的有符号元素之间的最小值 (四个有符号的 32 位整数) 和 5 位有符号 immediate  imm_s5 写入 vector (四个有符号的 32 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mini_s.w, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_mini_s_w<const IMM_S5: i32>(a: v4i32) -> v4i32 {
    static_assert_imm_s5!(IMM_S5);
    msa_mini_s_w(a, IMM_S5)
}

/// Immediate 有符号最小值
///
/// 将 vector `a` 中的有符号元素之间的最小值 (两个有符号的 64 位整数) 和 5 位有符号 immediate  imm_s5 写入 vector (两个有符号的 64 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mini_s.d, imm_s5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_mini_s_d<const IMM_S5: i32>(a: v2i64) -> v2i64 {
    static_assert_imm_s5!(IMM_S5);
    msa_mini_s_d(a, IMM_S5)
}

/// Vector 无符号最小值
///
/// 将 vector `a` 中的无符号元素 (十六个无符号 8 位整数) 和 vector `b` 中的无符号元素 (十六个无符号 8 位整数) 之间的最小值写入 vector (十六个无符号 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(min_u.b))]
pub unsafe fn __msa_min_u_b(a: v16u8, b: v16u8) -> v16u8 {
    msa_min_u_b(a, mem::transmute(b))
}

/// Vector 无符号最小值
///
/// 将 vector `a` 中的无符号元素 (八个无符号 16 位整数) 和 vector `b` 中的无符号元素 (八个无符号 16 位整数) 之间的最小值写入 vector (八个无符号 16 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(min_u.h))]
pub unsafe fn __msa_min_u_h(a: v8u16, b: v8u16) -> v8u16 {
    msa_min_u_h(a, mem::transmute(b))
}

/// Vector 无符号最小值
///
/// 将 vector `a` 中的无符号元素 (四个无符号 32 位整数) 和 vector `b` 中的无符号元素 (四个无符号 32 位整数) 之间的最小值写入 vector (四个无符号 32 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(min_u.w))]
pub unsafe fn __msa_min_u_w(a: v4u32, b: v4u32) -> v4u32 {
    msa_min_u_w(a, mem::transmute(b))
}

/// Vector 无符号最小值
///
/// 将 vector `a` 中的无符号元素 (两个无符号 64 位整数) 和 vector `b` 中的无符号元素 (两个无符号 64 位整数) 之间的最小值写入 vector (两个无符号 64 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(min_u.d))]
pub unsafe fn __msa_min_u_d(a: v2u64, b: v2u64) -> v2u64 {
    msa_min_u_d(a, mem::transmute(b))
}

/// Immediate 无符号最小值
///
/// vector `a` 中的无符号元素之间的最小值 (十六个无符号 8 位整数) 和 5 位无符号 immediate  `imm5` 被写入 vector (十六个无符号 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mini_u.b, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_mini_u_b<const IMM5: i32>(a: v16u8) -> v16u8 {
    static_assert_imm5!(IMM5);
    msa_mini_u_b(a, IMM5)
}

/// Immediate 无符号最小值
///
/// vector `a` (八个无符号的 16 位整数) 和 5 位无符号 immediate  `imm5` 中的无符号元素之间的最小值被写入 vector (八个无符号的 16 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mini_u.h, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_mini_u_h<const IMM5: i32>(a: v8u16) -> v8u16 {
    static_assert_imm5!(IMM5);
    msa_mini_u_h(a, IMM5)
}

/// Immediate 无符号最小值
///
/// 将 vector `a` (四个无符号的 32 位整数) 和 5 位无符号 immediate  `imm5` 中的无符号元素之间的最小值写入 vector (四个无符号的 32 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mini_u.w, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_mini_u_w<const IMM5: i32>(a: v4u32) -> v4u32 {
    static_assert_imm5!(IMM5);
    msa_mini_u_w(a, IMM5)
}

/// Immediate 无符号最小值
///
/// vector `a` (两个无符号的 64 位整数) 和 5 位无符号 immediate  `imm5` 中的无符号元素之间的最小值被写入 vector (两个无符号的 64 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mini_u.d, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_mini_u_d<const IMM5: i32>(a: v2u64) -> v2u64 {
    static_assert_imm5!(IMM5);
    msa_mini_u_d(a, IMM5)
}

/// Vector 有符号模数
///
/// vector `a` (十六个带符号的 8 位整数) 中的有符号整数元素除以 vector `b` (十六个带符号的 8 位整数) 中的有符号整数元素。
///
/// 与被除数相同符号的其余部分写入 vector (十六个带符号的 8 位整数)。
/// 如果除数元素 vector `b` 为零，则结果值为 UNPREDICTABLE。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mod_s.b))]
pub unsafe fn __msa_mod_s_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_mod_s_b(a, mem::transmute(b))
}

/// Vector 有符号模数
///
/// vector `a` (八个带符号的 16 位整数) 中的有符号整数元素除以 vector `b` (八个带符号的 16 位整数) 中的有符号整数元素。
///
/// 与被除数相同符号的其余部分写入 vector (八个带符号的 16 位整数)。
/// 如果除数元素 vector `b` 为零，则结果值为 UNPREDICTABLE。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mod_s.h))]
pub unsafe fn __msa_mod_s_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_mod_s_h(a, mem::transmute(b))
}

/// Vector 有符号模数
///
/// vector `a` (四个有符号的 32 位整数) 中的有符号整数元素除以 vector `b` (四个有符号的 32 位整数) 中的有符号整数元素。
///
/// 与被除数相同符号的其余部分写入 vector (四个带符号的 32 位整数)。
/// 如果除数元素 vector `b` 为零，则结果值为 UNPREDICTABLE。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mod_s.w))]
pub unsafe fn __msa_mod_s_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_mod_s_w(a, mem::transmute(b))
}

/// Vector 有符号模数
///
/// vector `a` (两个有符号的 64 位整数) 中的有符号整数元素除以 vector `b` (两个有符号的 64 位整数) 中的有符号整数元素。
///
/// 与被除数相同符号的其余部分写入 vector (两个带符号的 64 位整数)。
/// 如果除数元素 vector `b` 为零，则结果值为 UNPREDICTABLE。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mod_s.d))]
pub unsafe fn __msa_mod_s_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_mod_s_d(a, mem::transmute(b))
}

/// Vector 无符号模数
///
/// vector `a` 中的无符号整数元素 (十六个无符号 8 位整数) 除以 vector `b` 中的无符号整数元素 (十六个无符号 8 位整数)。
///
/// 与除数相同的符号的其余部分写入 vector (十六个无符号的 8 位整数)。
/// 如果除数元素 vector `b` 为零，则结果值为 UNPREDICTABLE。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mod_u.b))]
pub unsafe fn __msa_mod_u_b(a: v16u8, b: v16u8) -> v16u8 {
    msa_mod_u_b(a, mem::transmute(b))
}

/// Vector 无符号模数
///
/// vector `a` 中的无符号整数元素 (八个无符号 16 位整数) 除以 vector `b` 中的无符号整数元素 (八个无符号 16 位整数)。
///
/// 与被除数相同符号的其余部分写入 vector (八个无符号的 16 位整数)。
/// 如果除数元素 vector `b` 为零，则结果值为 UNPREDICTABLE。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mod_u.h))]
pub unsafe fn __msa_mod_u_h(a: v8u16, b: v8u16) -> v8u16 {
    msa_mod_u_h(a, mem::transmute(b))
}

/// Vector 无符号模数
///
/// vector `a` 中的无符号整数元素 (四个无符号 32 位整数) 除以 vector `b` 中的无符号整数元素 (四个无符号 32 位整数)。
///
/// 与被除数相同符号的其余部分写入 vector (四个无符号的 32 位整数)。
/// 如果除数元素 vector `b` 为零，则结果值为 UNPREDICTABLE。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mod_u.w))]
pub unsafe fn __msa_mod_u_w(a: v4u32, b: v4u32) -> v4u32 {
    msa_mod_u_w(a, mem::transmute(b))
}

/// Vector 无符号模数
///
/// vector `a` (两个无符号的 64 位整数) 中的无符号整数元素除以 vector `b` (两个无符号的 64 位整数) 中的无符号整数元素。
///
/// 与被除数相同符号的其余部分写入 vector (两个无符号的 64 位整数)。
/// 如果除数元素 vector `b` 为零，则结果值为 UNPREDICTABLE。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mod_u.d))]
pub unsafe fn __msa_mod_u_d(a: v2u64, b: v2u64) -> v2u64 {
    msa_mod_u_d(a, mem::transmute(b))
}

/// Vector 移动
///
/// 将 vector `a` (八个带符号的 16 位整数) 中的所有 WRLEN 位复制到 vector (八个带符号的 16 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(move.v))]
pub unsafe fn __msa_move_v(a: v16i8) -> v16i8 {
    msa_move_v(a)
}

/// Vector 定点的乘法和减法
///
/// 从 vector `a` (八个符号) 中减去 vector `c` (八个带符号的 16 位整数) 中定点元素与 vector `b` (八个符号的 16 位整数) 中定点元素的乘积。16 位整数)。
///
/// 乘法结果未饱和，即从目标中减去精确的 (-1) * (-1) =1。
/// 饱和的定点结果将存储回 vector `a`。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(msub_q.h))]
pub unsafe fn __msa_msub_q_h(a: v8i16, b: v8i16, c: v8i16) -> v8i16 {
    msa_msub_q_h(a, mem::transmute(b), c)
}

/// Vector 定点的乘法和减法
///
/// 从 vector `a` (四个带符号的整数) 中减去 vector `c` (四个带符号的 32 位整数) 中的定点元素与 vector `b` (四个带符号的 32 位整数) 中的定点元素的乘积。32 位整数)。
///
/// 乘法结果未饱和，即从目标中减去精确的 (-1) * (-1) =1。
/// 饱和的定点结果将存储回 vector `a`。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(msub_q.w))]
pub unsafe fn __msa_msub_q_w(a: v4i32, b: v4i32, c: v4i32) -> v4i32 {
    msa_msub_q_w(a, mem::transmute(b), c)
}

/// Vector 定点乘法和减法四舍五入
///
/// 从 vector `a` (八个符号) 中减去 vector `c` (八个带符号的 16 位整数) 中定点元素与 vector `b` (八个符号的 16 位整数) 中定点元素的乘积。16 位整数)。
///
/// 乘法结果未饱和，即从目标中减去精确的 (-1) * (-1) =1。
/// 取整和饱和的定点结果将存储回 vector `a`。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(msubr_q.h))]
pub unsafe fn __msa_msubr_q_h(a: v8i16, b: v8i16, c: v8i16) -> v8i16 {
    msa_msubr_q_h(a, mem::transmute(b), c)
}

/// Vector 定点乘法和减法四舍五入
///
/// 从 vector `a` (四个带符号的整数) 中减去 vector `c` (四个带符号的 32 位整数) 中的定点元素与 vector `b` (四个带符号的 32 位整数) 中的定点元素的乘积。32 位整数)。
///
/// 乘法结果未饱和，即从目标中减去精确的 (-1) * (-1) =1。
/// 取整和饱和的定点结果将存储回 vector `a`。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(msubr_q.w))]
pub unsafe fn __msa_msubr_q_w(a: v4i32, b: v4i32, c: v4i32) -> v4i32 {
    msa_msubr_q_w(a, mem::transmute(b), c)
}

/// Vector 乘法和减法
///
/// 将 vector `c` (十六个带符号的 8 位整数) 中的整数元素与 vector `b` (十六个带符号的 8 位整数) 中的整数元素相乘，然后从 vector `a` (十六个带符号的 8 位整数中) 的整数元素中减去。。
///
/// 乘法结果中最重要的一半将被丢弃。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(msubv.b))]
pub unsafe fn __msa_msubv_b(a: v16i8, b: v16i8, c: v16i8) -> v16i8 {
    msa_msubv_b(a, mem::transmute(b), c)
}

/// Vector 乘法和减法
///
/// vector `c` (八个带符号的 16 位整数) 中的整数元素与 vector `b` (八个带符号的 16 位整数) 中的整数元素相乘，然后从 vector `a` (八个带符号的 16 位整数中) 的整数元素中减去。。
///
/// 乘法结果中最重要的一半将被丢弃。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(msubv.h))]
pub unsafe fn __msa_msubv_h(a: v8i16, b: v8i16, c: v8i16) -> v8i16 {
    msa_msubv_h(a, mem::transmute(b), c)
}

/// Vector 乘法和减法
///
/// vector `c` (四个带符号的 32 位整数) 中的整数元素与 vector `b` (四个带符号的 32 位整数) 中的整数元素相乘，然后从 vector `a` (四个带符号的 32 位整数中) 的整数元素中减去。。
///
/// 乘法结果中最重要的一半将被丢弃。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(msubv.w))]
pub unsafe fn __msa_msubv_w(a: v4i32, b: v4i32, c: v4i32) -> v4i32 {
    msa_msubv_w(a, mem::transmute(b), c)
}

/// Vector 乘法和减法
///
/// vector `c` (两个带符号的 64 位整数) 中的整数元素与 vector `b` (两个带符号的 64 位整数) 中的整数元素相乘，然后从 vector `a` (两个带符号的 64 位整数中) 的整数元素中减去。。
///
/// 乘法结果中最重要的一半将被丢弃。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(msubv.d))]
pub unsafe fn __msa_msubv_d(a: v2i64, b: v2i64, c: v2i64) -> v2i64 {
    msa_msubv_d(a, mem::transmute(b), c)
}

/// Vector 定点乘法
///
/// vector `a` (八个带符号的 16 位整数) 中的定点元素乘以 vector `b` (八个带符号的 16 位整数) 中的定点元素。
///
/// 结果写入 vector (八个带符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mul_q.h))]
pub unsafe fn __msa_mul_q_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_mul_q_h(a, mem::transmute(b))
}

/// Vector 定点乘法
///
/// vector `a` (四个带符号的 32 位整数) 中的定点元素乘以 vector `b` (四个带符号的 32 位整数) 中的定点元素。
///
/// 结果写入 vector (四个带符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mul_q.w))]
pub unsafe fn __msa_mul_q_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_mul_q_w(a, mem::transmute(b))
}

/// Vector 定点乘法四舍五入
///
/// vector `a` (八个带符号的 16 位整数) 中的定点元素乘以 vector `b` (八个带符号的 16 位整数) 中的定点元素。
///
/// 舍入后的结果将写入 vector (八个带符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mulr_q.h))]
pub unsafe fn __msa_mulr_q_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_mulr_q_h(a, mem::transmute(b))
}

/// Vector 定点乘法四舍五入
///
/// vector `a` (四个带符号的 32 位整数) 中的定点元素乘以 vector `b` (四个带符号的 32 位整数) 中的定点元素。
///
/// 舍入后的结果将写入 vector (四个带符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mulr_q.w))]
pub unsafe fn __msa_mulr_q_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_mulr_q_w(a, mem::transmute(b))
}

/// Vector 相乘
///
/// vector `a` 中的整数元素 (十六个带符号的 8 位整数) 与 vector `b` 中的整数元素 (十六个带符号 8 位整数) 相乘。
///
/// 结果写入 vector (十六个带符号的 8 位整数)。
/// 乘法结果中最重要的一半将被丢弃。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mulv.b))]
pub unsafe fn __msa_mulv_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_mulv_b(a, mem::transmute(b))
}

/// Vector 相乘
///
/// vector `a` (八个带符号的 16 位整数) 中的整数元素与 vector `b` (八个带符号的 16 位整数) 中的整数元素相乘。
///
/// 结果写入 vector (八个带符号的 16 位整数)。
/// 乘法结果中最重要的一半将被丢弃。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mulv.h))]
pub unsafe fn __msa_mulv_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_mulv_h(a, mem::transmute(b))
}

/// Vector 相乘
///
/// vector `a` (四个带符号的 32 位整数) 中的整数元素与 vector `b` (四个带符号的 32 位整数) 中的整数元素相乘。
///
/// 结果写入 vector (四个带符号的 32 位整数)。
/// 乘法结果中最重要的一半将被丢弃。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mulv.w))]
pub unsafe fn __msa_mulv_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_mulv_w(a, mem::transmute(b))
}

/// Vector 相乘
///
/// vector `a` (两个带符号的 64 位整数) 中的整数元素与 vector `b` (两个带符号的 64 位整数) 中的整数元素相乘。
///
/// 结果写入 vector (两个带符号的 64 位整数)。
/// 乘法结果中最重要的一半将被丢弃。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(mulv.d))]
pub unsafe fn __msa_mulv_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_mulv_d(a, mem::transmute(b))
}

/// Vector 领先的计数
///
/// vector `a` 中的元素的前导数字 (十六个带符号的 8 位整数) 存储到 vector 中的元素 (十六个带符号的 8 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(nloc.b))]
pub unsafe fn __msa_nloc_b(a: v16i8) -> v16i8 {
    msa_nloc_b(a)
}

/// Vector 领先的计数
///
/// vector `a` 中的元素的前导数字 (八个带符号的 16 位整数) 存储到 vector 中的元素 (八个带符号的 16 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(nloc.h))]
pub unsafe fn __msa_nloc_h(a: v8i16) -> v8i16 {
    msa_nloc_h(a)
}

/// Vector 领先的计数
///
/// vector `a` 中的元素的前导数字 (四个有符号 32 位整数) 存储到 vector 中的元素 (四个有符号 32 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(nloc.w))]
pub unsafe fn __msa_nloc_w(a: v4i32) -> v4i32 {
    msa_nloc_w(a)
}

/// Vector 领先的计数
///
/// vector `a` 中的元素的前导数字 (两个带符号的 64 位整数) 存储到 vector 中的元素 (两个带符号的 64 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(nloc.d))]
pub unsafe fn __msa_nloc_d(a: v2i64) -> v2i64 {
    msa_nloc_d(a)
}

/// Vector 领先的零计数
///
/// vector `a` 中的元素的前导零数目 (十六个带符号的 8 位整数) 存储到 vector 中的元素 (十六个带符号的 8 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(nlzc.b))]
pub unsafe fn __msa_nlzc_b(a: v16i8) -> v16i8 {
    msa_nlzc_b(a)
}

/// Vector 领先的零计数
///
/// vector `a` 中的元素的前导零数 (八个带符号的 16 位整数) 存储到 vector 中的元素 (八个带符号的 16 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(nlzc.h))]
pub unsafe fn __msa_nlzc_h(a: v8i16) -> v8i16 {
    msa_nlzc_h(a)
}

/// Vector 领先的零计数
///
/// vector `a` 中的元素的前导零数 (四个有符号的 32 位整数) 存储到 vector 中的元素 (四个有符号的 32 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(nlzc.w))]
pub unsafe fn __msa_nlzc_w(a: v4i32) -> v4i32 {
    msa_nlzc_w(a)
}

/// Vector 领先的零计数
///
/// vector `a` 中的元素的前导零数 (两个有符号的 64 位整数) 存储到 vector 中的元素 (两个有符号的 64 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(nlzc.d))]
pub unsafe fn __msa_nlzc_d(a: v2i64) -> v2i64 {
    msa_nlzc_d(a)
}

/// Vector 逻辑取反或
///
/// vector `a` 的每个位 (十六个无符号的 8 位整数) 与 vector `b` 的对应位 (十六个无符号的 8 位整数) 按位逻辑或运算进行组合。
/// 结果写入 vector (十六个无符号的 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(nor.v))]
pub unsafe fn __msa_nor_v(a: v16u8, b: v16u8) -> v16u8 {
    msa_nor_v(a, mem::transmute(b))
}

/// Immediate 逻辑否定或
///
/// vector `a` 的每个位 (十六个无符号的 8 位整数) 与 8 位 immediate  `imm8` 进行按位逻辑或运算。
/// 结果写入 vector (十六个无符号的 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(nori.b, imm8 = 0b11111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_nori_b<const IMM8: i32>(a: v16u8) -> v16u8 {
    static_assert_imm8!(IMM8);
    msa_nori_b(a, IMM8)
}

/// Vector 逻辑或
///
/// vector `a` 的每个位 (十六个无符号的 8 位整数) 与 vector `b` 的对应位 (十六个无符号的 8 位整数) 按位逻辑或运算进行组合。
/// 结果写入 vector (十六个无符号的 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(or.v))]
pub unsafe fn __msa_or_v(a: v16u8, b: v16u8) -> v16u8 {
    msa_or_v(a, mem::transmute(b))
}

/// Immediate 逻辑或
///
/// vector `a` 的每一位 (十六个无符号的 8 位整数) 与 8 位 immediate  `imm8` 进行按位逻辑或运算。
/// 结果写入 vector (十六个无符号的 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(ori.b, imm8 = 0b11111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_ori_b<const IMM8: i32>(a: v16u8) -> v16u8 {
    static_assert_imm8!(IMM8);
    msa_ori_b(a, IMM8)
}

/// Vector 包 Even
///
/// vectors `a` 中的偶数元素 (十六个带符号的 8 位整数) 复制到结果 vector 的左半部分，而 vector `b` 中的偶数元素 (十六个带符号的 8 位整数数字) 被复制到结果 vector 的右半部分。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(pckev.b))]
pub unsafe fn __msa_pckev_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_pckev_b(a, mem::transmute(b))
}

/// Vector 包 Even
///
/// vectors `a` 中的偶数元素 (八个带符号的 16 位整数) 被复制到结果 vector 的左半部分，而 vector `b` 中的偶数元素 (八个带符号的 16 位整数) 被复制到结果 vector 的右半部分。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(pckev.h))]
pub unsafe fn __msa_pckev_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_pckev_h(a, mem::transmute(b))
}

/// Vector 包 Even
///
/// vectors `a` 中的偶数元素 (四个带符号的 32 位整数) 被复制到结果 vector 的左半部分，vector `b` 中的偶数元素 (四个带符号的 32 位整数) 被复制到结果 vector 的右半部分。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(pckev.w))]
pub unsafe fn __msa_pckev_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_pckev_w(a, mem::transmute(b))
}

/// Vector 包 Even
///
/// vectors `a` 中的偶数元素 (两个有符号的 64 位整数) 复制到结果 vector 的左半部分，而 vector `b` 中的偶数元素 (两个有符号 64 位整数的整数) 复制到结果 vector 的右半部分。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(pckev.d))]
pub unsafe fn __msa_pckev_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_pckev_d(a, mem::transmute(b))
}

/// Vector Pack 奇数
///
/// 将 vectors `a` 中的奇数元素 (十六个带符号的 8 位整数) 复制到结果 vector 的左半部分，将 vector `b` 中的奇数元素 (十六个带符号的 8 位整数) 复制到结果 vector 的右半部分。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(pckod.b))]
pub unsafe fn __msa_pckod_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_pckod_b(a, mem::transmute(b))
}

/// Vector Pack 奇数
///
/// 将 vectors `a` 中的奇数元素 (八个有符号的 16 位整数) 复制到结果 vector 的左半部分，并将 vector `b` 中的奇数元素 (八个有符号 16 位整数的整数) 复制到结果 vector 的右半部分。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(pckod.h))]
pub unsafe fn __msa_pckod_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_pckod_h(a, mem::transmute(b))
}

/// Vector Pack 奇数
///
/// 将 vectors `a` 中的奇数元素 (四个有符号的 32 位整数) 复制到结果 vector 的左半部分，并将 vector `b` 中的奇数元素 (四个有符号 32 位整数的整数) 复制到结果 vector 的右半部分。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(pckod.w))]
pub unsafe fn __msa_pckod_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_pckod_w(a, mem::transmute(b))
}

/// Vector Pack 奇数
///
/// 将 vectors `a` 中的奇数元素 (两个有符号的 64 位整数) 复制到结果 vector 的左半部分，并将 vector `b` 中的奇数元素 (两个有符号 64 位整数的整数) 复制到结果 vector 的右半部分。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(pckod.d))]
pub unsafe fn __msa_pckod_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_pckod_d(a, mem::transmute(b))
}

/// Vector 人口计数
///
/// 将 vector `a` 中的元素设置为 1 的位数 (十六个带符号的 8 位整数) 存储到结果 vector 中的元素 (十六个带符号的 8 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(pcnt.b))]
pub unsafe fn __msa_pcnt_b(a: v16i8) -> v16i8 {
    msa_pcnt_b(a)
}

/// Vector 人口计数
///
/// 将 vector `a` 中的元素设置为 1 的位数 (八个带符号的 16 位整数) 存储到结果 vector 中的元素 (八个带符号的 16 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(pcnt.h))]
pub unsafe fn __msa_pcnt_h(a: v8i16) -> v8i16 {
    msa_pcnt_h(a)
}

/// Vector 人口计数
///
/// 将 vector `a` 中的元素设置为 1 的位数 (四个有符号的 32 位整数) 存储到结果 vector (四个有符号的 32 位整数) 中的元素。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(pcnt.w))]
pub unsafe fn __msa_pcnt_w(a: v4i32) -> v4i32 {
    msa_pcnt_w(a)
}

/// Vector 人口计数
///
/// 将 vector `a` 中的元素设置为 1 的位数 (两个带符号的 64 位整数) 存储到结果 vector 中的元素 (两个带符号的 64 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(pcnt.d))]
pub unsafe fn __msa_pcnt_d(a: v2i64) -> v2i64 {
    msa_pcnt_d(a)
}

/// Immediate 有符号饱和
///
/// vector `a` 中的带符号元素 (十六个带符号的 8 位整数) 在不更改数据宽度的情况下已饱和为 `imm3+1` 位的带符号值。
///
/// 结果存储在 vector (十六个带符号的 8 位整数) 中。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sat_s.b, imm4 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_sat_s_b<const IMM3: i32>(a: v16i8) -> v16i8 {
    static_assert_imm3!(IMM3);
    msa_sat_s_b(a, IMM3)
}

/// Immediate 有符号饱和
///
/// vector `a` (八个带符号的 16 位整数) 中的带符号元素在不更改数据宽度的情况下已饱和为 `imm4+1` 位的带符号值。
///
/// 结果存储在 vector (八个带符号的 16 位整数) 中。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sat_s.h, imm3 = 0b1111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_sat_s_h<const IMM4: i32>(a: v8i16) -> v8i16 {
    static_assert_imm4!(IMM4);
    msa_sat_s_h(a, IMM4)
}

/// Immediate 有符号饱和
///
/// vector `a` 中的带符号元素 (四个带符号的 32 位整数) 在不更改数据宽度的情况下已饱和为 `imm5+1` 位的带符号值。
///
/// 结果存储在 vector (四个带符号的 32 位整数) 中。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sat_s.w, imm2 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_sat_s_w<const IMM5: i32>(a: v4i32) -> v4i32 {
    static_assert_imm5!(IMM5);
    msa_sat_s_w(a, IMM5)
}

/// Immediate 有符号饱和
///
/// vector `a` 中的带符号元素 (两个带符号的 64 位整数) 已饱和为 `imm6+1` 位的带符号值，而不会更改数据宽度。
///
/// 结果存储在 vector (两个带符号的 64 位整数) 中。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sat_s.d, imm1 = 0b111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_sat_s_d<const IMM6: i32>(a: v2i64) -> v2i64 {
    static_assert_imm6!(IMM6);
    msa_sat_s_d(a, IMM6)
}

/// Immediate 无符号饱和
///
/// vector `a` 中的无符号元素 (十六个无符号的 8 位整数) 将饱和为 `imm3+1` 位的无符号值，而不会更改数据宽度。
///
/// 结果存储在 vector (十六个无符号的 8 位整数) 中。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sat_u.b, imm4 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_sat_u_b<const IMM3: i32>(a: v16u8) -> v16u8 {
    static_assert_imm3!(IMM3);
    msa_sat_u_b(a, IMM3)
}

/// Immediate 无符号饱和
///
/// vector `a` 中的无符号元素 (八个无符号的 16 位整数) 将饱和为 `imm4+1` 位的无符号值，而不会更改数据宽度。
///
/// 结果存储在 vector (八个无符号的 16 位整数) 中。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sat_u.h, imm3 = 0b1111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_sat_u_h<const IMM4: i32>(a: v8u16) -> v8u16 {
    static_assert_imm4!(IMM4);
    msa_sat_u_h(a, IMM4)
}

/// Immediate 无符号饱和
///
/// vector `a` 中的无符号元素 (四个无符号的 32 位整数) 将饱和为 `imm5+1` 位的无符号值，而不会更改数据宽度。
///
/// 结果存储在 vector (四个无符号的 32 位整数) 中。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sat_u.w, imm2 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_sat_u_w<const IMM5: i32>(a: v4u32) -> v4u32 {
    static_assert_imm5!(IMM5);
    msa_sat_u_w(a, IMM5)
}

/// Immediate 无符号饱和
///
/// vector `a` 中的无符号元素 (两个无符号 64 位整数) 将饱和为 `imm6+1` 位的无符号值，而不会更改数据宽度。
///
/// 结果存储在 vector (两个无符号的 64 位整数) 中。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sat_u.d, imm1 = 0b111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_sat_u_d<const IMM6: i32>(a: v2u64) -> v2u64 {
    static_assert_imm6!(IMM6);
    msa_sat_u_d(a, IMM6)
}

/// Immediate 设置随机元素
///
/// set shuffle 指令适用于 4 元素集。
/// 所有集合都以相同的方式洗牌：将 `a` 中的元素 i82i + 1..2i (十六个带符号的 8 位整数) 复制到元素 i 的结果 vector (十六个带符号的 8 位整数) 中，其中 i 为 0、1、2、3。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(shf.b, imm8 = 0b11111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_shf_b<const IMM8: i32>(a: v16i8) -> v16i8 {
    static_assert_imm8!(IMM8);
    msa_shf_b(a, IMM8)
}

/// Immediate 设置随机元素
///
/// set shuffle 指令适用于 4 元素集。
/// 所有集合都以相同的方式洗牌: `a` 中的元素 i82i + 1..2i (八个带符号的 16 位整数) 复制到元素 i 上，结果为 vector (八个带符号的 16 位整数)，其中 i 为 0、1、2、3。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(shf.h, imm8 = 0b11111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_shf_h<const IMM8: i32>(a: v8i16) -> v8i16 {
    static_assert_imm8!(IMM8);
    msa_shf_h(a, IMM8)
}

/// Immediate 设置随机元素
///
/// set shuffle 指令适用于 4 元素集。
/// 所有集合都以相同的方式洗牌：将 `a` 中的元素 i82i + 1..2i (四个带符号的 32 位整数) 复制到元素 i 的结果 vector (四个带符号的 32 位整数) 中，其中 i 为 0、1、2、3。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(shf.w, imm8 = 0b11111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_shf_w<const IMM8: i32>(a: v4i32) -> v4i32 {
    static_assert_imm8!(IMM8);
    msa_shf_w(a, IMM8)
}

/// GPR 列幻灯片
///
/// Vector 寄存器 `a` (十六个带符号的 8 位整数) 和 `b` (十六个带符号的 8 位整数) 包含二维字节数组 (rectangles)，按行存储，其行数与整数数据格式 df 中的字节数相同。
///
/// 两个源矩形 `b` 和 `a` 按照它们在语法中出现的顺序水平连接，即，首先是 `a`，然后是 `b`。
/// 在 `b` 上放置一个新的目标矩形，然后按 GPR `c` 中给定的列数将其向 `a` 和 `b` 的串联上方的左侧滑动。
/// 结果写入 vector (十六个带符号的 8 位整数)。
/// GPR `c` 值以目标矩形中的列数为模，或者等效地以目标 vector 中的数据格式 df 元素数来解释。
///
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sld.b))]
pub unsafe fn __msa_sld_b(a: v16i8, b: v16i8, c: i32) -> v16i8 {
    msa_sld_b(a, mem::transmute(b), c)
}

/// GPR 列幻灯片
///
/// Vector 寄存器 `a` (八个带符号的 16 位整数) 和 `b` (八个带符号的 16 位整数) 包含按行存储的二维字节数组 (rectangles)，其行数与整数数据格式 df 中的字节数相同。
///
/// 两个源矩形 `b` 和 `a` 按照它们在语法中出现的顺序水平连接，即，首先是 `a`，然后是 `b`。
/// 在 `b` 上放置一个新的目标矩形，然后按 GPR `c` 中给定的列数将其向 `a` 和 `b` 的串联上方的左侧滑动。
/// 结果写入 vector (八个带符号的 16 位整数)。
/// GPR `c` 值以目标矩形中的列数为模，或者等效地以目标 vector 中的数据格式 df 元素数来解释。
///
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sld.h))]
pub unsafe fn __msa_sld_h(a: v8i16, b: v8i16, c: i32) -> v8i16 {
    msa_sld_h(a, mem::transmute(b), c)
}

/// GPR 列幻灯片
///
/// Vector 寄存器 `a` (四个带符号的 32 位整数) 和 `b` (四个带符号的 32 位整数) 包含二维字节数组 (rectangles)，按行存储，其行数与整数数据格式 df 中的字节数相同。
///
/// 两个源矩形 `b` 和 `a` 按照它们在语法中出现的顺序水平连接，即，首先是 `a`，然后是 `b`。
/// 在 `b` 上放置一个新的目标矩形，然后按 GPR `c` 中给定的列数将其向 `a` 和 `b` 的串联上方的左侧滑动。
/// 结果写入 vector (四个带符号的 32 位整数)。
/// GPR `c` 值以目标矩形中的列数为模，或者等效地以目标 vector 中的数据格式 df 元素数来解释。
///
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sld.w))]
pub unsafe fn __msa_sld_w(a: v4i32, b: v4i32, c: i32) -> v4i32 {
    msa_sld_w(a, mem::transmute(b), c)
}

/// GPR 列幻灯片
///
/// Vector 寄存器 `a` (两个带符号的 64 位整数) 和 `b` (两个带符号的 64 位整数) 包含按行存储的二维字节数组 (rectangles)，其行数与整数数据格式 df 中的字节数相同。
///
/// 两个源矩形 `b` 和 `a` 按照它们在语法中出现的顺序水平连接，即，首先是 `a`，然后是 `b`。
/// 在 `b` 上放置一个新的目标矩形，然后按 GPR `c` 中给定的列数将其向 `a` 和 `b` 的串联上方的左侧滑动。
/// 结果写入 vector (两个带符号的 64 位整数)。
/// GPR `c` 值以目标矩形中的列数为模，或者等效地以目标 vector 中的数据格式 df 元素数来解释。
///
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sld.d))]
pub unsafe fn __msa_sld_d(a: v2i64, b: v2i64, c: i32) -> v2i64 {
    msa_sld_d(a, mem::transmute(b), c)
}

/// Immediate 列幻灯片
///
/// Vector 寄存器 `a` (十六个带符号的 8 位整数) 和 `b` (十六个带符号的 8 位整数) 包含二维字节数组 (rectangles)，按行存储，其行数与整数数据格式 df 中的字节数相同。
///
/// 两个源矩形 `b` 和 `a` 按照它们在语法中出现的顺序水平连接，即，首先是 `a`，然后是 `b`。
/// 在 `b` 上放置一个新的目标矩形，然后将其滑动到 `a` 和 `b` 的 `imm1` 列的串联上方的左侧。
/// 结果写入 vector (十六个带符号的 8 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sldi.b, imm4 = 0b1111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_sldi_b<const IMM4: i32>(a: v16i8, b: v16i8) -> v16i8 {
    static_assert_imm4!(IMM4);
    msa_sldi_b(a, mem::transmute(b), IMM4)
}

/// Immediate 列幻灯片
///
/// Vector 寄存器 `a` (八个带符号的 16 位整数) 和 `b` (八个带符号的 16 位整数) 包含按行存储的二维字节数组 (rectangles)，其行数与整数数据格式 df 中的字节数相同。
///
/// 两个源矩形 `b` 和 `a` 按照它们在语法中出现的顺序水平连接，即，首先是 `a`，然后是 `b`。
/// 在 `b` 上放置一个新的目标矩形，然后将其滑动到 `a` 和 `b` 的 `imm1` 列的串联上方的左侧。
/// 结果写入 vector (八个带符号的 16 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sldi.h, imm3 = 0b111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_sldi_h<const IMM3: i32>(a: v8i16, b: v8i16) -> v8i16 {
    static_assert_imm3!(IMM3);
    msa_sldi_h(a, mem::transmute(b), IMM3)
}

/// Immediate 列幻灯片
///
/// Vector 寄存器 `a` (四个带符号的 32 位整数) 和 `b` (四个带符号的 32 位整数) 包含二维字节数组 (rectangles)，按行存储，其行数与整数数据格式 df 中的字节数相同。
///
/// 两个源矩形 `b` 和 `a` 按照它们在语法中出现的顺序水平连接，即，首先是 `a`，然后是 `b`。
/// 在 `b` 上放置一个新的目标矩形，然后将其滑动到 `a` 和 `b` 的 `imm1` 列的串联上方的左侧。
/// 结果写入 vector (四个带符号的 32 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sldi.w, imm2 = 0b11))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_sldi_w<const IMM2: i32>(a: v4i32, b: v4i32) -> v4i32 {
    static_assert_imm2!(IMM2);
    msa_sldi_w(a, mem::transmute(b), IMM2)
}

/// Immediate 列幻灯片
///
/// Vector 寄存器 `a` (两个带符号的 64 位整数) 和 `b` (两个带符号的 64 位整数) 包含按行存储的二维字节数组 (rectangles)，其行数与整数数据格式 df 中的字节数相同。
///
/// 两个源矩形 `b` 和 `a` 按照它们在语法中出现的顺序水平连接，即，首先是 `a`，然后是 `b`。
/// 在 `b` 上放置一个新的目标矩形，然后将其滑动到 `a` 和 `b` 的 `imm1` 列的串联上方的左侧。
/// 结果写入 vector (两个带符号的 64 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sldi.d, imm1 = 0b1))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_sldi_d<const IMM1: i32>(a: v2i64, b: v2i64) -> v2i64 {
    static_assert_imm1!(IMM1);
    msa_sldi_d(a, mem::transmute(b), IMM1)
}

/// Vector 左移
///
/// vector `a` (十六个带符号的 8 位整数) 中的元素向左移动 vector `b` (十六个带符号的 8 位整数) 中的元素以位为模的位数。
/// 结果写入 vector (十六个带符号的 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sll.b))]
pub unsafe fn __msa_sll_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_sll_b(a, mem::transmute(b))
}

/// Vector 左移
///
/// vector `a` (八个带符号的 16 位整数) 中的元素向左移动 vector `b` (八个带符号的 16 位整数) 中的元素向左移动的位数。
/// 结果写入 vector (八个带符号的 16 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sll.h))]
pub unsafe fn __msa_sll_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_sll_h(a, mem::transmute(b))
}

/// Vector 左移
///
/// vector `a` (四个带符号的 32 位整数) 中的元素向左移动 vector `b` (四个带符号的 32 位整数) 中的元素向左移动的位数。
/// 结果写入 vector (四个带符号的 32 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sll.w))]
pub unsafe fn __msa_sll_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_sll_w(a, mem::transmute(b))
}

/// Vector 左移
///
/// vector `a` (两个带符号的 64 位整数) 中的元素向左移动 vector `b` (两个带符号的 64 位整数) 中的元素向左移动的位数。
/// 结果写入 vector (两个带符号的 64 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sll.d))]
pub unsafe fn __msa_sll_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_sll_d(a, mem::transmute(b))
}

/// Immediate 左移
///
/// vector `a` 中的元素 (十六个带符号的 8 位整数) 向左移动 `imm4` 位。
///
/// 结果写入 vector (十六个带符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(slli.b, imm4 = 0b1111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_slli_b<const IMM4: i32>(a: v16i8) -> v16i8 {
    static_assert_imm4!(IMM4);
    msa_slli_b(a, IMM4)
}

/// Immediate 左移
///
/// vector `a` (八个带符号的 16 位整数) 中的元素向左移动 `imm3` 位。
///
/// 结果写入 vector (八个带符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(slli.h, imm3 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_slli_h<const IMM3: i32>(a: v8i16) -> v8i16 {
    static_assert_imm3!(IMM3);
    msa_slli_h(a, IMM3)
}

/// Immediate 左移
///
/// vector `a` (四个带符号的 32 位整数) 中的元素向左移动 `imm2` 位。
///
/// 结果写入 vector (四个带符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(slli.w, imm2 = 0b11))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_slli_w<const IMM2: i32>(a: v4i32) -> v4i32 {
    static_assert_imm2!(IMM2);
    msa_slli_w(a, IMM2)
}

/// Immediate 左移
///
/// vector `a` (两个带符号的 64 位整数) 中的元素向左移动 `imm1` 位。
///
/// 结果写入 vector (两个带符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(slli.d, imm1 = 0b1))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_slli_d<const IMM1: i32>(a: v2i64) -> v2i64 {
    static_assert_imm1!(IMM1);
    msa_slli_d(a, IMM1)
}

/// GPR 元素图示
///
/// 将具有 GPR `b` 给定索引的 vector `a` (十六个带符号的 8 位整数) 元素复制到 vector (十六个带符号的 8 位整数) 中的所有元素 GPR `b` 值以目标 vector 中数据格式 df 元素的数量为模。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(splat.b))]
pub unsafe fn __msa_splat_b(a: v16i8, b: i32) -> v16i8 {
    msa_splat_b(a, mem::transmute(b))
}

/// GPR 元素图示
///
/// 将具有 GPR `b` 给定索引的 vector `a` (八个带符号的 16 位整数) 元素复制到 vector (八个带符号的 16 位整数) 中的所有元素 GPR `b` 值以目标 vector 中的数据格式 df 元素个数为模。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(splat.h))]
pub unsafe fn __msa_splat_h(a: v8i16, b: i32) -> v8i16 {
    msa_splat_h(a, mem::transmute(b))
}

/// GPR 元素图示
///
/// 将具有 GPR `b` 给定索引的 vector `a` (四个带符号的 32 位整数) 元素复制到 vector (四个带符号的 32 位整数) 中的所有元素 GPR `b` 值以目标 vector 中的数据格式 df 元素个数为模。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(splat.w))]
pub unsafe fn __msa_splat_w(a: v4i32, b: i32) -> v4i32 {
    msa_splat_w(a, mem::transmute(b))
}

/// GPR 元素图示
///
/// 将具有 GPR `b` 给定索引的 vector `a` (两个带符号的 64 位整数) 元素复制到 vector (两个带符号的 64 位整数) 中的所有元素 GPR `b` 值以目标 vector 中的数据格式 df 元素个数为模。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(splat.d))]
pub unsafe fn __msa_splat_d(a: v2i64, b: i32) -> v2i64 {
    msa_splat_d(a, mem::transmute(b))
}

/// Immediate 元素 Splat
///
/// 将 vector `a` 中的元素 `imm4` (十六个带符号的 8 位整数) 复制到 vector 中的所有元素 (十六个带符号的 8 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(splati.b, imm4 = 0b1111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_splati_b<const IMM4: i32>(a: v16i8) -> v16i8 {
    static_assert_imm4!(IMM4);
    msa_splati_b(a, IMM4)
}

/// Immediate 元素 Splat
///
/// 将 vector `a` 中的元素 `imm3` (八个带符号的 16 位整数) 复制到 vector 中的所有元素 (八个带符号的 16 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(splati.h, imm3 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_splati_h<const IMM3: i32>(a: v8i16) -> v8i16 {
    static_assert_imm3!(IMM3);
    msa_splati_h(a, IMM3)
}

/// Immediate 元素 Splat
///
/// 将 vector `a` (四个带符号的 32 位整数) 中的元素 `imm2` 复制到 vector (四个带符号的 32 位整数中) 的所有元素。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(splati.w, imm2 = 0b11))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_splati_w<const IMM2: i32>(a: v4i32) -> v4i32 {
    static_assert_imm2!(IMM2);
    msa_splati_w(a, IMM2)
}

/// Immediate 元素 Splat
///
/// 将 vector `a` (两个带符号的 64 位整数) 中的元素 `imm1` 复制到 vector (两个带符号的 64 位整数中) 的所有元素。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(splati.d, imm1 = 0b1))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_splati_d<const IMM1: i32>(a: v2i64) -> v2i64 {
    static_assert_imm1!(IMM1);
    msa_splati_d(a, IMM1)
}

/// Vector 右移算术
///
/// vector `a` (十六个带符号的 8 位整数) 中的元素按 vector `b` (十六个带符号的 8 位整数) 中元素的位数向右算术移位，以模的形式指定将 bits.The 结果中元素的大小写入其中 vector (十六个带符号的 8 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sra.b))]
pub unsafe fn __msa_sra_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_sra_b(a, mem::transmute(b))
}

/// Vector 右移算术
///
/// 将 vector `a` (八个带符号的 16 位整数) 中的元素右移算术 vector `b` (八个带符号的 16 位整数) 中的元素的位数，以模数形式指定将 bits.The 结果中的元素的大小写入其中 vector (八个带符号的 16 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sra.h))]
pub unsafe fn __msa_sra_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_sra_h(a, mem::transmute(b))
}

/// Vector 右移算术
///
/// vector `a` (四个带符号的 32 位整数) 中的元素按 vector `b` (四个带符号的 32 位整数) 中元素的位数向右算术移位，以模的形式指定将 bits.The 结果中的元素的大小写入其中 vector (四个带符号的 32 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sra.w))]
pub unsafe fn __msa_sra_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_sra_w(a, mem::transmute(b))
}

/// Vector 右移算术
///
/// 将 vector `a` (两个带符号的 64 位整数) 中的元素右移算术 vector `b` (两个带符号的 64 位整数) 中的元素的位数，以模数形式指定将 bits.The 结果中的元素大小写入 vector (两个带符号的 64 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(sra.d))]
pub unsafe fn __msa_sra_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_sra_d(a, mem::transmute(b))
}

/// Immediate 右移算术
///
/// vector `a` (十六个带符号的 8 位整数) 中的元素右移 `imm3` 位。
///
/// 结果写入 vector (十六个带符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srai.b, imm3 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srai_b<const IMM3: i32>(a: v16i8) -> v16i8 {
    static_assert_imm3!(IMM3);
    msa_srai_b(a, IMM3)
}

/// Immediate 右移算术
///
/// vector `a` (八个带符号的 16 位整数) 中的元素右移 `imm4` 位。
///
/// 结果写入 vector (八个带符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srai.h, imm4 = 0b1111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srai_h<const IMM4: i32>(a: v8i16) -> v8i16 {
    static_assert_imm4!(IMM4);
    msa_srai_h(a, IMM4)
}

/// Immediate 右移算术
///
/// vector `a` (四个带符号的 32 位整数) 中的元素右移 `imm5` 位。
///
/// 结果写入 vector (四个带符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srai.w, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srai_w<const IMM5: i32>(a: v4i32) -> v4i32 {
    static_assert_imm5!(IMM5);
    msa_srai_w(a, IMM5)
}

/// Immediate 右移算术
///
/// vector `a` (两个带符号的 64 位整数) 中的元素右移 `imm6` 位。
///
/// 结果写入 vector (两个带符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srai.d, imm6 = 0b111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srai_d<const IMM6: i32>(a: v2i64) -> v2i64 {
    static_assert_imm6!(IMM6);
    msa_srai_d(a, IMM6)
}

/// Vector 右移算术四舍五入
///
/// vector `a` (十六个带符号的 8 位整数) 中的元素按 vector `b` (十六个带符号的 8 位整数) 中元素的位数右移算术，以 bits.The 最高有效舍弃位中的元素大小为模被加到移位值上 (用于四舍五入)，并将结果写入 vector (十六个带符号的 8 位整数)。
///
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srar.b))]
pub unsafe fn __msa_srar_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_srar_b(a, mem::transmute(b))
}

/// Vector 右移算术四舍五入
///
/// vector `a` (八个带符号的 16 位整数) 中的元素按 vector `b` (八个带符号的 16 位整数) 中元素的位数右移算术，以 bits.The 最高有效舍弃位中的元素大小为模被加到移位后的值 (用于四舍五入)，并将结果写入 vector (八个带符号的 16 位整数)。
///
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srar.h))]
pub unsafe fn __msa_srar_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_srar_h(a, mem::transmute(b))
}

/// Vector 右移算术四舍五入
///
/// vector `a` (四个带符号的 32 位整数) 中的元素按 vector `b` (四个带符号的 32 位整数) 中的元素右移的位数指定以 bits.The 中最高有效舍弃位的元素大小为模被加到移位值上 (用于四舍五入)，并将结果写入 vector (四个带符号的 32 位整数)。
///
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srar.w))]
pub unsafe fn __msa_srar_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_srar_w(a, mem::transmute(b))
}

/// Vector 右移算术四舍五入
///
/// vector `a` (两个带符号的 64 位整数) 中的元素按算术右移 vector `b` (两个带符号的 64 位整数) 中的元素以模数形式指定 bits.The 中最高有效舍弃位的元素大小被加到移位值上 (用于四舍五入)，并将结果写入 vector (两个带符号的 64 位整数)。
///
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srar.d))]
pub unsafe fn __msa_srar_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_srar_d(a, mem::transmute(b))
}

/// Immediate 右移算术四舍五入
///
/// `imm3` bits.The 将 vector `a` (十六个带符号的 8 位整数) 中的元素右移，将最高有效舍弃位加到移位值 (用于舍入)，并将结果写入 vector (十六个带符号的 8 位整数。)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srari.b, imm3 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srari_b<const IMM3: i32>(a: v16i8) -> v16i8 {
    static_assert_imm3!(IMM3);
    msa_srari_b(a, IMM3)
}

/// Immediate 右移算术四舍五入
///
/// `imm4` bits.The 将 vector `a` (八个带符号的 16 位整数) 中的元素右移，将最高有效舍弃位加到移位值 (用于舍入)，并将结果写入 vector (八个带符号的 16 位整数。)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srari.h, imm4 = 0b1111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srari_h<const IMM4: i32>(a: v8i16) -> v8i16 {
    static_assert_imm4!(IMM4);
    msa_srari_h(a, IMM4)
}

/// Immediate 右移算术四舍五入
///
/// `imm5` bits.The 将 vector `a` (四个带符号的 32 位整数) 中的元素右移，将最高有效舍弃位加到移位值 (用于舍入)，并将结果写入 vector (四个带符号的 32 位整数。)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srari.w, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srari_w<const IMM5: i32>(a: v4i32) -> v4i32 {
    static_assert_imm5!(IMM5);
    msa_srari_w(a, IMM5)
}

/// Immediate 右移算术四舍五入
///
/// `imm6` bits.The 将 vector `a` (两个带符号的 64 位整数) 中的元素右移，将最高有效舍弃位加到移位值 (用于舍入)，并将结果写入 vector (两个带符号的 64 位整数。)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srari.d, imm6 = 0b111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srari_d<const IMM6: i32>(a: v2i64) -> v2i64 {
    static_assert_imm6!(IMM6);
    msa_srari_d(a, IMM6)
}

/// Vector 右移逻辑
///
/// vector `a` (十六个带符号的 8 位整数) 中的元素向右逻辑移位 vector `b` (十六个带符号的 8 位整数) 中的元素的位数，以模方式指定将 bits.The 结果中的元素的大小写入 vector (十六个带符号的 8 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srl.b))]
pub unsafe fn __msa_srl_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_srl_b(a, mem::transmute(b))
}

/// Vector 右移逻辑
///
/// vector `a` (八个带符号的 16 位整数) 中的元素向右逻辑移位 vector `b` (八个带符号的 16 位整数) 中的元素的位数，以模为单位，指定将 bits.The 结果中的元素的大小写入其中 vector (八个带符号的 16 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srl.h))]
pub unsafe fn __msa_srl_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_srl_h(a, mem::transmute(b))
}

/// Vector 右移逻辑
///
/// vector `a` (四个带符号的 32 位整数) 中的元素向右逻辑移位 vector `b` (四个带符号的 32 位整数) 中的元素的位数，以模方式指定 bits.The 结果中元素的大小写入 vector (四个带符号的 32 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srl.w))]
pub unsafe fn __msa_srl_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_srl_w(a, mem::transmute(b))
}

/// Vector 右移逻辑
///
/// vector `a` (两个带符号的 64 位整数) 中的元素向右逻辑移位 vector `b` (两个带符号的 64 位整数) 中的元素的位数，以模为单位，指定将 bits.The 结果中的元素的大小写入其中 vector (两个带符号的 64 位整数)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srl.d))]
pub unsafe fn __msa_srl_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_srl_d(a, mem::transmute(b))
}

/// Immediate 右移逻辑
///
/// vector `a` 中的元素 (十六个带符号的 8 位整数) 向逻辑上右移 `imm4` 位。
///
/// 结果写入 vector (十六个带符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srli.b, imm4 = 0b1111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srli_b<const IMM4: i32>(a: v16i8) -> v16i8 {
    static_assert_imm4!(IMM4);
    msa_srli_b(a, IMM4)
}

/// Immediate 右移逻辑
///
/// vector `a` (八个带符号的 16 位整数) 中的元素在逻辑上右移 `imm3` 位。
///
/// 结果写入 vector (八个带符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srli.h, imm3 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srli_h<const IMM3: i32>(a: v8i16) -> v8i16 {
    static_assert_imm3!(IMM3);
    msa_srli_h(a, IMM3)
}

/// Immediate 右移逻辑
///
/// vector `a` (四个带符号的 32 位整数) 中的元素在逻辑上右移 `imm2` 位。
///
/// 结果写入 vector (四个带符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srli.w, imm2 = 0b11))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srli_w<const IMM2: i32>(a: v4i32) -> v4i32 {
    static_assert_imm2!(IMM2);
    msa_srli_w(a, IMM2)
}

/// Immediate 右移逻辑
///
/// vector `a` (两个带符号的 64 位整数) 中的元素在逻辑上右移 `imm1` 位。
///
/// 结果写入 vector (两个带符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srli.d, imm1 = 0b1))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srli_d<const IMM1: i32>(a: v2i64) -> v2i64 {
    static_assert_imm1!(IMM1);
    msa_srli_d(a, IMM1)
}

/// Vector 右移逻辑舍入
///
/// vector `a` (十六个带符号的 8 位整数) 中的元素按逻辑逻辑右移 vector `b` (十六个带符号的 8 位整数) 中的元素以模数形式指定 bits.The 最高有效舍弃位中元素的大小被加到移位值上 (用于四舍五入)，并将结果写入 vector (十六个带符号的 8 位整数)。
///
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srlr.b))]
pub unsafe fn __msa_srlr_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_srlr_b(a, mem::transmute(b))
}

/// Vector 右移逻辑舍入
///
/// vector `a` (八个带符号的 16 位整数) 中的元素按逻辑右移 vector `b` (八个带符号的 16 位整数) 中的元素的位数，以模数形式指定 bits.The 中最高有效舍弃位的元素大小被加到移位后的值 (用于四舍五入)，结果被写入 vector (八个带符号的 16 位整数)。
///
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srlr.h))]
pub unsafe fn __msa_srlr_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_srlr_h(a, mem::transmute(b))
}

/// Vector 右移逻辑舍入
///
/// vector `a` (四个带符号的 32 位整数) 中的元素按逻辑右移 vector `b` (四个带符号的 32 位整数) 中的元素的位数，以模数形式指定 bits.The 中最高有效舍弃位的元素大小被加到移位值上 (用于四舍五入)，并将结果写入 vector (四个带符号的 32 位整数)。
///
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srlr.w))]
pub unsafe fn __msa_srlr_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_srlr_w(a, mem::transmute(b))
}

/// Vector 右移逻辑舍入
///
/// vector `a` (两个带符号的 64 位整数) 中的元素向右逻辑移位 vector `b` (两个带符号的 64 位整数) 中的元素的位数，以模数形式指定 bits.The 中最高有效舍弃位的元素大小被加到移位值上 (用于四舍五入)，并将结果写入 vector (两个带符号的 64 位整数)。
///
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srlr.d))]
pub unsafe fn __msa_srlr_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_srlr_d(a, mem::transmute(b))
}

/// Immediate 右移逻辑舍入
///
/// `a` 将 vector `a` (十六个带符号的 8 位整数) 中的元素右移逻辑 `imm6` bits.The 将最高有效舍弃位添加到移位值中 (用于舍入)，并将结果写入 vector (十六个带符号的 8 位整数中) )。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srlri.b, imm3 = 0b111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srlri_b<const IMM3: i32>(a: v16i8) -> v16i8 {
    static_assert_imm3!(IMM3);
    msa_srlri_b(a, IMM3)
}

/// Immediate 右移逻辑舍入
///
/// `imm6` bits.The 将 vector `a` (八个带符号的 16 位整数) 中的元素右移逻辑，将最高有效舍弃位加到移位值 (用于舍入)，并将结果写入 vector (八个带符号的 16 位整数。)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srlri.h, imm4 = 0b1111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srlri_h<const IMM4: i32>(a: v8i16) -> v8i16 {
    static_assert_imm4!(IMM4);
    msa_srlri_h(a, IMM4)
}

/// Immediate 右移逻辑舍入
///
/// vector `a` 中的元素 (四个带符号的 32 位整数) 通过 `imm6` bits.The 进行逻辑右移。最高有效舍弃位被添加到移位值中 (用于舍入)，结果被写入 vector (四个带符号的 32 位整数)。)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srlri.w, imm5 = 0b11111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srlri_w<const IMM5: i32>(a: v4i32) -> v4i32 {
    static_assert_imm5!(IMM5);
    msa_srlri_w(a, IMM5)
}

/// Immediate 右移逻辑舍入
///
/// vector `a` (两个带符号的 64 位整数) 中的元素被逻辑右移 `imm6` bits.The 最高有效舍弃位被加到移位值 (用于舍入)，结果被写入 vector (两个带符号的 64 位整数。)。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(srlri.d, imm6 = 0b111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_srlri_d<const IMM6: i32>(a: v2i64) -> v2i64 {
    static_assert_imm6!(IMM6);
    msa_srlri_d(a, IMM6)
}

/// Vector 商店
///
/// vector `a` 中的 WRLEN/8 字节 (十六个带符号的 8 位整数) 作为数据格式 df 的元素存储在基本 `mem_addr` 和 10 位带符号的 immediate 偏移量 `imm_s10` 寻址的有效存储位置。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(st.b, imm_s10 = 0b1111111111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_st_b<const IMM_S10: i32>(a: v16i8, mem_addr: *mut u8) -> () {
    static_assert_imm_s10!(IMM_S10);
    msa_st_b(a, mem_addr, IMM_S10)
}

/// Vector 商店
///
/// vector `a` 中的 WRLEN/8 字节 (八个带符号的 16 位整数) 作为数据格式 df 的元素存储在基本 `mem_addr` 和 11 位带符号 immediate 偏移量 `imm_s11` 寻址的有效存储位置。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(st.h, imm_s11 = 0b11111111111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_st_h<const IMM_S11: i32>(a: v8i16, mem_addr: *mut u8) -> () {
    static_assert_imm_s11!(IMM_S11);
    static_assert!(IMM_S11: i32 where IMM_S11 % 2 == 0);
    msa_st_h(a, mem_addr, IMM_S11)
}

/// Vector 商店
///
/// vector `a` 中的 WRLEN/8 字节 (四个带符号的 32 位整数) 作为数据格式 df 的元素存储在基本 `mem_addr` 和 12 位带符号 immediate 偏移量 `imm_s12` 寻址的有效存储位置。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(st.w, imm_s12 = 0b111111111111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_st_w<const IMM_S12: i32>(a: v4i32, mem_addr: *mut u8) -> () {
    static_assert_imm_s12!(IMM_S12);
    static_assert!(IMM_S12: i32 where IMM_S12 % 4 == 0);
    msa_st_w(a, mem_addr, IMM_S12)
}

/// Vector 商店
///
/// vector `a` 中的 WRLEN/8 字节 (两个带符号的 64 位整数) 作为数据格式 df 的元素存储在基本 `mem_addr` 和 13 位带符号 immediate 偏移量 `imm_s13` 寻址的有效存储位置。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(st.d, imm_s13 = 0b1111111111111))]
#[rustc_legacy_const_generics(2)]
pub unsafe fn __msa_st_d<const IMM_S13: i32>(a: v2i64, mem_addr: *mut u8) -> () {
    static_assert_imm_s13!(IMM_S13);
    static_assert!(IMM_S13: i32 where IMM_S13 % 8 == 0);
    msa_st_d(a, mem_addr, IMM_S13)
}

/// Vector 有符号值的饱和减法
///
/// 从 vector `a` (十六个带符号的 8 位整数) 中减去 vector `b` 中的元素 (十六个带符号的 8 位整数)。
/// 在将结果写入 vector (十六个带符号的 8 位整数) 之前，将执行带符号的算术运算并将溢出钳位到最大或者最小的可表示符号值。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subs_s.b))]
pub unsafe fn __msa_subs_s_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_subs_s_b(a, mem::transmute(b))
}

/// Vector 有符号值的饱和减法
///
/// 从 vector `a` (八个带符号的 16 位整数) 中减去 vector `b` (八个带符号的 16 位整数) 中的元素。
/// 在将结果写入 vector (八个有符号的 16 位整数) 之前，将执行有符号算术，并且溢出将钳位到最大或者最小可表示的有符号值。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subs_s.h))]
pub unsafe fn __msa_subs_s_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_subs_s_h(a, mem::transmute(b))
}

/// Vector 有符号值的饱和减法
///
/// 从 vector `a` (四个带符号的 32 位整数) 中减去 vector `b` (四个带符号的 32 位整数) 中的元素。
/// 在将结果写入 vector (四个有符号的 32 位整数) 之前，将执行有符号算术，并且溢出将钳位到最大或者最小可表示的有符号值。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subs_s.w))]
pub unsafe fn __msa_subs_s_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_subs_s_w(a, mem::transmute(b))
}

/// Vector 有符号值的饱和减法
///
/// 从 vector `a` (两个带符号的 64 位整数) 中的元素减去 vector `b` (两个带符号的 64 位整数) 中的元素。
/// 在将结果写入 vector (两个有符号的 64 位整数) 之前，将执行有符号算术，并且溢出将钳位到最大或者最小可表示的有符号值。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subs_s.d))]
pub unsafe fn __msa_subs_s_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_subs_s_d(a, mem::transmute(b))
}

/// Vector 无符号值的无符号饱和减
///
/// 从 vector `a` 中的元素 (十六个无符号的 8 位整数) 中减去 vector `b` 中的元素 (十六个无符号 8 位整数)。
///
/// 在将结果写入 vector (十六个无符号的 8 位整数) 之前，将执行无符号算术并将下溢钳位为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subs_u.b))]
pub unsafe fn __msa_subs_u_b(a: v16u8, b: v16u8) -> v16u8 {
    msa_subs_u_b(a, mem::transmute(b))
}

/// Vector 无符号值的无符号饱和减
///
/// 从 vector `a` (八个无符号 16 位整数) 中减去 vector `b` 中的元素 (八个无符号 16 位整数)。
///
/// 在将结果写入 vector (八个无符号的 16 位整数) 之前，将执行无符号算术并将下溢钳位为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subs_u.h))]
pub unsafe fn __msa_subs_u_h(a: v8u16, b: v8u16) -> v8u16 {
    msa_subs_u_h(a, mem::transmute(b))
}

/// Vector 无符号值的无符号饱和减
///
/// 从 vector `a` (四个无符号 32 位整数) 中减去 vector `b` 中的元素 (四个无符号 32 位整数)。
///
/// 在将结果写入 vector (四个无符号的 32 位整数) 之前，将执行无符号算术并将下溢钳位为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subs_u.w))]
pub unsafe fn __msa_subs_u_w(a: v4u32, b: v4u32) -> v4u32 {
    msa_subs_u_w(a, mem::transmute(b))
}

/// Vector 无符号值的无符号饱和减
///
/// 从 vector `a` (两个无符号 64 位整数) 中减去 vector `b` (两个无符号 64 位整数) 中的元素。
///
/// 在将结果写入 vector (两个无符号的 64 位整数) 之前，将执行无符号算术并将下溢钳位为 0。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subs_u.d))]
pub unsafe fn __msa_subs_u_d(a: v2u64, b: v2u64) -> v2u64 {
    msa_subs_u_d(a, mem::transmute(b))
}

/// Vector 无符号的无符号饱和减法
///
/// vector `b` (十六个带符号的 8 位整数) 中的有符号元素被从 vector `a` (十六个无符号的 8 位整数中) 的无符号元素中减去。
///
/// 带符号的结果被无符号饱和，并写入 vector (十六个无符号的 8 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subsus_u.b))]
pub unsafe fn __msa_subsus_u_b(a: v16u8, b: v16i8) -> v16u8 {
    msa_subsus_u_b(a, mem::transmute(b))
}

/// Vector 无符号的无符号饱和减法
///
/// vector `b` (八个带符号的 16 位整数) 中的有符号元素被从 vector `a` (八个无符号的 16 位整数) 中的无符号元素中减去。
///
/// 带符号的结果被无符号饱和，并写入 vector (八个无符号的 16 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subsus_u.h))]
pub unsafe fn __msa_subsus_u_h(a: v8u16, b: v8i16) -> v8u16 {
    msa_subsus_u_h(a, mem::transmute(b))
}

/// Vector 无符号的无符号饱和减法
///
/// 从 vector `a` 中的无符号元素 (四个无符号 32 位整数) 中减去 vector `b` 中的有符号元素 (四个有符号 6432it 整数)。
///
/// 带符号的结果被无符号饱和，并写入 vector (四个无符号的 32 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subsus_u.w))]
pub unsafe fn __msa_subsus_u_w(a: v4u32, b: v4i32) -> v4u32 {
    msa_subsus_u_w(a, mem::transmute(b))
}

/// Vector 无符号的无符号饱和减法
///
/// 从 vector `a` (两个无符号 64 位整数) 中的未有符号元素中减去 vector `b` (两个有符号 64 位整数) 中的有符号元素。
///
/// 带符号的结果被无符号饱和，并写入 vector (两个无符号的 64 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subsus_u.d))]
pub unsafe fn __msa_subsus_u_d(a: v2u64, b: v2i64) -> v2u64 {
    msa_subsus_u_d(a, mem::transmute(b))
}

/// Vector 有符号饱和值的有符号减法
///
/// 从 vector `a` (十六个无符号 8 位整数) 中减去 vector `b` 中的无符号元素 (十六个无符号 8 位整数)。
///
/// 带符号的结果被饱和并写入 vector (十六个无符号的 8 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subsuu_s.b))]
pub unsafe fn __msa_subsuu_s_b(a: v16u8, b: v16u8) -> v16i8 {
    msa_subsuu_s_b(a, mem::transmute(b))
}

/// Vector 有符号饱和值的有符号减法
///
/// vector `b` 中的无符号元素 (八个无符号 16 位整数) 从 vector `a` 中的无符号元素 (八个无符号 16 位整数) 中减去。
///
/// 将带符号的结果进行饱和并写入 vector (八个无符号的 16 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subsuu_s.h))]
pub unsafe fn __msa_subsuu_s_h(a: v8u16, b: v8u16) -> v8i16 {
    msa_subsuu_s_h(a, mem::transmute(b))
}

/// Vector 有符号饱和值的有符号减法
///
/// vector `b` 中的无符号元素 (四个无符号 32 位整数) 从 vector `a` 中的无符号元素 (四个无符号 32 位整数) 中减去。
///
/// 带符号的结果被饱和并写入 vector (四个无符号的 32 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subsuu_s.w))]
pub unsafe fn __msa_subsuu_s_w(a: v4u32, b: v4u32) -> v4i32 {
    msa_subsuu_s_w(a, mem::transmute(b))
}

/// Vector 有符号饱和值的有符号减法
///
/// vector `b` 中的无符号元素 (两个无符号 64 位整数) 从 vector `a` 中的无符号元素 (两个无符号 64 位整数) 中减去。
///
/// 带符号的结果被饱和并写入 vector (两个无符号的 64 位整数)。
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subsuu_s.d))]
pub unsafe fn __msa_subsuu_s_d(a: v2u64, b: v2u64) -> v2i64 {
    msa_subsuu_s_d(a, mem::transmute(b))
}

/// Vector 减法
///
/// 从 vector `a` (十六个带符号的 8 位整数) 中减去 vector `b` 中的元素 (十六个带符号的 8 位整数)。
///
/// 结果写入 vector (十六个带符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subv.b))]
pub unsafe fn __msa_subv_b(a: v16i8, b: v16i8) -> v16i8 {
    msa_subv_b(a, mem::transmute(b))
}

/// Vector 减法
///
/// 从 vector `a` (八个带符号的 16 位整数) 中减去 vector `b` (八个带符号的 16 位整数) 中的元素。
///
/// 结果写入 vector (八个带符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subv.h))]
pub unsafe fn __msa_subv_h(a: v8i16, b: v8i16) -> v8i16 {
    msa_subv_h(a, mem::transmute(b))
}

/// Vector 减法
///
/// 从 vector `a` (四个带符号的 32 位整数) 中减去 vector `b` (四个带符号的 32 位整数) 中的元素。
///
/// 结果写入 vector (四个带符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subv.w))]
pub unsafe fn __msa_subv_w(a: v4i32, b: v4i32) -> v4i32 {
    msa_subv_w(a, mem::transmute(b))
}

/// Vector 减法
///
/// 从 vector `a` (两个带符号的 64 位整数) 中的元素减去 vector `b` (两个带符号的 64 位整数) 中的元素。
///
/// 结果写入 vector (两个带符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subv.d))]
pub unsafe fn __msa_subv_d(a: v2i64, b: v2i64) -> v2i64 {
    msa_subv_d(a, mem::transmute(b))
}

/// Immediate 减法
///
/// 从 vector `a` 中的元素中减去 5 位 immediate 无符号值 `imm5` (16 个有符号 8 位整数)。
///
/// 结果写入 vector (十六个带符号的 8 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subvi.b, imm5 = 0b10111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_subvi_b<const IMM5: i32>(a: v16i8) -> v16i8 {
    static_assert_imm5!(IMM5);
    msa_subvi_b(a, IMM5)
}

/// Immediate 减法
///
/// 从 vector `a` 中的元素中减去 5 位 immediate 无符号值 `imm5` (8 个有符号的 16 位整数)。
///
/// 结果写入 vector (八个带符号的 16 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subvi.h, imm5 = 0b10111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_subvi_h<const IMM5: i32>(a: v8i16) -> v8i16 {
    static_assert_imm5!(IMM5);
    msa_subvi_h(a, IMM5)
}

/// Immediate 减法
///
/// 从 vector `a` 中的元素中减去 5 位 immediate 无符号值 `imm5` (四个有符号的 32 位整数)。
///
/// 结果写入 vector (四个带符号的 32 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subvi.w, imm5 = 0b10111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_subvi_w<const IMM5: i32>(a: v4i32) -> v4i32 {
    static_assert_imm5!(IMM5);
    msa_subvi_w(a, IMM5)
}

/// Immediate 减法
///
/// 从 vector `a` 中的元素中减去 5 位 immediate 无符号值 `imm5` (两个有符号的 64 位整数)。
///
/// 结果写入 vector (两个带符号的 64 位整数)。
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(subvi.d, imm5 = 0b10111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_subvi_d<const IMM5: i32>(a: v2i64) -> v2i64 {
    static_assert_imm5!(IMM5);
    msa_subvi_d(a, IMM5)
}

/// Vector 数据保持随机
///
/// vector 重排指令基于 vectors `b` (十六个带符号的 8 位整数) 和 `c` (十六个带符号的 8 位整数) 的组合，将数据元素选择性地复制到 vector `a` (十六个带符号的 8 位整数) 中。`a` 中相应的控制元素。
///
/// `a` 控制元素中的最低有效 6 位以级联 vectors `b`，`a` 中元素的数量为模，指定源元素的索引。
/// 如果位 6 或位 7 为 1，将没有副本，而是将目标元素设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(vshf.b))]
pub unsafe fn __msa_vshf_b(a: v16i8, b: v16i8, c: v16i8) -> v16i8 {
    msa_vshf_b(a, mem::transmute(b), c)
}

/// Vector 数据保持随机
///
/// vector 重排指令根据 vectors `b` (八个带符号的 16 位整数) 和 `c` (八个带符号的 16 位整数) 的连接，有选择地将数据元素复制到 vector `a` (八个带符号的 16 位整数)。`a` 中相应的控制元素。
///
/// `a` 控制元素中的最低有效 6 位以级联 vectors `b`，`a` 中元素的数量为模，指定源元素的索引。
/// 如果位 6 或位 7 为 1，将没有副本，而是将目标元素设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(vshf.h))]
pub unsafe fn __msa_vshf_h(a: v8i16, b: v8i16, c: v8i16) -> v8i16 {
    msa_vshf_h(a, mem::transmute(b), c)
}

/// Vector 数据保持随机
///
/// vector 重排指令基于 vectors `b` (四个带符号的 32 位整数) 和 `c` (四个带符号的 32 位整数) 的连接，有选择地将数据元素复制到 vector `a` (四个带符号的 32 位整数) 中。`a` 中相应的控制元素。
///
/// `a` 控制元素中的最低有效 6 位以级联 vectors `b`，`a` 中元素的数量为模，指定源元素的索引。
/// 如果位 6 或位 7 为 1，将没有副本，而是将目标元素设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(vshf.w))]
pub unsafe fn __msa_vshf_w(a: v4i32, b: v4i32, c: v4i32) -> v4i32 {
    msa_vshf_w(a, mem::transmute(b), c)
}

/// Vector 数据保持随机
///
/// vector 重排指令根据 vectors `b` (两个带符号的 64 位整数) 和 `c` (两个带符号的 64 位整数) 的连接，有选择地将数据元素复制到 vector `a` (两个带符号的 64 位整数)。`a` 中相应的控制元素。
///
/// `a` 控制元素中的最低有效 6 位以级联 vectors `b`，`a` 中元素的数量为模，指定源元素的索引。
/// 如果位 6 或位 7 为 1，将没有副本，而是将目标元素设置为 0。
///
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(vshf.d))]
pub unsafe fn __msa_vshf_d(a: v2i64, b: v2i64, c: v2i64) -> v2i64 {
    msa_vshf_d(a, mem::transmute(b), c)
}

/// Vector 逻辑异或
///
/// vector `a` 的每个位 (十六个无符号的 8 位整数) 与 vector `b` 的对应位 (十六个无符号的 8 位整数) 在按位逻辑 XOR 运算中组合在一起。
/// 结果写入 vector (十六个无符号的 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(xor.v))]
pub unsafe fn __msa_xor_v(a: v16u8, b: v16u8) -> v16u8 {
    msa_xor_v(a, mem::transmute(b))
}

/// Immediate 逻辑异或
///
/// vector `a` 的每个字节 (十六个无符号的 8 位整数) 与 8 位 immediate  `imm8` 进行按位逻辑 XOR 运算组合。
/// 结果写入 vector (十六个无符号的 8 位整数)。
///
///
///
#[inline]
#[target_feature(enable = "msa")]
#[cfg_attr(test, assert_instr(xori.b, imm8 = 0b11111111))]
#[rustc_legacy_const_generics(1)]
pub unsafe fn __msa_xori_b<const IMM8: i32>(a: v16u8) -> v16u8 {
    static_assert_imm8!(IMM8);
    msa_xori_b(a, IMM8)
}

#[cfg(test)]
mod tests {
    use crate::{
        core_arch::{mips::msa::*, simd::*},
        mem,
    };
    use std::{f32, f64};
    use stdarch_test::simd_test;

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_add_a_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -4, -3, -2, -1,
            -4, -3, -2, -1,
            -4, -3, -2, -1,
            -4, -3, -2, -1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            5, 5, 5, 5,
            5, 5, 5, 5, 
            5, 5, 5, 5, 
            5, 5, 5, 5
        );

        assert_eq!(
            r,
            mem::transmute(__msa_add_a_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_add_a_h() {
        #[rustfmt::skip]
        let a = i16x8::new(1, 2, 3, 4, 1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i16x8::new(-4, -3, -2, -1, -4, -3, -2, -1);
        #[rustfmt::skip]
        let r = i16x8::new(5, 5, 5, 5, 5, 5, 5, 5);

        assert_eq!(
            r,
            mem::transmute(__msa_add_a_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_add_a_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i32x4::new(-4, -3, -2, -1);
        #[rustfmt::skip]
        let r = i32x4::new(5, 5, 5, 5);

        assert_eq!(
            r,
            mem::transmute(__msa_add_a_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_add_a_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(-4, -3);
        #[rustfmt::skip]
        let r = i64x2::new(5, 5);

        assert_eq!(
            r,
            mem::transmute(__msa_add_a_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_adds_a_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            100, i8::MAX, 100, i8::MAX,
            100, i8::MAX, 100, i8::MAX,
            100, i8::MAX, 100, i8::MAX,
            100, i8::MAX, 100, i8::MAX
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -4, -3, -2, -100,
            -4, -3, -2, -100,
            -4, -3, -2, -100,
            -4, -3, -2, -100
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            104, 127, 102, 127, 
            104, 127, 102, 127, 
            104, 127, 102, 127, 
            104, 127, 102, 127
        );

        assert_eq!(
            r,
            mem::transmute(__msa_adds_a_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_adds_a_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            100, i16::MAX, 100, i16::MAX, 
            100, i16::MAX, 100, i16::MAX
        );
        #[rustfmt::skip]
        let b = i16x8::new(-4, -3, -2, -1, -4, -3, -2, -1);
        #[rustfmt::skip]
        let r = i16x8::new(
            104, i16::MAX, 102, i16::MAX,
            104, i16::MAX, 102, i16::MAX
        );

        assert_eq!(
            r,
            mem::transmute(__msa_adds_a_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_adds_a_w() {
        #[rustfmt::skip]
        let a = i32x4::new(100, i32::MAX, 100, i32::MAX);
        #[rustfmt::skip]
        let b = i32x4::new(-4, -3, -2, -1);
        #[rustfmt::skip]
        let r = i32x4::new(104, i32::MAX, 102, i32::MAX);

        assert_eq!(
            r,
            mem::transmute(__msa_adds_a_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_adds_a_d() {
        #[rustfmt::skip]
        let a = i64x2::new(100, i64::MAX);
        #[rustfmt::skip]
        let b = i64x2::new(-4, -3);
        #[rustfmt::skip]
        let r = i64x2::new(104, i64::MAX);

        assert_eq!(
            r,
            mem::transmute(__msa_adds_a_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_adds_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            100, i8::MIN, 100, i8::MAX,
            100, i8::MIN, 100, i8::MAX,
            100, i8::MIN, 100, i8::MAX,
            100, i8::MIN, 100, i8::MAX
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -4, -3, -2, 100,
            -4, -3, -2, 100,
            -4, -3, -2, 100,
            -4, -3, -2, 100
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            96, i8::MIN, 98, i8::MAX, 
            96, i8::MIN, 98, i8::MAX, 
            96, i8::MIN, 98, i8::MAX, 
            96, i8::MIN, 98, i8::MAX
        );

        assert_eq!(
            r,
            mem::transmute(__msa_adds_s_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_adds_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            100, i16::MIN, 100, i16::MAX, 
            100, i16::MIN, 100, i16::MAX
        );
        #[rustfmt::skip]
        let b = i16x8::new(-4, -3, -2, 1, -4, -3, -2, 1);
        #[rustfmt::skip]
        let r = i16x8::new(
            96, i16::MIN, 98, i16::MAX, 
            96, i16::MIN, 98, i16::MAX
        );

        assert_eq!(
            r,
            mem::transmute(__msa_adds_s_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_adds_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(100, i32::MAX, 100, i32::MIN);
        #[rustfmt::skip]
        let b = i32x4::new(-4, 3, -2, -1);
        #[rustfmt::skip]
        let r = i32x4::new(96, i32::MAX, 98, i32::MIN);

        assert_eq!(
            r,
            mem::transmute(__msa_adds_s_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_adds_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(100, i64::MIN);
        #[rustfmt::skip]
        let b = i64x2::new(-4, -3);
        #[rustfmt::skip]
        let r = i64x2::new(96, i64::MIN);

        assert_eq!(
            r,
            mem::transmute(__msa_adds_s_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_adds_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            100, u8::MAX, 100, u8::MAX,
            100, u8::MAX, 100, u8::MAX,
            100, u8::MAX, 100, u8::MAX,
            100, u8::MAX, 100, u8::MAX
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            4, 3, 2, 100,
            4, 3, 2, 100,
            4, 3, 2, 100,
            4, 3, 2, 100
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            104, u8::MAX, 102, u8::MAX, 
            104, u8::MAX, 102, u8::MAX, 
            104, u8::MAX, 102, u8::MAX, 
            104, u8::MAX, 102, u8::MAX
        );

        assert_eq!(
            r,
            mem::transmute(__msa_adds_u_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_adds_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(
            100, u16::MAX, 100, u16::MAX, 
            100, u16::MAX, 100, u16::MAX
        );
        #[rustfmt::skip]
        let b = u16x8::new(4, 3, 2, 1, 4, 3, 2, 1);
        #[rustfmt::skip]
        let r = u16x8::new(
            104, u16::MAX, 102, u16::MAX, 
            104, u16::MAX, 102, u16::MAX
        );

        assert_eq!(
            r,
            mem::transmute(__msa_adds_u_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_adds_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(100, u32::MAX, 100, u32::MAX);
        #[rustfmt::skip]
        let b = u32x4::new(4, 3, 2, 1);
        #[rustfmt::skip]
        let r = u32x4::new(104, u32::MAX, 102, u32::MAX);

        assert_eq!(
            r,
            mem::transmute(__msa_adds_u_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_adds_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(100, u64::MAX);
        #[rustfmt::skip]
        let b = u64x2::new(4, 3);
        #[rustfmt::skip]
        let r = u64x2::new(104, u64::MAX);

        assert_eq!(
            r,
            mem::transmute(__msa_adds_u_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_addv_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            100, i8::MIN, 100, i8::MAX,
            100, i8::MIN, 100, i8::MAX,
            100, i8::MIN, 100, i8::MAX,
            100, i8::MIN, 100, i8::MAX
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -4, -3, -2, 100,
            -4, -3, -2, 100,
            -4, -3, -2, 100,
            -4, -3, -2, 100
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            96, 125, 98, -29, 
            96, 125, 98, -29, 
            96, 125, 98, -29, 
            96, 125, 98, -29
        );

        assert_eq!(
            r,
            mem::transmute(__msa_addv_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_addv_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            100, i16::MIN, 100, i16::MAX, 
            100, i16::MIN, 100, i16::MAX
        );
        #[rustfmt::skip]
        let b = i16x8::new(-4, -3, -2, 1, -4, -3, -2, 1);
        #[rustfmt::skip]
        let r = i16x8::new(96, 32765, 98, -32768, 96, 32765, 98, -32768);

        assert_eq!(
            r,
            mem::transmute(__msa_addv_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_addv_w() {
        #[rustfmt::skip]
        let a = i32x4::new(100, i32::MAX, 100, i32::MIN);
        #[rustfmt::skip]
        let b = i32x4::new(-4, 3, -2, -1);
        #[rustfmt::skip]
        let r = i32x4::new(96, -2147483646, 98, 2147483647);

        assert_eq!(
            r,
            mem::transmute(__msa_addv_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_addv_d() {
        #[rustfmt::skip]
        let a = i64x2::new(100, i64::MIN);
        #[rustfmt::skip]
        let b = i64x2::new(-4, -3);
        #[rustfmt::skip]
        let r = i64x2::new(96, 9223372036854775805);

        assert_eq!(
            r,
            mem::transmute(__msa_addv_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_addvi_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            100, i8::MAX, 100, i8::MAX,
            100, i8::MAX, 100, i8::MAX,
            100, i8::MAX, 100, i8::MAX,
            100, i8::MAX, 100, i8::MAX
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            103, -126, 103, -126, 
            103, -126, 103, -126, 
            103, -126, 103, -126, 
            103, -126, 103, -126
        );

        assert_eq!(r, mem::transmute(__msa_addvi_b(mem::transmute(a), 67)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_addvi_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            i16::MAX, 3276, -100, -127, 
            i16::MAX, 3276, -100, -127
        );
        #[rustfmt::skip]
        let r = i16x8::new(
            -32766, 3279, -97, -124, 
            -32766, 3279, -97, -124
        );

        assert_eq!(r, mem::transmute(__msa_addvi_h(mem::transmute(a), 67)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_addvi_w() {
        #[rustfmt::skip]
        let a = i32x4::new(100, i32::MAX, 100, i32::MIN);
        #[rustfmt::skip]
        let r = i32x4::new(103, -2147483646, 103, -2147483645);

        assert_eq!(r, mem::transmute(__msa_addvi_w(mem::transmute(a), 67)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_addvi_d() {
        #[rustfmt::skip]
        let a = i64x2::new(100, i64::MIN);
        #[rustfmt::skip]
        let r = i64x2::new(117, -9223372036854775791);

        assert_eq!(r, mem::transmute(__msa_addvi_d(mem::transmute(a), 17)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_and_v() {
        #[rustfmt::skip]
        let a = u8x16::new(
            100, u8::MAX, 100, u8::MAX,
            100, u8::MAX, 100, u8::MAX,
            100, u8::MAX, 100, u8::MAX,
            100, u8::MAX, 100, u8::MAX
    );
        #[rustfmt::skip]
        let b = u8x16::new(
            4, 3, 2, 100,
            4, 3, 2, 100,
            4, 3, 2, 100,
            4, 3, 2, 100
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            4, 3, 0, 100, 
            4, 3, 0, 100, 
            4, 3, 0, 100, 
            4, 3, 0, 100
        );

        assert_eq!(
            r,
            mem::transmute(__msa_and_v(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_andi_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            100, u8::MAX, 100, u8::MAX,
            100, u8::MAX, 100, u8::MAX,
            100, u8::MAX, 100, u8::MAX,
            100, u8::MAX, 100, u8::MAX
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            4, 5, 4, 5, 
            4, 5, 4, 5, 
            4, 5, 4, 5, 
            4, 5, 4, 5
        );

        assert_eq!(r, mem::transmute(__msa_andi_b(mem::transmute(a), 5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_asub_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -1, -2, -3, -4,
            -1, -2, -3, -4,
            -1, -2, -3, -4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -6, -7, -8, -9,
            -6, -7, -8, -9,
            -6, -7, -8, -9,
            -6, -7, -8, -9
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            5, 5, 5, 5, 
            5, 5, 5, 5, 
            5, 5, 5, 5, 
            5, 5, 5, 5
        );

        assert_eq!(
            r,
            mem::transmute(__msa_asub_s_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_asub_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(-1, -2, -3, -4, -1, -2, -3, -4);
        #[rustfmt::skip]
        let b = i16x8::new(-6, -7, -8, -9, -6, -7, -8, -9);
        #[rustfmt::skip]
        let r = i16x8::new(5, 5, 5, 5, 5, 5, 5, 5);

        assert_eq!(
            r,
            mem::transmute(__msa_asub_s_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_asub_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(-1, -2, -3, -4);
        #[rustfmt::skip]
        let b = i32x4::new(-6, -7, -8, -9);
        #[rustfmt::skip]
        let r = i32x4::new(5, 5, 5, 5);

        assert_eq!(
            r,
            mem::transmute(__msa_asub_s_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_asub_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(-1, -2);
        #[rustfmt::skip]
        let b = i64x2::new(-6, -7);
        #[rustfmt::skip]
        let r = i64x2::new(5, 5);

        assert_eq!(
            r,
            mem::transmute(__msa_asub_s_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_asub_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            5, 5, 5, 5, 
            5, 5, 5, 5, 
            5, 5, 5, 5, 
            5, 5, 5, 5
        );

        assert_eq!(
            r,
            mem::transmute(__msa_asub_u_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_asub_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(1, 2, 3, 4, 1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u16x8::new(6, 7, 8, 9, 6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u16x8::new(5, 5, 5, 5, 5, 5, 5, 5);

        assert_eq!(
            r,
            mem::transmute(__msa_asub_u_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_asub_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u32x4::new(5, 5, 5, 5);

        assert_eq!(
            r,
            mem::transmute(__msa_asub_u_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_asub_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(1, 2);
        #[rustfmt::skip]
        let b = u64x2::new(6, 7);
        #[rustfmt::skip]
        let r = u64x2::new(5, 5);

        assert_eq!(
            r,
            mem::transmute(__msa_asub_u_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ave_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -1, -2, -3, -4,
            -1, -2, -3, -4,
            -1, -2, -3, -4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            6, -7, 8, -9,
            6, -7, 8, -9,
            6, -7, 8, -9,
            6, -7, 8, -9
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            2, -5, 2, -7, 
            2, -5, 2, -7, 
            2, -5, 2, -7, 
            2, -5, 2, -7
        );

        assert_eq!(
            r,
            mem::transmute(__msa_ave_s_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ave_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(-1, -2, -3, -4, -1, -2, -3, -4);
        #[rustfmt::skip]
        let b = i16x8::new(6, -7, 8, -9, 6, -7, 8, -9);
        #[rustfmt::skip]
        let r = i16x8::new(2, -5, 2, -7, 2, -5, 2, -7);

        assert_eq!(
            r,
            mem::transmute(__msa_ave_s_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ave_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(-1, -2, -3, -4);
        #[rustfmt::skip]
        let b = i32x4::new(6, -7, 8, -9);
        #[rustfmt::skip]
        let r = i32x4::new(2, -5, 2, -7);

        assert_eq!(
            r,
            mem::transmute(__msa_ave_s_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ave_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(-1, -2);
        #[rustfmt::skip]
        let b = i64x2::new(-6, -7);
        #[rustfmt::skip]
        let r = i64x2::new(-4, -5);

        assert_eq!(
            r,
            mem::transmute(__msa_ave_s_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ave_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            3, 4, 5, 6, 
            3, 4, 5, 6, 
            3, 4, 5, 6, 
            3, 4, 5, 6
        );

        assert_eq!(
            r,
            mem::transmute(__msa_ave_u_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ave_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(1, 2, 3, 4, 1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u16x8::new(6, 7, 8, 9, 6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u16x8::new(3, 4, 5, 6, 3, 4, 5, 6);

        assert_eq!(
            r,
            mem::transmute(__msa_ave_u_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ave_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u32x4::new(3, 4, 5, 6);

        assert_eq!(
            r,
            mem::transmute(__msa_ave_u_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ave_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(1, 2);
        #[rustfmt::skip]
        let b = u64x2::new(6, 7);
        #[rustfmt::skip]
        let r = u64x2::new(3, 4);

        assert_eq!(
            r,
            mem::transmute(__msa_ave_u_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_aver_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -1, -2, 3, -4,
            -1, -2, 3, -4,
            -1, -2, 3, -4,
            -1, -2, 3, -4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -6, 7, -8, -9,
            -6, 7, -8, -9,
            -6, 7, -8, -9,
            -6, 7, -8, -9
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -3, 3, -2, -6, 
            -3, 3, -2, -6, 
            -3, 3, -2, -6, 
            -3, 3, -2, -6
        );

        assert_eq!(
            r,
            mem::transmute(__msa_aver_s_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_aver_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(-1, -2, 3, -4, -1, -2, 3, -4);
        #[rustfmt::skip]
        let b = i16x8::new(-6, 7, -8, -9, -6, 7, -8, -9);
        #[rustfmt::skip]
        let r = i16x8::new(-3, 3, -2, -6, -3, 3, -2, -6);

        assert_eq!(
            r,
            mem::transmute(__msa_aver_s_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_aver_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(-1, -2, 3, -4);
        #[rustfmt::skip]
        let b = i32x4::new(-6, 7, -8, -9);
        #[rustfmt::skip]
        let r = i32x4::new(-3, 3, -2, -6);

        assert_eq!(
            r,
            mem::transmute(__msa_aver_s_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_aver_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(-1, -2);
        #[rustfmt::skip]
        let b = i64x2::new(-6, -7);
        #[rustfmt::skip]
        let r = i64x2::new(-3, -4);

        assert_eq!(
            r,
            mem::transmute(__msa_aver_s_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_aver_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            4, 5, 6, 7, 
            4, 5, 6, 7, 
            4, 5, 6, 7, 
            4, 5, 6, 7
        );

        assert_eq!(
            r,
            mem::transmute(__msa_aver_u_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_aver_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(1, 2, 3, 4, 1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u16x8::new(6, 7, 8, 9, 6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u16x8::new(4, 5, 6, 7, 4, 5, 6, 7);

        assert_eq!(
            r,
            mem::transmute(__msa_aver_u_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_aver_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u32x4::new(4, 5, 6, 7);

        assert_eq!(
            r,
            mem::transmute(__msa_aver_u_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_aver_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(1, 2);
        #[rustfmt::skip]
        let b = u64x2::new(6, 7);
        #[rustfmt::skip]
        let r = u64x2::new(4, 5);

        assert_eq!(
            r,
            mem::transmute(__msa_aver_u_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bclr_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            191, 27, 54, 1, 
            191, 27, 54, 1, 
            191, 27, 54, 1, 
            191, 27, 54, 1
        );

        assert_eq!(
            r,
            mem::transmute(__msa_bclr_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bclr_h() {
        #[rustfmt::skip]
        let a = u16x8::new(255, 155, 55, 1, 255, 155, 55, 1);
        #[rustfmt::skip]
        let b = u16x8::new(6, 7, 8, 9, 6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u16x8::new(191, 27, 55, 1, 191, 27, 55, 1);

        assert_eq!(
            r,
            mem::transmute(__msa_bclr_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bclr_w() {
        #[rustfmt::skip]
        let a = u32x4::new(255, 155, 55, 1);
        #[rustfmt::skip]
        let b = u32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u32x4::new(191, 27, 55, 1);

        assert_eq!(
            r,
            mem::transmute(__msa_bclr_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bclr_d() {
        #[rustfmt::skip]
        let a = u64x2::new(255, 155);
        #[rustfmt::skip]
        let b = u64x2::new(6, 7);
        #[rustfmt::skip]
        let r = u64x2::new(191, 27);

        assert_eq!(
            r,
            mem::transmute(__msa_bclr_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bclri_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            247, 147, 55, 1, 
            247, 147, 55, 1, 
            247, 147, 55, 1, 
            247, 147, 55, 1
        );

        assert_eq!(r, mem::transmute(__msa_bclri_b(mem::transmute(a), 3)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bclri_h() {
        #[rustfmt::skip]
        let a = u16x8::new(2155, 1155, 155, 1, 2155, 1155, 155, 1);
        #[rustfmt::skip]
        let r = u16x8::new(107, 1155, 155, 1, 107, 1155, 155, 1);

        assert_eq!(r, mem::transmute(__msa_bclri_h(mem::transmute(a), 11)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bclri_w() {
        #[rustfmt::skip]
        let a = u32x4::new(211111155, 111111155, 11111155, 1);
        #[rustfmt::skip]
        let r = u32x4::new(202722547, 102722547, 2722547, 1);

        assert_eq!(r, mem::transmute(__msa_bclri_w(mem::transmute(a), 23)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bclri_d() {
        #[rustfmt::skip]
        let a = u64x2::new(211111111155, 11111111111111155);
        #[rustfmt::skip]
        let r = u64x2::new(73672157683, 11110973672157683);

        assert_eq!(r, mem::transmute(__msa_bclri_d(mem::transmute(a), 37)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsl_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let c = u8x16::new(
            1, 3, 5, 9,
            1, 3, 5, 9,
            1, 3, 5, 9,
            1, 3, 5, 9
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            63, 11, 11, 1, 
            63, 11, 11, 1, 
            63, 11, 11, 1, 
            63, 11, 11, 1
        );

        assert_eq!(
            r,
            mem::transmute(__msa_binsl_b(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsl_h() {
        #[rustfmt::skip]
        let a = u16x8::new(
            32767, 16384, 8192, 4096, 
            32767, 16384, 8192, 4096
        );
        #[rustfmt::skip]
        let b = u16x8::new(
            21656, 5273, 7081, 2985, 
            21656, 5273, 7081, 2985
        );
        #[rustfmt::skip]
        let c = u16x8::new(
            3, 7, 9, 13, 
            15, 17, 21, 23
        );
        #[rustfmt::skip]
        let r = u16x8::new(
            24575, 5120, 7040, 2984, 
            21656, 0, 6144, 2816
        );

        assert_eq!(
            r,
            mem::transmute(__msa_binsl_h(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsl_w() {
        #[rustfmt::skip]
        let a = u32x4::new(2147483647, 536870912, 67108864, 8388608);
        #[rustfmt::skip]
        let b = u32x4::new(1036372536, 259093134, 78219975, 1119499719);
        #[rustfmt::skip]
        let c = u32x4::new(11, 15, 31, 37);
        #[rustfmt::skip]
        let r = u32x4::new(1037041663, 259063808, 78219975, 1082130432);

        assert_eq!(
            r,
            mem::transmute(__msa_binsl_w(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsl_d() {
        #[rustfmt::skip]
        let a = u64x2::new(8006399338, 2882303762);
        #[rustfmt::skip]
        let b = u64x2::new(9223372036854775805, 536870912);
        #[rustfmt::skip]
        let c = u64x2::new(12, 48);
        #[rustfmt::skip]
        let r = u64x2::new(9221120245047489898, 536901394);

        assert_eq!(
            r,
            mem::transmute(__msa_binsl_d(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsli_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            u8::MAX, 155, 55, 1,
            u8::MAX, 155, 55, 1,
            u8::MAX, 155, 55, 1,
            u8::MAX, 155, 55, 1
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            7, 7, 11, 9, 
            7, 7, 11, 9, 
            7, 7, 11, 9, 
            7, 7, 11, 9
        );

        assert_eq!(
            r,
            mem::transmute(__msa_binsli_b(mem::transmute(a), mem::transmute(b), 5))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsli_h() {
        #[rustfmt::skip]
        let a = u16x8::new(
            32767, 16384, 8192, 4096, 
            32767, 16384, 8192, 4096
        );
        #[rustfmt::skip]
        let b = u16x8::new(
            21656, 5273, 7081, 2985, 
            21656, 5273, 7081, 2985
        );
        #[rustfmt::skip]
        let r = u16x8::new(
            21659, 5272, 7080, 2984, 
            21659, 5272, 7080, 2984
        );

        assert_eq!(
            r,
            mem::transmute(__msa_binsli_h(mem::transmute(a), mem::transmute(b), 13))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsli_w() {
        #[rustfmt::skip]
        let a = u32x4::new(2147483647, 536870912, 67108864, 8388608);
        #[rustfmt::skip]
        let b = u32x4::new(1036372536, 259093134, 78219975, 1119499719);
        #[rustfmt::skip]
        let r = u32x4::new(1036386303, 259080192, 78217216, 1119485952);

        assert_eq!(
            r,
            mem::transmute(__msa_binsli_w(mem::transmute(a), mem::transmute(b), 17))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsli_d() {
        #[rustfmt::skip]
        let a = u64x2::new(8006399338, 2882303762);
        #[rustfmt::skip]
        let b = u64x2::new(9223372036854775805, 536870912);
        #[rustfmt::skip]
        let r = u64x2::new(9223372036854773098, 536901394);

        assert_eq!(
            r,
            mem::transmute(__msa_binsli_d(mem::transmute(a), mem::transmute(b), 48))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsr_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let c = u8x16::new(
            1, 3, 5, 9,
            1, 3, 5, 9,
            1, 3, 5, 9,
            1, 3, 5, 9
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            254, 151, 8, 1, 
            254, 151, 8, 1, 
            254, 151, 8, 1, 
            254, 151, 8, 1
        );

        assert_eq!(
            r,
            mem::transmute(__msa_binsr_b(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsr_h() {
        #[rustfmt::skip]
        let a = u16x8::new(
            32767, 16384, 8192, 4096, 
            32767, 16384, 8192, 4096
        );
        #[rustfmt::skip]
        let b = u16x8::new(
            21656, 5273, 7081, 2985, 
            21656, 5273, 7081, 2985
        );
        #[rustfmt::skip]
        let c = u16x8::new(
            3, 7, 9, 13, 
            15, 17, 21, 23
        );
        #[rustfmt::skip]
        let r = u16x8::new(
            32760, 16537, 9129, 2985, 
            21656, 16385, 8233, 4265
        );

        assert_eq!(
            r,
            mem::transmute(__msa_binsr_h(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsr_w() {
        #[rustfmt::skip]
        let a = u32x4::new(2147483647, 536870912, 67108864, 8388608);
        #[rustfmt::skip]
        let b = u32x4::new(1036372536, 259093134, 78219975, 1119499719);
        #[rustfmt::skip]
        let c = u32x4::new(11, 15, 31, 37);
        #[rustfmt::skip]
        let r = u32x4::new(2147482168, 536900238, 78219975, 8388615);

        assert_eq!(
            r,
            mem::transmute(__msa_binsr_w(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsr_d() {
        #[rustfmt::skip]
        let a = u64x2::new(8006399338, 2882303762);
        #[rustfmt::skip]
        let b = u64x2::new(9223372036854775805, 536870912);
        #[rustfmt::skip]
        let c = u64x2::new(12, 48);
        #[rustfmt::skip]
        let r = u64x2::new(8006402045, 536870912);

        assert_eq!(
            r,
            mem::transmute(__msa_binsr_d(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsri_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            198, 135, 8, 9, 
            198, 135, 8, 9, 
            198, 135, 8, 9, 
            198, 135, 8, 9
        );

        assert_eq!(
            r,
            mem::transmute(__msa_binsri_b(mem::transmute(a), mem::transmute(b), 5))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsri_h() {
        #[rustfmt::skip]
        let a = u16x8::new(
            32767, 16384, 8192, 4096, 
            32767, 16384, 8192, 4096
        );
        #[rustfmt::skip]
        let b = u16x8::new(
            21656, 5273, 7081, 2985, 
            21656, 5273, 7081, 2985
        );
        #[rustfmt::skip]
        let r = u16x8::new(
            21656, 21657, 7081, 2985, 
            21656, 21657, 7081, 2985
        );

        assert_eq!(
            r,
            mem::transmute(__msa_binsri_h(mem::transmute(a), mem::transmute(b), 13))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsri_w() {
        #[rustfmt::skip]
        let a = u32x4::new(2147483647, 536870912, 67108864, 8388608);
        #[rustfmt::skip]
        let b = u32x4::new(1036372536, 259093134, 78219975, 1119499719);
        #[rustfmt::skip]
        let r = u32x4::new(2147338808, 536965774, 67209927, 8533447);

        assert_eq!(
            r,
            mem::transmute(__msa_binsri_w(mem::transmute(a), mem::transmute(b), 17))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_binsri_d() {
        #[rustfmt::skip]
        let a = u64x2::new(8006399338, 2882303762);
        #[rustfmt::skip]
        let b = u64x2::new(9223372036854775805, 536870912);
        #[rustfmt::skip]
        let r = u64x2::new(562949953421309, 536870912);

        assert_eq!(
            r,
            mem::transmute(__msa_binsri_d(mem::transmute(a), mem::transmute(b), 48))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bmnz_v() {
        #[rustfmt::skip]
        let a = u8x16::new(
            u8::MAX, 155, 55, 1,
            u8::MAX, 155, 55, 1,
            u8::MAX, 155, 55, 1,
            u8::MAX, 155, 55, 1
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
        );
        #[rustfmt::skip]
        let c = u8x16::new(
            3, 5, 7, 1,
            3, 5, 7, 1,
            3, 5, 7, 1,
            3, 5, 7, 1
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            254, 159, 48, 1, 
            254, 159, 48, 1, 
            254, 159, 48, 1, 
            254, 159, 48, 1
        );

        assert_eq!(
            r,
            mem::transmute(__msa_bmnz_v(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bmnzi_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            u8::MAX, 155, 55, 1,
            u8::MAX, 155, 55, 1,
            u8::MAX, 155, 55, 1,
            u8::MAX, 155, 55, 1
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            1, u8::MAX, 155, 55,
            1, u8::MAX, 155, 55,
            1, u8::MAX, 155, 55,
            1, u8::MAX, 155, 55
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            249, 159, 51, 7, 
            249, 159, 51, 7, 
            249, 159, 51, 7, 
            249, 159, 51, 7
        );

        assert_eq!(
            r,
            mem::transmute(__msa_bmnzi_b(mem::transmute(a), mem::transmute(b), 7))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bmz_v() {
        #[rustfmt::skip]
        let a = u8x16::new(
            u8::MAX, 155, 55, 1,
            u8::MAX, 155, 55, 1,
            u8::MAX, 155, 55, 1,
            u8::MAX, 155, 55, 1
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let c = u8x16::new(
            3, 5, 7, 1,
            3, 5, 7, 1,
            3, 5, 7, 1,
            3, 5, 7, 1
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            7, 3, 15, 9, 
            7, 3, 15, 9, 
            7, 3, 15, 9, 
            7, 3, 15, 9
        );

        assert_eq!(
            r,
            mem::transmute(__msa_bmz_v(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bmzi_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            u8::MAX, 155, 55, 1,
            u8::MAX, 155, 55, 1,
            u8::MAX, 155, 55, 1,
            u8::MAX, 155, 55, 1
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            1, 255, 155, 55,
            1, 255, 155, 55,
            1, 255, 155, 55,
            1, 255, 155, 55
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            7, 251, 159, 49, 
            7, 251, 159, 49, 
            7, 251, 159, 49, 
            7, 251, 159, 49
        );

        assert_eq!(
            r,
            mem::transmute(__msa_bmzi_b(mem::transmute(a), mem::transmute(b), 7))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bneg_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            191, 27, 54, 3, 
            191, 27, 54, 3, 
            191, 27, 54, 3, 
            191, 27, 54, 3
        );

        assert_eq!(
            r,
            mem::transmute(__msa_bneg_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bneg_h() {
        #[rustfmt::skip]
        let a = u16x8::new(255, 155, 55, 1, 255, 155, 55, 1);
        #[rustfmt::skip]
        let b = u16x8::new(6, 7, 8, 9, 6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u16x8::new(191, 27, 311, 513, 191, 27, 311, 513);

        assert_eq!(
            r,
            mem::transmute(__msa_bneg_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bneg_w() {
        #[rustfmt::skip]
        let a = u32x4::new(255, 155, 55, 1);
        #[rustfmt::skip]
        let b = u32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u32x4::new(191, 27, 311, 513);

        assert_eq!(
            r,
            mem::transmute(__msa_bneg_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bneg_d() {
        #[rustfmt::skip]
        let a = u64x2::new(255, 155);
        #[rustfmt::skip]
        let b = u64x2::new(6, 7);
        #[rustfmt::skip]
        let r = u64x2::new(191, 27);

        assert_eq!(
            r,
            mem::transmute(__msa_bneg_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bnegi_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            50, 100, 127, u8::MAX,
            50, 100, 127, u8::MAX,
            50, 100, 127, u8::MAX,
            50, 100, 127, u8::MAX
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            34, 116, 111, 239, 
            34, 116, 111, 239, 
            34, 116, 111, 239, 
            34, 116, 111, 239
        );

        assert_eq!(r, mem::transmute(__msa_bnegi_b(mem::transmute(a), 4)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bnegi_h() {
        #[rustfmt::skip]
        let a = u16x8::new(
            32767, 3276, 100, 127,
            32767, 3276, 100, 127
        );
        #[rustfmt::skip]
        let r = u16x8::new(
            30719, 1228, 2148, 2175, 
            30719, 1228, 2148, 2175
        );

        assert_eq!(r, mem::transmute(__msa_bnegi_h(mem::transmute(a), 11)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bnegi_w() {
        #[rustfmt::skip]
        let a = u32x4::new(100, 2147483647, 100, 2147483648);
        #[rustfmt::skip]
        let r = u32x4::new(16777316, 2130706431, 16777316, 2164260864);

        assert_eq!(r, mem::transmute(__msa_bnegi_w(mem::transmute(a), 24)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bnegi_d() {
        #[rustfmt::skip]
        let a = u64x2::new(100, 9223372036854775808);
        #[rustfmt::skip]
        let r = u64x2::new(4398046511204, 9223376434901286912);

        assert_eq!(r, mem::transmute(__msa_bnegi_d(mem::transmute(a), 42)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bnz_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 1, 1, 1,
            1, 1, 1, 1,
            2, 2, 2, 2,
            4, 4, 0, 4,
        );
        let r = 0 as i32;

        assert_eq!(r, mem::transmute(__msa_bnz_b(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bnz_h() {
        #[rustfmt::skip]
        let a = u16x8::new(
            32767, 3276, 100, 127,
            32767, 0, 100, 127
        );
        let r = 0 as i32;

        assert_eq!(r, mem::transmute(__msa_bnz_h(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bnz_w() {
        #[rustfmt::skip]
        let a = u32x4::new(100, 2147483647, 0, 2147483648);
        let r = 0 as i32;

        assert_eq!(r, mem::transmute(__msa_bnz_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bnz_d() {
        #[rustfmt::skip]
        let a = u64x2::new(100, 9223372036854775808);
        #[rustfmt::skip]
        let r = 1 as i32;

        assert_eq!(r, mem::transmute(__msa_bnz_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bnz_v() {
        #[rustfmt::skip]
        let a = u8x16::new(
            0, 0, 0, 1,
            0, 0, 0, 0,
            0, 0, 0, 0,
            0, 0, 0, 0,
        );
        let r = 1 as i32;

        assert_eq!(r, mem::transmute(__msa_bnz_v(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bsel_v() {
        #[rustfmt::skip]
        let a = u8x16::new(
            3, 5, 7, 1,
            3, 5, 7, 1,
            3, 5, 7, 1,
            3, 5, 7, 1
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let c = u8x16::new(
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            7, 3, 15, 9, 
            7, 3, 15, 9, 
            7, 3, 15, 9, 
            7, 3, 15, 9
        );

        assert_eq!(
            r,
            mem::transmute(__msa_bsel_v(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bseli_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            121, 29, 57, 9, 
            121, 29, 57, 9, 
            121, 29, 57, 9, 
            121, 29, 57, 9
        );

        assert_eq!(
            r,
            mem::transmute(__msa_bseli_b(mem::transmute(a), mem::transmute(b), 121))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bset_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            255, 155, 55, 3, 
            255, 155, 55, 3, 
            255, 155, 55, 3, 
            255, 155, 55, 3
        );

        assert_eq!(
            r,
            mem::transmute(__msa_bset_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bset_h() {
        #[rustfmt::skip]
        let a = u16x8::new(255, 155, 55, 1, 255, 155, 55, 1);
        #[rustfmt::skip]
        let b = u16x8::new(6, 7, 8, 9, 6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u16x8::new(255, 155, 311, 513, 255, 155, 311, 513);

        assert_eq!(
            r,
            mem::transmute(__msa_bset_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bset_w() {
        #[rustfmt::skip]
        let a = u32x4::new(255, 155, 55, 1);
        #[rustfmt::skip]
        let b = u32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u32x4::new(255, 155, 311, 513);

        assert_eq!(
            r,
            mem::transmute(__msa_bset_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bset_d() {
        #[rustfmt::skip]
        let a = u64x2::new(255, 155);
        #[rustfmt::skip]
        let b = u64x2::new(6, 7);
        #[rustfmt::skip]
        let r = u64x2::new(255, 155);

        assert_eq!(
            r,
            mem::transmute(__msa_bset_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bseti_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            255, 159, 55, 5, 
            255, 159, 55, 5, 
            255, 159, 55, 5, 
            255, 159, 55, 5
        );

        assert_eq!(r, mem::transmute(__msa_bseti_b(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bseti_h() {
        #[rustfmt::skip]
        let a = u16x8::new(255, 155, 55, 1, 255, 155, 55, 1);
        #[rustfmt::skip]
        let r = u16x8::new(255, 159, 55, 5, 255, 159, 55, 5);

        assert_eq!(r, mem::transmute(__msa_bseti_h(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bseti_w() {
        #[rustfmt::skip]
        let a = u32x4::new(255, 155, 55, 1);
        #[rustfmt::skip]
        let r = u32x4::new(255, 159, 55, 5);

        assert_eq!(r, mem::transmute(__msa_bseti_w(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bseti_d() {
        #[rustfmt::skip]
        let a = u64x2::new(255, 155);
        #[rustfmt::skip]
        let r = u64x2::new(255, 159);

        assert_eq!(r, mem::transmute(__msa_bseti_d(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bz_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1,
            255, 155, 55, 1
        );
        let r = 0 as i32;

        assert_eq!(r, mem::transmute(__msa_bz_b(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bz_h() {
        #[rustfmt::skip]
        let a = u16x8::new(0, 0, 0, 0, 0, 0, 0, 0);
        let r = 1 as i32;

        assert_eq!(r, mem::transmute(__msa_bz_h(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bz_w() {
        #[rustfmt::skip]
        let a = u32x4::new(255, 0, 55, 1);
        let r = 1 as i32;

        assert_eq!(r, mem::transmute(__msa_bz_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bz_d() {
        #[rustfmt::skip]
        let a = u64x2::new(255, 0);
        let r = 1 as i32;

        assert_eq!(r, mem::transmute(__msa_bz_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_bz_v() {
        #[rustfmt::skip]
        let a = u8x16::new(
            0, 0, 0, 0,
            0, 0, 0, 0,
            0, 0, 0, 0,
            0, 0, 0, 0
        );
        let r = 1 as i32;

        assert_eq!(r, mem::transmute(__msa_bz_v(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ceq_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -128, 127, 55, 1,
            -128, 127, 55, 1,
            -128, 127, 55, 1,
            -128, 127, 55, 1
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -128, 126, 55, 1,
            -128, 126, 55, 1,
            -128, 126, 55, 1,
            -128, 126, 55, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -1, 0, -1, -1, 
            -1, 0, -1, -1, 
            -1, 0, -1, -1, 
            -1, 0, -1, -1
        );

        assert_eq!(
            r,
            mem::transmute(__msa_ceq_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ceq_h() {
        #[rustfmt::skip]
        let a = i16x8::new(255, 155, 55, 1, 255, 155, 55, 1);
        #[rustfmt::skip]
        let b = i16x8::new(255, 155, 56, 1, 255, 155, 56, 1);
        #[rustfmt::skip]
        let r = i16x8::new(-1, -1, 0, -1, -1, -1, 0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_ceq_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ceq_w() {
        #[rustfmt::skip]
        let a = i32x4::new(255, 155, 55, 1);
        #[rustfmt::skip]
        let b = i32x4::new(255, 156, 55, 1);
        #[rustfmt::skip]
        let r = i32x4::new(-1, 0, -1, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_ceq_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ceq_d() {
        #[rustfmt::skip]
        let a = i64x2::new(255, 155);
        #[rustfmt::skip]
        let b = i64x2::new(255, 156);
        #[rustfmt::skip]
        let r = i64x2::new(-1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_ceq_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ceqi_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            100, -1, -4, 15,
            100, -1, -4, 15,
            100, -1, -4, 15,
            100, -1, -4, 15
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            0, 0, -1, 0,
            0, 0, -1, 0,
            0, 0, -1, 0,
            0, 0, -1, 0
        );

        assert_eq!(r, mem::transmute(__msa_ceqi_b(mem::transmute(a), -4)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ceqi_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            32767, 3276, 100, -11,
            32767, 3276, 100, -11
        );
        #[rustfmt::skip]
        let r = i16x8::new(0, 0, 0, -1, 0, 0, 0, -1);

        assert_eq!(r, mem::transmute(__msa_ceqi_h(mem::transmute(a), -11)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ceqi_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 3, 5, -3);
        #[rustfmt::skip]
        let r = i32x4::new(0, 0, -1, 0);

        assert_eq!(r, mem::transmute(__msa_ceqi_w(mem::transmute(a), 5)));
    }

    // FIXME: https://reviews.llvm.org/D59884
    // 如果目标类型为 i64，则负 immediate 将丢失符号如果使用 4294967293 而不是 vector `a` 中的 -3，则测试通过
    //
    // #[simd_test(enable = "msa")]
    // unsafe fn test_msa_ceqi_d() {
    //     #[rustfmt::skip]
    //     let a = i64x2::new(-3, 2);
    //     #[rustfmt::skip]
    //     let r = i64x2::new(-1, 0);

    // assert_eq!(r, mem::transmute(__msa_ceqi_d(mem::transmute(a), -3)));
    // }

    // 无法在用户模式下进行测试
    // #[simd_test(enable = "msa")]
    // unsafe fn test_msa_cfcmsa() {
    //     let r = 5;

    // assert_eq!(r, mem::transmute(__msa_cfcmsa(5));
    // }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_cle_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -128, 127, 55, 2,
            -128, 127, 55, 2,
            -128, 127, 55, 2,
            -128, 127, 55, 2
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -128, 126, 55, 1,
            -128, 126, 55, 1,
            -128, 126, 55, 1,
            -128, 126, 55, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -1, 0, -1, 0, 
            -1, 0, -1, 0, 
            -1, 0, -1, 0, 
            -1, 0, -1, 0
        );

        assert_eq!(
            r,
            mem::transmute(__msa_cle_s_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_cle_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(255, 155, 55, 2, 255, 155, 55, 2);
        #[rustfmt::skip]
        let b = i16x8::new(255, 155, 56, 1, 255, 155, 56, 1);
        #[rustfmt::skip]
        let r = i16x8::new(-1, -1, -1, 0, -1, -1, -1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_cle_s_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_cle_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(255, 155, 55, 2);
        #[rustfmt::skip]
        let b = i32x4::new(255, 156, 55, 1);
        #[rustfmt::skip]
        let r = i32x4::new(-1, -1, -1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_cle_s_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_cle_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(255, 155);
        #[rustfmt::skip]
        let b = i64x2::new(255, 156);
        #[rustfmt::skip]
        let r = i64x2::new(-1, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_cle_s_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_cle_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            u8::MAX, 127, 55, 2,
            u8::MAX, 127, 55, 2,
            u8::MAX, 127, 55, 2,
            u8::MAX, 127, 55, 2
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            u8::MAX, 126, 55, 1,
            u8::MAX, 126, 55, 1,
            u8::MAX, 126, 55, 1,
            u8::MAX, 126, 55, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(-1, 0, -1, 0, -1, 0, -1, 0, -1, 0, -1, 0, -1, 0, -1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_cle_u_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_cle_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(
            u16::MAX, 155, 55, 2, 
            u16::MAX, 155, 55, 2
        );
        #[rustfmt::skip]
        let b = u16x8::new(
            u16::MAX, 155, 56, 1, 
            u16::MAX, 155, 56, 1
        );
        #[rustfmt::skip]
        let r = i16x8::new(-1, -1, -1, 0, -1, -1, -1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_cle_u_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_cle_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(u32::MAX, 155, 55, 2);
        #[rustfmt::skip]
        let b = u32x4::new(u32::MAX, 156, 55, 1);
        #[rustfmt::skip]
        let r = i32x4::new(-1, -1, -1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_cle_u_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_cle_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(u64::MAX, 155);
        #[rustfmt::skip]
        let b = u64x2::new(u64::MAX, 156);
        #[rustfmt::skip]
        let r = i64x2::new(-1, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_cle_u_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clei_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -2, -127, 100, -127,
            -2, -127, 100, -127,
            -2, -127, 100, -127,
            -2, -127, 100, -127
        );
        #[rustfmt::skip]
        let r = i8x16::new(-1, -1, 0, -1, -1, -1, 0, -1, -1, -1, 0, -1, -1, -1, 0, -1);

        assert_eq!(r, mem::transmute(__msa_clei_s_b(mem::transmute(a), -2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clei_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            32767, 3276, 10, -1,
            32767, 3276, 10, -1,
        );
        #[rustfmt::skip]
        let r = i16x8::new(0, 0, 0, -1, 0, 0, 0, -1);

        assert_eq!(r, mem::transmute(__msa_clei_s_h(mem::transmute(a), -1)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clei_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(100, 2147483647, 6, 2147483647);
        #[rustfmt::skip]
        let r = i32x4::new(0, 0, -1, 0);

        assert_eq!(r, mem::transmute(__msa_clei_s_w(mem::transmute(a), 6)));
    }

    // FIXME: https://reviews.llvm.org/D59884
    // 如果目标类型为 i64，则负 immediate 将丢失符号
    // -3 表示为 4294967293
    // #[simd_test(enable = "msa")]
    // unsafe fn test_msa_clei_s_d() {
    //     #[rustfmt::skip]
    //     let a = i64x2::new(-3, 11);
    //     #[rustfmt::skip]
    //     let r = i64x2::new(-1, 0);

    // assert_eq!(r, mem::transmute(__msa_clei_s_d(mem::transmute(a), -3)));
    // }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clei_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            2, 127, 100, 127,
            2, 127, 100, 127,
            2, 127, 100, 127,
            2, 127, 100, 127,
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -1, 0, 0, 0, 
            -1, 0, 0, 0, 
            -1, 0, 0, 0, 
            -1, 0, 0, 0
        );

        assert_eq!(r, mem::transmute(__msa_clei_u_b(mem::transmute(a), 25)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clei_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(
            1, 26, 15, 36,
            1, 26, 15, 36
        );
        #[rustfmt::skip]
        let r = i16x8::new(-1, 0, -1, 0, -1, 0, -1, 0);

        assert_eq!(r, mem::transmute(__msa_clei_u_h(mem::transmute(a), 25)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clei_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(25, 32, 25, 32);
        #[rustfmt::skip]
        let r = i32x4::new(-1, 0, -1, 0);

        assert_eq!(r, mem::transmute(__msa_clei_u_w(mem::transmute(a), 31)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clei_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(10, 26);
        #[rustfmt::skip]
        let r = i64x2::new(-1, 0);

        assert_eq!(r, mem::transmute(__msa_clei_u_d(mem::transmute(a), 25)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clt_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -128, 127, 55, 2,
            -128, 127, 55, 2,
            -128, 127, 55, 2,
            -128, 127, 55, 2
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -127, 126, 56, 1,
            -127, 126, 56, 1,
            -127, 126, 56, 1,
            -127, 126, 56, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -1, 0, -1, 0, 
            -1, 0, -1, 0, 
            -1, 0, -1, 0, 
            -1, 0, -1, 0
        );

        assert_eq!(
            r,
            mem::transmute(__msa_clt_s_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clt_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(-255, 155, 55, 2, -255, 155, 55, 2);
        #[rustfmt::skip]
        let b = i16x8::new(255, 156, 56, 1, 255, 156, 56, 1);
        #[rustfmt::skip]
        let r = i16x8::new(-1, -1, -1, 0, -1, -1, -1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_clt_s_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clt_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(-255, 155, 55, 2);
        #[rustfmt::skip]
        let b = i32x4::new(255, 156, 55, 1);
        #[rustfmt::skip]
        let r = i32x4::new(-1, -1, 0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_clt_s_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clt_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(-255, 155);
        #[rustfmt::skip]
        let b = i64x2::new(255, 156);
        #[rustfmt::skip]
        let r = i64x2::new(-1, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_clt_s_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clt_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            128, 127, 55, 2,
            128, 127, 55, 2,
            128, 127, 55, 2,
            128, 127, 55, 2
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            127, 126, 56, 1,
            127, 126, 56, 1,
            127, 126, 56, 1,
            127, 126, 56, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            0, 0, -1, 0, 
            0, 0, -1, 0, 
            0, 0, -1, 0, 
            0, 0, -1, 0
        );

        assert_eq!(
            r,
            mem::transmute(__msa_clt_u_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clt_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(255, 155, 55, 2, 255, 155, 55, 2);
        #[rustfmt::skip]
        let b = u16x8::new(255, 156, 56, 1, 255, 156, 56, 1);
        #[rustfmt::skip]
        let r = i16x8::new(0, -1, -1, 0, 0, -1, -1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_clt_u_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clt_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(255, 155, 55, 2);
        #[rustfmt::skip]
        let b = u32x4::new(255, 156, 55, 1);
        #[rustfmt::skip]
        let r = i32x4::new(0, -1, 0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_clt_u_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clt_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(255, 155);
        #[rustfmt::skip]
        let b = u64x2::new(255, 156);
        #[rustfmt::skip]
        let r = i64x2::new(0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_clt_u_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clti_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            2, -127, -5, 127,
            2, -127, -5, 127,
            2, -127, -5, 127,
            2, -127, -5, 127
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            0, -1, 0, 0, 
            0, -1, 0, 0, 
            0, -1, 0, 0, 
            0, -1, 0, 0
        );

        assert_eq!(r, mem::transmute(__msa_clti_s_b(mem::transmute(a), -5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clti_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            -1024, 3276, 15, 127,
            -1024, 3276, 15, 127
        );
        #[rustfmt::skip]
        let r = i16x8::new(-1, 0, 0, 0, -1, 0, 0, 0);

        assert_eq!(r, mem::transmute(__msa_clti_s_h(mem::transmute(a), 15)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clti_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(-15, 2147483647, -15, 2147483647);
        #[rustfmt::skip]
        let r = i32x4::new(-1, 0, -1, 0);

        assert_eq!(r, mem::transmute(__msa_clti_s_w(mem::transmute(a), -10)));
    }

    // FIXME: https://reviews.llvm.org/D59884
    // 如果目标类型为 i64，则负 immediate 将丢失符号
    // -3 表示为 4294967293
    // #[simd_test(enable = "msa")]
    // unsafe fn test_msa_clti_s_d() {
    //     #[rustfmt::skip]
    //     let a = i64x2::new(-5, -2);
    //     #[rustfmt::skip]
    //     let r = i64x2::new(-1, 0);

    // assert_eq!(r, mem::transmute(__msa_clti_s_d(mem::transmute(a), -3)));
    // }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clti_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            2, 127, 49, 127,
            2, 127, 49, 127,
            2, 127, 49, 127,
            2, 127, 49, 127,
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -1, 0, 0, 0,
            -1, 0, 0, 0,
            -1, 0, 0, 0,
            -1, 0, 0, 0
        );

        assert_eq!(r, mem::transmute(__msa_clti_u_b(mem::transmute(a), 50)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clti_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(
            327, 3276, 100, 127,
            327, 3276, 100, 127
        );
        #[rustfmt::skip]
        let r = i16x8::new(0, 0, 0, 0, 0, 0, 0, 0);

        assert_eq!(r, mem::transmute(__msa_clti_u_h(mem::transmute(a), 30)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clti_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(100, 2147483647, 100, 2147483647);
        #[rustfmt::skip]
        let r = i32x4::new(0, 0, 0, 0);

        assert_eq!(r, mem::transmute(__msa_clti_u_w(mem::transmute(a), 10)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_clti_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(1, 9223372036854775807);
        #[rustfmt::skip]
        let r = i64x2::new(-1, 0);

        assert_eq!(r, mem::transmute(__msa_clti_u_d(mem::transmute(a), 10)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_copy_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -100, 127, 4, 127,
            -100, 127, 4, 127,
            -100, 127, 4, 127,
            -100, 127, 4, 127
        );
        #[rustfmt::skip]
        let r = -100 as i32;

        assert_eq!(r, mem::transmute(__msa_copy_s_b(mem::transmute(a), 12)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_copy_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            32767, 3276, 100, 11,
            32767, 3276, 100, 11
        );
        #[rustfmt::skip]
        let r = 32767 as i32;

        assert_eq!(r, mem::transmute(__msa_copy_s_h(mem::transmute(a), 4)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_copy_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(100, 2147483647, 5, -2147483647);
        let r = 2147483647 as i32;

        assert_eq!(r, mem::transmute(__msa_copy_s_w(mem::transmute(a), 1)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_copy_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(3, 9223372036854775807);
        #[rustfmt::skip]
        let r = 9223372036854775807 as i64;

        assert_eq!(r, mem::transmute(__msa_copy_s_d(mem::transmute(a), 1)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_copy_u_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            100, 127, 4, 127,
            100, 127, 4, 127,
            100, 127, 4, 127,
            100, 127, 4, 127
        );
        #[rustfmt::skip]
        let r = 100 as u32;

        assert_eq!(r, mem::transmute(__msa_copy_u_b(mem::transmute(a), 12)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_copy_u_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            32767, 3276, 100, 11,
            32767, 3276, 100, 11
        );
        #[rustfmt::skip]
        let r = 32767 as u32;

        assert_eq!(r, mem::transmute(__msa_copy_u_h(mem::transmute(a), 4)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_copy_u_w() {
        #[rustfmt::skip]
        let a = i32x4::new(100, 2147483647, 5, 2147483647);
        #[rustfmt::skip]
        let r = 2147483647 as u32;

        assert_eq!(r, mem::transmute(__msa_copy_u_w(mem::transmute(a), 1)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_copy_u_d() {
        #[rustfmt::skip]
        let a = i64x2::new(3, i64::MAX);
        #[rustfmt::skip]
        let r = 9223372036854775807 as u64;

        assert_eq!(r, mem::transmute(__msa_copy_u_d(mem::transmute(a), 1)));
    }

    // 无法在用户模式下进行测试
    //
    // #[simd_test(enable = "msa")]
    // unsafe fn test_msa_ctcmsa() {}

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_div_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -6, -7, -8, -9,
            -6, -7, -8, -9,
            -6, -7, -8, -9,
            -6, -7, -8, -9
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -1, -2, -3, -4,
            -1, -2, -3, -4,
            -1, -2, -3, -4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            6, 3, 2, 2, 
            6, 3, 2, 2, 
            6, 3, 2, 2, 
            6, 3, 2, 2
        );

        assert_eq!(
            r,
            mem::transmute(__msa_div_s_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_div_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(-6, -7, -8, -9, 6, 7, 8, 9);
        #[rustfmt::skip]
        let b = i16x8::new(-1, -2, -3, -4, -1, -2, -3, -4);
        #[rustfmt::skip]
        let r = i16x8::new(6, 3, 2, 2, -6, -3, -2, -2);

        assert_eq!(
            r,
            mem::transmute(__msa_div_s_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_div_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(-6, -7, 8, 9);
        #[rustfmt::skip]
        let b = i32x4::new(-1, -2, -3, -4);
        #[rustfmt::skip]
        let r = i32x4::new(6, 3, -2, -2);

        assert_eq!(
            r,
            mem::transmute(__msa_div_s_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_div_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(-6, 7);
        #[rustfmt::skip]
        let b = i64x2::new(-1, -2);
        #[rustfmt::skip]
        let r = i64x2::new(6, -3);

        assert_eq!(
            r,
            mem::transmute(__msa_div_s_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_div_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            6, 3, 2, 2, 
            6, 3, 2, 2, 
            6, 3, 2, 2, 
            6, 3, 2, 2
        );

        assert_eq!(
            r,
            mem::transmute(__msa_div_u_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_div_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(6, 7, 8, 9, 6, 7, 8, 9);
        #[rustfmt::skip]
        let b = u16x8::new(1, 2, 3, 4, 1, 2, 3, 4);
        #[rustfmt::skip]
        let r = u16x8::new(6, 3, 2, 2, 6, 3, 2, 2);

        assert_eq!(
            r,
            mem::transmute(__msa_div_u_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_div_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let b = u32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let r = u32x4::new(6, 3, 2, 2);

        assert_eq!(
            r,
            mem::transmute(__msa_div_u_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_div_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(6, 7);
        #[rustfmt::skip]
        let b = u64x2::new(1, 2);
        #[rustfmt::skip]
        let r = u64x2::new(6, 3);

        assert_eq!(
            r,
            mem::transmute(__msa_div_u_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dotp_s_h() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -1, -2, -3, 4,
            -1, -2, -3, -4,
            -1, -2, -3, 4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -6, -7, -8, -9,
            -6, -7, -8, -9,
            -6, -7, -8, -9,
            -6, -7, -8, -9
        );
        #[rustfmt::skip]
        let r = i16x8::new(20, -12, 20, 60, 20, -12, 20, 60);

        assert_eq!(
            r,
            mem::transmute(__msa_dotp_s_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dotp_s_w() {
        #[rustfmt::skip]
        let a = i16x8::new(-1, -2, -3, -4, -1, -2, -3, 4);
        #[rustfmt::skip]
        let b = i16x8::new(-6, -7, -8, -9, -6, -7, -8, -9);
        #[rustfmt::skip]
        let r = i32x4::new(20, 60, 20, -12);

        assert_eq!(
            r,
            mem::transmute(__msa_dotp_s_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dotp_s_d() {
        #[rustfmt::skip]
        let a = i32x4::new(-1, -2, -3, 4);
        #[rustfmt::skip]
        let b = i32x4::new(-6, -7, -8, -9);
        #[rustfmt::skip]
        let r = i64x2::new(20, -12);

        assert_eq!(
            r,
            mem::transmute(__msa_dotp_s_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dotp_u_h() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = u16x8::new(20, 60, 20, 60, 20, 60, 20, 60);

        assert_eq!(
            r,
            mem::transmute(__msa_dotp_u_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dotp_u_w() {
        #[rustfmt::skip]
        let a = u16x8::new(1, 2, 3, 4, 1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u16x8::new(6, 7, 8, 9, 6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u32x4::new(20, 60, 20, 60);

        assert_eq!(
            r,
            mem::transmute(__msa_dotp_u_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dotp_u_d() {
        #[rustfmt::skip]
        let a = u32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u64x2::new(20, 60);

        assert_eq!(
            r,
            mem::transmute(__msa_dotp_u_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dpadd_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(-1, -2, -3, -4, -1, -2, -3, 4);
        #[rustfmt::skip]
        let b = i8x16::new(
            -1, -2, -3, 4,
            -1, -2, -3, -4,
            -1, -2, -3, 4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let c = i8x16::new(
            -6, -7, -8, -9,
            -6, -7, -8, -9,
            -6, -7, -8, -9,
            -6, -7, -8, -9
        );
        #[rustfmt::skip]
        let r = i16x8::new(19, -14, 17, 56, 19, -14, 17, 64);

        assert_eq!(
            r,
            mem::transmute(__msa_dpadd_s_h(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dpadd_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(-1, -2, -3, -4);
        #[rustfmt::skip]
        let b = i16x8::new(
            -1, -2, -3, 4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let c = i16x8::new(
            -6, -7, -8, -9,
            -6, -7, -8, -9
        );
        #[rustfmt::skip]
        let r = i32x4::new(19, -14, 17, 56);

        assert_eq!(
            r,
            mem::transmute(__msa_dpadd_s_w(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dpadd_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(-1, -2);
        #[rustfmt::skip]
        let b = i32x4::new(-1, -2, -3, 4);
        #[rustfmt::skip]
        let c = i32x4::new(-6, -7, -8, -9);
        #[rustfmt::skip]
        let r = i64x2::new(19, -14);

        assert_eq!(
            r,
            mem::transmute(__msa_dpadd_s_d(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dpadd_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(1, 2, 3, 4, 1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let c = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = u16x8::new(21, 62, 23, 64, 21, 62, 23, 64);

        assert_eq!(
            r,
            mem::transmute(__msa_dpadd_u_h(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dpadd_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u16x8::new(
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let c = u16x8::new(
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = u32x4::new(21, 62, 23, 64);

        assert_eq!(
            r,
            mem::transmute(__msa_dpadd_u_w(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dpadd_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(1, 2);
        #[rustfmt::skip]
        let b = u32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let c = u32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u64x2::new(21, 62);

        assert_eq!(
            r,
            mem::transmute(__msa_dpadd_u_d(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dpsub_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(-1, -2, -3, -4, -1, -2, -3, 4);
        #[rustfmt::skip]
        let b = i8x16::new(
            -1, -2, -3, 4,
            -1, -2, -3, -4,
            -1, -2, -3, 4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let c = i8x16::new(
            -6, -7, -8, -9,
            -6, -7, -8, -9,
            -6, -7, -8, -9,
            -6, -7, -8, -9
        );
        #[rustfmt::skip]
        let r = i16x8::new(-21, 10, -23, -64, -21, 10, -23, -56);

        assert_eq!(
            r,
            mem::transmute(__msa_dpsub_s_h(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dpsub_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(-1, -2, -3, -4);
        #[rustfmt::skip]
        let b = i16x8::new(
            -1, -2, -3, 4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let c = i16x8::new(
            -6, -7, -8, -9,
            -6, -7, -8, -9
        );
        #[rustfmt::skip]
        let r = i32x4::new(-21, 10, -23, -64);

        assert_eq!(
            r,
            mem::transmute(__msa_dpsub_s_w(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dpsub_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(-1, -2);
        #[rustfmt::skip]
        let b = i32x4::new(-1, -2, -3, 4);
        #[rustfmt::skip]
        let c = i32x4::new(-6, -7, -8, -9);
        #[rustfmt::skip]
        let r = i64x2::new(-21, 10);

        assert_eq!(
            r,
            mem::transmute(__msa_dpsub_s_d(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dpsub_u_h() {
        #[rustfmt::skip]
        let a = i16x8::new(1, -2, 3, -4, -1, 2,-3, 4);
        #[rustfmt::skip]
        let b = u8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4        
        );
        #[rustfmt::skip]
        let c = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9       
        );
        #[rustfmt::skip]
        let r = i16x8::new(-19, -62, -17, -64, -21, -58, -23, -56);

        assert_eq!(
            r,
            mem::transmute(__msa_dpsub_u_h(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dpsub_u_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, -2, 3, -4);
        #[rustfmt::skip]
        let b = u16x8::new(
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let c = u16x8::new(
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = i32x4::new(-19, -62, -17, -64);

        assert_eq!(
            r,
            mem::transmute(__msa_dpsub_u_w(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_dpsub_u_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, -2);
        #[rustfmt::skip]
        let b = u32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let c = u32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = i64x2::new(-19, -62);

        assert_eq!(
            r,
            mem::transmute(__msa_dpsub_u_d(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fadd_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.1, -2.2, 3.3, -4.4);
        #[rustfmt::skip]
        let b = f32x4::new(4.4, -3.3, 2.2, -1.1);
        #[rustfmt::skip]
        let r = f32x4::new(5.5, -5.5, 5.5, -5.5);

        assert_eq!(
            r,
            mem::transmute(__msa_fadd_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fadd_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.1, -2.2);
        #[rustfmt::skip]
        let b = f64x2::new(4.4, -3.3);
        #[rustfmt::skip]
        let r = f64x2::new(5.5, -5.5);

        assert_eq!(
            r,
            mem::transmute(__msa_fadd_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    // 仅观察到的信号应该是 SIGFPE 信号无法测试
    //
    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcaf_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.1, -2.2, 3.3, -4.4);
        #[rustfmt::skip]
        let b = f32x4::new(0.0, -1.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let r = i32x4::new(0, 0, 0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fcaf_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    // 仅观察到的信号应该是 SIGFPE 信号无法测试
    //
    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcaf_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.1, -2.2);
        #[rustfmt::skip]
        let b = f64x2::new(-2.2, 1.1);
        #[rustfmt::skip]
        let r = i64x2::new(0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fcaf_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fceq_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.1, -2.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let b = f32x4::new(-4.4, -2.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let r = i32x4::new(0, -1, -1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fceq_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fceq_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.1, -2.2);
        #[rustfmt::skip]
        let b = f64x2::new(1.1, 1.1);
        #[rustfmt::skip]
        let r = i64x2::new(-1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fceq_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fclass_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.1, -2.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let r = i32x4::new(128, 8, 128, 2);

        assert_eq!(r, mem::transmute(__msa_fclass_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fclass_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.1, -2.2);
        #[rustfmt::skip]
        let r = i64x2::new(128, 8);

        assert_eq!(r, mem::transmute(__msa_fclass_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcle_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.1, -2.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let b = f32x4::new(-4.4, -1.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let r = i32x4::new(0, -1, -1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fcle_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcle_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.1, -2.2);
        #[rustfmt::skip]
        let b = f64x2::new(1.1, 1.1);
        #[rustfmt::skip]
        let r = i64x2::new(-1, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fcle_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fclt_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.1, -2.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let b = f32x4::new(-4.4, -1.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let r = i32x4::new(0, -1, 0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fclt_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fclt_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.1, -2.2);
        #[rustfmt::skip]
        let b = f64x2::new(1.1, 1.1);
        #[rustfmt::skip]
        let r = i64x2::new(0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fclt_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcne_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.1, -2.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let b = f32x4::new(-4.4, -1.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let r = i32x4::new(-1, -1, 0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fcne_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcne_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.1, -2.2);
        #[rustfmt::skip]
        let b = f64x2::new(1.1, 1.1);
        #[rustfmt::skip]
        let r = i64x2::new(0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fcne_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcor_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.1, -2.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let b = f32x4::new(f32::NAN, -1.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let r = i32x4::new(0, -1, -1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fcor_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcor_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.1, f64::NAN);
        #[rustfmt::skip]
        let b = f64x2::new(1.1, 1.1);
        #[rustfmt::skip]
        let r = i64x2::new(-1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fcor_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcueq_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.1, -2.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let b = f32x4::new(f32::NAN, -1.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let r = i32x4::new(-1, 0, -1, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fcueq_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcueq_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.1, f64::NAN);
        #[rustfmt::skip]
        let b = f64x2::new(1.1, 1.1);
        #[rustfmt::skip]
        let r = i64x2::new(-1, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fcueq_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcule_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.1, -2.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let b = f32x4::new(f32::NAN, -1.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let r = i32x4::new(-1, -1, -1, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fcule_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcule_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.1, f64::NAN);
        #[rustfmt::skip]
        let b = f64x2::new(1.1, 1.1);
        #[rustfmt::skip]
        let r = i64x2::new(-1, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fcule_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcult_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.1, -2.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let b = f32x4::new(f32::NAN, -1.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let r = i32x4::new(-1, -1, 0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fcult_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcult_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.1, f64::NAN);
        #[rustfmt::skip]
        let b = f64x2::new(1.1, 1.1);
        #[rustfmt::skip]
        let r = i64x2::new(0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fcult_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcun_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.1, -2.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let b = f32x4::new(f32::NAN, -1.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let r = i32x4::new(-1, 0, 0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fcun_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcun_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.1, f64::NAN);
        #[rustfmt::skip]
        let b = f64x2::new(1.1, 1.1);
        #[rustfmt::skip]
        let r = i64x2::new(0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fcun_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcune_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.1, -2.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let b = f32x4::new(f32::NAN, -1.2, 3.3, f32::NAN);
        #[rustfmt::skip]
        let r = i32x4::new(-1, -1, 0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fcune_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fcune_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.1, f64::NAN);
        #[rustfmt::skip]
        let b = f64x2::new(1.1, 1.1);
        #[rustfmt::skip]
        let r = i64x2::new(0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fcune_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fdiv_w() {
        #[rustfmt::skip]
        let a = f32x4::new(5.25, -20.2, 333.333, -425.0);
        #[rustfmt::skip]
        let b = f32x4::new(4.0, -2.1, 11.11, 8.2);
        #[rustfmt::skip]
        let r = f32x4::new(1.3125, 9.619048, 30.002972, -51.82927);

        assert_eq!(
            r,
            mem::transmute(__msa_fdiv_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fdiv_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1111.11, -222222.2);
        #[rustfmt::skip]
        let b = f64x2::new(-4.85, 3.33);
        #[rustfmt::skip]
        let r = f64x2::new(-229.09484536082473, -66733.3933933934);

        assert_eq!(
            r,
            mem::transmute(__msa_fdiv_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    /*// FIXME: 16-bit floats
    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fexdo_h() {
        #[rustfmt::skip]
        let a = f32x4::new(20.5, 2.3, 4.5, 5.4);
        #[rustfmt::skip]
        let b = f32x4::new(1.1, 1.0, 1.0, 1.0);
        let r = i16x8::new(1, 9, 30, 51, 1, 9, 30, 51);

        assert_eq!(r, mem::transmute(__msa_fexdo_h(mem::transmute(a), mem::transmute(b))));
    }*/

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fexdo_w() {
        #[rustfmt::skip]
        let a = f64x2::new(2000005.5, 2.3);
        #[rustfmt::skip]
        let b = f64x2::new(1235689784512.1, 2147483649998.5);
        #[rustfmt::skip]
        let r = f32x4::new(
            1235689800000.0, 2147483600000.0, 
            2000005.5, 2.3
        );

        assert_eq!(
            r,
            mem::transmute(__msa_fexdo_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fexp2_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.1, -2.2, 3.3, -4.4);
        #[rustfmt::skip]
        let b = i32x4::new(4, -3, 2, 1);
        #[rustfmt::skip]
        let r = f32x4::new(17.6, -0.275, 13.2, -8.8);

        assert_eq!(
            r,
            mem::transmute(__msa_fexp2_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fexp2_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.1, -2.2);
        #[rustfmt::skip]
        let b = i64x2::new(-4, 3);
        #[rustfmt::skip]
        let r = f64x2::new(0.06875, -17.6);

        assert_eq!(
            r,
            mem::transmute(__msa_fexp2_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    // FIXME: 16 位浮点数
    // #[simd_test(enable = "msa")]
    // unsafe fn test_msa_fexupl_w() {
    //     #[rustfmt::skip]
    //     let a = f16x8(1.5, 2.5, 3.5, 4.5, 5.5, 6.5, 7.5, 8.5);
    //     #[rustfmt::skip]
    //     let r = f32x4::new(5.5, 6.5, 7.5, 8.5);

    // assert_eq!(r, mem::transmute(__msa_fexupl_w(mem::transmute(a))));
    // }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fexupl_d() {
        #[rustfmt::skip]
        let a = f32x4::new(5.5, 6.5, 7.5, 8.5);
        #[rustfmt::skip]
        let r = f64x2::new(7.5, 8.5);

        assert_eq!(r, mem::transmute(__msa_fexupl_d(mem::transmute(a))));
    }

    // FIXME: 16 位浮点数
    //     #[simd_test(enable = "msa")]
    // unsafe fn test_msa_fexupr_w() {
    //     #[rustfmt::skip]
    //     let a = f16x8(1.5, 2.5, 3.5, 4.5, 5.5, 6.5, 7.5, 8.5);
    //     #[rustfmt::skip]
    //     let r = f32x4::new(1.5, 2.5, 3.5, 4.5);

    // assert_eq!(r, mem::transmute(__msa_fexupr_w(mem::transmute(a))));
    // }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fexupr_d() {
        #[rustfmt::skip]
        let a = f32x4::new(5.5, 6.5, 7.5, 8.5);
        #[rustfmt::skip]
        let r = f64x2::new(5.5, 6.5);

        assert_eq!(r, mem::transmute(__msa_fexupr_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ffint_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(-1, 2, -3, 4);
        #[rustfmt::skip]
        let r = f32x4::new(-1.0, 2.0, -3.0, 4.0);

        assert_eq!(r, mem::transmute(__msa_ffint_s_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ffint_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(-1, 2);
        #[rustfmt::skip]
        let r = f64x2::new(-1.0,     2.0);

        assert_eq!(r, mem::transmute(__msa_ffint_s_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ffint_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let r = f32x4::new(1.0, 2.0, 3.0, 4.0);

        assert_eq!(r, mem::transmute(__msa_ffint_u_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ffint_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(1, 2);
        #[rustfmt::skip]
        let r = f64x2::new(1.0, 2.0);

        assert_eq!(r, mem::transmute(__msa_ffint_u_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ffql_w() {
        #[rustfmt::skip]
        let a = i16x8::new(11, 25, 33, 47, 11, 25, 33, 47);
        #[rustfmt::skip]
        let r = f32x4::new(
            0.00033569336, 0.00076293945,
            0.0010070801, 0.0014343262
        );

        assert_eq!(r, mem::transmute(__msa_ffql_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ffql_d() {
        #[rustfmt::skip]
        let a = i32x4::new(1111, 2222, 3333, 4444);
        #[rustfmt::skip]
        let r = f64x2::new(
            0.000001552049070596695,
            0.0000020693987607955933
        );

        assert_eq!(r, mem::transmute(__msa_ffql_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ffqr_w() {
        #[rustfmt::skip]
        let a = i16x8::new(12, 26, 34, 48, 11, 25, 33, 47);
        #[rustfmt::skip]
        let r = f32x4::new(
            0.00036621094, 0.00079345703, 
            0.0010375977, 0.0014648438
        );

        assert_eq!(r, mem::transmute(__msa_ffqr_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ffqr_d() {
        #[rustfmt::skip]
        let a = i32x4::new(1111, 2555, 3333, 475);
        #[rustfmt::skip]
        let r = f64x2::new(
            0.0000005173496901988983, 
            0.0000011897645890712738
        );

        assert_eq!(r, mem::transmute(__msa_ffqr_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fill_b() {
        #[rustfmt::skip]
        let r = i8x16::new(
            2, 2, 2, 2,
            2, 2, 2, 2,
            2, 2, 2, 2,
            2, 2, 2, 2
        );

        assert_eq!(r, mem::transmute(__msa_fill_b(2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fill_h() {
        #[rustfmt::skip]
        let r = i16x8::new(2, 2, 2, 2, 2, 2, 2, 2);

        assert_eq!(r, mem::transmute(__msa_fill_h(2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fill_w() {
        #[rustfmt::skip]
        let r = i32x4::new(2, 2, 2, 2);

        assert_eq!(r, mem::transmute(__msa_fill_w(2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fill_d() {
        #[rustfmt::skip]
        let r = i64x2::new(2, 2);

        assert_eq!(r, mem::transmute(__msa_fill_d(2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_flog2_w() {
        #[rustfmt::skip]
        let a = f32x4::new(8.0, 16.0, 32.0, 64.0);
        #[rustfmt::skip]
        let r = f32x4::new(3.0, 4.0, 5.0, 6.0);

        assert_eq!(r, mem::transmute(__msa_flog2_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_flog2_d() {
        #[rustfmt::skip]
        let a = f64x2::new(8.0, 16.0);
        #[rustfmt::skip]
        let r = f64x2::new(3.0, 4.0);

        assert_eq!(r, mem::transmute(__msa_flog2_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fmadd_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.0, 2.0, 3.0, 4.0);
        #[rustfmt::skip]
        let b = f32x4::new(5.0, 6.0, 7.0, 8.0);
        #[rustfmt::skip]
        let c = f32x4::new(9.0, 10.0, 11.0, 12.0);
        #[rustfmt::skip]
        let r = f32x4::new(46.0, 62.0, 80.0, 100.0);

        assert_eq!(
            r,
            mem::transmute(__msa_fmadd_w(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fmadd_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.0, 2.0);
        #[rustfmt::skip]
        let b = f64x2::new(3.0, 4.0);
        #[rustfmt::skip]
        let c = f64x2::new(5.0, 6.0);
        #[rustfmt::skip]
        let r = f64x2::new(16.0, 26.0);

        assert_eq!(
            r,
            mem::transmute(__msa_fmadd_d(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fmax_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.0, -6.0, 7.0, 8.0);
        #[rustfmt::skip]
        let b = f32x4::new(5.0, -2.0, 3.0, 4.0);
        #[rustfmt::skip]
        let r = f32x4::new(5.0, -2.0, 7.0, 8.0);

        assert_eq!(
            r,
            mem::transmute(__msa_fmax_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fmax_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.0, 4.0);
        #[rustfmt::skip]
        let b = f64x2::new(3.0, 2.0);
        #[rustfmt::skip]
        let r = f64x2::new(3.0, 4.0);

        assert_eq!(
            r,
            mem::transmute(__msa_fmax_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fmax_a_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.0, -6.0, -7.0, -8.0);
        #[rustfmt::skip]
        let b = f32x4::new(5.0, -2.0, 3.0, 4.0);
        #[rustfmt::skip]
        let r = f32x4::new(5.0, -6.0, -7.0, -8.0);

        assert_eq!(
            r,
            mem::transmute(__msa_fmax_a_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fmax_a_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.0, -4.0);
        #[rustfmt::skip]
        let b = f64x2::new(3.0, 2.0);
        #[rustfmt::skip]
        let r = f64x2::new(3.0, -4.0);

        assert_eq!(
            r,
            mem::transmute(__msa_fmax_a_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fmin_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.0, -6.0, 7.0, 8.0);
        #[rustfmt::skip]
        let b = f32x4::new(5.0, -2.0, 3.0, 4.0);
        #[rustfmt::skip]
        let r = f32x4::new(1.0, -6.0, 3.0, 4.0);

        assert_eq!(
            r,
            mem::transmute(__msa_fmin_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fmin_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.0, 4.0);
        #[rustfmt::skip]
        let b = f64x2::new(3.0, 2.0);
        #[rustfmt::skip]
        let r = f64x2::new(1.0, 2.0);

        assert_eq!(
            r,
            mem::transmute(__msa_fmin_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fmin_a_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.0, -6.0, -7.0, -8.0);
        #[rustfmt::skip]
        let b = f32x4::new(5.0, -2.0, 3.0, 4.0);
        #[rustfmt::skip]
        let r = f32x4::new(1.0, -2.0, 3.0, 4.0);

        assert_eq!(
            r,
            mem::transmute(__msa_fmin_a_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fmin_a_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.0, -4.0);
        #[rustfmt::skip]
        let b = f64x2::new(3.0, 2.0);
        #[rustfmt::skip]
        let r = f64x2::new(1.0, 2.0);

        assert_eq!(
            r,
            mem::transmute(__msa_fmin_a_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fmsub_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.0, 2.0, 3.0, 4.0);
        #[rustfmt::skip]
        let b = f32x4::new(5.0, 6.0, 7.0, 8.0);
        #[rustfmt::skip]
        let c = f32x4::new(9.0, 10.0, 11.0, 12.0);
        #[rustfmt::skip]
        let r = f32x4::new(-44.0, -58.0, -74.0, -92.0);

        assert_eq!(
            r,
            mem::transmute(__msa_fmsub_w(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fmsub_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.0, 2.0);
        #[rustfmt::skip]
        let b = f64x2::new(3.0, 4.0);
        #[rustfmt::skip]
        let c = f64x2::new(5.0, 6.0);
        #[rustfmt::skip]
        let r = f64x2::new(-14.0, -22.0);

        assert_eq!(
            r,
            mem::transmute(__msa_fmsub_d(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fmul_w() {
        #[rustfmt::skip]
        let a = f32x4::new(1.1, -2.2, 3.3, 4.4);
        #[rustfmt::skip]
        let b = f32x4::new(4.4, 3.3, 2.2, -1.1);
        #[rustfmt::skip]
        let r = f32x4::new(4.84, -7.26, 7.26, -4.84);

        assert_eq!(
            r,
            mem::transmute(__msa_fmul_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fmul_d() {
        #[rustfmt::skip]
        let a = f64x2::new(1.1, -2.2);
        #[rustfmt::skip]
        let b = f64x2::new(4.0, -3.3);
        #[rustfmt::skip]
        let r = f64x2::new(4.4, 7.26);

        assert_eq!(
            r,
            mem::transmute(__msa_fmul_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_frint_w() {
        #[rustfmt::skip]
        let a = f32x4::new(2.6, -2.7, 1.3, -1.7);
        #[rustfmt::skip]
        let r = f32x4::new(3.0, -3.0, 1.0, -2.0);

        assert_eq!(r, mem::transmute(__msa_frint_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_frint_d() {
        #[rustfmt::skip]
        let a = f64x2::new(2.6, 1.3);
        #[rustfmt::skip]
        let r = f64x2::new(3.0, 1.0);

        assert_eq!(r, mem::transmute(__msa_frint_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_frcp_w() {
        #[rustfmt::skip]
        let a = f32x4::new(2.6, -2.7, 1.3, -1.7);
        #[rustfmt::skip]
        let r = f32x4::new(
            0.3846154, -0.37037036, 
            0.7692308, -0.58823526
        );

        assert_eq!(r, mem::transmute(__msa_frcp_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_frcp_d() {
        #[rustfmt::skip]
        let a = f64x2::new(2.6, 1.3);
        #[rustfmt::skip]
        let r = f64x2::new(0.3846153846153846, 0.7692307692307692);

        assert_eq!(r, mem::transmute(__msa_frcp_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_frsqrt_w() {
        #[rustfmt::skip]
        let a = f32x4::new(2.6, 2.7, 1.3, 1.7);
        #[rustfmt::skip]
        let r = f32x4::new(
            0.6201737, 0.6085806, 
            0.87705797, 0.766965
        );

        assert_eq!(r, mem::transmute(__msa_frsqrt_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_frsqrt_d() {
        #[rustfmt::skip]
        let a = f64x2::new(2.6, 1.3);
        #[rustfmt::skip]
        let r = f64x2::new(0.6201736729460422, 0.8770580193070292);

        assert_eq!(r, mem::transmute(__msa_frsqrt_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsaf_w() {
        #[rustfmt::skip]
        let a = f32x4::new(-5.5, 5.5, 5.5, 5.5);
        #[rustfmt::skip]
        let b = f32x4::new(-5.5, 5.5, 5.5, 5.5);
        #[rustfmt::skip]
        let r = i32x4::new(0, 0, 0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fsaf_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsaf_d() {
        #[rustfmt::skip]
        let a = f64x2::new(-125.5, 5.5);
        #[rustfmt::skip]
        let b = f64x2::new(125.5, 3.3);
        #[rustfmt::skip]
        let r = i64x2::new(0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fsaf_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fseq_w() {
        #[rustfmt::skip]
        let a = f32x4::new(-5.5, -3.3, f32::NAN, f32::NAN);
        #[rustfmt::skip]
        let b = f32x4::new(5.5, -3.3, f32::NAN, 1.1);
        #[rustfmt::skip]
        let r = i32x4::new(0, -1, 0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fseq_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fseq_d() {
        #[rustfmt::skip]
        let a = f64x2::new(-125.5, 5.5);
        #[rustfmt::skip]
        let b = f64x2::new(125.5, 5.5);
        #[rustfmt::skip]
        let r = i64x2::new(0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fseq_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsle_w() {
        #[rustfmt::skip]
        let a = f32x4::new(5.5, 5.5, 5.5, f32::NAN);
        #[rustfmt::skip]
        let b = f32x4::new(-5.5, 3.3, 5.5, f32::NAN);
        #[rustfmt::skip]
        let r = i32x4::new(0, 0, -1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fsle_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsle_d() {
        #[rustfmt::skip]
        let a = f64x2::new(-125.5, 5.5);
        #[rustfmt::skip]
        let b = f64x2::new(125.5, 3.3);
        #[rustfmt::skip]
        let r = i64x2::new(-1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fsle_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fslt_w() {
        #[rustfmt::skip]
        let a = f32x4::new(-5.5, 5.5, 5.5, 5.5);
        #[rustfmt::skip]
        let b = f32x4::new(5.5, 3.3, 5.5, 1.1);
        #[rustfmt::skip]
        let r = i32x4::new(-1, 0, 0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fslt_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fslt_d() {
        #[rustfmt::skip]
        let a = f64x2::new(-125.5, 5.5);
        #[rustfmt::skip]
        let b = f64x2::new(125.5, 3.3);
        #[rustfmt::skip]
        let r = i64x2::new(-1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fslt_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsne_w() {
        #[rustfmt::skip]
        let a = f32x4::new(-5.5, 5.5, 5.5, 5.5);
        #[rustfmt::skip]
        let b = f32x4::new(5.5, 3.3, 5.5, 1.1);
        #[rustfmt::skip]
        let r = i32x4::new(-1, -1, 0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fsne_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsne_d() {
        #[rustfmt::skip]
        let a = f64x2::new(-125.5, 5.5);
        #[rustfmt::skip]
        let b = f64x2::new(125.5, 5.5);
        #[rustfmt::skip]
        let r = i64x2::new(-1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fsne_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsor_w() {
        #[rustfmt::skip]
        let a = f32x4::new(-5.5, f32::NAN, 5.5, 5.5);
        #[rustfmt::skip]
        let b = f32x4::new(5.5, 3.3, 5.5, 1.1);
        #[rustfmt::skip]
        let r = i32x4::new(-1, 0, -1, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fsor_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsor_d() {
        #[rustfmt::skip]
        let a = f64x2::new(-125.5, 5.5);
        #[rustfmt::skip]
        let b = f64x2::new(125.5, f64::NAN);
        #[rustfmt::skip]
        let r = i64x2::new(-1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fsor_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsqrt_w() {
        #[rustfmt::skip]
        let a = f32x4::new(9.0, 81.0, 1089.0, 10000.0);
        #[rustfmt::skip]
        let r = f32x4::new(3.0, 9.0, 33.0, 100.0);

        assert_eq!(r, mem::transmute(__msa_fsqrt_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsqrt_d() {
        #[rustfmt::skip]
        let a = f64x2::new(81.0, 10000.0);
        #[rustfmt::skip]
        let r = f64x2::new(9.0, 100.0);

        assert_eq!(r, mem::transmute(__msa_fsqrt_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsub_w() {
        #[rustfmt::skip]
        let a = f32x4::new(5.5, 6.5, 7.5, 8.5);
        #[rustfmt::skip]
        let b = f32x4::new(1.25, 1.75, 2.25, 2.75);
        #[rustfmt::skip]
        let r = f32x4::new(4.25, 4.75, 5.25, 5.75);

        assert_eq!(
            r,
            mem::transmute(__msa_fsub_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsub_d() {
        #[rustfmt::skip]
        let a = f64x2::new(555.5, 55.5);
        #[rustfmt::skip]
        let b = f64x2::new(4.25, 3.25);
        #[rustfmt::skip]
        let r = f64x2::new(551.25, 52.25);

        assert_eq!(
            r,
            mem::transmute(__msa_fsub_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsueq_w() {
        #[rustfmt::skip]
        let a = f32x4::new(5.5, f32::NAN, 5.5, 5.5);
        #[rustfmt::skip]
        let b = f32x4::new(5.5, 5.5, -5.5, 5.5);
        #[rustfmt::skip]
        let r = i32x4::new(-1, -1, 0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fsueq_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsueq_d() {
        #[rustfmt::skip]
        let a = f64x2::new(-5.5, 5.5);
        #[rustfmt::skip]
        let b = f64x2::new(5.5, f64::NAN);
        #[rustfmt::skip]
        let r = i64x2::new(0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fsueq_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsule_w() {
        #[rustfmt::skip]
        let a = f32x4::new(5.7, 5.8, 5.9, f32::NAN);
        #[rustfmt::skip]
        let b = f32x4::new(5.6, 5.9, 5.9, f32::NAN);
        #[rustfmt::skip]
        let r = i32x4::new(0, -1, -1, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fsule_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsule_d() {
        #[rustfmt::skip]
        let a = f64x2::new(5.5, 5.5);
        #[rustfmt::skip]
        let b = f64x2::new(5.5, 5.5);
        #[rustfmt::skip]
        let r = i64x2::new(-1, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fsule_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsult_w() {
        #[rustfmt::skip]
        let a = f32x4::new(5.5, 5.5, 5.5, 5.5);
        #[rustfmt::skip]
        let b = f32x4::new(5.6, f32::NAN, 2.2, 1.1);
        #[rustfmt::skip]
        let r = i32x4::new(-1, -1, 0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fsult_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsult_d() {
        #[rustfmt::skip]
        let a = f64x2::new(5.5, f64::NAN);
        #[rustfmt::skip]
        let b = f64x2::new(4.4, 3.3);
        #[rustfmt::skip]
        let r = i64x2::new(0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fsult_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsun_w() {
        #[rustfmt::skip]
        let a = f32x4::new(5.5, 5.5, f32::NAN, 5.5);
        #[rustfmt::skip]
        let b = f32x4::new(4.4, 3.3, 2.2, f32::NAN);
        #[rustfmt::skip]
        let r = i32x4::new(0, 0, -1, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fsun_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsun_d() {
        #[rustfmt::skip]
        let a = f64x2::new(5.5, f64::NAN);
        #[rustfmt::skip]
        let b = f64x2::new(4.4, 3.3);
        #[rustfmt::skip]
        let r = i64x2::new(0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fsun_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsune_w() {
        #[rustfmt::skip]
        let a = f32x4::new(5.5, 5.5, f32::NAN, 5.5);
        #[rustfmt::skip]
        let b = f32x4::new(4.4, 3.3, 2.2, 5.5);
        #[rustfmt::skip]
        let r = i32x4::new(-1, -1, -1, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_fsune_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_fsune_d() {
        #[rustfmt::skip]
        let a = f64x2::new(5.5, f64::NAN);
        #[rustfmt::skip]
        let b = f64x2::new(5.5, 3.3);
        #[rustfmt::skip]
        let r = i64x2::new(0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_fsune_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ftint_s_w() {
        #[rustfmt::skip]
        let a = f32x4::new(-5.5, 75.6, -1000.7, 1219.3);
        #[rustfmt::skip]
        let r = i32x4::new(-6, 76, -1001, 1219);

        assert_eq!(r, mem::transmute(__msa_ftint_s_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ftint_s_d() {
        #[rustfmt::skip]
        let a = f64x2::new(-5.5, 25656.4);
        #[rustfmt::skip]
        let r = i64x2::new(-6, 25656);

        assert_eq!(r, mem::transmute(__msa_ftint_s_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ftint_u_w() {
        #[rustfmt::skip]
        let a = f32x4::new(-5.5, 75.6, -1000.7, 1219.3);
        #[rustfmt::skip]
        let r = u32x4::new(0, 76, 0, 1219);

        assert_eq!(r, mem::transmute(__msa_ftint_u_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ftint_u_d() {
        #[rustfmt::skip]
        let a = f64x2::new(5.5, -25656.4);
        #[rustfmt::skip]
        let r = u64x2::new(6, 0);

        assert_eq!(r, mem::transmute(__msa_ftint_u_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ftq_h() {
        #[rustfmt::skip]
        let a = f32x4::new(0.00001, 0.0002, 0.00001, -0.0002);
        #[rustfmt::skip]
        let b = f32x4::new(0.0001, -0.002, 0.0001, 0.002);
        #[rustfmt::skip]
        let r = i16x8::new(3, -66, 3, 66, 0, 7, 0, -7);

        assert_eq!(
            r,
            mem::transmute(__msa_ftq_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ftq_w() {
        #[rustfmt::skip]
        let a = f64x2::new(0.00001, -0.0002);
        #[rustfmt::skip]
        let b = f64x2::new(0.00000045, 0.000015);
        #[rustfmt::skip]
        let r = i32x4::new(966, 32212, 21475, -429497);

        assert_eq!(
            r,
            mem::transmute(__msa_ftq_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ftrunc_s_w() {
        #[rustfmt::skip]
        let a = f32x4::new(-5.5, 75.6, -1000.7, 1219.3);
        #[rustfmt::skip]
        let r = i32x4::new(-5, 75, -1000, 1219);

        assert_eq!(r, mem::transmute(__msa_ftrunc_s_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ftrunc_s_d() {
        #[rustfmt::skip]
        let a = f64x2::new(-5.5, 25656.4);
        #[rustfmt::skip]
        let r = i64x2::new(-5, 25656);

        assert_eq!(r, mem::transmute(__msa_ftrunc_s_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ftrunc_u_w() {
        #[rustfmt::skip]
        let a = f32x4::new(-5.5, 75.6, -1000.7, 1219.3);
        #[rustfmt::skip]
        let r = u32x4::new(0, 75, 0, 1219);

        assert_eq!(r, mem::transmute(__msa_ftrunc_u_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ftrunc_u_d() {
        #[rustfmt::skip]
        let a = f64x2::new(5.5, -25656.4);
        #[rustfmt::skip]
        let r = u64x2::new(5, 0);

        assert_eq!(r, mem::transmute(__msa_ftrunc_u_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_hadd_s_h() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            -1, -2, -3, -4,
            1, 2, 3, 4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i16x8::new(6, 6, 2, -2, 6, 6, 2, -2);

        assert_eq!(
            r,
            mem::transmute(__msa_hadd_s_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_hadd_s_w() {
        #[rustfmt::skip]
        let a = i16x8::new(
            1, 2, 3, 4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i32x4::new(6, 6, 2, -2);

        assert_eq!(
            r,
            mem::transmute(__msa_hadd_s_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_hadd_s_d() {
        #[rustfmt::skip]
        let a = i32x4::new(1, -2, 3, -4);
        #[rustfmt::skip]
        let b = i32x4::new(4, 3, 2, 1);
        #[rustfmt::skip]
        let r = i64x2::new(2, -2);

        assert_eq!(
            r,
            mem::transmute(__msa_hadd_s_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_hadd_u_h() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = u16x8::new(6, 6, 6, 6, 6, 6, 6, 6);

        assert_eq!(
            r,
            mem::transmute(__msa_hadd_u_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_hadd_u_w() {
        #[rustfmt::skip]
        let a = u16x8::new(
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = u16x8::new(
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = u32x4::new(6, 6, 6, 6);

        assert_eq!(
            r,
            mem::transmute(__msa_hadd_u_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_hadd_u_d() {
        #[rustfmt::skip]
        let a = u32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u32x4::new(4, 3, 2, 1);
        #[rustfmt::skip]
        let r = u64x2::new(6, 6);

        assert_eq!(
            r,
            mem::transmute(__msa_hadd_u_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_hsub_s_h() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            -1, -2, -3, -4,
            1, 2, 3, 4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i16x8::new(-2, 2, -6, -6, -2, 2, -6, -6);

        assert_eq!(
            r,
            mem::transmute(__msa_hsub_s_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_hsub_s_w() {
        #[rustfmt::skip]
        let a = i16x8::new(
            1, 2, 3, 4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i32x4::new(-2, 2, -6, -6);

        assert_eq!(
            r,
            mem::transmute(__msa_hsub_s_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_hsub_s_d() {
        #[rustfmt::skip]
        let a = i32x4::new(1, -2, 3, -4);
        #[rustfmt::skip]
        let b = i32x4::new(4, 3, 2, 1);
        #[rustfmt::skip]
        let r = i64x2::new(-6, -6);

        assert_eq!(
            r,
            mem::transmute(__msa_hsub_s_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_hsub_u_h() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i16x8::new(-2, 2, -2, 2, -2, 2, -2, 2);

        assert_eq!(
            r,
            mem::transmute(__msa_hsub_u_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_hsub_u_w() {
        #[rustfmt::skip]
        let a = u16x8::new(
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = u16x8::new(
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i32x4::new(-2, 2, -2, 2);

        assert_eq!(
            r,
            mem::transmute(__msa_hsub_u_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_hsub_u_d() {
        #[rustfmt::skip]
        let a = u32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u32x4::new(4, 3, 2, 1);
        #[rustfmt::skip]
        let r = i64x2::new(-2, 2);

        assert_eq!(
            r,
            mem::transmute(__msa_hsub_u_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvev_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            4, 1, 2, 3,
            4, 1, 2, 3,
            4, 1, 2, 3,
            4, 1, 2, 3
        );

        assert_eq!(
            r,
            mem::transmute(__msa_ilvev_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvev_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i16x8::new(4, 1, 2, 3, 4, 1, 2, 3);

        assert_eq!(
            r,
            mem::transmute(__msa_ilvev_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvev_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i32x4::new(4, 3, 2, 1);
        #[rustfmt::skip]
        let r = i32x4::new(4, 1, 2, 3);

        assert_eq!(
            r,
            mem::transmute(__msa_ilvev_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvev_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(4, 3);
        #[rustfmt::skip]
        let r = i64x2::new(4, 1);

        assert_eq!(
            r,
            mem::transmute(__msa_ilvev_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvl_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            9, 10, 11, 12,
            13, 14, 15, 16
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            16, 15, 14, 13,
            12, 11, 10, 9,
            8, 7, 6, 5,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            8, 9, 7, 10,
            6, 11, 5, 12,
            4, 13, 3, 14,
            2, 15, 1, 16
        );

        assert_eq!(
            r,
            mem::transmute(__msa_ilvl_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvl_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            1, 2, 3, 4,
            5, 6, 7, 8
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            8, 7, 6, 5,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i16x8::new(4, 5, 3, 6, 2, 7, 1, 8);

        assert_eq!(
            r,
            mem::transmute(__msa_ilvl_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvl_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i32x4::new(4, 3, 2, 1);
        #[rustfmt::skip]
        let r = i32x4::new(2, 3, 1, 4);

        assert_eq!(
            r,
            mem::transmute(__msa_ilvl_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvl_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(2, 1);
        #[rustfmt::skip]
        let r = i64x2::new(1, 2);

        assert_eq!(
            r,
            mem::transmute(__msa_ilvl_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvod_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            9, 10, 11, 12,
            13, 14, 15, 16
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            16, 15, 14, 13,
            12, 11, 10, 9,
            8, 7, 6, 5,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            15, 2, 13, 4,
            11, 6, 9, 8,
            7, 10, 5, 12,
            3, 14, 1, 16
        );

        assert_eq!(
            r,
            mem::transmute(__msa_ilvod_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvod_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            1, 2, 3, 4,
            5, 6, 7, 8
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            8, 7, 6, 5,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i16x8::new(7, 2, 5, 4, 3, 6, 1, 8);

        assert_eq!(
            r,
            mem::transmute(__msa_ilvod_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvod_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i32x4::new(4, 3, 2, 1);
        #[rustfmt::skip]
        let r = i32x4::new(3, 2, 1, 4);

        assert_eq!(
            r,
            mem::transmute(__msa_ilvod_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvod_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(2, 1);
        #[rustfmt::skip]
        let r = i64x2::new(1, 2);

        assert_eq!(
            r,
            mem::transmute(__msa_ilvod_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvr_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            9, 10, 11, 12,
            13, 14, 15, 16
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            16, 15, 14, 13,
            12, 11, 10, 9,
            8, 7, 6, 5,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            16, 1, 15, 2,
            14, 3, 13, 4,
            12, 5, 11, 6,
            10, 7, 9, 8
        );

        assert_eq!(
            r,
            mem::transmute(__msa_ilvr_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvr_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            8, 7, 6, 5,
            4, 3, 2, 1,
        );
        #[rustfmt::skip]
        let r = i16x8::new(8, 1, 7, 2, 6, 3, 5, 4);

        assert_eq!(
            r,
            mem::transmute(__msa_ilvr_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvr_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i32x4::new(4, 3, 2, 1);
        #[rustfmt::skip]
        let r = i32x4::new(4, 1, 3, 2);

        assert_eq!(
            r,
            mem::transmute(__msa_ilvr_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ilvr_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(2, 1);
        #[rustfmt::skip]
        let r = i64x2::new(2, 1);

        assert_eq!(
            r,
            mem::transmute(__msa_ilvr_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_insert_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -100, 127, 4, 127,
            -100, 127, 4, 127,
            -100, 127, 4, 127,
            -100, 127, 4, 127
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -100, 127, 4, 127,
            -100, 127, 4, 127,
            -100, 127, 4, 127,
            5, 127, 4, 127
        );

        assert_eq!(r, mem::transmute(__msa_insert_b(mem::transmute(a), 12, 5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_insert_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            32767, 3276, 100, 11,
            32767, 3276, 100, 11
        );
        #[rustfmt::skip]
        let r = i16x8::new(
            32767, 3276, 100, 11,
            5, 3276, 100, 11
        );

        assert_eq!(r, mem::transmute(__msa_insert_h(mem::transmute(a), 4, 5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_insert_w() {
        #[rustfmt::skip]
        let a = i32x4::new(100, 2147483647, 5, -2147483647);
        #[rustfmt::skip]
        let r = i32x4::new(100, 7, 5, -2147483647);

        assert_eq!(r, mem::transmute(__msa_insert_w(mem::transmute(a), 1, 7)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_insert_d() {
        #[rustfmt::skip]
        let a = i64x2::new(3, i64::MAX);
        #[rustfmt::skip]
        let r = i64x2::new(3, 100);

        assert_eq!(r, mem::transmute(__msa_insert_d(mem::transmute(a), 1, 100)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_insve_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -100, i8::MAX, 4, i8::MAX,
            -100, i8::MAX, 4, i8::MAX,
            -100, i8::MAX, 4, i8::MAX,
            -100, i8::MAX, 4, i8::MAX
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            5, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -100, 127, 4, 127,
            -100, 127, 4, 127,
            -100, 127, 4, 127,
            5, 127, 4, 127
        );

        assert_eq!(
            r,
            mem::transmute(__msa_insve_b(mem::transmute(a), 12, mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_insve_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            i16::MAX, 3276, 100, 11,
            i16::MAX, 3276, 100, 11
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let r = i16x8::new(
            32767, 3276, 100, 11,
            1, 3276, 100, 11
        );

        assert_eq!(
            r,
            mem::transmute(__msa_insve_h(mem::transmute(a), 4, mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_insve_w() {
        #[rustfmt::skip]
        let a = i32x4::new(100, 2147483647, 5, -2147483647);
        #[rustfmt::skip]
        let b = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let r = i32x4::new(100, 2147483647, 5, 1);

        assert_eq!(
            r,
            mem::transmute(__msa_insve_w(mem::transmute(a), 3, mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_insve_d() {
        #[rustfmt::skip]
        let a = i64x2::new(3, i64::MAX);
        #[rustfmt::skip]
        let b = i64x2::new(1, 2);
        #[rustfmt::skip]
        let r = i64x2::new(3, 1);

        assert_eq!(
            r,
            mem::transmute(__msa_insve_d(mem::transmute(a), 1, mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ld_b() {
        #[rustfmt::skip]
        let mut a : [i8; 32] = [
            0, 1, 2, 3, 4, 5, 6, 7,
            8, 9, 10, 11, 12, 13, 14, 15,
            16, 17, 18, 19, 20, 21, 22, 23,
            24, 25, 26, 27, 28, 29, 30, 31
        ];
        let p = &mut a[4] as *mut _ as *mut u8;
        #[rustfmt::skip]
        let r = i8x16::new(
            13, 14, 15, 16, 
            17, 18, 19, 20, 
            21, 22, 23, 24, 
            25, 26, 27, 28
        );

        assert_eq!(r, mem::transmute(__msa_ld_b(p, 9)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ld_h() {
        #[rustfmt::skip]
        let mut a : [i16; 16] = [
            0, 1, 2, 3, 4, 5, 6, 7, 
            8, 9, 10, 11, 12, 13, 14, 15
        ];
        let p = &mut a[4] as *mut _ as *mut u8;
        #[rustfmt::skip]
        let r = i16x8::new(3, 4, 5, 6, 7, 8, 9, 10);

        assert_eq!(r, mem::transmute(__msa_ld_h(p, -2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ld_w() {
        #[rustfmt::skip]
        let mut a : [i32; 8] = [0, 1, 2, 3, 4, 5, 6, 7];
        let p = &mut a[3] as *mut _ as *mut u8;
        #[rustfmt::skip]
        let r = i32x4::new(2, 3, 4, 5);

        assert_eq!(r, mem::transmute(__msa_ld_w(p, -4)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ld_d() {
        #[rustfmt::skip]
        let mut a : [i64; 8] = [0, 1, 2, 3, 4, 5, 6, 7];
        let p = &mut a[4] as *mut _ as *mut u8;
        #[rustfmt::skip]
        let r = i64x2::new(0, 1);

        assert_eq!(r, mem::transmute(__msa_ld_d(p, -32)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ldi_b() {
        #[rustfmt::skip]
        let r = i8x16::new(
            -20, -20, -20, -20,
            -20, -20, -20, -20,
            -20, -20, -20, -20,
            -20, -20, -20, -20
        );

        assert_eq!(r, mem::transmute(__msa_ldi_b(-20)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ldi_h() {
        #[rustfmt::skip]
        let r = i16x8::new(
            255, 255, 255, 255,
            255, 255, 255, 255
        );

        assert_eq!(r, mem::transmute(__msa_ldi_h(255)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ldi_w() {
        #[rustfmt::skip]
        let r = i32x4::new(-509, -509, -509, -509);

        assert_eq!(r, mem::transmute(__msa_ldi_w(-509)));
    }

    // FIXME: https://reviews.llvm.org/D59884
    // 如果目标类型为 i64，则负 immediate 将丢失符号如果使用 4294967185 而不是 vector `r` 中的 -111，则测试通过
    //
    // #[simd_test(enable = "msa")]
    // unsafe fn test_msa_ldi_d() {
    //     let r = i64x2::new(-111, -111);

    // assert_eq!(r, mem::transmute(__msa_ldi_d(-111)));
    // }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_madd_q_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            i16::MAX, 1024, i16::MIN, -1024,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            1024, 1024, 1024, 1024,
            1024, 1024, 1024, 1024
        );
        #[rustfmt::skip]
        let c = i16x8::new(
            i16::MAX, i16::MAX, 1, -1,
            33, 66, 99, 132
        );
        #[rustfmt::skip]
        let r = i16x8::new(32767, 2047, -32768, -1025, 2, 4, 6, 8);

        assert_eq!(
            r,
            mem::transmute(__msa_madd_q_h(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_madd_q_w() {
        #[rustfmt::skip]
        let a = i32x4::new(i32::MAX, i32::MIN, 1, 2);
        #[rustfmt::skip]
        let b = i32x4::new(102401, 102401, 102401, 102401);
        #[rustfmt::skip]
        let c = i32x4::new(10240, 20480, 30720, 40960);
        #[rustfmt::skip]
        let r = i32x4::new(2147483647, -2147483648, 2, 3);

        assert_eq!(
            r,
            mem::transmute(__msa_madd_q_w(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_maddr_q_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            32767, 1024, -32768, -1024,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            1024, 1024, 1024, 1024,
            1024, 1024, 1024, 1024
        );
        #[rustfmt::skip]
        let c = i16x8::new(
            32767, 32767, 32767, 32767,
            33, 66, 99, 132
        );
        #[rustfmt::skip]
        let r = i16x8::new(32767, 2048, -31744, 0, 2, 4, 6, 8);

        assert_eq!(
            r,
            mem::transmute(__msa_maddr_q_h(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_maddr_q_w() {
        #[rustfmt::skip]
        let a = i32x4::new(i32::MAX, i32::MIN, 1, 2);
        #[rustfmt::skip]
        let b = i32x4::new(102401, 102401, 102401, 102401);
        #[rustfmt::skip]
        let c = i32x4::new(10240, 20480, 30720, 40960);
        #[rustfmt::skip]
        let r = i32x4::new(2147483647, -2147483647, 2, 4);

        assert_eq!(
            r,
            mem::transmute(__msa_maddr_q_w(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_maddv_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            5, 6, 7, 8,
            5, 6, 7, 8,
            5, 6, 7, 8,
            5, 6, 7, 8
        );
        #[rustfmt::skip]
        let c = i8x16::new(
            9, 10, 11, 12,
            9, 10, 11, 12,
            9, 10, 11, 12,
            9, 10, 11, 12
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            46, 62, 80, 100,
            46, 62, 80, 100,
            46, 62, 80, 100,
            46, 62, 80, 100
        );

        assert_eq!(
            r,
            mem::transmute(__msa_maddv_b(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_maddv_h() {
        #[rustfmt::skip]
        let a = i16x8::new(1, 2, 3, 4, 1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i16x8::new(5, 6, 7, 8, 5, 6, 7, 8);
        #[rustfmt::skip]
        let c = i16x8::new(9, 10, 11, 12, 9, 10, 11, 12);
        #[rustfmt::skip]
        let r = i16x8::new(46, 62, 80, 100, 46, 62, 80, 100);

        assert_eq!(
            r,
            mem::transmute(__msa_maddv_h(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_maddv_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 1, 2);
        #[rustfmt::skip]
        let b = i32x4::new(3, 4, 3, 4);
        #[rustfmt::skip]
        let c = i32x4::new(5, 6, 5, 6);
        #[rustfmt::skip]
        let r = i32x4::new(16, 26, 16, 26);

        assert_eq!(
            r,
            mem::transmute(__msa_maddv_w(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_maddv_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(3, 4);
        #[rustfmt::skip]
        let c = i64x2::new(5, 6);
        #[rustfmt::skip]
        let r = i64x2::new(16, 26);

        assert_eq!(
            r,
            mem::transmute(__msa_maddv_d(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_max_a_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            -1, -2, -3, -4,
            1, 2, 3, 4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -6, -7, -8, -9,
            6, 7, 8, 9,
            -6, -7, -8, -9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -6, -7, -8, -9,
            6, 7, 8, 9,
            -6, -7, -8, -9,
            6, 7, 8, 9
        );

        assert_eq!(
            r,
            mem::transmute(__msa_max_a_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_max_a_h() {
        #[rustfmt::skip]
        let a = i16x8::new(1, -2, 3, -4, 1, -2, 3, -4);
        #[rustfmt::skip]
        let b = i16x8::new(-6, 7, -8, 9, -6, 7, -8, 9);
        #[rustfmt::skip]
        let r = i16x8::new(-6, 7, -8, 9, -6, 7, -8, 9);

        assert_eq!(
            r,
            mem::transmute(__msa_max_a_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_max_a_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, -2, 3, -4);
        #[rustfmt::skip]
        let b = i32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = i32x4::new(6, 7, 8, 9);

        assert_eq!(
            r,
            mem::transmute(__msa_max_a_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_max_a_d() {
        #[rustfmt::skip]
        let a = i64x2::new(-1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(6, -7);
        #[rustfmt::skip]
        let r = i64x2::new(6, -7);

        assert_eq!(
            r,
            mem::transmute(__msa_max_a_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_max_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            -1, -2, -3, -4,
            1, 2, 3, 4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -6, -7, -8, -9,
            6, 7, 8, 9,
            -6, -7, -8, -9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            1, 2, 3, 4,
            6, 7, 8, 9,
            1, 2, 3, 4,
            6, 7, 8, 9
        );

        assert_eq!(
            r,
            mem::transmute(__msa_max_s_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_max_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(1, -2, 3, -4, 1, -2, 3, -4);
        #[rustfmt::skip]
        let b = i16x8::new(-6, 7, -8, 9, -6, 7, -8, 9);
        #[rustfmt::skip]
        let r = i16x8::new(1, 7, 3, 9, 1, 7, 3, 9);

        assert_eq!(
            r,
            mem::transmute(__msa_max_s_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_max_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, -2, 3, -4);
        #[rustfmt::skip]
        let b = i32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = i32x4::new(6, 7, 8, 9);

        assert_eq!(
            r,
            mem::transmute(__msa_max_s_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_max_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(-1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(6, -7);
        #[rustfmt::skip]
        let r = i64x2::new(6, 2);

        assert_eq!(
            r,
            mem::transmute(__msa_max_s_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_max_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );

        assert_eq!(
            r,
            mem::transmute(__msa_max_u_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_max_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(1, 2, 3, 4, 1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u16x8::new(6, 7, 8, 9, 6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u16x8::new(6, 7, 8, 9, 6, 7, 8, 9);

        assert_eq!(
            r,
            mem::transmute(__msa_max_u_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_max_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u32x4::new(6, 7, 8, 9);

        assert_eq!(
            r,
            mem::transmute(__msa_max_u_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_max_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(1, 2);
        #[rustfmt::skip]
        let b = u64x2::new(6, 7);
        #[rustfmt::skip]
        let r = u64x2::new(6, 7);

        assert_eq!(
            r,
            mem::transmute(__msa_max_u_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_maxi_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, -20, -6, 8,
            1, -20, -6, 8,
            1, -20, -6, 8,
            1, -20, -6, 8
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            1, -16, -6, 8,
            1, -16, -6, 8,
            1, -16, -6, 8,
            1, -16, -6, 8
        );

        assert_eq!(r, mem::transmute(__msa_maxi_s_b(mem::transmute(a), -16)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_maxi_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(1, 3, -60, -8, 1, 3, -6, -8);
        #[rustfmt::skip]
        let r = i16x8::new(15, 15, 15, 15, 15, 15, 15, 15);

        assert_eq!(r, mem::transmute(__msa_maxi_s_h(mem::transmute(a), 15)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_maxi_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 3, -6, -8);
        #[rustfmt::skip]
        let r = i32x4::new(1, 3, -5, -5);

        assert_eq!(r, mem::transmute(__msa_maxi_s_w(mem::transmute(a), -5)));
    }

    // FIXME: https://reviews.llvm.org/D59884
    // 如果目标类型为 i64，则负 immediate 将丢失符号如果使用 4294967293 而不是 vector `r` 中的 -3，则测试通过
    //
    // #[simd_test(enable = "msa")]
    // unsafe fn test_msa_maxi_s_d() {
    //     #[rustfmt::skip]
    //     let a = i64x2::new(1, -8);
    //     #[rustfmt::skip]
    //     let r = i64x2::new(-3, -3);

    // assert_eq!(r, mem::transmute(__msa_maxi_s_d(mem::transmute(a), -3)));
    // }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_maxi_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 3, 6, 8,
            1, 3, 6, 8,
            1, 3, 6, 8,
            1, 3, 6, 8
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            5, 5, 6, 8,
            5, 5, 6, 8,
            5, 5, 6, 8,
            5, 5, 6, 8
        );

        assert_eq!(r, mem::transmute(__msa_maxi_u_b(mem::transmute(a), 5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_maxi_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(1, 3, 6, 8, 1, 3, 6, 8);
        #[rustfmt::skip]
        let r = u16x8::new(5, 5, 6, 8, 5, 5, 6, 8);

        assert_eq!(r, mem::transmute(__msa_maxi_u_h(mem::transmute(a), 5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_maxi_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(1, 3, 6, 8);
        #[rustfmt::skip]
        let r = u32x4::new(5, 5, 6, 8);

        assert_eq!(r, mem::transmute(__msa_maxi_u_w(mem::transmute(a), 5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_maxi_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(1, 8);
        #[rustfmt::skip]
        let r = u64x2::new(5, 8);

        assert_eq!(r, mem::transmute(__msa_maxi_u_d(mem::transmute(a), 5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_min_a_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            -1, -2, -3, -4,
            1, 2, 3, 4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -6, -7, -8, -9,
            6, 7, 8, 9,
            -6, -7, -8, -9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            1, 2, 3, 4,
            -1, -2, -3, -4,
            1, 2, 3, 4,
            -1, -2, -3, -4
        );

        assert_eq!(
            r,
            mem::transmute(__msa_min_a_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_min_a_h() {
        #[rustfmt::skip]
        let a = i16x8::new(1, -2, 3, -4, 1, -2, 3, -4);
        #[rustfmt::skip]
        let b = i16x8::new(-6, 7, -8, 9, -6, 7, -8, 9);
        #[rustfmt::skip]
        let r = i16x8::new(1, -2, 3, -4, 1, -2, 3, -4);

        assert_eq!(
            r,
            mem::transmute(__msa_min_a_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_min_a_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, -2, 3, -4);
        #[rustfmt::skip]
        let b = i32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = i32x4::new(1, -2, 3, -4);

        assert_eq!(
            r,
            mem::transmute(__msa_min_a_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_min_a_d() {
        #[rustfmt::skip]
        let a = i64x2::new(-1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(6, -7);
        #[rustfmt::skip]
        let r = i64x2::new(-1, 2);

        assert_eq!(
            r,
            mem::transmute(__msa_min_a_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_min_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            -1, -2, -3, -4,
            1, 2, 3, 4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -6, -7, -8, -9,
            6, 7, 8, 9,
            -6, -7, -8, -9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -6, -7, -8, -9,
            -1, -2, -3, -4,
            -6, -7, -8, -9,
            -1, -2, -3, -4
        );

        assert_eq!(
            r,
            mem::transmute(__msa_min_s_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_min_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(1, -2, 3, -4, 1, -2, 3, -4);
        #[rustfmt::skip]
        let b = i16x8::new(-6, 7, -8, 9, -6, 7, -8, 9);
        #[rustfmt::skip]
        let r = i16x8::new(-6, -2, -8, -4, -6, -2, -8, -4);

        assert_eq!(
            r,
            mem::transmute(__msa_min_s_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_min_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, -2, 3, -4);
        #[rustfmt::skip]
        let b = i32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = i32x4::new(1, -2, 3, -4);

        assert_eq!(
            r,
            mem::transmute(__msa_min_s_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_min_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(-1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(6, -7);
        #[rustfmt::skip]
        let r = i64x2::new(-1, -7);

        assert_eq!(
            r,
            mem::transmute(__msa_min_s_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mini_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            -1, -2, -3, -4,
            1, 2, 3, 4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -10, -10, -10, -10,
            -10, -10, -10, -10,
            -10, -10, -10, -10,
            -10, -10, -10, -10
        );

        assert_eq!(r, mem::transmute(__msa_mini_s_b(mem::transmute(a), -10)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mini_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(1, -2, 3, -4, 1, -2, 3, -4);
        #[rustfmt::skip]
        let r = i16x8::new(-3, -3, -3, -4, -3, -3, -3, -4);

        assert_eq!(r, mem::transmute(__msa_mini_s_h(mem::transmute(a), -3)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mini_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, -2, 3, -4);
        #[rustfmt::skip]
        let r = i32x4::new(-3, -3, -3, -4);

        assert_eq!(r, mem::transmute(__msa_mini_s_w(mem::transmute(a), -3)));
    }

    // FIXME: https://reviews.llvm.org/D59884
    // 如果目标类型为 i64，则负 immediate 将丢失符号
    // -3 表示为 4294967293
    // #[simd_test(enable = "msa")]
    // unsafe fn test_msa_mini_s_d() {
    //     #[rustfmt::skip]
    //     let a = i64x2::new(-3, 2);
    //     #[rustfmt::skip]
    //     let r = i64x2::new(-1, -3);

    // assert_eq!(r, mem::transmute(__msa_mini_s_d(mem::transmute(a), -3)));
    // }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_min_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );

        assert_eq!(
            r,
            mem::transmute(__msa_min_u_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_min_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(1, 2, 3, 4, 1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u16x8::new(6, 7, 8, 9, 6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u16x8::new(1, 2, 3, 4, 1, 2, 3, 4,);

        assert_eq!(
            r,
            mem::transmute(__msa_min_u_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_min_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = u32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u32x4::new(1, 2, 3, 4,);

        assert_eq!(
            r,
            mem::transmute(__msa_min_u_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_min_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(1, 2);
        #[rustfmt::skip]
        let b = u64x2::new(6, 7);
        #[rustfmt::skip]
        let r = u64x2::new(1, 2,);

        assert_eq!(
            r,
            mem::transmute(__msa_min_u_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mini_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 3, 6, 8,
            1, 3, 6, 8,
            1, 3, 6, 8,
            1, 3, 6, 8
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            1, 3, 5, 5,
            1, 3, 5, 5,
            1, 3, 5, 5,
            1, 3, 5, 5
        );

        assert_eq!(r, mem::transmute(__msa_mini_u_b(mem::transmute(a), 5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mini_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(1, 3, 6, 8, 1, 3, 6, 8);
        #[rustfmt::skip]
        let r = u16x8::new(1, 3, 5, 5, 1, 3, 5, 5);

        assert_eq!(r, mem::transmute(__msa_mini_u_h(mem::transmute(a), 5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mini_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(1, 3, 6, 8);
        #[rustfmt::skip]
        let r = u32x4::new(1, 3, 5, 5);

        assert_eq!(r, mem::transmute(__msa_mini_u_w(mem::transmute(a), 5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mini_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(1, 8);
        #[rustfmt::skip]
        let r = u64x2::new(1, 5);

        assert_eq!(r, mem::transmute(__msa_mini_u_d(mem::transmute(a), 5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mod_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -6, -7, -8, -9,
            6, 7, 8, 9,
            -6, -7, -8, -9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            1, 2, 3, 4,
            -1, -2, -3, -4,
            1, 2, 3, 4,
            -1, -2, -3, -4
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            0, -1, -2, -1,
            0, 1, 2, 1,
            0, -1, -2, -1,
            0, 1, 2, 1
        );

        assert_eq!(
            r,
            mem::transmute(__msa_mod_s_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mod_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(-6, 7, -8, 9, -6, 7, -8, 9);
        #[rustfmt::skip]
        let b = i16x8::new(1, -2, 3, -4, 1, -2, 3, -4);
        #[rustfmt::skip]
        let r = i16x8::new(0, 1, -2, 1, 0, 1, -2, 1);

        assert_eq!(
            r,
            mem::transmute(__msa_mod_s_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mod_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let b = i32x4::new(1, -2, 3, -4);
        #[rustfmt::skip]
        let r = i32x4::new(0, 1, 2, 1);

        assert_eq!(
            r,
            mem::transmute(__msa_mod_s_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mod_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(6, -7);
        #[rustfmt::skip]
        let b = i64x2::new(-1, 2);
        #[rustfmt::skip]
        let r = i64x2::new(0, -1);

        assert_eq!(
            r,
            mem::transmute(__msa_mod_s_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mod_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            0, 1, 2, 1,
            0, 1, 2, 1,
            0, 1, 2, 1,
            0, 1, 2, 1
        );

        assert_eq!(
            r,
            mem::transmute(__msa_mod_u_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mod_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(6, 7, 8, 9, 6, 7, 8, 9);
        #[rustfmt::skip]
        let b = u16x8::new(1, 2, 3, 4, 1, 2, 3, 4);
        #[rustfmt::skip]
        let r = u16x8::new(0, 1, 2, 1, 0, 1, 2, 1);

        assert_eq!(
            r,
            mem::transmute(__msa_mod_u_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mod_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let b = u32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let r = u32x4::new(0, 1, 2, 1);

        assert_eq!(
            r,
            mem::transmute(__msa_mod_u_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mod_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(6, 7);
        #[rustfmt::skip]
        let b = u64x2::new(1, 2);
        #[rustfmt::skip]
        let r = u64x2::new(0, 1);

        assert_eq!(
            r,
            mem::transmute(__msa_mod_u_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_move_v() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            1, 2, 3, 4,
            5, 6, 7, 8
            );
        #[rustfmt::skip]
        let r = i8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            1, 2, 3, 4,
            5, 6, 7, 8
            );

        assert_eq!(r, mem::transmute(__msa_move_v(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_msub_q_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            1024, -1024, 1024, -1024,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            1025, 1025, 1025, 1025,
            1025, 1025, 1025, 1025
        );
        #[rustfmt::skip]
        let c = i16x8::new(
            1024, 2048, 3072, 4096,
            1024, 2048, 3072, 4096
        );
        #[rustfmt::skip]
        let r = i16x8::new(991, -1089, 927, -1153, -32, -63, -94, -125);

        assert_eq!(
            r,
            mem::transmute(__msa_msub_q_h(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_msub_q_w() {
        #[rustfmt::skip]
        let a = i32x4::new(2147483647, -2147483647, 1, 2);
        #[rustfmt::skip]
        let b = i32x4::new(10240, 10240, 10240, 10240);
        #[rustfmt::skip]
        let c = i32x4::new(10240, 20480, 30720, 40960);
        #[rustfmt::skip]
        let r = i32x4::new(2147483646, -2147483648, 0, 1);

        assert_eq!(
            r,
            mem::transmute(__msa_msub_q_w(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_msubr_q_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            1024, -1024, 1024, -1024,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            1025, 1025, 1025, 1025, 
            1025, 1025, 1025, 1025
        );
        #[rustfmt::skip]
        let c = i16x8::new(
            1024, 2048, 3072, 4096,
            1024, 2048, 3072, 4096
        );
        #[rustfmt::skip]
        let r = i16x8::new(992, -1088, 928, -1152, -31, -62, -93, -124);

        assert_eq!(
            r,
            mem::transmute(__msa_msubr_q_h(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_msubr_q_w() {
        #[rustfmt::skip]
        let a = i32x4::new(i32::MAX, -2147483647, 1, 2);
        #[rustfmt::skip]
        let b = i32x4::new(10240, 10240, 10240, 10240);
        #[rustfmt::skip]
        let c = i32x4::new(10240, 20480, 30720, 40960);
        #[rustfmt::skip]
        let r = i32x4::new(2147483647, -2147483647, 1, 2);

        assert_eq!(
            r,
            mem::transmute(__msa_msubr_q_w(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_msubv_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            5, 6, 7, 8,
            5, 6, 7, 8,
            5, 6, 7, 8,
            5, 6, 7, 8
        );
        #[rustfmt::skip]
        let c = i8x16::new(
            9, 10, 11, 12,
            9, 10, 11, 12,
            9, 10, 11, 12,
            9, 10, 11, 12
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -44, -58, -74, -92,
            -44, -58, -74, -92,
            -44, -58, -74, -92,
            -44, -58, -74, -92
        );

        assert_eq!(
            r,
            mem::transmute(__msa_msubv_b(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_msubv_h() {
        #[rustfmt::skip]
        let a = i16x8::new(1, 2, 3, 4, 1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i16x8::new(5, 6, 7, 8, 5, 6, 7, 8);
        #[rustfmt::skip]
        let c = i16x8::new(9, 10, 11, 12, 9, 10, 11, 12);
        #[rustfmt::skip]
        let r = i16x8::new(-44, -58, -74, -92, -44, -58, -74, -92);

        assert_eq!(
            r,
            mem::transmute(__msa_msubv_h(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_msubv_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 1, 2);
        #[rustfmt::skip]
        let b = i32x4::new(3, 4, 3, 4);
        #[rustfmt::skip]
        let c = i32x4::new(5, 6, 5, 6);
        #[rustfmt::skip]
        let r = i32x4::new(-14, -22, -14, -22);

        assert_eq!(
            r,
            mem::transmute(__msa_msubv_w(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_msubv_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(3, 4);
        #[rustfmt::skip]
        let c = i64x2::new(5, 6);
        #[rustfmt::skip]
        let r = i64x2::new(-14, -22);

        assert_eq!(
            r,
            mem::transmute(__msa_msubv_d(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mul_q_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            12500, -20, -300, 400,
            12500, 20, 300, 400
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            1250, 10240, -7585, 8456,
            1250, 10240, -7585, 8456
        );
        #[rustfmt::skip]
        let r = i16x8::new(476, -7, 69, 103, 476, 6, -70, 103);

        assert_eq!(
            r,
            mem::transmute(__msa_mul_q_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mul_q_w() {
        #[rustfmt::skip]
        let a = i32x4::new(
            i32::MAX, i32::MAX,
            i32::MIN, i32::MIN
        );
        #[rustfmt::skip]
        let b = i32x4::new(30, 60, 30, 60);
        #[rustfmt::skip]
        let r = i32x4::new(29, 59, -30, -60);

        assert_eq!(
            r,
            mem::transmute(__msa_mul_q_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mulr_q_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            12500, -20, -300, 400,
            12500, 20, 300, 400
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            1250, 10240, -7585, 8456,
            1250, 10240, -7585, 8456
        );
        #[rustfmt::skip]
        let r = i16x8::new(477, -6, 69, 103, 477, 6, -69, 103);

        assert_eq!(
            r,
            mem::transmute(__msa_mulr_q_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mulr_q_w() {
        #[rustfmt::skip]
        let a = i32x4::new(
            i32::MAX, i32::MAX,
            i32::MIN, i32::MIN
        );
        #[rustfmt::skip]
        let b = i32x4::new(30, 60, 30, 60);
        #[rustfmt::skip]
        let r = i32x4::new(30, 60, -30, -60);

        assert_eq!(
            r,
            mem::transmute(__msa_mulr_q_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mulv_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            9, 10, 11, 12,
            13, 14, 15, 16
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            16, 15, 14, 13,
            12, 11, 10, 9,
            8, 7, 6, 5,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            16, 30, 42, 52,
            60, 66, 70, 72,
            72, 70, 66, 60,
            52, 42, 30, 16
        );

        assert_eq!(
            r,
            mem::transmute(__msa_mulv_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mulv_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            1, 2, 3, 4,
            5, 6, 7, 8
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            8, 7, 6, 5,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i16x8::new(8, 14, 18, 20, 20, 18, 14, 8);

        assert_eq!(
            r,
            mem::transmute(__msa_mulv_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mulv_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i32x4::new(4, 3, 2, 1);
        #[rustfmt::skip]
        let r = i32x4::new(4, 6, 6, 4);

        assert_eq!(
            r,
            mem::transmute(__msa_mulv_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_mulv_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(2, 1);
        #[rustfmt::skip]
        let r = i64x2::new(2, 2);

        assert_eq!(
            r,
            mem::transmute(__msa_mulv_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_nloc_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -128, -64, -32, -16,
            -8, -4, -2, -1,
            1, 2, 4, 8,
            16, 32, 64, 127
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            0, 0, 0, 0,
            0, 0, 0, 0
        );

        assert_eq!(r, mem::transmute(__msa_nloc_b(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_nloc_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            -32768, -16384, -8192, -4096,
            4096, 8192, 16384, 32767
        );
        #[rustfmt::skip]
        let r = i16x8::new(1, 2, 3, 4, 0, 0, 0, 0);

        assert_eq!(r, mem::transmute(__msa_nloc_h(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_nloc_w() {
        #[rustfmt::skip]
        let a = i32x4::new(
            i32::MIN, -1073741824,
            1073741824, i32::MAX
        );
        #[rustfmt::skip]
        let r = i32x4::new(1, 2, 0, 0);

        assert_eq!(r, mem::transmute(__msa_nloc_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_nloc_d() {
        #[rustfmt::skip]
        let a = i64x2::new(i64::MIN, i64::MAX);
        #[rustfmt::skip]
        let r = i64x2::new(1, 0);

        assert_eq!(r, mem::transmute(__msa_nloc_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_nlzc_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            9, 10, 11, 12,
            13, 14, 15, 16
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            7, 6, 6, 5,
            5, 5, 5, 4,
            4, 4, 4, 4,
            4, 4, 4, 3
        );

        assert_eq!(r, mem::transmute(__msa_nlzc_b(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_nlzc_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            1, 2, 3, 4,
            5, 6, 7, 8
        );
        #[rustfmt::skip]
        let r = i16x8::new(15, 14, 14, 13, 13, 13, 13, 12);

        assert_eq!(r, mem::transmute(__msa_nlzc_h(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_nlzc_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let r = i32x4::new(31, 30, 30, 29);

        assert_eq!(r, mem::transmute(__msa_nlzc_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_nlzc_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let r = i64x2::new(63, 62);

        assert_eq!(r, mem::transmute(__msa_nlzc_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_nor_v() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            9, 10, 11, 12,
            13, 14, 15, 16
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            9, 10, 11, 12,
            13, 14, 15, 16
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            254, 253, 252, 251,
            250, 249, 248, 247,
            246, 245, 244, 243,
            242, 241, 240, 239
        );

        assert_eq!(
            r,
            mem::transmute(__msa_nor_v(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_nori_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            9, 10, 11, 12,
            13, 14, 15, 16
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            250, 249, 248, 251,
            250, 249, 248, 243,
            242, 241, 240, 243,
            242, 241, 240, 235
        );

        assert_eq!(r, mem::transmute(__msa_nori_b(mem::transmute(a), 4)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_or_v() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            9, 10, 11, 12,
            13, 14, 15, 16
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            9, 10, 11, 12,
            13, 14, 15, 16
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            9, 10, 11, 12,
            13, 14, 15, 16
        );

        assert_eq!(
            r,
            mem::transmute(__msa_or_v(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_ori_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            9, 10, 11, 12,
            13, 14, 15, 16
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            5, 6, 7, 4,
            5, 6, 7, 12,
            13, 14, 15, 12,
            13, 14, 15, 20
        );

        assert_eq!(r, mem::transmute(__msa_ori_b(mem::transmute(a), 4)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_pckev_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            4, 2, 4, 2,
            4, 2, 4, 2,
            1, 3, 1, 3,
            1, 3, 1, 3
        );

        assert_eq!(
            r,
            mem::transmute(__msa_pckev_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_pckev_h() {
        #[rustfmt::skip]
        let a = i16x8::new(1, 2, 3, 4, 1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i16x8::new(4, 3, 2, 1, 4, 3, 2, 1);
        #[rustfmt::skip]
        let r = i16x8::new(4, 2, 4, 2, 1, 3, 1, 3);

        assert_eq!(
            r,
            mem::transmute(__msa_pckev_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_pckev_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i32x4::new(4, 3, 2, 1);
        #[rustfmt::skip]
        let r = i32x4::new(4, 2, 1, 3);

        assert_eq!(
            r,
            mem::transmute(__msa_pckev_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_pckev_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(4, 3);
        #[rustfmt::skip]
        let r = i64x2::new(4, 1);

        assert_eq!(
            r,
            mem::transmute(__msa_pckev_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_pckod_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            3, 1, 3, 1,
            3, 1, 3, 1,
            2, 4, 2, 4,
            2, 4, 2, 4
        );

        assert_eq!(
            r,
            mem::transmute(__msa_pckod_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_pckod_h() {
        #[rustfmt::skip]
        let a = i16x8::new(1, 2, 3, 4, 1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i16x8::new(4, 3, 2, 1, 4, 3, 2, 1);
        #[rustfmt::skip]
        let r = i16x8::new(3, 1, 3, 1, 2, 4, 2, 4);

        assert_eq!(
            r,
            mem::transmute(__msa_pckod_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_pckod_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i32x4::new(4, 3, 2, 1);
        #[rustfmt::skip]
        let r = i32x4::new(3, 1, 2, 4);

        assert_eq!(
            r,
            mem::transmute(__msa_pckod_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_pckod_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(4, 3);
        #[rustfmt::skip]
        let r = i64x2::new(3, 2);

        assert_eq!(
            r,
            mem::transmute(__msa_pckod_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_pcnt_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -128, -64, -32, -16,
            -8, -4, -2, -1,
            1, 2, 4, 8,
            16, 32, 64, 127
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            1, 1, 1, 1,
            1, 1, 1, 7
        );

        assert_eq!(r, mem::transmute(__msa_pcnt_b(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_pcnt_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            -32768, -16384, -8192, -4096,
            4096, 8192, 16384, 32767
        );
        #[rustfmt::skip]
        let r = i16x8::new(1, 2, 3, 4, 1, 1, 1, 15);

        assert_eq!(r, mem::transmute(__msa_pcnt_h(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_pcnt_w() {
        #[rustfmt::skip]
        let a = i32x4::new(
            i32::MIN, -1073741824,
            1073741824, i32::MAX
        );
        #[rustfmt::skip]
        let r = i32x4::new(1, 2, 1, 31);

        assert_eq!(r, mem::transmute(__msa_pcnt_w(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_pcnt_d() {
        #[rustfmt::skip]
        let a = i64x2::new(-2147483648, 2147483647);
        #[rustfmt::skip]
        let r = i64x2::new(33, 31);

        assert_eq!(r, mem::transmute(__msa_pcnt_d(mem::transmute(a))));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sat_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            i8::MAX, 105, 30, 1,
            i8::MAX, 105, 30, 1,
            i8::MAX, 105, 30, 1,
            i8::MAX, 105, 30, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            3, 3, 3, 1,
            3, 3, 3, 1,
            3, 3, 3, 1,
            3, 3, 3, 1
        );

        assert_eq!(r, mem::transmute(__msa_sat_s_b(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sat_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            i16::MAX, 1155, 155, 1,
            i16::MAX, 1155, 155, 1
        );
        #[rustfmt::skip]
        let r = i16x8::new(127, 127, 127, 1, 127, 127, 127, 1);

        assert_eq!(r, mem::transmute(__msa_sat_s_h(mem::transmute(a), 7)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sat_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(i32::MAX, 111111155, i32::MAX, 1);
        #[rustfmt::skip]
        let r = i32x4::new(131071, 131071, 131071, 1);

        assert_eq!(r, mem::transmute(__msa_sat_s_w(mem::transmute(a), 17)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sat_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(i64::MAX, 1);
        #[rustfmt::skip]
        let r = i64x2::new(137438953471, 1);

        assert_eq!(r, mem::transmute(__msa_sat_s_d(mem::transmute(a), 37)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sat_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            u8::MAX, 105, 30, 1,
            u8::MAX, 105, 30, 1,
            u8::MAX, 105, 30, 1,
            u8::MAX, 105, 30, 1
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            7, 7, 7, 1,
            7, 7, 7, 1,
            7, 7, 7, 1,
            7, 7, 7, 1
        );

        assert_eq!(r, mem::transmute(__msa_sat_u_b(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sat_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(
            u16::MAX, 1155, 155, 1,
            u16::MAX, 1155, 155, 1
        );
        #[rustfmt::skip]
        let r = u16x8::new(255, 255, 155, 1, 255, 255, 155, 1);

        assert_eq!(r, mem::transmute(__msa_sat_u_h(mem::transmute(a), 7)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sat_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(u32::MAX, 111111155, u32::MAX, 1);
        #[rustfmt::skip]
        let r = u32x4::new(262143, 262143, 262143, 1);

        assert_eq!(r, mem::transmute(__msa_sat_u_w(mem::transmute(a), 17)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sat_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(u64::MAX, 1);
        #[rustfmt::skip]
        let r = u64x2::new(274877906943, 1);

        assert_eq!(r, mem::transmute(__msa_sat_u_d(mem::transmute(a), 37)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_shf_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            11, 12, 3, 4,
            11, 12, 3, 4,
            11, 12, 3, 4,
            11, 12, 3, 4
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            11, 3, 4, 12,
            11, 3, 4, 12,
            11, 3, 4, 12,
            11, 3, 4, 12
        );

        assert_eq!(r, mem::transmute(__msa_shf_b(mem::transmute(a), 120)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_shf_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            11, 12, 13, 14,
            11, 12, 13, 14
        );
        #[rustfmt::skip]
        let r = i16x8::new(11, 14, 12, 13, 11, 14, 12, 13);

        assert_eq!(r, mem::transmute(__msa_shf_h(mem::transmute(a), 156)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_shf_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let r = i32x4::new(1, 3, 2, 4);

        assert_eq!(r, mem::transmute(__msa_shf_w(mem::transmute(a), 216)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sld_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            0, 1, 2, 3,
            4, 5, 6, 7,
            8, 9, 10, 11,
            12, 13, 14, 15
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            16, 17, 18, 19,
            20, 21, 22, 23,
            24, 25, 26, 27,
            28, 29, 30, 31
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            21, 22, 23, 24,
            25, 26, 27, 28,
            29, 30, 31, 0,
            1, 2, 3, 4
        );

        assert_eq!(
            r,
            mem::transmute(__msa_sld_b(mem::transmute(a), mem::transmute(b), 5))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sld_h() {
        #[rustfmt::skip]
        let a = i16x8::new(0, 1, 2, 3, 4, 5, 6, 7);
        #[rustfmt::skip]
        let b = i16x8::new(8, 9, 10, 11, 12, 13, 14, 15);
        // let c = 5 as i32;
        let r = i16x8::new(9, 10, 11, 0, 13, 14, 15, 4);

        assert_eq!(
            r,
            mem::transmute(__msa_sld_h(mem::transmute(a), mem::transmute(b), 2))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sld_w() {
        #[rustfmt::skip]
        let a = i32x4::new(0, 1, 2, 3);
        #[rustfmt::skip]
        let b = i32x4::new(4, 5, 6, 7);
        #[rustfmt::skip]
        let r = i32x4::new(4, 5, 6, 7);

        assert_eq!(
            r,
            mem::transmute(__msa_sld_w(mem::transmute(a), mem::transmute(b), 4))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sld_d() {
        #[rustfmt::skip]
        let a = i64x2::new(0, 1);
        #[rustfmt::skip]
        let b = i64x2::new(2, 3);
        #[rustfmt::skip]
        let r = i64x2::new(2, 3);

        assert_eq!(
            r,
            mem::transmute(__msa_sld_d(mem::transmute(a), mem::transmute(b), 2))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sldi_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            0, 1, 2, 3,
            4, 5, 6, 7,
            8, 9, 10, 11,
            12, 13, 14, 15
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            16, 17, 18, 19,
            20, 21, 22, 23,
            24, 25, 26, 27,
            28, 29, 30, 31
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            21, 22, 23, 24,
            25, 26, 27, 28,
            29, 30, 31, 0,
            1, 2, 3, 4
        );

        assert_eq!(
            r,
            mem::transmute(__msa_sldi_b(mem::transmute(a), mem::transmute(b), 5))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sldi_h() {
        #[rustfmt::skip]
        let a = i16x8::new(0, 1, 2, 3, 4, 5, 6, 7);
        #[rustfmt::skip]
        let b = i16x8::new(8, 9, 10, 11, 12, 13, 14, 15);
        // let c = 5 as i32;
        let r = i16x8::new(9, 10, 11, 0, 13, 14, 15, 4);

        assert_eq!(
            r,
            mem::transmute(__msa_sldi_h(mem::transmute(a), mem::transmute(b), 2))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sldi_w() {
        #[rustfmt::skip]
        let a = i32x4::new(0, 1, 2, 3);
        #[rustfmt::skip]
        let b = i32x4::new(4, 5, 6, 7);
        #[rustfmt::skip]
        let r = i32x4::new(4, 5, 6, 7);

        assert_eq!(
            r,
            mem::transmute(__msa_sldi_w(mem::transmute(a), mem::transmute(b), 4))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sldi_d() {
        #[rustfmt::skip]
        let a = i64x2::new(0, 1);
        #[rustfmt::skip]
        let b = i64x2::new(2, 3);
        #[rustfmt::skip]
        let r = i64x2::new(2, 3);

        assert_eq!(
            r,
            mem::transmute(__msa_sldi_d(mem::transmute(a), mem::transmute(b), 2))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sll_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            16, 16, 12, 8,
            16, 16, 12, 8,
            16, 16, 12, 8,
            16, 16, 12, 8
        );

        assert_eq!(
            r,
            mem::transmute(__msa_sll_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sll_h() {
        #[rustfmt::skip]
        let a = i16x8::new(1, 2, 3, 4, 1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i16x8::new(4, 3, 2, 1, 4, 3, 2, 1);
        #[rustfmt::skip]
        let r = i16x8::new(16, 16, 12, 8, 16, 16, 12, 8);

        assert_eq!(
            r,
            mem::transmute(__msa_sll_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sll_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i32x4::new(4, 3, 2, 1);
        #[rustfmt::skip]
        let r = i32x4::new(16, 16, 12, 8);

        assert_eq!(
            r,
            mem::transmute(__msa_sll_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sll_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(4, 3);
        #[rustfmt::skip]
        let r = i64x2::new(16, 16);

        assert_eq!(
            r,
            mem::transmute(__msa_sll_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_slli_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            4, 8, 12, 16,
            4, 8, 12, 16,
            4, 8, 12, 16,
            4, 8, 12, 16
        );

        assert_eq!(r, mem::transmute(__msa_slli_b(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_slli_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            1, 2, 3, 4,
            1, 2, 3, 4  
        );
        #[rustfmt::skip]
        let r = i16x8::new(4, 8, 12, 16, 4, 8, 12, 16);

        assert_eq!(r, mem::transmute(__msa_slli_h(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_slli_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let r = i32x4::new(4, 8, 12, 16);

        assert_eq!(r, mem::transmute(__msa_slli_w(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_slli_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let r = i64x2::new(2, 4);

        assert_eq!(r, mem::transmute(__msa_slli_d(mem::transmute(a), 1)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_splat_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            4, 4, 4, 4,
            4, 4, 4, 4,
            4, 4, 4, 4,
            4, 4, 4, 4
        );

        assert_eq!(r, mem::transmute(__msa_splat_b(mem::transmute(a), 3)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_splat_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
        );
        #[rustfmt::skip]
        let r = i16x8::new(4, 4, 4, 4, 4, 4, 4, 4);

        assert_eq!(r, mem::transmute(__msa_splat_h(mem::transmute(a), 3)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_splat_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let r = i32x4::new(4, 4, 4, 4);

        assert_eq!(r, mem::transmute(__msa_splat_w(mem::transmute(a), 3)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_splat_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let r = i64x2::new(2, 2);

        assert_eq!(r, mem::transmute(__msa_splat_d(mem::transmute(a), 3)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_splati_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            3, 3, 3, 3,
            3, 3, 3, 3,
            3, 3, 3, 3,
            3, 3, 3, 3
        );

        assert_eq!(r, mem::transmute(__msa_splati_b(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_splati_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
        );
        #[rustfmt::skip]
        let r = i16x8::new(3, 3, 3, 3, 3, 3, 3, 3);

        assert_eq!(r, mem::transmute(__msa_splati_h(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_splati_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let r = i32x4::new(3, 3, 3, 3);

        assert_eq!(r, mem::transmute(__msa_splati_w(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_splati_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let r = i64x2::new(2, 2);

        assert_eq!(r, mem::transmute(__msa_splati_d(mem::transmute(a), 1)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sra_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -128, -64, -32, -16,
            -8, -4, -2, -1,
            1, 2, 4, 8,
            16, 32, 64, 127
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            8, 7, 6, 5,
            4, 3, 2, 1,
            8, 7, 6, 5,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -128, -1, -1, -1,
            -1, -1, -1, -1,
            1, 0, 0, 0,
            1, 4, 16, 63
        );

        assert_eq!(
            r,
            mem::transmute(__msa_sra_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sra_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            -32768, -16384, -8192, -4096,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            15, 14, 13, 12,
            12, 13, 14, 15
        );
        #[rustfmt::skip]
        let r = i16x8::new(
            -1, -1, -1, -1,
            0, 0, 0, 0
        );

        assert_eq!(
            r,
            mem::transmute(__msa_sra_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sra_w() {
        #[rustfmt::skip]
        let a = i32x4::new(i32::MIN, -1073741824, 1, 2);
        #[rustfmt::skip]
        let b = i32x4::new(16, 15, 16, 15);
        #[rustfmt::skip]
        let r = i32x4::new(-32768, -32768, 0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_sra_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_sra_d() {
        #[rustfmt::skip]
        let a = i64x2::new(i64::MIN, i64::MAX);
        #[rustfmt::skip]
        let b = i64x2::new(32, 31);
        #[rustfmt::skip]
        let r = i64x2::new(-2147483648, 4294967295);

        assert_eq!(
            r,
            mem::transmute(__msa_sra_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srai_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            i8::MAX, 125, 55, 1,
            i8::MAX, 125, 55, 1,
            i8::MAX, 125, 55, 1,
            i8::MAX, 125, 55, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            31, 31, 13, 0,
            31, 31, 13, 0,
            31, 31, 13, 0,
            31, 31, 13, 0
        );

        assert_eq!(r, mem::transmute(__msa_srai_b(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srai_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            i16::MAX, 125, 55, 1, 
            i16::MAX, 125, 55, 1
        );
        #[rustfmt::skip]
        let r = i16x8::new(8191, 31, 13, 0, 8191, 31, 13, 0);

        assert_eq!(r, mem::transmute(__msa_srai_h(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srai_w() {
        #[rustfmt::skip]
        let a = i32x4::new(i32::MAX, 125, 55, 1);
        let r = i32x4::new(536870911, 31, 13, 0);

        assert_eq!(r, mem::transmute(__msa_srai_w(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srai_d() {
        #[rustfmt::skip]
        let a = i64x2::new(i64::MAX, 55);
        #[rustfmt::skip]
        let r = i64x2::new(2305843009213693951, 13);

        assert_eq!(r, mem::transmute(__msa_srai_d(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srar_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -128, -64, -32, -16,
            -8, -4, -2, -1,
            1, 2, 4, 8,
            16, 32, 64, 127
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            4, 3, 2, 1,
            4, 3, 2, 1,
            8, 7, 6, 5,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -8, -8, -8, -8,
            0, 0, 0, 0,
            1, 0, 0, 0,
            1, 4, 16, 64
        );

        assert_eq!(
            r,
            mem::transmute(__msa_srar_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srar_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            i16::MIN, -16384, -8192, -4096,
            150, 50, 25, 15
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            4, 3, 2, 1,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let r = i16x8::new(
            -2048, -2048, -2048, -2048, 
            75, 13, 3, 1
        );

        assert_eq!(
            r,
            mem::transmute(__msa_srar_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srar_w() {
        #[rustfmt::skip]
        let a = i32x4::new(i32::MIN, -1073741824, 100, 50);
        #[rustfmt::skip]
        let b = i32x4::new(16, 15, 1, 2);
        #[rustfmt::skip]
        let r = i32x4::new(-32768, -32768, 50, 13);

        assert_eq!(
            r,
            mem::transmute(__msa_srar_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srar_d() {
        #[rustfmt::skip]
        let a = i64x2::new(i64::MIN, i64::MAX);
        #[rustfmt::skip]
        let b = i64x2::new(32, 31);
        #[rustfmt::skip]
        let r = i64x2::new(-2147483648, 4294967296);

        assert_eq!(
            r,
            mem::transmute(__msa_srar_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srari_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            125, i8::MAX, 55, 1,
            125, i8::MAX, 55, 1,
            125, i8::MAX, 55, 1,
            125, i8::MAX, 55, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            31, 32, 14, 0,
            31, 32, 14, 0,
            31, 32, 14, 0,
            31, 32, 14, 0
        );

        assert_eq!(r, mem::transmute(__msa_srari_b(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srari_h() {
        #[rustfmt::skip]
        let a = i16x8::new(2155, 1155, 155, 1, 2155, 1155, 155, 1);
        #[rustfmt::skip]
        let r = i16x8::new(539, 289, 39, 0, 539, 289, 39, 0);

        assert_eq!(r, mem::transmute(__msa_srari_h(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srari_w() {
        #[rustfmt::skip]
        let a = i32x4::new(211111155, 111111155, 11111155, 1);
        #[rustfmt::skip]
        let r = i32x4::new(52777789, 27777789, 2777789, 0);

        assert_eq!(r, mem::transmute(__msa_srari_w(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srari_d() {
        #[rustfmt::skip]
        let a = i64x2::new(211111111155, 111111111155);
        #[rustfmt::skip]
        let r = i64x2::new(52777777789, 27777777789);

        assert_eq!(r, mem::transmute(__msa_srari_d(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srl_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -128, -64, -32, -16,
            -8, -4, -2, -1,
            1, 2, 4, 8,
            16, 32, 64, 127
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            8, 7, 6, 5,
            4, 3, 2, 1,
            8, 7, 6, 5,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -128, 1, 3, 7,
            15, 31, 63, 127,
            1, 0, 0, 0,
            1, 4, 16, 63
        );

        assert_eq!(
            r,
            mem::transmute(__msa_srl_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srl_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            -32768, -16384, -8192, -4096,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            15, 14, 13, 12,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i16x8::new(1, 3, 7, 15, 0, 0, 0, 2);

        assert_eq!(
            r,
            mem::transmute(__msa_srl_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srl_w() {
        #[rustfmt::skip]
        let a = i32x4::new(i32::MIN, -1073741824, 1, 2);
        #[rustfmt::skip]
        let b = i32x4::new(16, 15, 16, 15);
        #[rustfmt::skip]
        let r = i32x4::new(32768, 98304, 0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_srl_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srl_d() {
        #[rustfmt::skip]
        let a = i64x2::new(i64::MIN, i64::MAX);
        #[rustfmt::skip]
        let b = i64x2::new(32, 31);
        #[rustfmt::skip]
        let r = i64x2::new(2147483648, 4294967295);

        assert_eq!(
            r,
            mem::transmute(__msa_srl_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srli_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            25, 50, 100, 127,
            25, 50, 100, 127,
            25, 50, 100, 127,
            25, 50, 100, 127
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            6, 12, 25, 31,
            6, 12, 25, 31, 
            6, 12, 25, 31, 
            6, 12, 25, 31
        );

        assert_eq!(r, mem::transmute(__msa_srli_b(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srli_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            i16::MAX, 3276, 100, 127,
            i16::MAX, 3276, 100, 127
        );
        #[rustfmt::skip]
        let r = i16x8::new(
            8191, 819, 25, 31,
            8191, 819, 25, 31
        );

        assert_eq!(r, mem::transmute(__msa_srli_h(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srli_w() {
        #[rustfmt::skip]
        let a = i32x4::new(100, i32::MAX, 100, i32::MAX);
        #[rustfmt::skip]
        let r = i32x4::new(25, 536870911, 25, 536870911);

        assert_eq!(r, mem::transmute(__msa_srli_w(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srli_d() {
        #[rustfmt::skip]
        let a = i64x2::new(100, i64::MAX);
        #[rustfmt::skip]
        let r = i64x2::new(50, 4611686018427387903);

        assert_eq!(r, mem::transmute(__msa_srli_d(mem::transmute(a), 1)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srlr_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            -128, -64, -32, -16,
            -8, -4, -2, -1,
            1, 2, 4, 8,
            16, 32, 64, 127
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            8, 7, 6, 5,
            4, 3, 2, 1,
            8, 7, 6, 5,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            -128, 2, 4, 8,
            16, 32, 64, -128,
            1, 0, 0, 0,
            1, 4, 16, 64
        );

        assert_eq!(
            r,
            mem::transmute(__msa_srlr_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srlr_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            -32768, -16384, -8192, -4096,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            15, 14, 13, 12,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i16x8::new(1, 3, 7, 15, 0, 0, 1, 2);

        assert_eq!(
            r,
            mem::transmute(__msa_srlr_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srlr_w() {
        #[rustfmt::skip]
        let a = i32x4::new(i32::MIN, -1073741824, 1, 2);
        #[rustfmt::skip]
        let b = i32x4::new(16, 15, 16, 15);
        let r = i32x4::new(32768, 98304, 0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_srlr_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srlr_d() {
        #[rustfmt::skip]
        let a = i64x2::new(i64::MIN, i64::MAX);
        #[rustfmt::skip]
        let b = i64x2::new(32, 31);
        #[rustfmt::skip]
        let r = i64x2::new(2147483648, 4294967296);

        assert_eq!(
            r,
            mem::transmute(__msa_srlr_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srlri_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            25, 50, 100, i8::MAX,
            25, 50, 100, i8::MAX,
            25, 50, 100, i8::MAX,
            25, 50, 100, i8::MAX
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            6, 13, 25, 32, 
            6, 13, 25, 32, 
            6, 13, 25, 32, 
            6, 13, 25, 32
        );

        assert_eq!(r, mem::transmute(__msa_srlri_b(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srlri_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            i16::MAX, 3276, 100, 127,
            i16::MAX, 3276, 100, 127
        );
        let r = i16x8::new(8192, 819, 25, 32, 8192, 819, 25, 32);

        assert_eq!(r, mem::transmute(__msa_srlri_h(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srlri_w() {
        #[rustfmt::skip]
        let a = i32x4::new(100, 150, 200, i32::MAX);
        #[rustfmt::skip]
        let r = i32x4::new(25, 38, 50, 536870912);

        assert_eq!(r, mem::transmute(__msa_srlri_w(mem::transmute(a), 2)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_srlri_d() {
        #[rustfmt::skip]
        let a = i64x2::new(100, i64::MAX);
        #[rustfmt::skip]
        let r = i64x2::new(50, 4611686018427387904);

        assert_eq!(r, mem::transmute(__msa_srlri_d(mem::transmute(a), 1)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_st_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            13, 14, 15, 16, 
            17, 18, 19, 20, 
            21, 22, 23, 24, 
            25, 26, 27, 28
        );
        #[rustfmt::skip]
        let mut arr : [i8; 16] = [
            0, 0, 0, 0,
            0, 0, 0, 0,
            0, 0, 0, 0,
            0, 0, 0, 0
        ];
        #[rustfmt::skip]
        let r : [i8; 16] = [
            13, 14, 15, 16, 
            17, 18, 19, 20, 
            21, 22, 23, 24, 
            25, 26, 27, 28
        ];
        __msa_st_b(mem::transmute(a), arr.as_mut_ptr() as *mut u8, 0);
        assert_eq!(arr, r);
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_st_h() {
        #[rustfmt::skip]
        let a = i16x8::new(13, 14, 15, 16, 17, 18, 19, 20);
        let mut arr: [i16; 8] = [0, 0, 0, 0, 0, 0, 0, 0];
        #[rustfmt::skip]
        let r  : [i16; 8] = [13, 14, 15, 16, 17, 18, 19, 20];
        __msa_st_h(mem::transmute(a), arr.as_mut_ptr() as *mut u8, 0);
        assert_eq!(arr, r);
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_st_w() {
        #[rustfmt::skip]
        let a = i32x4::new(13, 14, 15, 16);
        let mut arr: [i32; 4] = [0, 0, 0, 0];
        #[rustfmt::skip]
        let r  : [i32; 4] = [13, 14, 15, 16];
        __msa_st_w(mem::transmute(a), arr.as_mut_ptr() as *mut u8, 0);
        assert_eq!(arr, r);
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_st_d() {
        #[rustfmt::skip]
        let a = i64x2::new(13, 14);
        let mut arr: [i64; 2] = [0, 0];
        #[rustfmt::skip]
        let r : [i64; 2] = [13, 14];
        __msa_st_d(mem::transmute(a), arr.as_mut_ptr() as *mut u8, 0);
        assert_eq!(arr, r);
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subs_s_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            i8::MIN, -2, -3, -4,
            i8::MIN, -2, -3, -4,
            i8::MIN, -2, -3, -4,
            i8::MIN, -2, -3, -4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            6, -7, 8, -9,
            6, -7, 8, -9,
            6, -7, 8, -9,
            6, -7, 8, -9
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            i8::MIN, 5, -11, 5,
            i8::MIN, 5, -11, 5,
            i8::MIN, 5, -11, 5,
            i8::MIN, 5, -11, 5
        );

        assert_eq!(
            r,
            mem::transmute(__msa_subs_s_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subs_s_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            i16::MIN, -2, -3, -4,
            i16::MIN, -2, -3, -4
        );
        #[rustfmt::skip]
        let b = i16x8::new(6, -7, 8, -9, 6, -7, 8, -9);
        #[rustfmt::skip]
        let r = i16x8::new(
            i16::MIN, 5, -11, 5,
            i16::MIN, 5, -11, 5
        );

        assert_eq!(
            r,
            mem::transmute(__msa_subs_s_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subs_s_w() {
        #[rustfmt::skip]
        let a = i32x4::new(i32::MIN, -2, -3, -4);
        #[rustfmt::skip]
        let b = i32x4::new(6, -7, 8, -9);
        #[rustfmt::skip]
        let r = i32x4::new(i32::MIN, 5, -11, 5);

        assert_eq!(
            r,
            mem::transmute(__msa_subs_s_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subs_s_d() {
        #[rustfmt::skip]
        let a = i64x2::new(i64::MIN, -2);
        #[rustfmt::skip]
        let b = i64x2::new(6, -7);
        #[rustfmt::skip]
        let r = i64x2::new(i64::MIN, 5);

        assert_eq!(
            r,
            mem::transmute(__msa_subs_s_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subs_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            u8::MAX, 2, 3, 4,
            u8::MAX, 2, 3, 4,
            u8::MAX, 2, 3, 4,
            u8::MAX, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9,
            6, 7, 8, 9
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            249, 0, 0, 0,
            249, 0, 0, 0,
            249, 0, 0, 0,
            249, 0, 0, 0
        );

        assert_eq!(
            r,
            mem::transmute(__msa_subs_u_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subs_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(
            u16::MAX, 2, 3, 4, 
            u16::MAX, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = u16x8::new(6, 7, 8, 9, 6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u16x8::new(65529, 0, 0, 0, 65529, 0, 0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_subs_u_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subs_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(u32::MAX, 2, 3, 4);
        #[rustfmt::skip]
        let b = u32x4::new(6, 7, 8, 9);
        #[rustfmt::skip]
        let r = u32x4::new(4294967289, 0, 0, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_subs_u_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subs_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(u64::MAX, 2);
        #[rustfmt::skip]
        let b = u64x2::new(6, 7);
        #[rustfmt::skip]
        let r = u64x2::new(18446744073709551609, 0);

        assert_eq!(
            r,
            mem::transmute(__msa_subs_u_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subsus_u_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            u8::MAX, 2, 3, 4,
            u8::MAX, 2, 3, 4,
            u8::MAX, 2, 3, 4,
            u8::MAX, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            -6, -7, -8, -9,
            -6, -7, -8, -9,
            -6, -7, -8, -9,
            -6, -7, -8, -9
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            255, 9, 11, 13,
            255, 9, 11, 13,
            255, 9, 11, 13,
            255, 9, 11, 13
        );

        assert_eq!(
            r,
            mem::transmute(__msa_subsus_u_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subsus_u_h() {
        #[rustfmt::skip]
        let a = u16x8::new(
            u16::MAX, 2, 3, 4,
            u16::MAX, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i16x8::new(-6, -7, -8, -9, -6, -7, -8, -9);
        #[rustfmt::skip]
        let r = u16x8::new(65535, 9, 11, 13, 65535, 9, 11, 13);

        assert_eq!(
            r,
            mem::transmute(__msa_subsus_u_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subsus_u_w() {
        #[rustfmt::skip]
        let a = u32x4::new(u32::MAX, 2, 3, 4);
        #[rustfmt::skip]
        let b = i32x4::new(-6, -7, -8, -9);
        #[rustfmt::skip]
        let r = u32x4::new(4294967295, 9, 11, 13);

        assert_eq!(
            r,
            mem::transmute(__msa_subsus_u_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subsus_u_d() {
        #[rustfmt::skip]
        let a = u64x2::new(u64::MAX, 2);
        #[rustfmt::skip]
        let b = i64x2::new(-6, -7);
        #[rustfmt::skip]
        let r = u64x2::new(18446744073709551615, 9);

        assert_eq!(
            r,
            mem::transmute(__msa_subsus_u_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subsuu_s_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            u8::MAX, 2, 3, 4,
            u8::MAX, 2, 3, 4,
            u8::MAX, 2, 3, 4,
            u8::MAX, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            6, 7, 8, u8::MAX,
            6, 7, 8, u8::MAX,
            6, 7, 8, u8::MAX,
            6, 7, 8, u8::MAX
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            127, -5, -5, -128,
            127, -5, -5, -128,
            127, -5, -5, -128,
            127, -5, -5, -128
        );

        assert_eq!(
            r,
            mem::transmute(__msa_subsuu_s_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subsuu_s_h() {
        #[rustfmt::skip]
        let a = u16x8::new(
            u16::MAX, 2, 3, 
            4, u16::MAX, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = u16x8::new(6, 7, 8, 65535, 6, 7, 8, 65535);
        #[rustfmt::skip]
        let r = i16x8::new(32767, -5, -5, -32768, 32767, -5, -5, -32768);

        assert_eq!(
            r,
            mem::transmute(__msa_subsuu_s_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subsuu_s_w() {
        #[rustfmt::skip]
        let a = u32x4::new(u32::MAX, 2, 3, 4);
        #[rustfmt::skip]
        let b = u32x4::new(6, 7, 8, 4294967295);
        #[rustfmt::skip]
        let r = i32x4::new(2147483647, -5, -5, -2147483648);

        assert_eq!(
            r,
            mem::transmute(__msa_subsuu_s_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subsuu_s_d() {
        #[rustfmt::skip]
        let a = u64x2::new(u64::MAX, 2);
        #[rustfmt::skip]
        let b = u64x2::new(6, 7);
        #[rustfmt::skip]
        let r = i64x2::new(i64::MAX, -5);

        assert_eq!(
            r,
            mem::transmute(__msa_subsuu_s_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subv_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            i8::MIN, -2, -3, -4,
            i8::MIN, -2, -3, -4,
            i8::MIN, -2, -3, -4,
            i8::MIN, -2, -3, -4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            6, -7, 8, -9,
            6, -7, 8, -9,
            6, -7, 8, -9,
            6, -7, 8, -9
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            122, 5, -11, 5,
            122, 5, -11, 5,
            122, 5, -11, 5,
            122, 5, -11, 5
        );

        assert_eq!(
            r,
            mem::transmute(__msa_subv_b(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subv_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            i16::MIN, -2, -3, -4,
            i16::MIN, -2, -3, -4
        );
        #[rustfmt::skip]
        let b = i16x8::new(6, -7, 8, -9, 6, -7, 8, -9);
        #[rustfmt::skip]
        let r = i16x8::new(32762, 5, -11, 5, 32762, 5, -11, 5);

        assert_eq!(
            r,
            mem::transmute(__msa_subv_h(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subv_w() {
        #[rustfmt::skip]
        let a = i32x4::new(i32::MIN, -2, -3, -4);
        #[rustfmt::skip]
        let b = i32x4::new(6, -7, 8, -9);
        #[rustfmt::skip]
        let r = i32x4::new(2147483642, 5, -11, 5);

        assert_eq!(
            r,
            mem::transmute(__msa_subv_w(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subv_d() {
        #[rustfmt::skip]
        let a = i64x2::new(i64::MAX, -2);
        #[rustfmt::skip]
        let b = i64x2::new(6, -7);
        #[rustfmt::skip]
        let r = i64x2::new(9223372036854775801, 5);

        assert_eq!(
            r,
            mem::transmute(__msa_subv_d(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subvi_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            100, i8::MAX, 50, i8::MIN,
            100, i8::MAX, 50, i8::MIN,
            100, i8::MAX, 50, i8::MIN,
            100, i8::MAX, 50, i8::MIN
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            95, 122, 45, 123,
            95, 122, 45, 123,
            95, 122, 45, 123,
            95, 122, 45, 123
        );

        assert_eq!(r, mem::transmute(__msa_subvi_b(mem::transmute(a), 5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subvi_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            i16::MAX, 3276, -100, i16::MIN,
            i16::MAX, 3276, -100, i16::MIN
        );
        #[rustfmt::skip]
        let r = i16x8::new(
            32762, 3271, -105, 32763,
            32762, 3271, -105, 32763
        );

        assert_eq!(r, mem::transmute(__msa_subvi_h(mem::transmute(a), 5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subvi_w() {
        #[rustfmt::skip]
        let a = i32x4::new(100, 150, 200, i32::MAX);
        #[rustfmt::skip]
        let r = i32x4::new(95, 145, 195, 2147483642);

        assert_eq!(r, mem::transmute(__msa_subvi_w(mem::transmute(a), 5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_subvi_d() {
        #[rustfmt::skip]
        let a = i64x2::new(100, i64::MAX);
        #[rustfmt::skip]
        let r = i64x2::new(95, 9223372036854775802);

        assert_eq!(r, mem::transmute(__msa_subvi_d(mem::transmute(a), 5)));
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_vshf_b() {
        #[rustfmt::skip]
        let a = i8x16::new(
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i8x16::new(
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let c = i8x16::new(
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = i8x16::new(
            3, 2, 1, 4, 
            3, 2, 1, 4, 
            3, 2, 1, 4, 
            3, 2, 1, 4
        );

        assert_eq!(
            r,
            mem::transmute(__msa_vshf_b(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_vshf_h() {
        #[rustfmt::skip]
        let a = i16x8::new(
            1, 2, 3, 4,
            1, 2, 3, 4
        );
        #[rustfmt::skip]
        let b = i16x8::new(
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let c = i16x8::new(
            4, 3, 2, 1,
            4, 3, 2, 1
        );
        let r = i16x8::new(3, 2, 1, 4, 3, 2, 1, 4);

        assert_eq!(
            r,
            mem::transmute(__msa_vshf_h(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_vshf_w() {
        #[rustfmt::skip]
        let a = i32x4::new(1, 2, 3, 4);
        #[rustfmt::skip]
        let b = i32x4::new(4, 3, 2, 1);
        #[rustfmt::skip]
        let c = i32x4::new(4, 3, 2, 1);
        #[rustfmt::skip]
        let r = i32x4::new(3, 2, 1, 4);

        assert_eq!(
            r,
            mem::transmute(__msa_vshf_w(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_vshf_d() {
        #[rustfmt::skip]
        let a = i64x2::new(1, 2);
        #[rustfmt::skip]
        let b = i64x2::new(4, 3);
        #[rustfmt::skip]
        let c = i64x2::new(4, 3);
        #[rustfmt::skip]
        let r = i64x2::new(3, 4);

        assert_eq!(
            r,
            mem::transmute(__msa_vshf_d(
                mem::transmute(a),
                mem::transmute(b),
                mem::transmute(c)
            ))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_xor_v() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            9, 10, 11, 12,
            13, 14, 15, 16
        );
        #[rustfmt::skip]
        let b = u8x16::new(
            16, 15, 14, 13,
            12, 11, 10, 9,
            8, 7, 6, 5,
            4, 3, 2, 1
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            17, 13, 13, 9, 
            9, 13, 13, 1, 
            1, 13, 13, 9, 
            9, 13, 13, 17
        );

        assert_eq!(
            r,
            mem::transmute(__msa_xor_v(mem::transmute(a), mem::transmute(b)))
        );
    }

    #[simd_test(enable = "msa")]
    unsafe fn test_msa_xori_b() {
        #[rustfmt::skip]
        let a = u8x16::new(
            1, 2, 3, 4,
            5, 6, 7, 8,
            9, 10, 11, 12,
            13, 14, 15, 16
        );
        #[rustfmt::skip]
        let r = u8x16::new(
            5, 6, 7, 0, 
            1, 2, 3, 12, 
            13, 14, 15, 8, 
            9, 10, 11, 20
        );

        assert_eq!(r, mem::transmute(__msa_xori_b(mem::transmute(a), 4)));
    }
}
